;
; IDL Event Callback Procedures
; ID_GUI_eventcb
;
; Generated on:	08/24/2011 10:21.03
;
;	to call this package from another, do the following in the
;	calling routine:
;
;		my_path = PTR_NEW(image_path_on_disk)
;		ID_GUI, _EXTRA={ptr_path: my_path}
;		.
;		.   when done:
;		PTR_FREE, my_path
;
;-----------------------------------------------------------------
;
pro ID_PostCreation, wWidget, _EXTRA=path_ptr
	;
	;	_VWBExtra_ may contain a structure (containing a pointer) to the image pathname on disk
	;
	test1 = N_ELEMENTS(path_ptr)
	if(test1 gt 0) then begin
		test = SIZE(path_ptr,/TYPE)
		if(test eq 8) then ptr_path = path_ptr.ptr_path else ptr_path = PTR_NEW()
	endif else begin
		ptr_path = PTR_NEW()
	endelse
    ;
    ;   set up and clear summary structure
    ;
    ;
    id_struct=	   {image_name:       					'',     $         ;
    				ptr_path:					  ptr_path,		$		  ; contains either a null pointer or the pointer to the file pathname
                    ncol:       						0L,     $         :
                    nrow:		        				0L,  	$         ;
                    ngrid:        						0L,     $         ;
                    grid_radius:    					0L,     $	      ;
                    fit_order:							4L,		$			; actual order fitted
                    ;
                    ;	pointers to total number of grid points and the list of x,y positions
                    ;
                    npts:				   				-1L,		$			; number of xy points on list
                    xy_points:	   			lonarr(2500,3), 	$			; up to 50x50 points allowed - 1st: x, 2nd: y, 3rd: 0 not used, 1 use, -1 deleted
                    ;
                    ;	pointers to image, model, and difference arrays
                    ;
                    ptr_model:					PTR_NEW(),		$			; will set size when needed
                    ptr_image:					PTR_NEW(),		$
                    ptr_diff:					PTR_NEW(),		$
                    ptr_header:					PTR_NEW(),		$
                    ;
                    ;
                    ;
                    id_draw_window_wid:					0L, 	$
                    id_draw_xscr:						0L, 	$			; window size in x
                    id_draw_yscr:						0L,		$			; window size in y
                    id_draw_x:							0.0,		$			; returned cursor x value in image pixel coordinates
                    id_draw_y:							0.0,		$
                    id_mouse_x_last:					0L,		$
                    id_mouse_y_last:					0L,		$
                    id_draw_window_index:				0L,		$
                    ;
                    id_show_points_checkbox_wid:		0L,		$
                    id_show_points_flag:				1L,		$
                    ;
                    id_image_name_textbox_wid:			0L,		$
                    id_image_size_textbox_wid:			0L,		$
                    ;
                    id_offset_slider_wid:				0L,		$
                    offset_slider_low:				  -2.0,		$			; dex equivalent to '0'
                    offset_slider_high:				  +6.0,		$			; dex equivalent to '100'
                    offset_slider_value:			  50.0,		$			; current value
                    ;
                    id_scale_slider_wid:				0L,		$
                    scale_slider_low:				   0.5,		$
                    scale_slider_high:				   5.0,		$
                    scale_slider_value:				  17.0,		$
                    ;
                    id_ngrid_slider_wid:				0L,		$
                    ngrid_slider_low:					1L,		$
                    ngrid_slider_high:				   50L,		$
                    ngrid_slider_value:					7L,		$
                    ;
                    id_grid_radius_slider_wid:			0L,		$
                    radius_slider_low:					1L,		$
                    radius_slider_high:				  400L,		$
                    radius_slider_value:			   30L,     $
                    ;
                    id_max_order_slider_wid:			0L, 	$
                    max_order_slider_low:				0L,		$
                    max_order_slider_high:				4L,		$
                    max_order_slider_value:				4L,		$
                    ;
                    id_read_grid_file_button_wid:		0L,		$
                    ;
                    id_generate_grid_button_wid:		0L,		$
                    id_delete_point_button_wid:			0L,		$
                    id_add_point_button_wid:			0L, 	$
                    id_undelete_point_button_wid:		0L, 	$
                    id_reset_all_deletes_button_wid:	0L,		$
                    ;
                    id_remove_all_points_button_wid:	0L, 	$
                    ;
                    id_display_image_button_wid:		0L, 	$
                    ;

                    id_fit_model_button_wid:			0L,		$
                    ;
                    id_display_model_button_wid:		0L, 	$
                    id_display_difference_button_wid:	0L, 	$
                    id_display_flag:					0L, 	$	; 0=none, 1=image, 2=model, 3=difference
                    ;
                    id_display_grid_button_wid:			0L, 	$
                    id_save_image_wid:					0L, 	$
                    ;
                    id_bottom_textbox_wid:				0L,		$
                    id_exit_bad_button_wid:				0L, 	$
                    id_exit_good_button_wid:			0L 	$
                    ;
                    }
    ;
    PTR_ID = PTR_NEW(id_struct,/NO_COPY)
    ;
    ;	load the pointer(s) into the pointer structure
    ;
    WIDGET_CONTROL, wWidget, SET_UVALUE=PTR_ID
end
;
;-----------------------------------------------------------------
pro ID_Realize, wWidget
	;
	;	get key window information
	;
	WIDGET_CONTROL, wWidget, GET_UVALUE=PTR_ID
	;
	;	get and save all the wids
	;
	event = {top: wWidget}
	ID_get_wids, event
	;
	;	set the slider values on the GUIs
	;
	WIDGET_CONTROL, (*PTR_ID).id_offset_slider_wid, SET_VALUE=(*PTR_ID).offset_slider_value
	;WIDGET_CONTROL, (*PTR_ID).id_bottom_textbox_wid, SET_VALUE="Offset Slider set to "+STRING((*PTR_ID).offset_slider_value,format='(i4)'), /APPEND
	;
	WIDGET_CONTROL, (*PTR_ID).id_scale_slider_wid, SET_VALUE=(*PTR_ID).scale_slider_value
	;WIDGET_CONTROL, (*PTR_ID).id_bottom_textbox_wid, SET_VALUE="Scale Slider set to "+STRING((*PTR_ID).scale_slider_value,format='(i4)'), /APPEND
	;
	WIDGET_CONTROL, (*PTR_ID).id_max_order_slider_wid, SET_VALUE=(*PTR_ID).max_order_slider_value
	;WIDGET_CONTROL, (*PTR_ID).id_bottom_textbox_wid, SET_VALUE="Maximum Fit Order Slider set to "+STRING((*PTR_ID).max_order_slider_value,format='(i4)'), /APPEND
	;
	WIDGET_CONTROL, (*PTR_ID).id_ngrid_slider_wid, SET_VALUE=(*PTR_ID).ngrid_slider_value
	;WIDGET_CONTROL, (*PTR_ID).id_bottom_textbox_wid, SET_VALUE="N_Grid Slider set to "+STRING((*PTR_ID).ngrid_slider_value,format='(i4)'), /APPEND
	;
	WIDGET_CONTROL, (*PTR_ID).id_grid_radius_slider_wid, SET_VALUE=(*PTR_ID).radius_slider_value
	;WIDGET_CONTROL, (*PTR_ID).id_bottom_textbox_wid, SET_VALUE="Radius Slider set to "+STRING((*PTR_ID).radius_slider_value,format='(i4)'), /APPEND
	;
	for is = 0, 4 do begin
		redisplay_flag = 0
		ID_Change_Slider, Event, is, redisplay_flag
	endfor
	;
	;	set the checkbox
	;
	WIDGET_CONTROL, (*PTR_ID).id_show_points_checkbox_wid, SET_BUTTON=(*PTR_ID).id_show_points_flag
	;
	;	desensitize all the cursor-driven action buttons
	;
	WIDGET_CONTROL, (*PTR_ID).id_delete_point_button_wid, SENSITIVE=0
    WIDGET_CONTROL, (*PTR_ID).id_add_point_button_wid, SENSITIVE=0
    WIDGET_CONTROL, (*PTR_ID).id_undelete_point_button_wid, SENSITIVE=0
    WIDGET_CONTROL, (*PTR_ID).id_reset_all_deletes_button_wid, SENSITIVE=0
    WIDGET_CONTROL, (*PTR_ID).id_remove_all_points_button_wid, SENSITIVE=0
	;
	;	set up the draw window
	;
	WIDGET_CONTROL, (*PTR_ID).id_draw_window_wid, GET_VALUE=win_ind
    window_geo = WIDGET_INFO( (*PTR_ID).id_draw_window_wid, /GEOMETRY)
    ;
    (*PTR_ID).id_draw_xscr = window_geo.scr_xsize
    (*PTR_ID).id_draw_yscr = window_geo.scr_ysize
    ;
	WSET, win_ind
	(*PTR_ID).id_draw_window_index = win_ind
	;
	device, decomposed=0
	loadct, 39, /silent					; 0 = BW, 3 = BB, 39 = Rainbow+white
	tvlct, [0,255,0,170,255],[0,255,255,170,0],[0,255,0,170,0],0
	black=0
	white=1
	;
	;	message to text box
	;
	WIDGET_CONTROL, (*PTR_ID).id_bottom_textbox_wid, SET_VALUE="Image Display GUI - Initialized!", /APPEND
	ttl = WIDGET_INFO( (*PTR_ID).id_bottom_textbox_wid, /TEXT_TOP_LINE)
			WIDGET_CONTROL, (*PTR_ID).id_bottom_textbox_wid, SET_TEXT_TOP_LINE = ttl+6
	;
	;	if the ptr_path is NOT a null pointer, load the image and display it (otherwise just start)
	;
	test = PTR_VALID( (*PTR_ID).ptr_path )
	if(test eq 1) then begin
		;
		;	valid pointer found, get path
		;
		(*PTR_ID).image_name = (*(*PTR_ID).ptr_path)
		;
		(*PTR_ID).ptr_image = PTR_NEW(readfits((*PTR_ID).image_name, header))
		(*PTR_ID).ptr_header = PTR_NEW(header)
		;
		WIDGET_CONTROL, (*PTR_ID).id_image_name_textbox_wid, SET_VALUE=FILE_BASENAME((*PTR_ID).image_name)
		;
		;	get image size
		;
		isize = SIZE( (*(*PTR_ID).ptr_image) )
		(*PTR_ID).ncol = isize[1]
		(*PTR_ID).nrow = isize[2]
		;
		WIDGET_CONTROL, (*PTR_ID).id_image_size_textbox_wid, SET_VALUE=STRING( isize[1], isize[2], format='(i4,",",i4)')
		;
		(*PTR_ID).id_display_flag = 1
		;
		ID_Display_Image, Event, (*PTR_ID).id_display_flag
	endif
end
;
;-------------------------------------------------------------------
pro ID_Exit, Event, igood
	;
	;	if igood = 1, this is a 'good' exit, so save points
	;	to a save directory
	;
	;	otherwise, return a null pointer
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_ID
	;
	;free the pointers
	;
	;		first the ones contained in structures
	;
	PTR_FREE, (*PTR_ID).ptr_image
	PTR_FREE, (*PTR_ID).ptr_model
	PTR_FREE, (*PTR_ID).ptr_diff
	PTR_FREE, (*PTR_ID).ptr_header
	;
	;	put the igood mode into the same heap variable
	;	that brought in the image path
	;
	(*(*PTR_ID).ptr_path) = STRING(igood,format='(i1)')
	;
	if(igood eq 1) then begin
		;
		;	write points to a file
		;
		;		first, find origin of input file name: if "S7_*" then
		;		look for or create S11_Full_Field_Polarimetry directory
		;		if not found, put into current directory
		;
		dir_in = FILE_DIRNAME( (*PTR_ID).image_name )
		file_in = FILE_BASENAME( (*PTR_ID).image_name )
		;
		; get header
		;
		this_header = HEADFITS(dir_in + PATH_SEP() + file_in)
		;
		;		create output filename
		;
		test_fits = STRPOS( STRUPCASE(file_in), '.FITS')
		if(test_fits ne -1) then begin
			new_file = STRMID( file_in, 0, test_fits) + '_bg_points.dat'
		endif else begin
			WIDGET_CONTROL, (*PTR_ID).id_bottom_textbox_wid, SET_VALUE="ERROR - Could not parse filename", /APPEND
    		ttl = WIDGET_INFO( (*PTR_ID).id_bottom_textbox_wid, /TEXT_TOP_LINE)
			WIDGET_CONTROL, (*PTR_ID).id_bottom_textbox_wid, SET_TEXT_TOP_LINE = ttl+1
			goto, do_not_write
		endelse
		;
		;		and directory
		;
		test_S7 = STRPOS( dir_in, 'S7_' )
		if(test_S7 ne -1) then begin
			new_dir = STRMID( dir_in, 0, test_S7 ) + 'S11_Full_Field_Polarimetry'
			test = FILE_INFO(new_dir)
			if(test.exists eq 1 and test.directory eq 1) then begin
				dir_out = new_dir
			endif else begin
				FILE_MKDIR, new_dir
				dir_out = new_dir
			endelse
		endif else begin
			dir_out = dir_in
		endelse
		;
		path_out = dir_out + PATH_SEP() + new_file
		;
		;	write file of grid points, with embedded astrometry information
		;
		openw, lu_out, path_out, /GET_LUN
		;
		;print, 'Successfully read '+(*PTR_ID).image_name+' at line 264 of ID_GUI_eventcb'
		;
		EXTAST, this_header, this_astr, noparams
		if(noparams eq -1) then stop
		;
		; make_astr, astr,CD = cd, crval = crval1, crpix = crpix1+1, ctype = ['RA---','DEC--'] + 'TAN'
		ast_header = strarr(1)
		putast, ast_header, this_astr, equi=2000.0,cd_type=2
		;
		ind = WHERE( (*PTR_ID).xy_points[*,2] eq 1, np)
		printf, lu_out, ';; Background points file written by PPOL S11 on '+SYSTIME()
		printf, lu_out, ';;    npts, radius, fit order, the points (as i, x, y)
		printf, lu_out, np,format='(";NPTS    =                   ",i4)'
		printf, lu_out, (*PTR_ID).grid_radius, format='(";GRIDRADI=                   ",i4)'
		printf, lu_out, (*PTR_ID).fit_order,format='(";FITORDER=                   ",i4)'
		;
		; add the astrometry stuff
		;
		nast = N_ELEMENTS(ast_header)
		for iast = 0, nast-1 do begin
		  printf, lu_out, ast_header[iast], format='(";",a)'
		endfor
		;
		; then, the individual grid points
		for i = 0, np-1 do begin
			j = ind[i]
			printf, lu_out, i, (*PTR_ID).xy_points[j,0],(*PTR_ID).xy_points[j,1],format='(i4,",",i5,",",i5)'
		endfor
		;
		;	close the file
		;
		FREE_LUN, lu_out
		;
		WIDGET_CONTROL, (*PTR_ID).id_bottom_textbox_wid, SET_VALUE="Wrote file "+new_file, /APPEND
		WIDGET_CONTROL, (*PTR_ID).id_bottom_textbox_wid, SET_VALUE="  To Directory "+dir_out, /APPEND
    	ttl = WIDGET_INFO( (*PTR_ID).id_bottom_textbox_wid, /TEXT_TOP_LINE)
		WIDGET_CONTROL, (*PTR_ID).id_bottom_textbox_wid, SET_TEXT_TOP_LINE = ttl+1
	endif
	;
	do_not_write:
	;
	;	and then the full structure pointer
	;
	PTR_FREE, PTR_ID	;
	;
	HEAP_GC
	;
	WIDGET_CONTROL, Event.top, /DESTROY
end
;
;-----------------------------------------------------------
pro ID_get_wids, Event
	;
	;	get and save all the wids for the GUI
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_ID
	;
	(*PTR_ID).id_draw_window_wid					= WIDGET_INFO( Event.top, Find_by_uname="ID_Draw_Window")
	;
	(*PTR_ID).id_offset_slider_wid 				= WIDGET_INFO( Event.top, Find_by_uname="ID_Image_Offset_Slider")
    (*PTR_ID).id_scale_slider_wid				= WIDGET_INFO( Event.top, Find_by_uname="ID_Image_Scale_Slider")
    (*PTR_ID).id_ngrid_slider_wid				= WIDGET_INFO( Event.top, Find_by_uname="ID_NGrid_Slider")
    (*PTR_ID).id_grid_radius_slider_wid			= WIDGET_INFO( Event.top, Find_by_uname="ID_Grid_Radius_Slider")
    (*PTR_ID).id_max_order_slider_wid			= WIDGET_INFO( Event.top, Find_by_uname="ID_Max_Fit_Order_Slider")
    ;
    (*PTR_ID).id_delete_point_button_wid		= WIDGET_INFO( Event.top, Find_by_uname="ID_Delete_Grid_Point_Button")
    (*PTR_ID).id_add_point_button_wid			= WIDGET_INFO( Event.top, Find_by_uname="ID_Add_Point_Button")
    (*PTR_ID).id_undelete_point_button_wid		= WIDGET_INFO( Event.top, Find_by_uname="ID_Undelete_Point_Button")
    (*PTR_ID).id_reset_all_deletes_button_wid	= WIDGET_INFO( Event.top, Find_by_uname="ID_Reset_All_Deleted_Button")
    (*PTR_ID).id_remove_all_points_button_wid			= WIDGET_INFO( Event.top, Find_by_uname="ID_Remove_All_Points_Button")
    ;
    (*PTR_ID).id_display_image_button_wid		= WIDGET_INFO( Event.top, Find_by_uname="ID_Redisplay_Image_Button")
    (*PTR_ID).id_fit_model_button_wid			= WIDGET_INFO( Event.top, Find_by_uname="ID_Fit_Background_Button")
    (*PTR_ID).id_display_model_button_wid		= WIDGET_INFO( Event.top, Find_by_uname="ID_Display_Background_Button")
    (*PTR_ID).id_display_difference_button_wid	= WIDGET_INFO( Event.top, Find_by_uname="ID_Display_Image_Minus_Background_Button")
    (*PTR_ID).id_save_image_wid					= WIDGET_INFO( Event.top, Find_by_uname="ID_Save_Image_Button")
    (*PTR_ID).id_exit_bad_button_wid			= WIDGET_INFO( Event.top, Find_by_uname="ID_Exit_No_Background_Button")
    (*PTR_ID).id_exit_good_button_wid			= WIDGET_INFO( Event.top, Find_by_uname="ID_Exit_With_Background_Button")
    ;
    (*PTR_ID).id_image_name_textbox_wid			= WIDGET_INFO( Event.top, Find_by_uname="ID_Image_Name_Textbox")
    (*PTR_ID).id_image_size_textbox_wid			= WIDGET_INFO( Event.top, Find_by_uname="ID_Image_Size_Textbox")
    (*PTR_ID).id_bottom_textbox_wid				= WIDGET_INFO( Event.top, Find_by_uname="ID_Bottom_Textbox")
    (*PTR_ID).id_generate_grid_button_wid		= WIDGET_INFO( Event.top, Find_by_uname="ID_Generate_Grid_Button")
    (*PTR_ID).id_read_grid_file_button_wid		= WIDGET_INFO( Event.top, Find_by_uname="ID_Read_Grid_File_Button")
    (*PTR_ID).id_show_points_checkbox_wid		= WIDGET_INFO( Event.top, Find_by_uname="ID_Show_Points_Checkbox")
    ;
    WIDGET_CONTROL, (*PTR_ID).id_bottom_textbox_wid, SET_VALUE="WIDs all set up", /APPEND
    ttl = WIDGET_INFO( (*PTR_ID).id_bottom_textbox_wid, /TEXT_TOP_LINE)
	WIDGET_CONTROL, (*PTR_ID).id_bottom_textbox_wid, SET_TEXT_TOP_LINE = ttl+1
end
;
;----------------------------- Event Handler
pro ID_Handler, Event
	;
	;	figure out which button was pushed
	;
	buttons = [		"ID_Image_Offset_Slider",		$	; done
    				"ID_Image_Scale_Slider",		$	; done
    				"ID_NGrid_Slider",				$	; done
    				"ID_Grid_Radius_Slider",		$	; done
    				"ID_Max_Fit_Order_Slider",		$	; done
					;
					"ID_Generate_Grid_Button",		$	; done
    				"ID_Delete_Grid_Point_Button",	$	; done
    				"ID_Add_Point_Button",			$	; done
    				"ID_Undelete_Point_Button",		$	; done
    				"ID_Reset_All_Deleted_Button",	$	; done
    				"ID_Remove_All_Points_Button",	$	; done

    				"ID_Redisplay_Image_Button",	$	; done
    				"ID_Display_Background_Button",	$
    				"ID_Display_Image_Minus_Background_Button",	$
    				"ID_Show_Points_Checkbox",		$	; done

    				"ID_Fit_Background_Button",		$
    				"ID_Save_Image_Button",			$	; done

					"ID_Draw_Window",				$	; done, MOUSE BUTTON GENERATED EVENTS
    				"ID_Exit_No_Background_Button",	$
    				"ID_Exit_With_Background_Button", $
    				"ID_Read_Grid_File_Button"		$   ; brings up another selector for current file or another file
			   ]
	wid_button = Event.id
	n_buttons = N_ELEMENTS(buttons)
	button_wids = lonarr(n_buttons)
	for iw = 0, n_buttons-1 do button_wids[iw] = WIDGET_INFO(Event.top, FIND_BY_UNAME=buttons[iw])
	;
	;print, 'Entered ID_Handler with wid_button = ',wid_button
	;
	ind = WHERE(button_wids[*] eq wid_button, count)
	if(count ne 0) then begin
		case wid_button of
			button_wids[0]: ID_Change_Slider, Event, 0, 1		; type, redraw flag
			button_wids[1]: ID_Change_Slider, Event, 1, 1
			button_wids[2]: ID_Change_Slider, Event, 2, 0
			button_wids[3]: ID_Change_Slider, Event, 3, 0
			button_wids[4]: ID_Change_Slider, Event, 4, 0
			;
			button_wids[5]: ID_Generate_Grid, Event
			button_wids[6]: ID_Change_Point, Event, 0			; 0 = single point, 1 = undelete one point; 2 = undelete all; 3 = clear all
			button_wids[7]: ID_Add_Point, Event
			button_wids[8]: ID_Change_Point, Event, 1
			button_wids[9]: ID_Change_Point, Event, 2
			button_wids[10]: ID_Change_Point, Event, 3
			;
			button_wids[11]: ID_Display_Image, Event, 1
			button_wids[12]: ID_Display_Image, Event, 2
			button_wids[13]: ID_Display_Image, Event, 3
			button_wids[14]: ID_Toggle_Show_Points, Event
			;
			button_wids[15]: ID_Fit_Background, Event
			button_wids[16]: ID_Write_FITS_Image, Event
			;
			button_wids[17]: ID_Mouse, Event
			button_wids[18]: ID_Exit, Event, 0
			button_wids[19]: ID_Exit, Event, 1
			button_wids[20]: ID_Read_Grid_File, Event

			else: begin
					WIDGET_CONTROL, (*PTR_ID).id_bottom_textbox_wid, SET_VALUE="Error - Image Display GUI button push not recognized", /APPEND
					ttl = WIDGET_INFO( (*PTR_ID).id_bottom_textbox_wid, /TEXT_TOP_LINE)
					WIDGET_CONTROL, (*PTR_ID).id_bottom_textbox_wid, SET_TEXT_TOP_LINE = ttl+1
				  end
	    endcase
	endif else begin
		WIDGET_CONTROL, (*PTR_ID).id_bottom_textbox_wid, SET_VALUE="Image Display GUI - No button pushed.", /APPEND
		ttl = WIDGET_INFO( (*PTR_ID).id_bottom_textbox_wid, /TEXT_TOP_LINE)
		WIDGET_CONTROL, (*PTR_ID).id_bottom_textbox_wid, SET_TEXT_TOP_LINE = ttl+1
	endelse
end
;
;
;-----------------------------------------------------------
pro ID_Read_Grid_File, Event
	;
	;	tries to read an existing grid file already in the S11
	;	directory.
	;
	;	puts up a selector to ask if grid file is for this group or another group
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_ID
	;
	dir_in = FILE_DIRNAME( (*PTR_ID).image_name )
	file_in = FILE_BASENAME( (*PTR_ID).image_name )
	;
	; get header of this image
	;
	this_header = HEADFITS(dir_in + PATH_SEP() + file_in)
	;
	; and this astrometry
	;
	EXTAST, this_header, this_astr, noparams
	if(noparams eq -1) then stop
	;
	;print, 'Successfully read '+(*PTR_ID).image_name+' at line 480 of ID_GUI_eventcb'
	;
	;  ask about current group grid points file or another
	;
	yn = dialog_message(['Use grid point file', 'for this group [Yes],', 'or another group [No]?'],$
	           /QUESTION,Title='Grid Point File Origin')
	;
	;  if 'No' - open file chooser
	;
	if(STRTRIM(STRUPCASE(yn),2) eq 'NO') then begin
	 ;
	 test_S7 = STRPOS( dir_in, 'S7_' )
	 if(test_S7 ne -1) then begin
	    new_dir = STRMID( dir_in, 0, test_S7 ) + 'S11_Full_Field_Polarimetry'
	    test = FILE_INFO(new_dir)
	    if(test.exists eq 1 and test.directory eq 1) then begin
	      dir_out = new_dir
	    endif
	 endif
	 ;
    grid_path = dialog_pickfile(title='Select Grid Points File', file=dir_out+PATH_SEP()+'dummy.dat',filter='*.dat')
    this_grid = 0
   endif else begin
    this_grid = 1
   endelse
  ;
   if(this_grid eq 1) then begin
    ;
  	;		create input filename
  	;
  	test_fits = STRPOS( STRUPCASE(file_in), '.FITS')
  	if(test_fits ne -1) then begin
  		new_file = STRMID( file_in, 0, test_fits) + '_bg_points.dat'
  	endif else begin
  		WIDGET_CONTROL, (*PTR_ID).id_bottom_textbox_wid, SET_VALUE="ERROR - Could not parse filename", /APPEND
  		ttl = WIDGET_INFO( (*PTR_ID).id_bottom_textbox_wid, /TEXT_TOP_LINE)
  		WIDGET_CONTROL, (*PTR_ID).id_bottom_textbox_wid, SET_TEXT_TOP_LINE = ttl+1
  		goto, do_not_read
  	endelse
  	;
  	;		and directory
  	;
  	test_S7 = STRPOS( dir_in, 'S7_' )
  	if(test_S7 ne -1) then begin
  		new_dir = STRMID( dir_in, 0, test_S7 ) + 'S11_Full_Field_Polarimetry'
  		test = FILE_INFO(new_dir)
  		if(test.exists eq 1 and test.directory eq 1) then begin
  			dir_out = new_dir
  		endif else begin
  			FILE_MKDIR, new_dir
  			dir_out = new_dir
  		endelse
  	endif else begin
  		dir_out = dir_in
  	endelse
  	;
  	path_in = dir_out + PATH_SEP() + new_file
  endif else begin
    path_in = grid_path
    new_file = FILE_BASENAME(grid_path)
  endelse
  ;
	WIDGET_CONTROL, (*PTR_ID).id_bottom_textbox_wid, SET_VALUE="Requested Read of filename "+new_file, /APPEND
	ttl = WIDGET_INFO( (*PTR_ID).id_bottom_textbox_wid, /TEXT_TOP_LINE)
	WIDGET_CONTROL, (*PTR_ID).id_bottom_textbox_wid, SET_TEXT_TOP_LINE = ttl+1
	;
	;	test for it existing
	;
	test = FILE_INFO(path_in)
	if(NOT(test.exists)) then begin
		WIDGET_CONTROL, (*PTR_ID).id_bottom_textbox_wid, SET_VALUE="ERROR - Could not find grid points file", /APPEND
		ttl = WIDGET_INFO( (*PTR_ID).id_bottom_textbox_wid, /TEXT_TOP_LINE)
		WIDGET_CONTROL, (*PTR_ID).id_bottom_textbox_wid, SET_TEXT_TOP_LINE = ttl+1
		goto, do_not_read
	endif
	;
	;	exists - read into key variables
	;
	nx = SXPAR(this_header, 'NAXIS1')
	ny = SXPAR(this_header, 'NAXIS2')
	;
	grid_ptr = s7_read_points_file(Event, path_in)
	;
	if(NOT(PTR_VALID(grid_ptr))) then begin
		WIDGET_CONTROL, (*PTR_ID).id_bottom_textbox_wid, SET_VALUE="ERROR - failed to read grid points file", /APPEND
		ttl = WIDGET_INFO( (*PTR_ID).id_bottom_textbox_wid, /TEXT_TOP_LINE)
		WIDGET_CONTROL, (*PTR_ID).id_bottom_textbox_wid, SET_TEXT_TOP_LINE = ttl+1
		goto, do_not_read
	endif
	;
	(*PTR_ID).npts = (*grid_ptr).npts
	(*PTR_ID).radius_slider_value = (*grid_ptr).radius
	(*PTR_ID).max_order_slider_value = (*grid_ptr).order
	;
	;  note that the astrometry of the image used to create the grid points is stored in
	;  (*PTR_ID).astr
	;
	(*PTR_ID).ngrid = (*PTR_ID).ngrid_slider_value
	(*PTR_ID).grid_radius = (*PTR_ID).radius_slider_value
	;
	;  use the astrometry in the grid points to compare to the astrometry of this image to establish offsets
	;  of grid points and which ones to use or not use
	;
	rad = (*PTR_ID).grid_radius
	rad2 = rad/2.0
	;
	nuse0 = 0L
	for ipts = 0, (*grid_ptr).npts-1 do begin
	 ;
	 ; xy center from grid points file
	 ;
	 xg = (*grid_ptr).xy_points[ipts,0]
	 yg = (*grid_ptr).xy_points[ipts,1]
	 ;
	 ; convert to RA, Dec using grid astr
	 ;
	 xy2ad, xg, yg, (*grid_ptr).astr, a, d
	 ;
	 ; then back to x, y using astrometry for this image
	 ;
	 ad2xy, a, d, this_astr, x, y
	 ;
	 ; position tests - given the radius of the zone, is this center
	 ;   closer to the edges by more than half the radius? If so, don't use
	 ;
	 use = 1
	 if(x lt rad2 or y lt rad2) then use = -1
	 if(x gt (nx-1-rad2) or y gt (ny-1-rad2)) then use = -1
	 ;
	 ; load x, y positions
	 ;
		;(*PTR_ID).xy_points[ipts,0] = (*grid_ptr).xy_points[ipts,0]
		;(*PTR_ID).xy_points[ipts,1] = (*grid_ptr).xy_points[ipts,1]
		(*PTR_ID).xy_points[ipts,0] = ROUND(x)
		(*PTR_ID).xy_points[ipts,1] = ROUND(y)
		;
		;	set flag to use
		;
		(*PTR_ID).xy_points[ipts,2] = use
		;
		if(use eq -1) then nuse0++
	endfor
	;
	if(nuse0 ne 0) then begin
	 print, 'Number of edge-deleted grid points = ',nuse0
	endif
	;
	;	done!
	;
	PTR_FREE, grid_ptr
	;
	ID_Draw_Points, Event
	;
	WIDGET_CONTROL, (*PTR_ID).id_bottom_textbox_wid, SET_VALUE="Read grid points file with "+$
			STRING((*PTR_ID).npts,format='(I4)')+" points", /APPEND
	ttl = WIDGET_INFO( (*PTR_ID).id_bottom_textbox_wid, /TEXT_TOP_LINE)
	WIDGET_CONTROL, (*PTR_ID).id_bottom_textbox_wid, SET_TEXT_TOP_LINE = ttl+1
	;
	do_not_read:
end
;------------------------------------------------------------
pro ID_Fit_Background, Event
	;
	;	uses all the pixels within radius of each point
	;	to compute mfm means for each point, then fits
	;	for background, using maximum order as per the slider
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_ID
	;
	WIDGET_CONTROL, /hourglass
	;
	;	test for npts large enough
	;
	npts = (*PTR_ID).npts
	radius = (*PTR_ID).radius_slider_value
	ncol = (*PTR_ID).ncol
	nrow = (*PTR_ID).nrow
	;
	if(radius eq 0 or ncol eq 0 or nrow eq 0) then begin
		this_message = 'ERROR - radius = 0'
		goto, IDFB_end
	endif
	;
	if(npts lt 0) then begin
		this_message = 'no points on list'
		goto, IDFB_end
	endif
	;
	;	how many non-deleted points?
	;
	ind = WHERE( (*PTR_ID).xy_points[*,2] eq 1, ngood)
	if(ngood le 0) then begin
		this_message = 'no good points on list'
		goto, IDFB_end
	endif
	;
	ptr_list = PTR_NEW( fltarr(ngood,3) )		; 0 = x, 1 = y, 2 = background
	ptr_rms = PTR_NEW( fltarr(ngood) )
	;
	;	for each point, compute mfm value of the zone
	;
	icount = 0L
	for i = 0, ngood-1 do begin
		j = ind[i]
		(*ptr_list)[i,0] = (*PTR_ID).xy_points[j,0]
		cx = (*ptr_list)[i,0]
		(*ptr_list)[i,1] = (*PTR_ID).xy_points[j,1]
		cy = (*ptr_list)[i,1]
		;
		;	collect all the image pixels within radius of this point
		;
		x0 = 0 > (cx - radius)
		x1 = (cx + radius) < (ncol - 1)
		y0 = 0 > (cy - radius)
		y1 = (cy + radius) < (nrow - 1)
		;
		vec = fltarr( (x1 - x0 + 1) * (y1 - y0 + 1) )
		nvec = 0L
		;
		for ix = x0, x1 do begin
			for iy = y0, y1 do begin
				;
				dx = float(ix - cx)
				dy = float(iy - cy)
				dr = sqrt(dx * dx + dy * dy)
				if(dr le radius) then begin
					vec[nvec] = (*(*PTR_ID).ptr_image)[ix,iy]
					nvec++
				endif
			endfor
		endfor
		;
		vec = vec[0:nvec-1]
		ind = WHERE(FINITE(vec),nfinite)
		if(nfinite gt 0) then begin
			zz = median_filtered_mean(vec[ind])
			if(FINITE(zz[0]) and FINITE(zz[1])) then begin
				(*ptr_list)[icount,2] = zz[0]
				(*ptr_rms)[icount] = zz[1]
				icount++
			endif
		endif
	endfor
	;
	;	mean sky and mean rms
	;
	zz_sky = median_filtered_mean(REFORM( (*ptr_list)[0:icount-1,2] ))
	zz_rms = median_filtered_mean((*ptr_rms)[0:icount-1])
	;
	mean_sky = zz_sky[0]
	sky_noise = zz_rms[0]
	;
	;	clean out baddies
	;
	ind_good = WHERE( abs((*ptr_list)[0:icount-1,2] - mean_sky)/sky_noise lt 10.0,ngood)
	if(ngood ge 0) then begin
		(*ptr_list) = (*ptr_list)[ind_good,*]
	endif else begin
		this_message = 'Too few good points'
		goto, IDFB_end
	endelse
	;
	;	have all the points. Next, select maximum fitting order
	;	in both dimensions.
	;
	degree = ((nvec - 4) < (*PTR_ID).max_order_slider_value) < ngood
	;
	(*PTR_ID).fit_order = degree
	;
	Result = SFIT( TRANSPOSE( (*ptr_list)), degree, KX=coeffs, /IRREGULAR, /MAX_DEGREE )
	;
	;	create the model image (from original, times zero)
	;
	(*PTR_ID).ptr_model = PTR_NEW( (*(*PTR_ID).ptr_image) )
	(*(*PTR_ID).ptr_model) *= 0.0
	;
	this_y = findgen(nrow)+0.5
	for ix = 0, ncol-1 do begin
		this_x = float(ix)+0.5
		;
		(*(*PTR_ID).ptr_model)[ix,*] = 0.0
		;
		ik = 0
		for kx = 0, degree do begin
			for ky = 0, degree do begin
				if(kx+ky le degree) then begin
					(*(*PTR_ID).ptr_model)[ix,*] += coeffs[ik] * (this_y^ky) * (this_x^kx)
					ik++
				endif
			endfor
		endfor
	endfor
	;
	;	also compute the difference image
	;
	(*PTR_ID).ptr_diff = PTR_NEW( (*(*PTR_ID).ptr_image) - (*(*PTR_ID).ptr_model) )
	;
	;	add back in mean background - so the effect is background flattening, not removal
	;
	(*(*PTR_ID).ptr_diff) += mean_sky
	;
	PTR_FREE, ptr_list
	PTR_FREE, ptr_rms
	;
	;	and set display to show this model
	;
	(*PTR_ID).id_display_flag = 2
	;
	;	turn off showing of points
	;
	(*PTR_ID).id_show_points_flag = 0
	WIDGET_CONTROL, (*PTR_ID).id_show_points_checkbox_wid, SET_BUTTON=(*PTR_ID).id_show_points_flag
	ID_Display_Image, Event, (*PTR_ID).id_display_flag
	this_message = 'Model and Diff Images Computed'
	;
	IDFB_end:
	;
	WIDGET_CONTROL, (*PTR_ID).id_bottom_textbox_wid, SET_VALUE=this_message, /APPEND
	ttl = WIDGET_INFO( (*PTR_ID).id_bottom_textbox_wid, /TEXT_TOP_LINE)
	WIDGET_CONTROL, (*PTR_ID).id_bottom_textbox_wid, SET_TEXT_TOP_LINE = ttl+1
	;
end
;-----------------------------------------------------------
pro ID_Write_FITS_Image, Event
	;
	;	writes a FITS image to disk from the currently displayed image
	;	(based on display_flag)
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_ID
	;
	;	display ?
	;
	case (*PTR_ID).id_display_flag of
		0:	begin
			this_message = "no image displayed"
			end
		1:	begin
			writefits, dialog_pickfile(Title="Name of FITS image to write"),(*(*PTR_ID).ptr_image),(*(*PTR_ID).ptr_header)
			this_message = "Original image written to disk"
			end
		2:	begin
			SXADDHIST, 'Background Model from PPOL/S11',(*(*PTR_ID).ptr_header)
			writefits, dialog_pickfile(Title="Name of FITS image to write"),(*(*PTR_ID).ptr_model),(*(*PTR_ID).ptr_header)
			this_message = "Background model image written to disk"
			end
		3:	begin
			SXADDHIST, 'Difference (image - background) from PPOL/S11',(*(*PTR_ID).ptr_header)
			writefits, dialog_pickfile(Title="Name of FITS image to write"),(*(*PTR_ID).ptr_diff),(*(*PTR_ID).ptr_header)
			this_message = "Difference image written to disk"
			end
	endcase
	;
	WIDGET_CONTROL, (*PTR_ID).id_bottom_textbox_wid, SET_VALUE=this_message, /APPEND
	ttl = WIDGET_INFO( (*PTR_ID).id_bottom_textbox_wid, /TEXT_TOP_LINE)
	WIDGET_CONTROL, (*PTR_ID).id_bottom_textbox_wid, SET_TEXT_TOP_LINE = ttl+1
end
;
;-----------------------------------------------------------
pro ID_Toggle_Show_Points, Event
	;
	;	changes value of show points flag and redisplays image
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_ID
	;
	(*PTR_ID).id_show_points_flag = 1L - (*PTR_ID).id_show_points_flag
	;
	WIDGET_CONTROL, (*PTR_ID).id_show_points_checkbox_wid, SET_BUTTON=(*PTR_ID).id_show_points_flag
	;
	case (*PTR_ID).id_show_points_flag of
		0:	WIDGET_CONTROL, (*PTR_ID).id_bottom_textbox_wid, SET_VALUE="Turned off display of points", /APPEND
		1:	WIDGET_CONTROL, (*PTR_ID).id_bottom_textbox_wid, SET_VALUE="Turnd on display of points", /APPEND
	endcase
	;
	ttl = WIDGET_INFO( (*PTR_ID).id_bottom_textbox_wid, /TEXT_TOP_LINE)
	WIDGET_CONTROL, (*PTR_ID).id_bottom_textbox_wid, SET_TEXT_TOP_LINE = ttl+1
	;
	ID_Display_Image, Event, (*PTR_ID).id_display_flag
end
;
;------------------------------------------------------------
pro ID_Generate_Grid, Event
	;
	;	creates the grid of positions, and marks them on the
	;	current display
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_ID
	;
	;	get values for ngrid and radius
	;
	;		TBD - effects of slider functions - for now just copy them
	;
	(*PTR_ID).ngrid = (*PTR_ID).ngrid_slider_value
	(*PTR_ID).grid_radius = (*PTR_ID).radius_slider_value
	;
	ngrid = (*PTR_ID).ngrid
	radius = (*PTR_ID).grid_radius
	;
	;	and the image axes sizes
	;
	ncol = (*PTR_ID).ncol
	nrow = (*PTR_ID).nrow
	;
	if((*PTR_ID).npts eq -1) then (*PTR_ID).npts++
	;
	;	test for reasonable ngrid and radius values
	;
	if(ngrid gt 1 and ngrid lt 51 and radius gt 2 and radius lt 201) then begin
		;
		;	test for an image in the buffer
		;
		if( (*PTR_ID).id_display_flag ge 1) then begin
			;
			;	all OK, compute x, y grid points
			;
			dx = float(ncol) / float(ngrid)
			dy = float(nrow) / float(ngrid)
			;
			for ix = 0, ngrid-1 do begin
				for iy = 0, ngrid-1 do begin
					;
					this_x = FLOOR( dx * (float(ix) + 0.5))
					this_y = FLOOR( dy * (float(iy) + 0.5))
					;
					(*PTR_ID).xy_points[(*PTR_ID).npts, *] = [this_x, this_y, 1]
					;
					(*PTR_ID).npts++
				endfor
			endfor
			WIDGET_CONTROL, (*PTR_ID).id_bottom_textbox_wid, SET_VALUE="Added "+STRING(ngrid*ngrid,format='(I4)')+" grid points", /APPEND
			ttl = WIDGET_INFO( (*PTR_ID).id_bottom_textbox_wid, /TEXT_TOP_LINE)
			WIDGET_CONTROL, (*PTR_ID).id_bottom_textbox_wid, SET_TEXT_TOP_LINE = ttl+1
			;
			;	draw all points
			;
			ID_Draw_Points, Event
			;
		endif else begin
			WIDGET_CONTROL, (*PTR_ID).id_bottom_textbox_wid, SET_VALUE="ERROR - no image has been read yet", /APPEND
			ttl = WIDGET_INFO( (*PTR_ID).id_bottom_textbox_wid, /TEXT_TOP_LINE)
			WIDGET_CONTROL, (*PTR_ID).id_bottom_textbox_wid, SET_TEXT_TOP_LINE = ttl+1
		endelse
	endif else begin
		WIDGET_CONTROL, (*PTR_ID).id_bottom_textbox_wid, SET_VALUE="ERROR - ngrid or radius out of bounds",/APPEND
		WIDGET_CONTROL, (*PTR_ID).id_bottom_textbox_wid, SET_VALUE="ngrid, radius ="+$
			STRING(ngrid, radius, format='(i5,",",i5)')
		ttl = WIDGET_INFO( (*PTR_ID).id_bottom_textbox_wid, /TEXT_TOP_LINE)
			WIDGET_CONTROL, (*PTR_ID).id_bottom_textbox_wid, SET_TEXT_TOP_LINE = ttl+2
	endelse
end
;
;-------------------------------------------------------------
pro ID_Draw_Points, Event
	;
	;	draw all the valid points on the xy list on the image in green, deleted in yellow
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_ID
	;
	npts = (*PTR_ID).npts
	radius = (*PTR_ID).grid_radius
	;
	ncol = (*PTR_ID).ncol
	nrow = (*PTR_ID).nrow
	;
	circle_x = fltarr(32)
	circle_y = fltarr(32)
	;
	angle = (!PI / 180.0) * 360.0 / 31.0
	;
	ifirst = 1
	;
	if(npts ge 0) then begin
		;
		for ipt = 0, npts do begin
			;
			;	test this point for having been set or deleted
			;
			case (*PTR_ID).xy_points[ipt,2] of
				0: color = 'none'							; not set, skip
				1: color = 'green'							; set
				-1: color = 'red'						; deleted
				else: color = 'yellow'							; dunno
			endcase
			;
			if(color ne 'none') then begin
				this_x = (*PTR_ID).xy_points[ipt,0]
				this_y = (*PTR_id).xy_points[ipt,1]
				;
				for i = 0, 31 do begin
					circle_x[i] = this_x + radius * cos(angle * i)
					circle_y[i] = this_y + radius * sin(angle * i)
				endfor
				;
				case color of
					'green': begin
						this_color = 2
						this_thick=3
						     end
					'yellow': begin
						this_color = 3
						this_thick=1
							end
					'red':	begin
						this_color = 4
						this_thick=1
							end
				endcase
				;
				if(ifirst eq 1) then begin
					WSET, (*PTR_ID).id_draw_window_index
					plot, [0,ncol], [0,nrow], xr=[0,ncol], yr=[0,nrow], xs=5, ys=5,$
							position=[0.,0.,1.,1.], color=this_color, /NOERASE, /NODATA
					ifirst = 0
				endif
				WSET, (*PTR_ID).id_draw_window_index
				oplot, circle_x, circle_y,$
							color=this_color, thick=this_thick
			endif
		endfor
	endif
	;
	WIDGET_CONTROL, (*PTR_ID).id_delete_point_button_wid, SENSITIVE=0
    WIDGET_CONTROL, (*PTR_ID).id_add_point_button_wid, SENSITIVE=0
    WIDGET_CONTROL, (*PTR_ID).id_undelete_point_button_wid, SENSITIVE=0
    if( (*PTR_ID).npts ge 0) then isx = 1 else isx = 0
    WIDGET_CONTROL, (*PTR_ID).id_reset_all_deletes_button_wid, SENSITIVE=isx
    WIDGET_CONTROL, (*PTR_ID).id_remove_all_points_button_wid, SENSITIVE=isx
end
;
;-------------------------------------------------------------
pro ID_Mouse, Event
	;
	;	capture info from a Mouse button press over the draw window
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_ID
	;
	;wait, 0.1
	;
	;	get the event info, if this was a mouse button closure
	;
	if(Event.type eq 0 and Event.press eq 1) then begin
		;
		;	test for double values, skip if so
		;
		if( (*PTR_ID).id_mouse_x_last ne Event.x or (*PTR_ID).id_mouse_y_last ne Event.y) then begin
			;
			(*PTR_ID).id_draw_x = (double(Event.x)/double((*PTR_ID).id_draw_xscr)) * double((*PTR_ID).ncol)
			(*PTR_ID).id_draw_y = (double(Event.y)/double((*PTR_ID).id_draw_yscr)) * double((*PTR_ID).nrow)
			;
			;	get z value from current image
			;
			case (*PTR_ID).id_display_flag of
				1: z = (*(*PTR_ID).ptr_image)[(*PTR_ID).id_draw_x,(*PTR_ID).id_draw_y]
				2: z = (*(*PTR_ID).ptr_model)[(*PTR_ID).id_draw_x,(*PTR_ID).id_draw_y]
				3: z = (*(*PTR_ID).ptr_diff)[(*PTR_ID).id_draw_x,(*PTR_ID).id_draw_y]
			endcase
			WIDGET_CONTROL, (*PTR_ID).id_bottom_textbox_wid, SET_VALUE="Mouse button pressed at (x,y, z) = "+$
					STRING((*PTR_ID).id_draw_x, (*PTR_ID).id_draw_y,z, format='(f6.1,",",f6.1,",",f10.1)'), /APPEND
			ttl = WIDGET_INFO( (*PTR_ID).id_bottom_textbox_wid, /TEXT_TOP_LINE)
			WIDGET_CONTROL, (*PTR_ID).id_bottom_textbox_wid, SET_TEXT_TOP_LINE = ttl+2
		endif
	endif
	;
	;	sensitize all the buttons that could be invoked
	;
	WIDGET_CONTROL, (*PTR_ID).id_delete_point_button_wid, SENSITIVE=1
    WIDGET_CONTROL, (*PTR_ID).id_add_point_button_wid, SENSITIVE=1
    WIDGET_CONTROL, (*PTR_ID).id_undelete_point_button_wid, SENSITIVE=1
    WIDGET_CONTROL, (*PTR_ID).id_reset_all_deletes_button_wid, SENSITIVE=1
    WIDGET_CONTROL, (*PTR_ID).id_remove_all_points_button_wid, SENSITIVE=1
end
;
;-------------------------------------------------------------
pro ID_Add_Point, Event
	;
	;	add a new point/zone at the most recent mouse position
	;	- start by desensitizing all the buttons
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_ID
	;
	WIDGET_CONTROL, (*PTR_ID).id_delete_point_button_wid, SENSITIVE=0
    WIDGET_CONTROL, (*PTR_ID).id_add_point_button_wid, SENSITIVE=0
    WIDGET_CONTROL, (*PTR_ID).id_undelete_point_button_wid, SENSITIVE=0
    if( (*PTR_ID).npts ge 0) then isx = 1 else isx = 0
    WIDGET_CONTROL, (*PTR_ID).id_reset_all_deletes_button_wid, SENSITIVE=isx
    WIDGET_CONTROL, (*PTR_ID).id_remove_all_points_button_wid, SENSITIVE=isx
    ;
    if( (*PTR_ID).npts eq -1) then (*PTR_ID).npts++
    ;
    (*PTR_ID).xy_points[(*PTR_ID).npts,*] = [FLOOR((*PTR_ID).id_draw_x), FLOOR((*PTR_ID).id_draw_y), 1]
    ;
    (*PTR_ID).grid_radius = (*PTR_ID).radius_slider_value
    ;
    (*PTR_ID).npts++
	;
    ID_Draw_Points, Event
end
;
;--------------------------------------------------------------
pro ID_Change_Point, Event, cp_mode
	;
	;	modifies a point or points, depending on cp_mode
	;
	;	0 = delete point;	1 = undelete point
	;	2 = undelete all;	3 = remove all
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_ID
	;
	;	if mode 2 or 3, don't need to check cursor x,y value
	;
	if(cp_mode ge 2) then begin
		;
		for i = 0, (*PTR_ID).npts do begin
			;
			case cp_mode of
				2: 	(*PTR_ID).xy_points[i,2] = 1
				3:	(*PTR_ID).xy_points[i,2] = 0
			endcase
		endfor
		;
		if(cp_mode eq 3) then begin
			(*PTR_ID).npts = -1		; reset
			this_message = 'Removing all points'
		endif else begin
			this_message = 'Resetting all points'
		endelse
	endif else begin
		;
		;	get last cursor position
		;
		this_x = (*PTR_ID).id_draw_x
		this_y = (*PTR_ID).id_draw_y
		;
		;	compare to each point that has been set or reset
		;	find closest within radius
		;
		radius = (*PTR_ID).grid_radius
		;
		r_min = 1.0e6
		i_min = -1
		;
		for i = 0, (*PTR_ID).npts do begin
			if( (*PTR_ID).xy_points[i,2] ne 0) then begin
				dx = this_x - (*PTR_ID).xy_points[i,0]
				dy = this_y - (*PTR_ID).xy_points[i,1]
				dr = sqrt(dx*dx + dy*dy)
				if(dr le radius) then begin
					if(dr le r_min) then begin
						r_min = dr
						i_min = i
					endif
				endif
			endif
		endfor
		;
		;	if no matches, say so
		;
		if(i_min eq -1) then begin
			this_message = 'No points matched to cursor position'
		endif else begin
			;
			;	match found, set new status value as per mode
			;
			case cp_mode of
				0: begin
						(*PTR_ID).xy_points[i_min,2] = -1
						this_message = 'Deleted point at '+STRING(this_x, this_y, format='(F7.1,",",f7.1)')
					end
				1: begin
						(*PTR_ID).xy_points[i_min,2] = 1
						this_message = 'UnDeleted point at '+STRING(this_x, this_y, format='(F7.1,",",f7.1)')
					end
			endcase
		endelse
	endelse
	;
	;	redraw points
	;
	ID_Display_Image, Event, (*PTR_ID).id_display_flag
	;
	WIDGET_CONTROL, (*PTR_ID).id_bottom_textbox_wid, SET_VALUE=this_message, /APPEND
	ttl = WIDGET_INFO( (*PTR_ID).id_bottom_textbox_wid, /TEXT_TOP_LINE)
	WIDGET_CONTROL, (*PTR_ID).id_bottom_textbox_wid, SET_TEXT_TOP_LINE = ttl+1
end
;
;--------------------------------------------------------------
pro ID_Change_Slider, Event, slider_id, redisplay_flag
	;
	;	captures new slider value, figures out which one, saves
	;	result, and if selected, redraws image
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_ID
	;
	;	which one?
	;
	case slider_id of
				0: wid = (*PTR_ID).id_offset_slider_wid
				1: wid = (*PTR_ID).id_scale_slider_wid
				2: wid = (*PTR_ID).id_ngrid_slider_wid
				3: wid = (*PTR_ID).id_grid_radius_slider_wid
				4: wid = (*PTR_ID).id_max_order_slider_wid
	endcase
	;
	WIDGET_CONTROL, wid, GET_VALUE=value
	WIDGET_CONTROL, (*PTR_ID).id_bottom_textbox_wid, SET_VALUE="Slider set to "+STRING(value,format='(i4)'), /APPEND
	ttl = WIDGET_INFO( (*PTR_ID).id_bottom_textbox_wid, /TEXT_TOP_LINE)
			WIDGET_CONTROL, (*PTR_ID).id_bottom_textbox_wid, SET_TEXT_TOP_LINE = ttl+1
	;
	;	store
	;
	case slider_id of
				0: (*PTR_ID).offset_slider_value = value
				1: (*PTR_ID).scale_slider_value = value
				2: (*PTR_ID).ngrid_slider_value = value
				3: (*PTR_ID).radius_slider_value = value
				4: (*PTR_ID).max_order_slider_value = value
	endcase
	;
	;	redraw?
	;
	if(redisplay_flag eq 1) then begin
		;
		;	what is currently displayed?
		;
		this_ind = (*PTR_ID).id_display_flag
		;
		;	reshow it
		;
		ID_Display_Image, Event, this_ind
	endif
end
;
;---------------------------------------------------------------
pro ID_Display_Image, Event, index
	;
	;	displays an image to the draw window, based on the index
	;		(1=image, 2=model, 3=difference)
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_ID
	;
	;	check what is currently displayed. If nothing, and no image,
	;	open file chooser and read the image
	;
	if( (*PTR_ID).id_display_flag eq 0 and (*PTR_ID).image_name eq '') then begin
		file_name = dialog_pickfile(Title='FITS image to open')
		(*PTR_ID).ptr_image = PTR_NEW(readfits(file_name, header))
		(*PTR_ID).image_name = file_name
		(*PTR_ID).ptr_header = PTR_NEW(header)
		;
		WIDGET_CONTROL, (*PTR_ID).id_image_name_textbox_wid, SET_VALUE=FILE_BASENAME((*PTR_ID).image_name)
		;
		;	get image size
		;
		isize = SIZE( (*(*PTR_ID).ptr_image) )
		(*PTR_ID).ncol = isize[1]
		(*PTR_ID).nrow = isize[2]
		;
		WIDGET_CONTROL, (*PTR_ID).id_image_size_textbox_wid, SET_VALUE=STRING( isize[1], isize[2], format='(i4,",",i4)')
		;
		(*PTR_ID).id_display_flag = 1
	endif
	;
	case index of
		0:	begin
			print, 'ID/GUI - error - should not have gotten here'
			end
		1:	begin
				ID_Display_This, Event, (*PTR_ID).ptr_image
				(*PTR_ID).id_display_flag = 1
			end
		2:  begin
				ID_Display_This, Event, (*PTR_ID).ptr_model
				(*PTR_ID).id_display_flag = 2
			end
		3:	begin
				ID_Display_This, Event, (*PTR_ID).ptr_diff
				(*PTR_ID).id_display_flag = 3
			end
		else:	begin
					print, 'ID/GUI - error - no case recognized'
				end
	endcase
	;
	;	redraw grid, if points present and display flag set
	;
	if( (*PTR_ID).id_show_points_flag eq 1) then begin
			ID_Draw_Points, Event
	endif else begin
		WIDGET_CONTROL, (*PTR_ID).id_delete_point_button_wid, SENSITIVE=0
	    WIDGET_CONTROL, (*PTR_ID).id_add_point_button_wid, SENSITIVE=0
	    WIDGET_CONTROL, (*PTR_ID).id_undelete_point_button_wid, SENSITIVE=0
	    if( (*PTR_ID).npts ge 0) then isx = 1 else isx = 0
	    WIDGET_CONTROL, (*PTR_ID).id_reset_all_deletes_button_wid, SENSITIVE=isx
	    WIDGET_CONTROL, (*PTR_ID).id_remove_all_points_button_wid, SENSITIVE=isx
	endelse
	;
	;	done, return
	;
end
;
;----------------------------------------------------------------
pro	ID_Display_This, Event, ptr
	;
	;	displays to the draw window the image pointed to by the ptr
	;		using the slider values
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_ID
	;
	ptr_disp = PTR_NEW( (*ptr) )			; temporary image, allowed to be modified for display
	;
	;	offset, scale values
	;
	offset = ((*PTR_ID).offset_slider_value - 20.0) / 10.0 	; log value from -5.0 to +5.0
	scale = (((*PTR_ID).scale_slider_value - 20.0) / 10.0 + 2.1)
	;
	nrow = (*PTR_ID).nrow
	ncol = (*PTR_ID).ncol
	;
	vec = (*ptr_disp)[0.1*ncol:0.9*ncol:25,0.1*nrow:0.9*nrow:25]
	vec = vec[WHERE(vec)]
	mfm = median_filtered_mean(vec)
	;print, "median, rms = ",mfm[0],mfm[1]
	;
	;   do log scaling for values in excess of mean
	;
	(*ptr_disp) -= mfm[0]-5.0*mfm[1]							; remove mean background, but bias back with 5 sigma background noise
	;
		;
	ind_z = WHERE((*ptr_disp)-10.0^offset le 0.,count_z,COMPLEMENT=ind,NCOMPLEMENT=n_ind)
	;print, "number of pixels below 0 = ",count_z
   ;
	if(count_z gt 0) then (*ptr_disp)[ind_z] = -0.2
	if(n_ind gt 0) then (*ptr_disp)[ind] = -0.1 > alog10((*ptr_disp)[ind]) else (*ptr_disp)[*] = -0.1
	;
	scaled_disp = bytscl((255 < (0 > (((*ptr_disp)-offset) / scale)*255)),min=0,max=255,top=248)
	scaled_disp += 7
	;
	out_image = CONGRID(scaled_disp, (*PTR_ID).id_draw_xscr, (*PTR_ID).id_draw_yscr)
    TV, bytscl(out_image), xsize = (*PTR_ID).id_draw_xscr, ysize= (*PTR_ID).id_draw_yscr
    ;
    WIDGET_CONTROL, (*PTR_ID).id_bottom_textbox_wid, SET_VALUE="Offset, Scale = "+STRING(offset,scale,format='(f7.2,",",f7.2)'), /APPEND
	ttl = WIDGET_INFO( (*PTR_ID).id_bottom_textbox_wid, /TEXT_TOP_LINE)
	WIDGET_CONTROL, (*PTR_ID).id_bottom_textbox_wid, SET_TEXT_TOP_LINE = ttl+2
    ;
    PTR_FREE, ptr_disp
end
;
;---------------------------------------------------------------------------------------
; Empty stub procedure used for autoloading.
;
pro ID_GUI_eventcb
end


pro ID_Return, wWidget

end
