;
;	Step 10 Event Handler Routines - Compute Instrumental Polarization
;
pro S10_Init, Event
	;
	;	initialize S10 stuff when tabbing onto Step 10
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	g_ptr = PTR_struct.PTR_group
	;
	;	initialize options buttons and buffers
	;
	WIDGET_CONTROL, (*w_ptr).S10_nlines_textbox_wid, SET_VALUE=''
	;
;	geo = WIDGET_INFO( (*w_ptr).S10_nlines_textbox_wid, /GEOMETRY)
	;
	WIDGET_CONTROL, (*w_ptr).S10_n_good_stars_textbox_wid, SET_VALUE=''
	WIDGET_CONTROL, (*w_ptr).S10_actual_per_grid_textbox_wid, SET_VALUE=''
	WIDGET_CONTROL, (*w_ptr).S10_search_radius_textbox_wid, SET_VALUE=''
	;
	str = STRING( (*w_ptr).S10_sigma_cutoff,format='(f5.2)')
	t_wid = (*w_ptr).S10_cutoff_textbox_wid
	center_in_textbox, t_wid, str
	;
	;
	center_in_textbox, (*w_ptr).S10_stars_per_grid_textbox_wid, STRING( (*w_ptr).S10_stars_per_grid, $
								format='(I4.1)')
	center_in_textbox, (*w_ptr).S10_boxcar_size_textbox_wid, STRING( (*w_ptr).S10_boxcar_size, $
								format='(I4.1)')
	;
	(*w_ptr).S10_save_region_mode = 1 - (*w_ptr).S10_save_region_mode
	S10_Toggle_save_region_file, Event
	;
	(*w_ptr).S10_n_polcats = 0
	;
	;	set band from stored value
	;
	S10_Select_Band, (*w_ptr).S10_Band, Event
	;
	;	get High P status
	;
	(*w_ptr).S10_High_P_flag = 1 - (*w_ptr).S10_High_P_flag
	S10_Toggle_High_P, Event
	;
	;	turn off processing buttons
	;
	WIDGET_CONTROL, (*w_ptr).S10_apply_criteria_button_wid, SENSITIVE=0
	WIDGET_CONTROL, (*w_ptr).S10_compute_p_inst_button_wid, SENSITIVE=0
	WIDGET_CONTROL, (*w_ptr).S10_save_images_button_wid, SENSITIVE=0
	;
	;	part 2 - high-pol star detection - all on, set defaults
	;
	(*w_ptr).S10_avoid_stars_flag = 1 - (*w_ptr).S10_avoid_stars_flag
	S10_Toggle_avoid_stars, Event
	;
	WIDGET_CONTROL, (*w_ptr).S10_high_pol_threshold_textbox_wid, SET_VALUE=STRING((*w_ptr).S10_high_pol_threshold_value,format='(1x,f5.1,1x)')
	WIDGET_CONTROL, (*w_ptr).S10_field_number_threshold_textbox_wid, SET_VALUE=STRING((*w_ptr).S10_field_number_threshold_value,format='(1x,i2.1,1x)')
	;
	WIDGET_CONTROL, (*w_ptr).S10_number_high_pol_stars_textbox_wid, SET_VALUE=''
end
;
;-----------------------------------------------
pro S10_Toggle_High_P, Event
	;
	;	toggle this mode
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	;
	(*w_ptr).S10_High_P_flag = 1 - (*w_ptr).S10_High_P_flag
	;
	WIDGET_CONTROL, (*w_ptr).S10_High_P_Checkbox_wid, SET_BUTTON=(*w_ptr).S10_High_P_flag
	;
	case (*w_ptr).S10_High_P_flag of
		0: print_text, Event, "Changing mode to NOT not hold back High Polarization stars"
		1: print_text, Event, "Changing mode to hold back High Polarization stars"
	endcase
	;
end
;
;-----------------------------------------------
pro S10_Toggle_save_region_file, Event
	;
	;	toggle this mode
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	;
	(*w_ptr).S10_save_region_mode = 1 - (*w_ptr).S10_save_region_mode
	;
	WIDGET_CONTROL, (*w_ptr).S10_save_region_file_checkbox_wid, SET_BUTTON=(*w_ptr).S10_save_region_mode
	;
	case (*w_ptr).S10_save_region_mode of
		0: print_text, Event, "Changing mode to NOT save DS9 region file of stars"
		1: print_text, Event, "Changing mode to save DS9 region file of stars"
	endcase
	;
end
;
;-----------------------------------------------
pro S10_Select_Band, band, Event
	;
	;	sets or resets radio buttons
	;	for band
	;
	;	and rewrites buffer
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	;
	(*w_ptr).S10_Band = band
	;
	if(band eq 'H') then begin
		ih = 1
	endif
	if(band eq 'K') then begin
		ih = 0
	endif
	;
	WIDGET_CONTROL, (*w_ptr).S10_Select_H_RButton_wid, SET_BUTTON = ih
	WIDGET_CONTROL, (*w_ptr).S10_Select_K_RButton_wid, SET_BUTTON = 1-ih
	;
	print_text, Event, "Selecting "+band+"-Pol mode for P_Inst calculation"
end
;
;-----------------------------------------------
pro S10_Change_cutoff, Event
	;
	;	manage edits to the sigma cutoff textbox
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	;
	wid = (*w_ptr).S10_cutoff_textbox_wid
	value = (*w_ptr).S10_sigma_cutoff
	;
	S10_Modify_textbox, Event, wid, value, update_textbox
	;
	(*w_ptr).S10_sigma_cutoff = value
	str = STRTRIM(STRING(value,format='(F5.2)'),2)
	;
	if(update_textbox eq 1) then begin
		center_in_textbox, wid, str
		print_text, Event, "Changed sQ, sU upper cutoff to "+str+" percent"
	endif
end
;
;----------------------------------------------
pro S10_Change_boxcar_size, Event
	;
	;	manage edits to the boxcar size textbox
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	;
	wid = (*w_ptr).S10_boxcar_size_textbox_wid
	value = (*w_ptr).S10_boxcar_size
	;
	S10_Modify_textbox, Event, wid, value, update_textbox
	;
	(*w_ptr).S10_boxcar_size = value
	str = STRTRIM(STRING(value,format='(I4.1)'),2)
	;
	if(update_textbox eq 1) then begin
		center_in_textbox, wid, str
		print_text, Event, "Changed Smoothing Boxcar Radius to "+str+" pixels"
	endif
end
;
;----------------------------------------------
pro S10_Change_stars_per_grid, Event
	;
	;	manage edits to the stars per grid textbox
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	;
	wid = (*w_ptr).S10_stars_per_grid_textbox_wid
	value = (*w_ptr).S10_stars_per_grid
	;
	S10_Modify_textbox, Event, wid, value, update_textbox
	;
	(*w_ptr).S10_stars_per_grid = value
	str = STRTRIM(STRING(value,format='(I4.1)'),2)
	;
	if(update_textbox eq 1) then begin
		center_in_textbox, wid, str
		print_text, Event, "Changed Nominal Number of Stars to be sought per grid point to "+str
	endif
end
;----------------------------------------------------------------
pro S10_Modify_textbox, Event, wid, value, update_textbox
	;
	;	handle insert character events for P scale override value
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	;
	update_textbox = 0
	;
	;	ignore everything except a CR, then read textbox buffer
	;
	if(Event.type eq 0) then begin
		if(Event.ch eq 10B) then begin
			existing_text = ''
			WIDGET_CONTROL, wid, GET_VALUE=existing_text
			mch = STRLEN(existing_text[0])
			if(mch gt 0) then begin
				type = SIZE(value, /TYPE)
				if(type eq 2 or type eq 3) then begin
					reads, existing_text[0], format='(i)',value
					update_textbox = 1
				endif else begin
					if(mch ge 3) then reads, existing_text[0], format='(F)', value
					update_textbox = 1
				endelse
				;
			endif
		endif
	endif
end
;-----------------------------------------------
pro S10_Select_polcats, Event
	;
	;	use file chooser to load list of POLCATS to process
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	s_ptr = PTR_struct.PTR_summary
	w_ptr = PTR_struct.PTR_wid_id
	g_ptr = PTR_struct.PTR_group
	;
	;	band
	;
	band = (*w_ptr).S10_Band
	;
	files = dialog_pickfile(Title="Select POLCATS to process",/MULTIPLE_FILES)
	nfiles = N_ELEMENTS(files)
	if(nfiles gt 0) then begin
		;
		(*w_ptr).S10_polcat_list[0:nfiles-1] = files[*]
		(*w_ptr).S10_n_polcats = nfiles
		;
		print_text, Event, "Found a total of "+STRING(nfiles,format='(I)')+" POLCATS to read"
		;
		;	now, read the polcats and correct the stellar Q, U values to boresight means
		;
		pol_struct = s10_read_polcats_into_structure(Event, files, band)
		;
		;	and park the structure on heap
		;
		(*w_ptr).S10_ptr_pol_struct = PTR_NEW(pol_struct)		; remember to free this at the end of the calculation
		;
		n_pol = N_ELEMENTS(pol_struct.U)
		center_in_textbox, (*w_ptr).S10_nlines_textbox_wid, STRING(n_pol, format='(I6.1)')
		;
		WIDGET_CONTROL, (*w_ptr).S10_apply_criteria_button_wid, SENSITIVE=1
	endif else begin
		print_text, Event, "No files selected - try again, or select another option"
	endelse
end
;
;----------------------------------------------
pro S10_Apply_criteria, Event
	;
	;	uses the current criteria to select stars from the structure,
	;	making an index array of pointers to the good stars
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	;
	;	start with the high pol star avoidance, if the flag is set - or skip entirely, if zero
	;
	if( (*w_ptr).S10_avoid_stars_flag eq 1) then begin
		;
		;	check that the list pointer is valid. If not, have to
		;	reopen and read the file
		;
		if( PTR_VALID( (*w_ptr).S10_short_list_ptr ) eq 1) then begin
			;
		endif else begin
			;
			;	ptr not valid, get disk file name, open it, and reload
			;	the list
			;
			print_text, Event, 'Whoops! The high pol list is not in memory - must read the disk file'
			;
			file_in = dialog_pickfile(Title='Select high pol list disk file')
			if( STRLEN(file_in) lt 5) then begin
				print_text, Event, 'ERROR - no file selected. Jumping out of this routine.'
				goto, SAC_end
			endif
			;
			;	file name ok, open the file for reading
			;
			openr, lu_in, file_in, /GET_LUN
			;
			;	and load some buffers
			;
			(*w_ptr).S10_list_pathname = file_in
			(*w_ptr).S10_high_pol_file_lun = lu_in
			;
			nf = 0L
			short_list = strarr(1000)
			;
			WHILE ~EOF(lu_in) do begin
				line = ''
				readf, lu_in, line
				short_list[nf] = STRTRIM(line,2)
				nf++
			ENDWHILE
			;
			;	can leave file open for now. Write to heap
			;
			short_list = short_list[0:nf-1]
			;
			(*w_ptr).S10_short_list_ptr = PTR_NEW(short_list)
			(*w_ptr).S10_n_short_list = nf
			;
			print_text, Event, 'Successfully reloaded the high pol list from disk file '+FILE_BASENAME(file_in)
		endelse
		;
		;	now that we have a good list in memory, find all the input stars that do NOT
		;	match to any of these designations
		;
		;	start with an index array of the full input star struture
		;
		nstart = N_ELEMENTS( (*(*w_ptr).S10_ptr_pol_struct).U )
		ind_bad = bytarr(nstart)
		;
		;	must compare designations one at a time
		;
		for id = 0, (*w_ptr).S10_n_short_list-1 do begin
			bad_desig = STRTRIM( (*(*w_ptr).S10_short_list_ptr)[id], 2)
			;
			ind = WHERE( STRTRIM((*(*w_ptr).S10_ptr_pol_struct).g_desig,2) eq bad_desig, nbad)
			if(nbad gt 0) then begin
				ind_bad[ind] = 1
			endif
		endfor
		;
		tot_bad = TOTAL( ind_bad )
		print_text, Event, 'Rejected a total of '+STRING(tot_bad,format='(I5.1)')+' stars as having high polarizations'
	endif else begin
		nstart = N_ELEMENTS( (*(*w_ptr).S10_ptr_pol_struct).U )
		ind_bad = bytarr(nstart)
	endelse
	;
	;	next the sU, sQ cuts and traps
	;
	sigma_cut = (*w_ptr).S10_sigma_cutoff
	;
	;	if the High P flag is set, also apply the sigma-cutoff value to detect/reject high polarizations
	;	20131203 - DPC
	;	20140217 - DPC added low sigma trap at 0.2%
	;
	print_text, Event, 'Echo in Apply Criteria - High P flag = '+STRING((*w_ptr).S10_High_P_flag)
	;
	if( (*w_ptr).S10_High_P_flag eq 1) then begin
		;
		;	20140217 - turn back into P and sP to apply the high P cut
		;				to both U and Q
		;
		temp_p = sqrt((*(*w_ptr).S10_ptr_pol_struct).U^2 + (*(*w_ptr).S10_ptr_pol_struct).Q^2)
		temp_sp = (1./temp_p) * sqrt(((*(*w_ptr).S10_ptr_pol_struct).U * (*(*w_ptr).S10_ptr_pol_struct).sU)^2 + $
									 ((*(*w_ptr).S10_ptr_pol_struct).Q * (*(*w_ptr).S10_ptr_pol_struct).sQ)^2)
		temp_SNR = temp_p / temp_sp
		;
		;	the test is > 1% SNR > 3
		;				> 2% SNR > 2
		;				> 3% SNR > 1
		;
		high_P_scale = 2.0
		;
		ind_high_P = WHERE((temp_p gt 1.0*high_P_scale and temp_SNR gt 3.0) or $
						   (temp_p gt 2.0*high_P_scale and temp_SNR gt 2.0) or $
						   (temp_p gt 3.0*high_P_scale and temp_SNR gt 1.0), nhighP)
		;
		if(nhighP gt 0) then begin
			ind_u_reject = ind_high_P
			ind_q_reject = ind_high_P
			n_u_reject = nhighP
			n_q_reject = nhighP
			;
			ntest = N_ELEMENTS(temp_p)
			u_test = lonarr(ntest)
			q_test = lonarr(ntest)
			u_test += 1L
			u_test[ind_u_reject] = 0L
			q_test += 1L
			q_test[ind_q_reject] = 0L
		endif
		;
;		u_test_1 = abs((*(*w_ptr).S10_ptr_pol_struct).U) le sigma_cut
;		u_test_2 = (abs((*(*w_ptr).S10_ptr_pol_struct).U) / (*(*w_ptr).S10_ptr_pol_struct).sU) lt 2.5
;		u_test = u_test_1 or u_test_2
;		;
;		ind_u_reject = WHERE(u_test eq 0,n_u_reject)
;		;
;		q_test_1 = abs((*(*w_ptr).S10_ptr_pol_struct).Q) le sigma_cut
;		q_test_2 = (abs((*(*w_ptr).S10_ptr_pol_struct).Q) / (*(*w_ptr).S10_ptr_pol_struct).sQ) lt 2.5
;		q_test = q_test_1 or q_test_2
		;
		ind_q_reject = WHERE(q_test eq 0,n_q_reject)
		;
		if(n_u_reject + n_q_reject gt 0) then begin
			print_text, Event, "Number of stars rejected for high P values = "+$
						STRING(n_u_reject, format='(I5.1)')
		endif else begin
			print_text, Event, "No stars showed High P values"
		endelse
	endif else begin
		u_test = lonarr(N_ELEMENTS((*(*w_ptr).S10_ptr_pol_struct).U))
		u_test[*] += 1L
		q_test = lonarr(N_ELEMENTS((*(*w_ptr).S10_ptr_pol_struct).Q))
		q_test[*] += 1L
	endelse
	;
	u_index = WHERE( ((*(*w_ptr).S10_ptr_pol_struct).sU le sigma_cut) and $
					 u_test and (ind_bad ne 1), n_u)
	if(n_u gt 0) then begin
		(*w_ptr).S10_n_u_index = n_u
		(*w_ptr).S10_u_index[0:n_u-1] = u_index
		(*w_ptr).S10_u_index[n_u:*] = -1L
	endif
	;
	q_index = WHERE( ((*(*w_ptr).S10_ptr_pol_struct).sQ le sigma_cut) and $
					q_test and (ind_bad ne 1), n_q)
	if(n_q gt 0) then begin
		(*w_ptr).S10_n_q_index = n_q
		(*w_ptr).S10_q_index[0:n_q-1] = q_index
		(*w_ptr).S10_q_index[n_q:*] = -1L
	endif
	;
	;	finally, trap sU and sQ at no lower than 0.2% - 20140217 DPC
	;
	sig_trap = 0.14
	;
	(*(*w_ptr).S10_ptr_pol_struct).sU >= sig_trap
	(*(*w_ptr).S10_ptr_pol_struct).sQ >= sig_trap
	;
	;	populate the textbox with both numbers
	;
	center_in_textbox, (*w_ptr).S10_n_good_stars_textbox_wid, STRING(n_u, n_q, format='(I6.1,";",I6.1)')
	;
	;	and the other quantities
	;
	; 			radius for N data points
	;
	n_stars_per_grid = (*w_ptr).S10_stars_per_grid
	;
	dist = s10_estimate_distance(n_stars_per_grid,n_u > n_q)
	;
	center_in_textbox, (*w_ptr).S10_search_radius_textbox_wid, STRING(dist,format='(F6.1)')
	;
	;	actual number of stars per grid point
	;
	WIDGET_CONTROL, /HOURGLASS
	;
	pol_struct = (*(*w_ptr).S10_ptr_pol_struct)
	;
	x_full = pol_struct.x_det
	y_full = pol_struct.y_det
	u_full = pol_struct.U
	su_full = pol_struct.sU
	q_full = pol_struct.Q
	sq_full = pol_struct.sQ
	;
	sU_cut = sigma_cut
	sQ_cut = sigma_cut
	;
	n_coarse = 150
	trap = sig_trap
	n_save = 0L
	coarse_N = intarr(n_coarse, n_coarse)
	for ix = 0, n_coarse-1 do begin
		for iy = 0, n_coarse-1 do begin
			xi = (ix + 0.5) * (1024.0 / n_coarse)
			yi = (iy + 0.5) * (1026.0 / n_coarse)
			;
			s10_collect_data_points, xi, yi, dist, x_full, y_full, u_full, su_full, $
								 x_vec_u, y_vec_u, u_vec, su_vec, SIG_CUT = sU_cut, SIG_TRAP=trap
			s10_collect_data_points, xi, yi, dist, x_full, y_full, q_full, sq_full, $
								 x_vec_q, y_vec_q, q_vec, sq_vec, SIG_CUT = sQ_cut, SIG_TRAP=trap
			;
			n_in_vec_u = N_ELEMENTS(x_vec_u)
			;
			n_in_vec_q = N_ELEMENTS(x_vec_q)
			;
			coarse_N[ix,iy] = n_in_vec_u > n_in_vec_q
		endfor
	endfor
	;
	;	MFM ave N per pixel
	;
	zz_n = median_filtered_mean(coarse_N)
	;
	center_in_textbox, (*w_ptr).S10_actual_per_grid_textbox_wid, STRING(zz_n[0],zz_n[1],$
					format='(f6.1,"+/-",f6.1)')
	;
	;	turn on next button
	;
	WIDGET_CONTROL, (*w_ptr).S10_compute_p_inst_button_wid, SENSITIVE=1
	;
	SAC_end:
end
;
;---------------------------------------------
pro S10_Compute_p_inst, Event
	;
	;	compute P inst images from the selected stellar data
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	;
	pol_struct = (*(*w_ptr).S10_ptr_pol_struct)
	;
	WIDGET_CONTROL, /HOURGLASS
	;
	x_full = pol_struct.x_det
	y_full = pol_struct.y_det
	;
	n_stars = N_ELEMENTS(x_full)
	;
	;	and the U, Q data
	;
	u_full = pol_struct.U
	su_full = pol_struct.sU
	;
	q_full = pol_struct.Q
	sq_full = pol_struct.sQ
	;
	;	compute collection distance for N data points
	;
	n_stars_per_grid = (*w_ptr).S10_stars_per_grid
	;
	dist = s10_estimate_distance(n_stars_per_grid,n_stars)
	;
;	WIDGET_CONTROL, (*w_ptr).S10_search_radius_textbox_wid, SET_VALUE=STRING(dist,format='(F6.1)')
	;
	;	and compute coarse U, Q images
	;
	n_coarse = 150
	;
	coarse_U = fltarr(n_coarse,n_coarse)
	coarse_Q = coarse_U
	coarse_N = intarr(n_coarse,n_coarse)
	coarse_sQ = coarse_U
	coarse_sU = coarse_U
	;
	;	for each of the image points,
	;
	;	with uncertainties below the indicated cut levels:
	;
	sU_cut = (*w_ptr).S10_sigma_cutoff
	sQ_cut = sU_cut
	;
	;	save list of all stars used, too
	;
	n_save = 0L
	;
	sig_trap = 0.14			; trap at no uncertainties below this % - already done earlier
	;
	for ix = 0, n_coarse-1 do begin
		for iy = 0, n_coarse-1 do begin
			xi = (ix + 0.5) * (1024.0 / n_coarse)
			yi = (iy + 0.5) * (1026.0 / n_coarse)
			;
			s10_collect_data_points, xi, yi, dist, x_full, y_full, u_full, su_full, $
								 x_vec_u, y_vec_u, u_vec, su_vec, SIG_CUT = sU_cut,$
								 SIG_TRAP = sig_trap
			s10_collect_data_points, xi, yi, dist, x_full, y_full, q_full, sq_full, $
								 x_vec_q, y_vec_q, q_vec, sq_vec, SIG_CUT = sQ_cut, $
								 SIG_TRAP = sig_trap
			;
			if(n_save eq 0L) then begin
				x_save = [x_vec_u, x_vec_q]
				y_save = [y_vec_u, y_vec_q]
			endif else begin
				x_save = [x_save, x_vec_u, x_vec_q]
				y_save = [y_save, y_vec_u, y_vec_q]
			endelse
			n_save = N_ELEMENTS(x_save)
			;
			n_in_vec_u = N_ELEMENTS(x_vec_u)
			if(n_in_vec_u le 1) then begin
				if(x_vec_u[0] eq -1) then begin
					coarse_U[ix,iy] = 0.0
					coarse_sU[ix,iy] = 10.0
				endif else begin
					coarse_U[ix,iy] = u_vec[0]
					coarse_sU[ix,iy] = su_vec[0]
				endelse
			endif else begin
				U = s10_interpolate(xi, yi, x_vec_u, y_vec_u, u_vec, su_vec)
				coarse_U[ix,iy] = U[0]
				coarse_sU[ix,iy] = U[1]
			endelse
			;
			n_in_vec_q = N_ELEMENTS(x_vec_q)
			if(n_in_vec_q le 1) then begin
				if(x_vec_q[0] eq -1) then begin
					coarse_Q[ix,iy] = 0.0
					coarse_sQ[ix,iy] = 10.0
				endif else begin
					coarse_Q[ix,iy] = q_vec[0]
					coarse_sQ[ix,iy] = sq_vec[0]
				endelse
			endif else begin
				Q = s10_interpolate(xi, yi, x_vec_q, y_vec_q, q_vec, sq_vec)
				coarse_Q[ix,iy] = Q[0]
				coarse_sQ[ix,iy] = Q[1]
			endelse
			;
			coarse_N[ix,iy] = n_in_vec_u > n_in_vec_q
		endfor
	endfor
	;
	;	subtract ave_U and ave_Q
	;
	;coarse_U -= UQ_struct.ave_U
	;coarse_Q -= UQ_struct.ave_Q
	;
	coarse_P = sqrt(coarse_U * coarse_U + coarse_Q * coarse_Q)
	;
	;	sort for unique saved stars, using a hash table
	;
	hash = 10000L * long(x_save) + long(y_save)
	ind = SORT(hash)
	ind_uniq = UNIQ(hash[ind])
	;
	x_uniq = x_save[ind[ind_uniq]]
	y_uniq = y_save[ind[ind_uniq]]
	;
	;	MFM ave N per pixel
	;
	zz_n = median_filtered_mean(coarse_N)
;	WIDGET_CONTROL, (*w_ptr).S10_actual_per_grid_textbox_wid, SET_VALUE=STRING(zz_n[0],zz_n[1],$
;					format='(f5.1,"+/-",f5.1)')
	;
	;	smooth the U, Q arrays and their uncertainties
	;
	kernel = (*w_ptr).S10_boxcar_size
	ckernel = 3.0 * kernel * (n_coarse/1024.0)			; convert to coarse grid units
	;
	smooth_U = SMOOTH(coarse_U, ckernel,/EDGE_TRUNCATE)
	smooth_Q = SMOOTH(coarse_Q, ckernel,/EDGE_TRUNCATE)
	;smooth_P = sqrt(smooth_U * smooth_U + smooth_Q * smooth_Q)
	smooth_sU = SMOOTH(coarse_sU, ckernel,/EDGE_TRUNCATE)
	smooth_sQ = SMOOTH(coarse_sQ, ckernel,/EDGE_TRUNCATE)
	;smooth_sP = sqrt( (smooth_U * smooth_sU)^2 + (smooth_Q * smooth_sQ)^2) / smooth_P
	;
	;	expand to 1024x1024 images
	;
	interp = 0
	;
	u_image = SMOOTH(CONGRID(smooth_U,1024,1024,/CENTER,INTERP=interp),kernel, /EDGE_TRUNCATE)
	su_image = SMOOTH(CONGRID(smooth_sU,1024,1024,/CENTER,INTERP=interp),kernel, /EDGE_TRUNCATE)
	q_image = SMOOTH(CONGRID(smooth_Q,1024,1024,/CENTER,INTERP=interp),kernel, /EDGE_TRUNCATE)
	sq_image = SMOOTH(CONGRID(smooth_sQ,1024,1024,/CENTER,INTERP=interp),kernel, /EDGE_TRUNCATE)
	;
	p_image = sqrt(u_image * u_image + q_image * q_image)
	sp_image = sqrt( (u_image * su_image)^2 + (q_image * sq_image)^2) / p_image
	sn_image = p_image / sp_image
	n_image = CONGRID(coarse_N,1024,1024,/CENTER,INTERP=interp)
	;
	;	and return the large arrays
	;
	the_result = {u_image: u_image, su_image: su_image, q_image: q_image, $
					sq_image: sq_image, p_image: p_image, sp_image: sp_image, $
					sn_image: sn_image, n_image: n_image, $
					x_stars: x_uniq, y_stars: y_uniq}
	;
	(*w_ptr).S10_ptr_images = PTR_NEW(the_result)
	;
	;	done, so write message and unload ptr to input stars
	;
	print_text, Event, "Instrumental Polarization Images Finished Computing"
	;
	;	turn on button for saving
	;
	WIDGET_CONTROL, (*w_ptr).S10_save_images_button_wid, SENSITIVE=1
end
;
;----------------------------------------------
pro S10_Save_images, Event
	;
	;	writes the images to a designated directory
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	;
	dir_out = dialog_pickfile(title='Directory to Save P_Inst Images Into',/directory)
	len = STRLEN(dir_out)
	if(len lt 5) then begin
		print_text, Event, "Error - bad directory choice. Try again."
		return
	endif
	;
	;	test for exists
	;
	test = FILE_INFO(dir_out)
	if(test.exists ne 1 or test.directory ne 1) then FILE_MKDIR, dir_out
	;
	;	recapture the heap structure
	;
	result = (*(*w_ptr).S10_ptr_images)
	;
	path_u = dir_out + 'u_inst.fits'
	path_su = dir_out + 'u_sig_inst.fits'
	path_q = dir_out + 'q_inst.fits'
	path_sq = dir_out + 'q_sig_inst.fits'
	path_p = dir_out + 'p_inst.fits'
	path_sp = dir_out + 'p_sig_inst.fits'
	path_sn = dir_out + 'p_inst_signal_to_noise.fits'
	path_n = dir_out + 'nstars_used_per_pixel.fits'
	path_reg = dir_out + 'stars_used.reg'
	;
	writefits, path_u, result.u_image
	writefits, path_su, result.su_image
	writefits, path_q, result.q_image
	writefits, path_sq, result.sq_image
	writefits, path_p, result.p_image
	writefits, path_sp, result.sp_image
	writefits, path_sn, result.sn_image
	writefits, path_n, result.n_image
	;
	;	if reg file mode = 1, write that too
	;
	if( (*w_ptr).S10_save_region_mode eq 1) then begin
		;
		openw, lu_stars, path_reg, /GET_LUN
		n_stars = N_ELEMENTS(result.x_stars)
		for is = 0L, n_stars-1L do begin
			printf, lu_stars, result.x_stars[is], result.y_stars[is], $
			        format='("point ",f7.1," ",f7.1," # point=diamond")
		endfor
		;
		;	and if any high pol stars, call them out, also
		;
		;***** tbd
		FREE_LUN, lu_stars
	endif
	;
	S10_unload_heap, Event
	;
	print_text, Event, "Instrumental Polarization Images have been written to directory "+dir_out
end
;----------------------------------------------
pro S10_unload_heap, Event
	;
	;	frees the heap variable holding the polarization structure after the
	;	calculation is over
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	;
	PTR_FREE, (*w_ptr).S10_ptr_pol_struct
	PTR_FREE, (*w_ptr).S10_ptr_images
	;
end
;
;-----------------------------------------------
pro S10_Toggle_avoid_stars, Event
	;
	;	toggle this mode
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	;
	(*w_ptr).S10_avoid_stars_flag = 1 - (*w_ptr).S10_avoid_stars_flag
	;
	WIDGET_CONTROL, (*w_ptr).S10_avoid_stars_checkbox_wid, SET_BUTTON=(*w_ptr).S10_avoid_stars_flag
	;
	case (*w_ptr).S10_avoid_stars_flag of
		0: print_text, Event, "Changing mode to NOT avoid high-pol stars on the list (or no list yet)"
		1: print_text, Event, "Changing mode to avoid high-pol stars on the list"
	endcase
	;
end
;
;-----------------------------------------------
pro S10_Change_high_pol_threshold, Event
	;
	;	manage edits to the high pol S/N cutoff textbox
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	;
	wid = (*w_ptr).S10_high_pol_threshold_textbox_wid
	value = (*w_ptr).S10_high_pol_threshold_value
	;
	S10_Modify_textbox, Event, wid, value, update_textbox
	;
	(*w_ptr).S10_high_pol_threshold_value = value
	str = STRTRIM(STRING(value,format='(F5.2)'),2)
	;
	if(update_textbox eq 1) then begin
		center_in_textbox, wid, str
		print_text, Event, "Changed High Polarization S/N threshold to "+str+" for a star to be considered for the list"
	endif
end
;-----------------------------------------------
pro S10_Change_field_number_threshold, Event
	;
	;	manage edits to the minimum number of fields textbox
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	;
	wid = (*w_ptr).S10_field_number_threshold_textbox_wid
	value = (*w_ptr).S10_field_number_threshold_value
	;
	S10_Modify_textbox, Event, wid, value, update_textbox
	;
	(*w_ptr).S10_field_number_threshold_value = value
	str = STRTRIM(STRING(value,format='(I2.1)'),2)
	;
	if(update_textbox eq 1) then begin
		center_in_textbox, wid, str
		print_text, Event, "Changed Field Number threshold to "+str+" fields required per high-pol star to get on the list"
	endif
end
;
;--------------------------------------------
pro S10_Select_polcats_2, Event
	;
	;	use file chooser to load list of POLCATS to search for high-pol stars
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	;
	;	band
	;
	band = (*w_ptr).S10_Band
	;
	files = dialog_pickfile(Title="Select POLCATS for high-pol star search",/MULTIPLE_FILES)
	nfiles = N_ELEMENTS(files)
	if(nfiles gt 0) then begin
		;
		(*w_ptr).S10_polcat_list_2[0:nfiles-1] = files[*]
		(*w_ptr).S10_n_polcats_2 = nfiles
		;
		print_text, Event, "Found a total of "+STRING(nfiles,format='(I)')+" POLCATS to read"
		;
		;	now, read the polcats
		;
		pol_struct_2 = s10_read_polcats_into_structure(Event, files, band)
		;
		;	and park the structure on heap
		;
		(*w_ptr).S10_ptr_pol_struct_2 = PTR_NEW(pol_struct_2)		; remember to free this at the end of the calculation
		;
	endif else begin
		print_text, Event, "No files selected - try again, or select another option"
	endelse
end
;
;----------------------------------------------
pro S10_Find_high_pol_stars, Event
	;
	;	go through the polcat_2 structure to find high pol stars
	;
	;	save the gpips desig of those with P/sP exceeding the threshold
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	;
	pol_struct_2 = (*(*w_ptr).S10_ptr_pol_struct_2)
	n_pol_2 = N_ELEMENTS(pol_struct_2.U)
	;
	print_text, Event, 'Testing '+STRING(n_pol_2,format='(I5.1)')+' stars for high polarization'
	;
	;	select all entries with P/sP > threshold (and sP > 0.1%)
	;
	sn_threshold = (*w_ptr).S10_high_pol_threshold_value
	;
	P_vec = pol_struct_2.P
	sP_vec = pol_struct_2.sP
	;
	ind_pass_1 = WHERE( ((P_vec / sP_vec) gt sn_threshold) and (sP_vec gt 0.1) ,npass1)
	;
	if(npass1 gt 0) then begin
		print_text, Event, 'Found '+STRING(npass1,format='(i5.1)')+' stars had P/sP > '+$
			STRING( (*w_ptr).S10_high_pol_threshold_value,format='(F5.1)')
		;
		;	found at least one star passes the thresholding
		;
;		;	save the indices
;		;
;		desig_raw_list = pol_struct_2[ind_pass_1].t_desig
;		;
;		;	now, sort them
;		;
;		ind_sort = SORT(desig_raw_list)
;		;
;		desig_list = desig_raw_list[ind_sort]
;		;
;		;	how many of these pass the field number (frequency) test?
;		;
;		ind_uniq = UNIQ(desig_list)		; uniq returns the ending index of each group of repeaters
;		nsets = N_ELEMENTS(ind_uniq)
;		;
;		;	compute first differences
;		;
;		ind_uniq_2 = [0, ind_uniq[0:nsets-2]]
;		;
;		ind_dif = ind_uniq - ind_uniq_2
;		;
;		ind_nf = WHERE(ind_dif ge (*w_ptr).field_number_threshold_value ,npass2)
;		;
;		if(npass2 gt 0) then begin
;			print_text, Event, 'Found '+STRING(npass2,format='(i5.1)')+' stars were seen in at least '+$
;					STRING( (*w_ptr).field_number_threshold_value,format='(i3.1)')+' fields'
			;
			WIDGET_CONTROL, (*w_ptr).S10_number_high_pol_stars_textbox_wid, SET_VALUE=STRING(npass1,format='(i5.1)')
			;
			;	get those 2mass designations
			;
			(*w_ptr).S10_short_list_ptr = PTR_NEW(pol_struct_2[ind_pass_1].g_desig)		; remember to free this after appending
			(*w_ptr).S10_n_short_list = npass1
			;
;		endif else begin
;			print_text, Event, 'No stars were found in at least '+$
;				STRING( (*w_ptr).field_number_threshold_value,format='(i3.1)')+' fields'
;		endelse
	endif else begin
		print_text, Event, 'No stars were found to have P/sP > '+$
			STRING( (*w_ptr).S10_high_pol_threshold_value,format='(F5.1)')
	endelse
	;
	print_text, Event, 'Done testing POLCAT stars for high pol entries'
end
;
;------------------------------------------------------------
pro S10_Create_select_list, Event
	;
	;	either creates a file on disk to hold the high-polarization list,
	;	or, if the desired one already exists, opens it for writing.
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	;
	file_in = dialog_pickfile(Title='Select or Create high pol list disk file')
	nf = STRLEN(file_in)
	if(nf lt 5) then begin
		print_text, Event, 'ERROR - no file selected. Try again.'
		goto, SCSL_end
	endif
	;
	;	test for the file's presence
	;
	test = FILE_INFO(file_in)
	if(test.exists eq 1) then begin
		;
		;	file exists, just open it
		;
		openw, lu_out, file_in, /GET_LUN, /APPEND
		;
	endif else begin
		;
		;	create one on disk
		;
		openw, lu_out, file_in, /GET_LUN
		;
	endelse
	;
	(*w_ptr).S10_list_pathname = file_in
	;
	;	save the lu number
	;
	(*w_ptr).S10_high_pol_file_lun = lu_out
	;
	print_text, Event, 'Created or opened file '+FILE_BASENAME(file_in)+' for saving high pol star designations'
	;
	SCSL_end:
end
;
;----------------------------------------------------------
pro S10_Clear_or_append_list, action, Event
	;
	;	clears a list file (on disk and internally), or appends
	;	the short list to the current disk file and closes it.
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	;
	;	selection action
	;
	if(action eq 0) then begin						; clear the list(s)
		;
		;	flush the short list - dunno if this should be done, so comment out for now
		;
;		if( PTR_VALID( (*w_ptr).S10_short_list_ptr ) eq 1) then begin
;			PTR_FREE, (*w_ptr).S10_short_list_ptr
;			(*w_ptr).S10_n_short_list = 0
;		endif
		;
		;	and the disk file (delete, then reopen)
		;
		if( (*w_ptr).S10_high_pol_file_lun gt 0) then begin
			FREE_LUN, (*w_ptr).S10_high_pol_file_lun
			FILE_DELETE, (*w_ptr).S10_list_pathname
			;
			openw, lu_out, (*w_ptr).S10_list_pathname, /GET_LUN
			(*w_ptr).S10_high_pol_file_lun = lu_out
		endif
		;
		print_text, Event, 'Cleared disk file'
	endif else begin
		;
		;	appends the short list to the currently open file
		;
		n_desig = (*w_ptr).S10_n_short_list
		;
		if(n_desig gt 0) then begin
			lu_out = (*w_ptr).S10_high_pol_file_lun
			;
			for i = 0, n_desig-1 do begin
				;
				printf, lu_out, STRTRIM( (*(*w_ptr).S10_short_list_ptr)[i],2)
			endfor
			;
			print_text, Event, 'Wrote '+STRING( n_desig, format='(i5.1)')+' high pol star designations to disk list'
		endif
		;
		;	and close the open file
		;
		FREE_LUN, (*w_ptr).S10_high_pol_file_lun
		;
		;	but don't free the open pointers until the list has been used
		;
;		if( PTR_VALID( (*w_ptr).S10_short_list_ptr ) eq 1) then begin
;			PTR_FREE, (*w_ptr).S10_short_list_ptr
;			(*w_ptr).S10_n_short_list = 0
;		endif
	endelse
	;
end