;
;	Step 3 Event Handler Routines - Astrometry
;
pro S3_Init, Event
	;
	;	initialize S3 stuff when tabbing onto Step 3
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	g_ptr = PTR_struct.PTR_group
	;
	;
	;	populate the group table
	;
	n_groups = (*g_ptr).n_groups
	if(n_groups gt 0) then begin
		ind = WHERE((*g_ptr).S3_group_queue[0:n_groups-1] ge 1,count)
		if(count gt 0) then (*g_ptr).S3_group_queue[ind] = -2
	endif
	S3_populate_group_list, Event
	;
	;	clear LEDs to red
	;
	S3_LED_sensitize, Event, 1, 0
	S3_LED_sensitize, Event, 2, 0
	;
	;	and set processing radio buttons to reflect saved status
	;
	S3_camera, Event, 0				; set to F/5 initially
	;
	S3_Show_Radio_Buttons, Event
	;
	;	toggle defocussed state
	;
	(*w_ptr).S3_defocus_mode = 1 - (*w_ptr).S3_defocus_mode
	S3_Toggle_Defocus, Event
	;
	;  toggle background status
	;  
	(*w_ptr).S3_flatten_bg_mode = 1 - (*w_ptr).S3_flatten_bg_mode
	S3_Toggle_Flatten_BG, Event
	;
	;	populate the history window
	;
	h_wid = (*w_ptr).S3_history_textbox_wid
	SAll_populate_history, Event, h_wid, 3
	;
	;	display the latest bad astrometry count
	;
	WIDGET_CONTROL, (*w_ptr).S3_bad_fit_textbox_wid, SET_VALUE=STRING((*w_ptr).S3_bad_fit_count,format='(I4.1)')
end
;
;------------------------------------------------------------------
pro S3_Show_Radio_Buttons, Event
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	;
	for icamera = 0, 1 do begin
		case icamera of
			0: wid = (*w_ptr).S3_F5_wid
			1: wid = (*w_ptr).S3_F17_wid
		endcase
		;
		if((*w_ptr).S3_camera_mode eq icamera) then ib = 1 else ib = 0
		WIDGET_CONTROL, wid, SET_BUTTON=ib
	endfor
	;
	wid = (*w_ptr).S3_save_2mass_wid
	;
	if((*w_ptr).S3_2mass_mode eq 1) then ib = 1 else ib = 0
	WIDGET_CONTROL, wid, SET_BUTTON=ib
	;
	wid = (*w_ptr).S3_flatten_bg_wid
	if((*w_ptr).S3_flatten_bg_mode eq 1) then ib = 1 else ib = 0
	WIDGET_CONTROL, wid, SET_BUTTON=ib
	;
end
;
;-----------------------------------------------
pro S3_Camera, Event, ibutton
	;
	;	turns on the indicated radio button, turning off the others
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	;
	(*w_ptr).S3_camera_mode = ibutton
	case ibutton of
		0: print_text, Event, "Switching to F5 camera mode"
		1: print_text, Event, "Switching to F17 camera mode"
	endcase
	;
	S3_Show_Radio_Buttons, Event
end
;
;-----------------------------------------------
pro S3_2MASS, Event
	;
	;	toggles 2mass flag
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	;
	;	read button status
	;
	wid = (*w_ptr).S3_save_2mass_wid
	ibutton = WIDGET_INFO(wid, /BUTTON_SET)
	;
	(*w_ptr).S3_2mass_mode = ibutton
	case ibutton of
		0: print_text, Event, "Turning off 2MASS file save mode"
		1: print_text, Event, "Turning on 2MASS file save mode"
	endcase
	;
	S3_Show_Radio_Buttons, Event
end
;
;-----------------------------------------------
pro S3_Toggle_Flatten_BG, Event
  ;
  ; toggles flatten background flag
  ;
  WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
  w_ptr = PTR_struct.PTR_wid_id
  ;
  ; read button status
  ;
  wid = (*w_ptr).S3_flatten_bg_wid
  ibutton = WIDGET_INFO(wid, /BUTTON_SET)
  ;
  (*w_ptr).S3_flatten_bg_mode = ibutton
  case ibutton of
    0: print_text, Event, "Turning off flatten image backgrounds mode"
    1: print_text, Event, "Turning on flatten image backgrounds mode"
  endcase
  ;
  S3_Show_Radio_Buttons, Event
end
;-----------------------------------------------
pro S3_Toggle_Defocus, Event
	;
	;	toggles defocus flag
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	;
	(*w_ptr).S3_defocus_mode = 1 - (*w_ptr).S3_defocus_mode
	;
	WIDGET_CONTROL, (*w_ptr).S3_defocus_wid, SET_BUTTON=(*w_ptr).S3_defocus_mode
	;
	case (*w_ptr).S3_defocus_mode of
		0: print_text, Event, "Turning off defocus mode"
		1: print_text, Event, "Turning on defocus mode"
	endcase
	;
end
;-----------------------------------------------
pro S3_Select_Groups, Event
	;
	;	one or more items in the group listbox was/were selected
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	s_ptr = PTR_struct.PTR_summary
	w_ptr = PTR_struct.PTR_wid_id
	g_ptr = PTR_struct.PTR_group
	;
	wid = (*w_ptr).S3_group_listbox_wid
	index = WIDGET_INFO( wid, /LIST_SELECT)
	n = N_ELEMENTS(index)
	if(n eq 1 and index[0] eq -1) then goto, S3_SG_end
	;
	;	select only groups with Step >=2
	;
	ind = WHERE((*g_ptr).group_step[index] ge 2, count)
	if(count gt 0) then begin
		index = index[ind]
		if(count ne n) then print_text, Event, "Warning - some groups not processed through Step 2 - skipping those."
		n = count
	endif else begin
		print_text, Event, "Warning - no groups processed through Step 2. Go back"
		goto, S3_SG_end
	endelse
	;
	(*g_ptr).n_groups_selected = n
	;
	print_text, Event, "Number of selected groups = "+STRING(format='(I)',n)
	;
	;	set the index - it gets reset after a button action is handled
	;
	(*g_ptr).selected_groups[0:n-1] = index[*]
	if(n eq 1) then begin
		group = (*g_ptr).group_names[index[0]]
		print_text, Event, "Selected Group "+group+" from Group List"
	endif else begin
		print_text, Event, "Selected "+STRING(format='(I)',n)+" groups from list"
	endelse
	;
	S3_SG_end:
end
;
;-----------------------------------------------
pro S3_Add_To_Queue, Event
	;
	;	selected groups are added to the S3 processing queue
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	s_ptr = PTR_struct.PTR_summary
	g_ptr = PTR_struct.PTR_group
	;
	;	how many selected?
	;
	n = (*g_ptr).n_groups_selected
	;
	if(n gt 0) then begin
		for ig = 0, n-1 do begin
			j = (*g_ptr).selected_groups[ig]
			;
			;	set queue pointer to 2 (enqueued)
			;
			(*g_ptr).S3_group_queue[j] = 2
		endfor
	endif
	;
	;	reload listbox
	;
	S3_populate_group_list, Event
	(*g_ptr).n_groups_selected = 0
	(*g_ptr).selected_groups[*] = 0
end
;-----------------------------------------------
pro S3_Remove_From_Queue, Event
	;
	;	selected groups are removed (skipped) from the S3 processing queue
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	s_ptr = PTR_struct.PTR_summary
	g_ptr = PTR_struct.PTR_group
	;
	;	how many selected?
	;
	n = (*g_ptr).n_groups_selected
	;
	if(n gt 0) then begin
		for ig = 0, n-1 do begin
			j = (*g_ptr).selected_groups[ig]
			;
			(*g_ptr).S3_group_queue[j] = -1
		endfor
	endif
	;
	;	reload listbox
	;
	S3_populate_group_list, Event
	(*g_ptr).n_groups_selected = 0
	(*g_ptr).selected_groups[*] = 0
end
;
;-----------------------------------------------
pro S3_LED_sensitize, Event, S3_LED_number, sense
	;
	;	initializes LED stacks
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	;
	if(S3_LED_number eq 1) then begin
		wid = (*w_ptr).S3_LED1_images_wid
	endif else begin
		wid = (*w_ptr).S3_LED2_groups_wid
	endelse
	;
	WIDGET_CONTROL, wid, SENSITIVE=sense
	;
	;	set all colors to red upon sensitization
	;
	WIDGET_CONTROL, wid, GET_UVALUE=nLED
	;
	n_ele = N_ELEMENTS(nLED)
	if(n_ele eq 0) then nLED = 25
	if(nLED eq 0) then nLED = 25
	WIDGET_CONTROL, wid, SET_UVALUE=nLED
	colors = strarr(nLED)
	if(sense eq 1) then colors[*] = "red" else colors[*] = "gray"
	WCT_LED_stack, wid, colors
end
;
;-----------------------------------------------
pro S3_populate_group_list, Event
	;
	;	lists current groups and step status by name on the listbox
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	s_ptr = PTR_struct.PTR_summary
	g_ptr = PTR_struct.PTR_group
	;
	n_groups = (*g_ptr).n_groups
	if(n_groups gt 0) then begin
		;
		;	load group name,step and meta status, and queue status
		;
		text_vec = strarr(n_groups)
		for ig = 0, n_groups-1 do begin
			if((*g_ptr).group_step[ig] lt 2) then (*g_ptr).S3_group_queue[ig] = -2
			i_queue = (*g_ptr).S3_group_queue[ig]
			;
			case i_queue of
				-2:	queue = '?'
				-1: queue = 'Skip'
				 0: queue = 'Done'
				 1: queue = 'Active'
				 2: queue = 'Queued'
				else: queue = '...'
			endcase
			;
			;	parse group step if >30 <40
			;
			group_step = (*g_ptr).group_step[ig]
			if((group_step ge 30 and group_step le 40) or $
			  (group_step ge 60 and group_step le 69) or $
			  (group_step ge 110 and group_step le 119)) then begin
				step_0 = FLOOR(group_step / 10)
				step_1 = group_step - 10 * step_0
				text_vec[ig] = STRTRIM(STRING(format='(A," Step=",I2.1,".",I1.1,", Meta=",I2.1,", Action= ",A)',$
					(*g_ptr).group_names[ig],step_0,step_1,(*g_ptr).group_meta[ig],queue),1)
			endif else begin
				step_0 = group_step
				step_1 = 0
				text_vec[ig] = STRTRIM(STRING(format='(A," Step=",I2.1,", Meta=",I2.1,", Action= ",A)',$
					(*g_ptr).group_names[ig],step_0,(*g_ptr).group_meta[ig],queue),1)
			endelse
			;

		endfor
	endif else begin
		text_vec = ['','']
	endelse
	;
	WIDGET_CONTROL, (*w_ptr).S3_group_listbox_wid, SET_VALUE=text_vec
	;
end
;
;------------------------------------------------------
pro S3_Test_astrometry, Event
	;
	;	for the selected queued groups, examine CD matrices in
	;	FITS headers to test for problems:
	;		1. high variances
	;		2. way wrong values (inverted, rotated, bad platescale)
	;		3. image center RA, Decs off by more than 1/2 image size
	;
	;	20090409 - added test for sparse fields. If fewer than 10 stars,
	;				all numerical criteria relaxed by factor of 3. Obtain
	;				info from PPOLNSTR variable in FITS header
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	s_ptr = PTR_struct.PTR_summary
	g_ptr = PTR_struct.PTR_group
	h_ptr = PTR_struct.PTR_history
	;
	h_wid = (*w_ptr).S3_History_textbox_wid
	;
	;	write current group, SN textboxes
	;
	grp_wid = (*w_ptr).S3_group_name_textbox_wid
	sn_wid = (*w_ptr).S3_sn_textbox_wid
	;
	;	clear the bad fit counter
	;
	(*w_ptr).S3_bad_fit_count = 0L
	WIDGET_CONTROL, (*w_ptr).S3_bad_fit_textbox_wid, SET_VALUE=STRING((*w_ptr).S3_bad_fit_count,format='(I4.1)')
	;
	;	how many active items on the queue?
	;
	ind_queue = WHERE((*g_ptr).S3_group_queue[0:(*g_ptr).n_groups-1] eq 2,count)
	if(count le 0) then begin
		; if none, quit
		print_text, Event, "WARNING - no items in queue to process."
		goto, S3_TA_end
	endif
	;
	print_text, Event, "Found "+STRING(format='(I)',count)+" groups in the queue. Processing starting"
	;
	;	Clear LED2 to red with the number of groups to process
	;
	LED2_wid = (*w_ptr).S3_LED2_groups_wid
	LED1_wid = (*w_ptr).S3_LED1_images_wid
	;
	WIDGET_CONTROL, LED2_wid, SET_UVALUE=count
	group_colors = strarr(count)
	group_colors[*] = "red"
	S3_LED_sensitize, Event, 2, 1
	;
	;	for each of the enqueued groups:
	;
	for ig = 0, count-1 do begin
		;
		g_ind = ind_queue[ig]
		group_name = (*g_ptr).group_names[g_ind]
		print_text, Event, "Starting Astrometry Quality Testing for Group = "+group_name
		;
		WIDGET_CONTROL, grp_wid, SET_VALUE=STRTRIM(group_name,2)
		WIDGET_CONTROL, sn_wid, SET_VALUE = ''
		;
		;	verify that astrometry has already been performed for this group
		;
		if( (*g_ptr).group_step[g_ind] lt 3) then begin
			print_text, Event, "ERROR - No Astrometry performed yet for this group. Can't do test."
			goto, S3_TA_group_end
		endif
		;
		;	change the list box queue list to active for this group
		;
		(*g_ptr).S3_group_queue[g_ind] = 1
		S3_populate_group_list, Event
		;
		;	and reposition list to show this active group
		;
		if( g_ind gt 10) then begin
			WIDGET_CONTROL, (*w_ptr).S3_group_listbox_wid, SET_LIST_TOP= ( g_ind - 6)
		endif
		;
		;	change the LED2 bar to blue
		;
		group_colors[ig] = "blue"
		WCT_LED_stack, LED2_wid, group_colors
		;
		;	how many images to examine? Reset LED1 to reflect it
		;
		n_image = (*g_ptr).group_numbers[g_ind]
		;
		;	if > 130, use LED scaling
		;
		nLED = 130
		LED_scale = float(n_image) / float(nLED)
		if(LED_scale le 1.0) then LED_scale = 1.0
		;
		WIDGET_CONTROL, LED1_wid, SET_UVALUE=(nLED < n_image)
		image_colors = strarr(nLED < n_image)
		image_colors[*] = "red"
		S3_LED_sensitize, Event, 1, 1
		;
		;	for every image
		;
		ra_cen = dblarr(n_image)
		dec_cen = dblarr(n_image)
		cd_array = dblarr(n_image, 2, 2)
		good_image = strarr(n_image)
		;
		case (*w_ptr).S3_camera_mode of
			0: begin
				platescale_guess = 0.5788			; arcsec per pixel
				tolerance = 0.0005
			   end
			1: begin
				platescale_guess = 0.18
				tolerance = 0.0015
			   end
		endcase
		;
		i_count = 0L
		;
		i_bad_total = 0L
		i_first_good_image = -1L
		rich_scale_save = fltarr(n_image)
		i_image_save = lonarr(n_image)
		;
		for i_image = 0, n_image-1 do begin
			;
			;	set LED1 to blue
			;
			this_LED = ROUND(float(i_image) / LED_scale) & this_LED = 0 > ((nLED-1) < this_LED)
			image_colors[this_LED] = "blue"
			WCT_LED_stack, LED1_wid, image_colors
			;
			;	get the image name and check that its flag is valid
			;
			image_path = (*g_ptr).group_images[g_ind, i_image]
			;
;			print_text, Event, "Reading Astrometry data from FITS header of image " + $
;								FILE_BASENAME(image_path)
			;
			if((*g_ptr).group_image_flags[g_ind,i_image] eq 0) then begin
				str = FILE_BASENAME(image_path)+": use flag =0, no astro testing tried"
				print_text, Event, "Warning - image "+str
				SAll_add_to_history, Event, h_wid, 3, str
				SAll_populate_history, Event, h_wid, 3
				image_colors[this_LED] = "white"
				WCT_LED_stack, LED1_wid, image_colors
				goto, S3_test_image_end
			endif
			if(i_first_good_image eq -1) then i_first_good_image = i_image
			;
			;	build pathname to image
			;
			image_name = FILE_BASENAME(image_path)
			image_dir = (*s_ptr).analysis_dir + 'S3_Astrometry' + PATH_SEP()
			image_test_path = image_dir + image_name
			test = FILE_INFO(image_test_path)
			if(test.exists) then begin
				header = headfits(image_test_path, /SILENT)
				;
				;	pull out the astrometry info
				;
				CD1_1 = sxpar(header, "CD1_1")
				CD1_2 = sxpar(header, "CD1_2")
				CD2_1 = sxpar(header, "CD2_1")
				CD2_2 = sxpar(header, "CD2_2")
				;
;				tel_ra = 15.d*ten(strsplit(sxpar(header,'TELRA'),':',/EXTRACT)); telescope RA in degrees
;				tel_dec = ten(strsplit(sxpar(header,'TELDEC'),':',/EXTRACT)); telescope Dec in degrees
				tel_ra = sxpar(header, "CRVAL1")
				tel_dec = sxpar(header, "CRVAL2")
				;
				;	ascertain richness or sparseness of star field
				;
				nstars_found = sxpar(header, "PPOLNSTR", COUNT=ppol_count)
				if(ppol_count gt 0) then begin
					if(nstars_found le 1) then rich_scale = 50.0
					if(nstars_found ge 2) then rich_scale = 20.0
					if(nstars_found ge 6 and tel_dec lt 87.0) then rich_scale = 3.0
					if(nstars_found ge 10 and tel_dec lt 80.0) then rich_scale = 1.0
;					print_text, Event, "NStars found = "+STRING(nstars_found)+" so rich_scale = "+STRING(rich_scale)
				endif else begin
					rich_scale = 1.0
				endelse
				;
				;	test for craziness
				;
				;	1. do CD1_1 or CD2_2 have plate scales wildly at odds with
				;		camera platescale?
				;
				ibad = 0L
				if(CD1_1 ge 0.0 or CD2_2 le 0.0) then ibad = 1L		; shouldn't happen
				if((abs(CD1_1 + CD2_2)/abs(CD1_1)) gt 0.004*rich_scale) then begin	; was 0.005 20100804
					ibad = ibad + 2L
					print_text, Event, "X,Y platescales differ. X, Y PS = "+STRING(abs(CD1_1)*3600.0)+", "+STRING(abs(CD2_2)*3600.0)
				endif
				if(((abs(CD1_1)*3600.0 - platescale_guess)/platescale_guess) gt 0.004*rich_scale) then begin ; was 0.005 20100804
					ibad = ibad + 4L
					print_text, Event, "Bad Platescale = "+STRING(abs(CD1_1)*3600.0)
					print_text, Event, "threshold = "+STRING(0.004*rich_scale)
				endif
				;
				;	2. look for rotation - if more than 2 deg, this isn't right, unless at very high dec... (2/4/13)
				;
				angle = d_angle_zero((180.0/!DPI)*ATAN(CD1_2,CD1_1))
				if(abs(angle) gt 2.0*rich_scale) then begin
					ibad = ibad + 8L
					print_text, Event,"X axis rotation angle = "+STRING(angle)
				endif
				angle2 = d_angle_zero((180.0/!DPI)*ATAN(CD2_1,CD2_2))
				if(abs(angle2) gt 2.0*rich_scale) then begin
					ibad = ibad + 16L
					print_text, Event, "Y axis rotation angle = "+STRING(angle2)
				endif
				;
				;	3. check for RA, Dec centers way off from previous image
				;			(skip test for first image)
				;
				if(i_image ne i_first_good_image) then begin
					;
					d_ra = abs(tel_ra - ra_old)
					d_dec = abs(tel_dec - dec_old)
					cos_dec = COS(dec_old * !DPI / 180.0)
					;
					if(abs(d_ra * cos_dec) gt rich_scale*(512.0 * platescale_guess / 3600.0)) then ibad = ibad + 32L
					if(abs(d_dec) gt rich_scale*(512.0 * platescale_guess / 3600.0)) then ibad = ibad + 64L
				endif else begin
					ra_old = tel_ra
					dec_old = tel_dec
				endelse
				;
				;	load arrays for computation of means, dispersions
				;
				i_image_save[i_count] = i_image
				rich_scale_save[i_count] = rich_scale
				ra_cen[i_count] = tel_ra
				dec_cen[i_count] = tel_dec
				cd_array[i_count,0,0] = CD1_1
				cd_array[i_count,0,1] = CD1_2
				cd_array[i_count,1,0] = CD2_1
				cd_array[i_count,1,1] = CD2_2
				;
				good_image[i_count] = image_test_path
				;
				i_count++
				;
				;	if ibad > 0 already, write to the problem history array
				;
				if(ibad gt 0) then begin
					str = FILE_BASENAME(image_test_path) + " IBAD="+STRING(ibad, format="(B-8.8)")
					print_text, Event, str
					SAll_add_to_history, Event, h_wid, 3, str
					;
					i_bad_total = i_bad_total + ibad
					;
					;	set LED to orange
					;
					image_colors[this_LED] = "orange"
					WCT_LED_stack, LED1_wid, image_colors
					;
					;	and turn off use flag
					;
					(*g_ptr).group_image_flags[g_ind,i_image] = 0
				endif else begin
					;
					;	if passed tests, set LED to green
					;
					image_colors[this_LED] = "green"
					WCT_LED_stack, LED1_wid, image_colors
				endelse
				;
			endif else begin
				str = "ERROR - image "+FILE_BASENAME(image_test_path)+" not found"
				print_text,Event, str
				SAll_add_to_history, Event, h_wid, 3, str
				;
				;	turn off use flag
				;
				(*g_ptr).group_image_flags[g_ind,i_image] = 0
				image_colors[this_LED] = "yellow"
				WCT_LED_stack, LED1_wid, image_colors
			endelse
			;
			S3_test_image_end:
		endfor					; image end
		;
		;	this group done - compute mfm properties and look for outliers
		;
		zz_ra = median_filtered_mean(ra_cen[0:i_count-1])
		zz_dec = median_filtered_mean(dec_cen[0:i_count-1])
		zz_CD1_1 = median_filtered_mean(REFORM(cd_array[0:i_count-1,0,0]))
		zz_CD1_2 = median_filtered_mean(REFORM(cd_array[0:i_count-1,0,1]))
		zz_CD2_1 = median_filtered_mean(REFORM(cd_array[0:i_count-1,1,0]))
		zz_CD2_2 = median_filtered_mean(REFORM(cd_array[0:i_count-1,1,1]))
		;
		;	cap CD uncertainties at no smaller than 0.5 pix over 1024 (0.05%)
		;
		zz_CD1_1[1] = zz_CD1_1[1] > (0.0005 * abs(zz_CD1_1[0]))
		zz_CD2_2[1] = zz_CD2_2[1] > (0.0005 * abs(zz_CD2_2[0]))
		;
		;	cap RA, dec uncertainties to 1/2 frame size
		;
		rtod = (180.0 / !DPI)
		;
		cos_dec = COS(zz_dec[0]/rtod)
		zz_ra[1] = zz_ra[1] > (512.0 * platescale_guess / (3600.0 * cos_dec))
		zz_dec[1] = zz_dec[1] > (512.0 * platescale_guess / 3600.0)
		;
		sn = ROUND(abs(zz_CD1_1[0]/zz_CD1_1[1]))
		print_text, Event, "Computed platescale = "+STRING(abs(zz_CD1_1[0]*3600.0),sn,$
						format='(F9.6," S/N= ",I)')
		;
		;	compute angles
		;
		z_angle = dblarr(i_count)
		z_angle2 = dblarr(i_count)
		;
		;	cap angle uncertainties, though
		;
		for ii = 0, i_count-1 do begin
			z_angle[ii] = d_angle_zero(rtod * ATAN(cd_array[ii,0,1], cd_array[ii,0,0]))
			z_angle2[ii] = d_angle_zero(rtod * ATAN(cd_array[ii,1,0], cd_array[ii,1,1]))
		endfor
		;
		zz_angle =  median_filtered_mean(z_angle)
		zz_angle2 =  median_filtered_mean(z_angle2)
		;
		zz_angle[1] = zz_angle[1] > 0.1					; deg
		zz_angle2[1] = zz_angle2[1] > 0.1
		;
		;	for every image in the group, repeat the tests, but at the 0.4% level (was 0.5% 20100804)
		;
		if( (abs(zz_CD1_1[0] * 3600.0)-platescale_guess) /platescale_guess gt 0.004*rich_scale) then begin
			str = "Group Platescale bad. CD1_1= "+STRING(abs(3600.0*zz_CD1_1[0]),zz_CD1_1[1],$
								format='(F9.6,"+/-",F9.6)')
			print_text, Event, str
			SAll_add_to_history, Event, h_wid, 3, str
			;
			i_bad_total = i_bad_total + 256L
		endif
		;
		if(abs(zz_angle[0]) gt rich_scale*3. or abs(zz_angle2[0]) gt rich_scale*3.) then begin
			str = "Group Rot angle(s) bad. Ang, Ang2="+STRING(zz_angle[0], zz_angle2[0], format="(F,1x,F)")
			print_text, Event, str
			SAll_add_to_history, Event, h_wid, 3, str
			;
			i_bad_total = i_bad_total + 512L
		endif
		;
		for ii = 0, i_count-1 do begin
			;
			;	have to reaccess rich_scale
			;
			rich_scale = rich_scale_save[ii]
			i_image = i_image_save[ii]
			;
			ibad2 = 0
			;
			;	1. Platescale tests
			;
			dps1 = abs(cd_array[ii,0,0] - zz_CD1_1[0]) / zz_CD1_1[1]
			dps2 = abs(cd_array[ii,1,1] - zz_CD2_2[0]) / zz_CD2_2[1]
			if(dps1 gt 5.0*rich_scale) then ibad2 = ibad2 + 1
			if(dps2 gt 5.0*rich_scale) then ibad2 = ibad2 + 2
			;
			if(ibad2 gt 0) then begin
				print_text, Event, "Image "+FILE_BASENAME(good_image[ii])+" platescales = "+$
							STRING(abs(3600.0*dps1), abs(3600.0*dps2))
			endif
			;
			;	2. Rotation tests
			;
			if(abs(z_angle[ii] - zz_angle[0])/zz_angle[1] gt rich_scale*5.0) then begin
				ibad2 = ibad2 + 8
				print_text, Event, "Failed X-angle test. Angle = "+STRING(z_angle[ii])+" rich_scale = "+STRING(rich_scale)
			endif
			if(abs(z_angle2[ii] - zz_angle2[0])/zz_angle2[1] gt rich_scale*5.0) then begin
				ibad2 = ibad2 + 16
				print_text, Event, "Failed Y-angle test. Angle = "+STRING(z_angle2[ii])
			endif
			;
			;	3. RA, Dec centers
			;
			if(abs(ra_cen[ii] - zz_ra[0])/zz_ra[1] gt rich_scale*1.0) then begin
				ibad2 = ibad2 + 32
				print_text, Event, "ZZRA[0], ZZRA[1],  "+STRING(zz_ra[0],zz_ra[1],ra_cen[ii],$
			        format='(F20.8,2x,f20.8,1x,f20.8)')
			endif
			;
			if(abs(dec_cen[ii] - zz_dec[0])/zz_dec[1] gt rich_scale*1.0) then ibad2 = ibad2 + 64L
			;
			if(ibad2 gt 0) then begin
				str = FILE_BASENAME(good_image[ii]) + " IBAD2="+STRING(ibad2, format="(B-10.10)")+" USE->0"
				print_text, Event, str
				SAll_add_to_history, Event, h_wid, 3, str
				;
				;	turn its use flag off
				;
				(*g_ptr).group_image_flags[g_ind,i_image] = 0		; turn this guy off
				;
				i_bad_total = i_bad_total + ibad2
			endif
		endfor
		;
		;	final assessment - if any flags were set, write message
		;
		if(i_bad_total gt 0 and i_bad_total lt 256) then begin
			str = group_name+": Image-Level fail - BTOT="+STRING(i_bad_total, $
					format='(B-10.10)')
			print_text, Event, str
			SAll_add_to_history, Event, h_wid, 3, str
			;
			group_colors[ig] = "orange"
			;
			;	but permit continuing
			;
			if( (*g_ptr).group_step[g_ind] lt 3) then (*g_ptr).group_step[g_ind] = 3
			(*g_ptr).S3_group_queue[g_ind] = 0		; set queue to "done"
			;
			;	look for collossal errors
			;
			if(i_bad_total ge 256) then begin
				str = group_name+": Failed Astro Test - BTOT="+STRING(i_bad_total, $
					format='(B-10.10)')
				print_text, Event, str
				SAll_add_to_history, Event, h_wid, 3, str
				;
				group_colors[ig] = "red"
				;
				;	set step back to 31 to signify fail
				;
				(*g_ptr).group_step[g_ind] = 31
				(*g_ptr).S3_group_queue[g_ind] = -2		; set queue to "?"
			endif
			;
		endif else begin
			;
			print_text, Event, "Group = "+group_name+" passed Astro Testing with flying colors!"
			;
			WIDGET_CONTROL, sn_wid, SET_VALUE=STRTRIM(STRING(sn,format='(I)'),2)
			;
			;	change color of group LED to green
			;
			group_colors[ig] = "green"
			;
			;	and update step status value for this group
			;
			grp_step = (*g_ptr).group_step[g_ind]
			if((grp_step ge 30 and grp_step le 40) or grp_step eq 3) then begin
				(*g_ptr).group_step[g_ind] = 32
			endif
			;
			;	queue to done
			;
			(*g_ptr).S3_group_queue[g_ind] = 0
		endelse
		;
		WCT_LED_stack, LED2_wid, group_colors
		;
		S3_populate_group_list, Event
		if( g_ind gt 10) then begin
			WIDGET_CONTROL, (*w_ptr).S3_group_listbox_wid, SET_LIST_TOP= ( g_ind - 6)
		endif
		MSP_PPOL_Save_Group_File, Event
		;
		S3_TA_group_end:
	endfor						; group end
	;
	print_text, Event, "Astrometry Testing Completed"
	S3_TA_end:
end
;
;------------------------------------------------------
pro S3_Start_Astrometry, Event
	;
	;	Begin processing of all enqueued requests for astrometric determination
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	s_ptr = PTR_struct.PTR_summary
	g_ptr = PTR_struct.PTR_group
	h_ptr = PTR_struct.PTR_history
	;
	h_wid = (*w_ptr).S3_History_textbox_wid
	grp_wid = (*w_ptr).S3_group_name_textbox_wid
	;
	;	fetch the defocus mode flag
	;
	defocus_flag = (*w_ptr).S3_defocus_mode
	;
	;	clear the bad fit counter
	;
	(*w_ptr).S3_bad_fit_count = 0L
	WIDGET_CONTROL, (*w_ptr).S3_bad_fit_textbox_wid, SET_VALUE=STRING((*w_ptr).S3_bad_fit_count,format='(I4.1)')
	;
	;	how many active items on the queue?
	;
	ind_queue = WHERE((*g_ptr).S3_group_queue[0:(*g_ptr).n_groups-1] eq 2,count)
	if(count le 0) then begin
		; if none, quit
		print_text, Event, "WARNING - no items in queue to process."
		goto, S3_SA_end
	endif
	;
	print_text, Event, "Found "+STRING(format='(I)',count)+" groups in the queue. Processing starting"
	;
	;	Clear LED2 to red with the number of groups to process
	;
	LED2_wid = (*w_ptr).S3_LED2_groups_wid
	LED1_wid = (*w_ptr).S3_LED1_images_wid
	;
	WIDGET_CONTROL, LED2_wid, SET_UVALUE=count
	group_colors = strarr(count)
	group_colors[*] = "red"
	S3_LED_sensitize, Event, 2, 1
	;
	;	for each of the enqueued groups:
	;
	for ig = 0, count-1 do begin
		;
		g_ind = ind_queue[ig]
		group_name = (*g_ptr).group_names[g_ind]
		print_text, Event, "Starting Astrometry Processing for Group = "+group_name
		;
		WIDGET_CONTROL, grp_wid, SET_VALUE=STRTRIM(group_name,2)
		;
		;	change the list box queue list to active for this group
		;
		(*g_ptr).S3_group_queue[g_ind] = 1
		S3_populate_group_list, Event
		;
		;	and reposition the list
		;
		if( g_ind gt 10) then begin
			WIDGET_CONTROL, (*w_ptr).S3_group_listbox_wid, SET_LIST_TOP= ( g_ind - 6)
		endif
		;
		;	change the LED2 bar to blue
		;
		group_colors[ig] = "blue"
		WCT_LED_stack, LED2_wid, group_colors
		;
		;	how many images to examine? Reset LED1 to reflect it
		;
		n_image = (*g_ptr).group_numbers[g_ind]
		;
		;	if > 130, use LED scaling
		;
		nLED = 130
		LED_scale = float(n_image) / float(nLED)
		if(LED_scale le 1.0) then LED_scale = 1.0
		;
		WIDGET_CONTROL, LED1_wid, SET_UVALUE=(nLED < n_image)
		image_colors = strarr(nLED < n_image)
		image_colors[*] = "red"
		S3_LED_sensitize, Event, 1, 1
		;
		;	for every image
		;
		old_ra = 0.0
		old_dec = 0.0
		;
		;	fetch camera flag and 2mass file flag
		;
		if((*w_ptr).S3_2mass_mode eq 0) then two_mass=0 else two_mass=1
		;
		case (*w_ptr).S3_camera_mode of
			0: begin
				platescale_guess = 0.579
				tolerance = 0.0015
				f17 = 0
			   end
			1: begin
				platescale_guess = 0.18
				tolerance = 0.0015
				f17 = 1
			   end
		endcase
		;
		for i_image = 0, n_image-1 do begin
;		for i_image = 0, 10 do begin
			;
			;	set LED1 to blue
			;
			this_LED = ROUND(float(i_image) / LED_scale) & this_LED = 0 > ((nLED-1) < this_LED)
			image_colors[this_LED] = "blue"
			WCT_LED_stack, LED1_wid, image_colors
			;
			;	get the image name and check that its use flag is valid
			;
			image_path = (*g_ptr).group_images[g_ind, i_image]
			;
			;	also get its LLQ status
			;
			this_LLQ = (*g_ptr).group_llq_flags[g_ind,i_image]
			;
			print_text, Event, "Starting Astrometry processing of image "+FILE_BASENAME(image_path)
			;
			if(this_LLQ ne 0) then print_text, Event, "   respecting bad LLQ"
			;
			if((*g_ptr).group_image_flags[g_ind,i_image] eq 0) then begin
				str = FILE_BASENAME(image_path)+": use flag =0, no astrometry tried"
				print_text, Event, "Warning - image "+str
				SAll_add_to_history, Event, h_wid, 3, str
				SAll_populate_history, Event, h_wid, 3
				image_colors[this_LED] = "white"
				WCT_LED_stack, LED1_wid, image_colors
				goto, S3_image_end
			endif
			;
			; first, check to see if the image was ski-jump fixed by looking in the ski jump
			;	directory
			;
			image_name = FILE_BASENAME(image_path)
			ski_dir = (*s_ptr).analysis_dir + 'S2_Ski_Jump_Fixes' + PATH_SEP()
			ski_path = ski_dir + image_name
			test = FILE_INFO(ski_path)
			if(test.exists) then begin
				image_astrometry = ski_path
				print_text, Event, "...found image in Ski Jump directory - using it"
			endif else begin
				image_astrometry = image_path
			endelse
			;
			image_data = readfits(image_astrometry, old_header, /SILENT)
			;
			;  if the background flatten flag is set, flatten this image, using minimum curve surface fitting
			;  
			if( (*w_ptr).S3_flatten_bg_mode eq 1) then begin
			   S3_flatten_image_background, Event, image_data, flat_image_data, MAX_PTS=24, RADIUS=40, /MCS
			   image_data = flat_image_data
			   print_text, Event, "Flattening Background using MCS for Image "+FILE_BASENAME(image_astrometry)
			   FXADDPAR, old_header, 'HISTORY', 'PPOL Step 3 - flattened image background using MCS/P24/R40'
			endif
			;
			; ** 20120604 - DPC - added bad pixel cleaning via modeling
      ;
      image_data = model_bad_pixels(image_data)
      ;
      FXADDPAR, revised_header, 'HISTORY','  Image Cleaned of Bad Pixels via Model_Bad_Pixels.pro'
			;
			;	test to see if we can recycle the 2mass data set after the first fetch
			;
			text_crpix1 = sxpar(old_header, 'CRPIX1')
			if(text_crpix1 eq 1) then begin
			  	this_ra = 15.d*ten(double(strsplit(sxpar(old_header,'TELRA'),':',/EXTRACT))); telescope RA in degrees
				this_dec = ten(double(strsplit(sxpar(old_header,'TELDEC'),':',/EXTRACT))); telescope Dec in degrees
			endif else begin
			  	this_ra = sxpar(old_header, "CRVAL1")
			  	this_dec = sxpar(old_header, "CRVAL2")
			endelse
			;
			;	compare this to old to see if need to call 2mass or not
			;
			toler = 0.0333											; 1 arcmin
			if(abs(old_ra - this_ra) lt toler and abs(old_dec - this_dec) lt toler) then begin
			  		fetch_2mass = 1		; ie, don't fetch
			endif else begin
			  		fetch_2mass = 0
			  		old_ra = this_ra
			  		old_dec = this_dec
			endelse
			;
			itry = 0L
			;
			refetch:
			;
			revised_header = ppol_astrometry(Event,image_astrometry,image_data, old_header, result_flag, platescale_guess,$
							tolerance, find_scale, F17=f17, ARRAY_2MASS=array_2mass, TWO_MASS=fetch_2mass, $
							FILE_2MASS=save_file, DEFOCUS=defocus_flag, LLQ = this_LLQ)
			;
			;print, "return flag = ",result_flag
			;
			;	if a good result, write new image to new directory
			;
			output_dir = (*s_ptr).analysis_dir + 'S3_Astrometry' + PATH_SEP()
			output_path = output_dir + image_name
			if(result_flag ge 1 and result_flag le 12) then begin
				;
				FXADDPAR, revised_header, 'HISTORY', 'PPOL-Step 3-version '+(*s_ptr).version_short_string
				if(defocus_flag eq 1) then begin
					FXADDPAR, revised_header, 'DEFOCUS',1,'This image was deliberately defocussed'
				endif
				;
				;	add other flags
				;
				if( (*w_ptr).S3_camera_mode eq 1) then this_cam = 'F17' else this_cam = 'F5'
				;
				FXADDPAR, revised_header, 'HISTORY', '     Defocus Flag, Camera Mode = '+STRING(defocus_flag, $
								this_cam, format='(I1,",",a)' )
				;
        ;	add LLQ flag
        ;
        FXADDPAR, revised_header, 'PP3_LLQF', this_LLQ, '  Lower Left Quad Bad Flag (1=bad; 0=good)'
        ;
				writefits, output_path, image_data, revised_header
				;
				print_text, Event, "Wrote image "+output_path
				;
				this_LED = ROUND(float(i_image) / LED_scale) & this_LED = 0 > ((nLED-1) < this_LED)
				image_colors[this_LED] = "green"
				WCT_LED_stack, LED1_wid, image_colors
				;
				;	if 2MASS flag set, create sub directory, file, and write
				;
				if(two_mass) then begin
					;
					dir_2mass = output_dir + '2MASS_Files'
					test = FILE_INFO(dir_2mass)
					if(NOT(test.exists)) then begin
						FILE_MKDIR, dir_2mass
						print_text, Event, "Creating 2MASS files directory = "+dir_2mass
					endif
					;
					parts = STRSPLIT(FILE_BASENAME(output_path),'.fits',/extract,/regex)
					;
					path_2mass = dir_2mass + PATH_SEP() + '2MASS_' + parts[0] + '.dat'
					openw, lu_2mass, path_2mass, /GET_LUN
					;
					nlines = N_ELEMENTS(save_file)
					ntags = N_TAGS(save_file[0])
					for il = 0, nlines-1 do begin
						line = ''
						for itag = 0, ntags-1 do begin
							line = line + ' ' + STRTRIM(STRING(save_file[il].(itag)),2)
						endfor
						printf, lu_2mass, line
					endfor
					FREE_LUN, lu_2mass
					;
					WIDGET_CONTROL, (*w_ptr).S3_bad_fit_textbox_wid, SET_VALUE=STRING((*w_ptr).S3_bad_fit_count,format='(I4.1)')
					;
				endif
				;
				;	if either 1 or 2 star astrometry, tell history buffer
				;
				if(result_flag eq 11 or result_flag eq 12) then begin
					case result_flag of
						11:	ast_str = ' 1-Star Astrometry '
						12: ast_str = ' 2-Star Astrometry '
					endcase
					full_str = 'Warning - file '+FILE_BASENAME(image_astrometry)+' had only '+ast_str
					SAll_add_to_history, Event, h_wid, 3, full_str
					SAll_populate_history, Event, h_wid, 3
				endif
				;
			endif else begin
				;
				;	no good astrometry - why not?
				;
				if(result_flag eq 99) then begin
					if(itry le 5) then begin
						print_text, Event, "Warning - failed to fetch 2MASS data - trying again in 10 sec"
						image_colors[this_LED] = "violet"
						WCT_LED_stack, LED1_wid, image_colors
						wait, 10
						goto, refetch
					endif
					SAll_add_to_history, Event, h_wid, 3, FILE_BASENAME(image_astrometry)+": 2MASS fetch failed"
					SAll_populate_history, Event, h_wid, 3
				endif else begin
						;print_text, Event, "ERROR - Image quality inadequate - setting USE flag to zero"
						;
						;	and write to history buffer
						;
						str = FILE_BASENAME(image_path)+": poor quality image - use flag ->0"
						SAll_add_to_history, Event, h_wid, 3, str
						SAll_populate_history, Event, h_wid, 3
				endelse
				;
				print_text, Event, "ERROR - Bad Astrometry. Did not write image "+output_path+" to disk"
				(*g_ptr).group_image_flags[g_ind,i_image] = 0		; turn this guy off
				;
				this_LED = ROUND(float(i_image) / LED_scale) & this_LED = 0 > ((nLED-1) < this_LED)
				image_colors[this_LED] = "red"
				WCT_LED_stack, LED1_wid, image_colors
				;
				;	increment bad fit counter and display
				;
				(*w_ptr).S3_bad_fit_count++
				WIDGET_CONTROL, (*w_ptr).S3_bad_fit_textbox_wid, SET_VALUE=STRING((*w_ptr).S3_bad_fit_count,format='(I4.1)')
			endelse
			;
			S3_image_end:
		endfor
		;
		;	all images in this group are done
		;
		print_text, Event, "Finished Astrometry for Group = "+group_name+' with '+STRING( (*w_ptr).S3_bad_fit_count,format='(I)')+$
		                     ' bad images '
		;
		;	change color of group LED to green
		;
		group_colors[ig] = "green"
		WCT_LED_stack, LED2_wid, group_colors
		;
		;	clear queue tag for this group and rewrite listbox
		;
		(*g_ptr).S3_group_queue[g_ind] = 0
		;
		;	and update step status value for this group
		;
		(*g_ptr).group_step[g_ind] = 31
		S3_populate_group_list, Event
		if( g_ind gt 10) then begin
			WIDGET_CONTROL, (*w_ptr).S3_group_listbox_wid, SET_LIST_TOP= ( g_ind - 6)
		endif
		MSP_PPOL_Save_Group_File, Event
	endfor
	;
	;	all groups done.
	;
	print_text, Event, "All Queued Groups have been Astrometrized"
	;
	S3_SA_end:
end