;
;	Step 4 Event Handler Routines - Initial Photometry
;
pro S4_Init, Event
	;
	;	initialize S4 stuff when tabbing onto Step 4
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	g_ptr = PTR_struct.PTR_group
	;
	;	populate the group table
	;
	n_groups = (*g_ptr).n_groups
	if(n_groups gt 0) then begin
		ind = WHERE((*g_ptr).S4_group_queue[0:n_groups-1] ge 1,count)
		if(count gt 0) then (*g_ptr).S4_group_queue[ind] = -2
	endif
	S4_populate_group_list, Event
	;
	;	clear LEDs to red
	;
	S4_LED_sensitize, Event, 1, 0
	S4_LED_sensitize, Event, 2, 0
	;
	;	and set processing textboxes to reflect saved status
	;
	S4_Show_Textboxes, Event
	;
	;	and clear bad pixel, flattening checkboxes to saved values
	;
	WIDGET_CONTROL, (*w_ptr).S4_fix_bad_pixels_button_wid, SET_BUTTON=(*w_ptr).S4_fix_bad_pixels_flag
	WIDGET_CONTROL, (*w_ptr).S4_flatten_image_button_wid, SET_BUTTON=(*w_ptr).S4_flatten_image_flag
	;
	;	populate the history window
	;
	h_wid = (*w_ptr).S4_History_textbox_wid
	SAll_populate_history, Event, h_wid, 4
end
;
;------------------------------------------------------------------
pro S4_Toggle_Fix_Bad_Pixels, Event
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	;
	(*w_ptr).S4_fix_bad_pixels_flag = 1 - (*w_ptr).S4_fix_bad_pixels_flag
	;
	WIDGET_CONTROL, (*w_ptr).S4_fix_bad_pixels_button_wid, SET_BUTTON=(*w_ptr).S4_fix_bad_pixels_flag
	;
	case (*w_ptr).S4_fix_bad_pixels_flag of
		0: print_text, Event, "Changing mode to NOT fix bad pixels"
		1: print_text, Event, "Changing mode to FIX bad pixels"
	endcase
end
;
;---------------------------------------------------------------------
pro S4_Toggle_Flatten_Image, Event
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	;
	(*w_ptr).S4_flatten_image_flag = 1 - (*w_ptr).S4_flatten_image_flag
	;
	WIDGET_CONTROL, (*w_ptr).S4_flatten_image_button_wid, SET_BUTTON=(*w_ptr).S4_flatten_image_flag
	;
	case (*w_ptr).S4_flatten_image_flag of
		0: print_text, Event, "Changing mode to NOT flatten images (no background model subtraction)"
		1: print_text, Event, "Changing mode to FLATTEN images (background modeling and subtraction)"
	endcase
end
;
;------------------------------------------------------------------
pro S4_Show_Textboxes, Event
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	;
	for ibox = 0, 2 do begin
		case ibox of
			0: wid = (*w_ptr).S4_ng_textbox_wid
			1: wid = (*w_ptr).S4_nsig_textbox_wid
			2: wid = (*w_ptr).S4_saturation_textbox_wid
		endcase
		;
		WIDGET_CONTROL, wid, SET_VALUE=STRING(format='(F8.1)',(*w_ptr).S4_textbox_value[ibox])
	endfor
	;
end
;
;---------------------------------------------------
pro S4_Change_Textbox, Event, ibox
	;
	;	handle insert character events for options textboxes
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	;
	case ibox of
		0: wid = (*w_ptr).S4_ng_textbox_wid
		1: wid = (*w_ptr).S4_nsig_textbox_wid
		2: wid = (*w_ptr).S4_saturation_textbox_wid
	endcase
	;
	;	first, fetch current pointer into text box buffer
	;
	nch = (*w_ptr).S4_textbox_nch[ibox]
	;
	new_name = 0
	;
	if(Event.type eq 0) then begin
		;
		;	if character not CR, put it into buffer
		;
		ch = Event.ch
		if(ch ne 10B) then begin
			WIDGET_CONTROL, wid, GET_VALUE=existing_text
			(*w_ptr).S4_textbox_nch[ibox] = STRLEN(existing_text)
			for il = 0, STRLEN(existing_text[0])-1 do (*w_ptr).S4_textbox_buffer[ibox,il] = STRMID(existing_text, il, 1)
			WIDGET_CONTROL, wid, SET_TEXT_SELECT=[Event.offset,0]
			;
		endif else begin
			;
			;	was a character return - load the buffer into a single string
			;
			name = ''
			for ich = 0, (*w_ptr).S4_textbox_nch[ibox]-1 do begin
				name = name + (*w_ptr).S4_textbox_buffer[ibox,ich]
			endfor
			name = STRTRIM(name,2)
			;
			;	new name available - put it into current buffer and issue a relist command
			;
			new_name = 1
			;
		endelse
		;
		;print_text, Event, "Insert Character Event - Character = "+STRING(format='(a)',Event.ch)
	endif else begin
		if(Event.type eq 1) then begin
			WIDGET_CONTROL, wid, GET_VALUE=existing_text
			name = STRTRIM(Event.str,2)
			length = STRLEN(name)
			if(length gt 1) then begin
				this_name = strarr(length)
				for il=0, length-1 do begin
					this_name[il] = STRMID(name,il,1)
				endfor
			endif else begin
				if(length eq 0) then goto, S4_CT_end
				;
				this_name = strarr(1)
				this_name[0] = name
			endelse
			new_name = 1
			;
		endif else begin
			if(Event.type eq 2) then begin
				WIDGET_CONTROL, wid, GET_VALUE=existing_text
				;
				;	if length = 0 just skip
				;
				if(Event.length eq 0) then goto, S4_CT_end
				;

			endif else begin
				print_text, Event, "Event not recognized"
			endelse
		endelse
		(*w_ptr).S4_textbox_nch[ibox] = STRLEN(existing_text)
		for il = 0, STRLEN(existing_text[0])-1 do (*w_ptr).S4_textbox_buffer[ibox,il] = STRMID(existing_text, il, 1)
		WIDGET_CONTROL, wid, SET_TEXT_SELECT=[Event.offset,0]
	endelse
	;
	if(new_name eq 1) then begin
		;
		;	replace old value with new, but only accept good floating point value
		;
		CATCH, ibad
		;
		if(ibad ne 0) then begin
			print_text, Event, "Bad input, try again"
			;
			;	clear buffer to old values
			;
			(*w_ptr).S4_textbox_nch[ibox] = 0
			(*w_ptr).S4_textbox_buffer[ibox,*] = ''
			;
			;	and reshow
			;
			S4_Show_Textboxes, Event
			;
			CATCH, /CANCEL
			;
			goto, S4_CT_end
		endif
		;
		reads, name, format='(F)', value
		;
		print_text, Event, "Replacing value with "+STRING(format='(F)',value)
		;
		(*w_ptr).S4_textbox_value[ibox] = value
		;
		S4_Show_Textboxes, Event
		;
		;	clear buffer
		;
		(*w_ptr).S4_textbox_nch[ibox] = 0
		(*w_ptr).S4_textbox_buffer[ibox,*] = ''
		;
		CATCH, /CANCEL
	endif
	;
	S4_CT_end:
end
;
;-----------------------------------------------
pro S4_Select_Groups, Event
	;
	;	one or more items in the group listbox was/were selected
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	s_ptr = PTR_struct.PTR_summary
	w_ptr = PTR_struct.PTR_wid_id
	g_ptr = PTR_struct.PTR_group
	;
	wid = (*w_ptr).S4_group_listbox_wid
	index = WIDGET_INFO( wid, /LIST_SELECT)
	n = N_ELEMENTS(index)
	if(n eq 1 and index[0] eq -1) then goto, S4_SG_end
	;
	;	test for groups processed through step 3
	;
	grp_step = (*g_ptr).group_step[index]
	ind4 = WHERE(grp_step ge 3 and grp_step ne 31 and grp_step ne 32, count4)
	ind3 = WHERE(grp_step eq 32, count3)
	;
	count = count3 + count4
	if(count gt 0) then begin
		if(count3 gt 0) then begin
			new_index = index[ind3]
		endif
		if(count4 gt 0) then begin
			if(N_ELEMENTS(new_index) eq 0) then begin
				new_index = index[ind4]
			endif else begin
				new_index = [new_index, index[ind4]]
			endelse
		endif
		index = new_index
		;
		if(count ne n) then print_text, Event, "Warning - some groups not processed through Step 3, skipping those."
		n = count
	endif else begin
		print_text, Event, "Warning - no groups processed through Step 3. Go back"
		goto, S4_SG_end
	endelse
	;
	(*g_ptr).n_groups_selected = n
	;
	print_text, Event, "Number of selected groups = "+STRING(format='(I)',n)
	;
	;	set the index - it gets reset after a button action is handled
	;
	(*g_ptr).selected_groups[0:n-1] = index[*]
	if(n eq 1) then begin
		group = (*g_ptr).group_names[index[0]]
		print_text, Event, "Selected Group "+group+" from Group List"
	endif else begin
		print_text, Event, "Selected "+STRING(format='(I)',n)+" groups from list"
	endelse
	;
	S4_SG_end:
end
;
;-----------------------------------------------
pro S4_Add_To_Queue, Event
	;
	;	selected groups are added to the S4 processing queue
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	s_ptr = PTR_struct.PTR_summary
	g_ptr = PTR_struct.PTR_group
	;
	;	how many selected?
	;
	n = (*g_ptr).n_groups_selected
	;
	if(n gt 0) then begin
		for ig = 0, n-1 do begin
			j = (*g_ptr).selected_groups[ig]
			;
			;	set queue pointer to 2 (enqueued)
			;
			(*g_ptr).S4_group_queue[j] = 2
		endfor
	endif
	;
	;	reload listbox
	;
	S4_populate_group_list, Event
	(*g_ptr).n_groups_selected = 0
	(*g_ptr).selected_groups[*] = 0
end
;-----------------------------------------------
pro S4_Remove_From_Queue, Event
	;
	;	selected groups are removed (skipped) from the S4 processing queue
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	s_ptr = PTR_struct.PTR_summary
	g_ptr = PTR_struct.PTR_group
	;
	;	how many selected?
	;
	n = (*g_ptr).n_groups_selected
	;
	if(n gt 0) then begin
		for ig = 0, n-1 do begin
			j = (*g_ptr).selected_groups[ig]
			;
			(*g_ptr).S4_group_queue[j] = -1
		endfor
	endif
	;
	;	reload listbox
	;
	S4_populate_group_list, Event
	(*g_ptr).n_groups_selected = 0
	(*g_ptr).selected_groups[*] = 0
end
;
;-----------------------------------------------
pro S4_LED_sensitize, Event, S4_LED_number, sense
	;
	;	initializes LED stacks
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	;
	if(S4_LED_number eq 1) then begin
		wid = (*w_ptr).S4_LED1_images_wid
	endif else begin
		wid = (*w_ptr).S4_LED2_groups_wid
	endelse
	;
	WIDGET_CONTROL, wid, SENSITIVE=sense
	;
	;	set all colors to red upon sensitization
	;
	WIDGET_CONTROL, wid, GET_UVALUE=nLED
	;
	n_ele = N_ELEMENTS(nLED)
	if(n_ele eq 0) then nLED = 25
	if(nLED eq 0) then nLED = 25
	WIDGET_CONTROL, wid, SET_UVALUE=nLED
	colors = strarr(nLED)
	if(sense eq 1) then colors[*] = "red" else colors[*] = "gray"
	WCT_LED_stack, wid, colors
end
;
;-----------------------------------------------
pro S4_populate_group_list, Event
	;
	;	lists current groups and step status by name on the listbox
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	s_ptr = PTR_struct.PTR_summary
	g_ptr = PTR_struct.PTR_group
	;
	n_groups = (*g_ptr).n_groups
	if(n_groups gt 0) then begin
		;
		;	load group name,step and meta status, and queue status
		;
		text_vec = strarr(n_groups)
		for ig = 0, n_groups-1 do begin
			if((*g_ptr).group_step[ig] lt 3) then (*g_ptr).S4_group_queue[ig] = -2
			i_queue = (*g_ptr).S4_group_queue[ig]
			;
			case i_queue of
				-2:	queue = '?'
				-1: queue = 'Skip'
				 0: queue = 'Done'
				 1: queue = 'Active'
				 2: queue = 'Queued'
				else: queue = '...'
			endcase
			;
			grp_step = (*g_ptr).group_step[ig]
			if((grp_step ge 30 and grp_step le 40) or $
			   (grp_step ge 60 and grp_step le 70) or $
			   (grp_step ge 110 and grp_step le 119)) then begin
			   step_0 = FLOOR(grp_step / 10)
			   step_1 = grp_step - 10 * step_0
			   text_vec[ig] = STRTRIM(STRING(format='(A," Step=",I2.1,".",I1.1,", Meta=",I2.1,", Action= ",A)',$
					(*g_ptr).group_names[ig],step_0,step_1,(*g_ptr).group_meta[ig],queue),1)

			endif else begin
				text_vec[ig] = STRTRIM(STRING(format='(A," Step=",I2.1,", Meta=",I2.1,", Action= ",A)',$
					(*g_ptr).group_names[ig],(*g_ptr).group_step[ig],(*g_ptr).group_meta[ig],queue),1)
			endelse
			;
		endfor
	endif else begin
		text_vec = ['','']
	endelse
	;
	WIDGET_CONTROL, (*w_ptr).S4_group_listbox_wid, SET_VALUE=text_vec
	;
end
;
;------------------------------------------------------
pro S4_Start_PSF_Fitting, Event
	;
	;	Begin processing of all enqueued requests for ski jump processing
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	s_ptr = PTR_struct.PTR_summary
	g_ptr = PTR_struct.PTR_group
	d_ptr = PTR_Struct.PTR_Data
	h_ptr = PTR_Struct.PTR_History
	;
	h_wid = (*w_ptr).S4_History_textbox_wid
	;
	;	save residuals status
	;
;	isave_f = (*w_ptr).S4_save_residuals_mode
	isave_f = 0									; no residuals saved
	;
	;	how many active items on the queue?
	;
	ind_queue = WHERE((*g_ptr).S4_group_queue[0:(*g_ptr).n_groups-1] eq 2,count)
	if(count le 0) then begin
		; if none, quit
		print_text, Event, "WARNING - no items in queue to process."
		goto, S4_SPF_end
	endif
	;
	print_text, Event, "Found "+STRING(format='(I)',count)+" groups in the queue. Processing starting"
	;
	;	Clear LED2 to red with the number of groups to process
	;
	LED2_wid = (*w_ptr).S4_LED2_groups_wid
	LED1_wid = (*w_ptr).S4_LED1_images_wid
	;
	WIDGET_CONTROL, LED2_wid, SET_UVALUE=count
	group_colors = strarr(count)
	group_colors[*] = "red"
	S2_LED_sensitize, Event, 2, 1
	;
	;	clear the text boxes
	;
	WIDGET_CONTROL, (*w_ptr).S4_group_textbox_wid, SET_VALUE='0'
	WIDGET_CONTROL, (*w_ptr).S4_image_textbox_wid, SET_VALUE='0'
	WIDGET_CONTROL, (*w_ptr).S4_nstars_textbox_wid, SET_VALUE='0'
	;
	;	extract the processing value options
	;
	ng = ROUND((*w_ptr).S4_textbox_value[0])		; NG
	sigma = (*w_ptr).S4_textbox_value[1]			; sigma
	sat_limit = (*w_ptr).S4_textbox_value[2]		; saturation limit
	;
	;	for each of the enqueued groups:
	;
	this_ng = ng
	this_sigma = sigma
	;
	for ig = 0, count-1 do begin
		;
		;		reload the starting processing options
		;
		if(this_ng ne ng) then begin
			print_text, Event, "Resetting Default NG value back to "+STRING(float(ng),format='(F)')
			this_ng = ng
			WIDGET_CONTROL, (*w_ptr).S4_ng_textbox_wid, SET_VALUE=STRING(float(ng),format='(F)')
		endif
		if(this_sigma ne sigma) then begin
			print_text, Event, "Resetting Default Nsigma value back to "+STRING(float(sigma),format='(F)')
			this_sigma = sigma
			WIDGET_CONTROL, (*w_ptr).S4_nsig_textbox_wid, SET_VALUE=STRING(float(sigma),format='(F)')
		endif
		;
		g_ind = ind_queue[ig]
		WIDGET_CONTROL, (*w_ptr).S4_group_textbox_wid, SET_VALUE=STRING(format='(I)',g_ind)
		group_name = (*g_ptr).group_names[g_ind]
		;
		;	test -> has this group completed the astrometry step? (#3)
		;
		gstep = (*g_ptr).group_step[g_ind]
		if(gstep lt 3) then begin
			print_text, Event, "ERROR - group "+group_name+" has not completed astrometry step [3] - return"
			goto, S4_SPF_group_end
		endif
		;
		print_text, Event, "Starting PSF Fitting for Group = "+group_name
		;
		;	change the list box queue list to active for this group
		;
		(*g_ptr).S4_group_queue[g_ind] = 1
		S4_populate_group_list, Event
		;
		;	and reposition the list to show this active group
		;
		if( g_ind gt 10) then begin
			WIDGET_CONTROL, (*w_ptr).S4_group_listbox_wid, SET_LIST_TOP= ( g_ind - 6)
		endif
		;
		;	change the LED2 bar to blue
		;
		group_colors[ig] = "blue"
		WCT_LED_stack, LED2_wid, group_colors
		;
		;	how many images to examine? Reset LED1 to reflect it
		;
		n_image = (*g_ptr).group_numbers[g_ind]
		;
		;	if > 130, use LED scaling
		;
		nLED = 130
		LED_scale = float(n_image) / float(nLED)
		if(LED_scale le 1.0) then LED_scale = 1.0
		;
		WIDGET_CONTROL, LED1_wid, SET_UVALUE=(nLED < n_image)
		image_colors = strarr(nLED < n_image)
		image_colors[*] = "red"
		S2_LED_sensitize, Event, 1, 1
		;
		;	for every image
		;
		for i_image = 0, n_image-1 do begin
			;
			;	set LED1 to blue
			;
			this_LED = ROUND(float(i_image) / LED_scale) & this_LED = 0 > ((nLED-1) < this_LED)
			image_colors[this_LED] = "blue"
			WCT_LED_stack, LED1_wid, image_colors
			;
			image_path = (*g_ptr).group_images[g_ind,i_image]
			image_flag = (*g_ptr).group_image_flags[g_ind,i_image]
			llq_flag = (*g_ptr).group_llq_flags[g_ind,i_image]
			;
			;	if the image flag is set to zero, don't process
			;
			if(image_flag eq 0) then begin
				image_colors[this_LED] = "yellow"
				WCT_LED_stack, LED1_wid, image_colors
				WIDGET_CONTROL, (*w_ptr).S4_nstars_textbox_wid, SET_VALUE='skipped'
				str = "USE Flag 0 for "+FILE_BASENAME(image_path)+" no photometry done"
				print_text, Event,"Warning -  "+str
				SAll_add_to_history, Event, h_wid, 4, str
				SAll_populate_history, Event, h_wid, 4
				goto, S4_SPF_image_skip_end
			endif
			;
			;	get image from astrometry directory
			;
			image_name = FILE_BASENAME(image_path)
			image_dir = (*s_ptr).analysis_dir + 'S3_Astrometry' + PATH_SEP()
			;
			image_path = image_dir + image_name
			;
			print_text, Event, "PSF Fitting started for image "+FILE_BASENAME(image_path)
			WIDGET_CONTROL, (*w_ptr).S4_image_textbox_wid, SET_VALUE=FILE_BASENAME(image_path)
			;
			image_data = readfits(image_path, image_header, /SILENT)
			;
			;	if llq flag set, delete the LLQ
			;
			if(llq_flag eq 1) then begin
				image_data[0:511,0:512] = -1.0e6
				print_text, Event,'Masking Bad LLQ to deleted data values'
			endif
			;
			;	look in header for defocus keyword - if there, only do aperture photometry,
			;			no PSF fitting or neighbor removal
			;
			val = SXPAR(image_header, 'DEFOCUS', COUNT=d_count)
			if(d_count eq 1) then defocus_mode = 1 else defocus_mode = 0
			;
			;	also, if was a coadded image, scale limits by number of coadds
			;
			coadds = SXPAR(image_header, 'NUMCOADD')
			if(coadds eq 0) then coadds = 1
			;
			this_sat_limit = sat_limit * coadds
			;
			if(defocus_mode eq 0) then begin
				S4_PSF_fit, Event, image_data, image_header, image_path, status, residual_image, $
					NG=this_ng, SAVE_RESIDUALS=isave_f, $
					SIGMA=this_sigma, SAT_LIMIT=this_sat_limit, LLQ_FLAG = llq_flag
			endif else begin
				S4_defocus_aphot, Event, image_data, image_header, image_path, status, residual_image, $
					NG=this_ng, SAVE_RESIDUALS=isave_f, $
					SIGMA=this_sigma, SAT_LIMIT=this_sat_limit
			endelse
			;
			;	status return code -
			;		0 = OK
			;		-1 = bad
			;		-99 = really bad - no psf stars - turn off image use flag, tell history and continue
			;
			;	change LED to violet
			;
			image_colors[this_LED] = "violet"
			WCT_LED_stack, LED1_wid, image_colors
			;
			;	if good PSF fits, then save results to data file on disk
			;
			if(status eq 0) then begin
				;
				;	create output filename
				;
				output_dir = (*s_ptr).analysis_dir + 'S4_First_PSF_Fits' + PATH_SEP()
				output_name = FILE_BASENAME(image_path,'.fits') + '_phot.dat'
				output_path = output_dir + output_name
				;
				;	prepare for up to 10 tries to write the file
				;
				itry=10
				read_OK = 0
				WHILE (read_OK eq 0 and itry gt 0) do begin
					ON_IOERROR, retry_read
					openw, lu_out, output_path, /GET_LUN
					read_OK=1
					;
					retry_read: if(read_OK eq 0) then begin
						itry--
						wait, 3
					endif
				ENDWHILE
				;
				;
				;	write modified header, then phot info
				;
				n_header = N_ELEMENTS(image_header)
				;
				FXADDPAR, image_header, 'HISTORY','PPOL-Step 4-version '+(*s_ptr).version_short_string
				;
				for ih = 0, n_header-1 do begin
					printf, lu_out, format='(";;",a)', image_header[ih]
				endfor
				;
				part1 = ";; Star    X       Y      RA         Dec     Sky   Group   "
				part2 =  "Mag[0]  Mag[1]  Mag[2]  Mag[3]  Mag[4]  Mag[5]  Mag[6]  Mag[7]  Mag[8]  Mag[9] Mag[10] Mag[11]   "
				part3 = "sMag[0] sMag[1] sMag[2] sMag[3] sMag[4] sMag[5] sMag[6] sMag[7] sMag[8] sMag[9]sMag[10]sMag[11]"
				all_parts = part1 + part2 + part3
				printf, lu_out, format='(a)', all_parts
				;
				;	read from data structure, writing results to output file
				;
				nstars = (*d_ptr).nstars
				for is = 0, nstars-1 do begin
				    if((*d_ptr).dec[is] gt 0.0) then dsign = '+' else dsign = '-'
					d = abs((*d_ptr).dec[is])
					dten = FLOOR(d/10)
					d = d - 10 * dten
					printf, lu_out, format='(I6,1x,F7.2,1x,f7.2,1x,f10.6,1x,a1,i1.1,f8.6,1x,f8.2,1x,i4,1x,12(f7.4,1x),12(f7.4,1x))',$
									is,(*d_ptr).x[is],(*d_ptr).y[is],(*d_ptr).ra[is],dsign,dten,d,$
									(*d_ptr).sky[is],(*d_ptr).psf_group[is],(*d_ptr).mag[is,*],(*d_ptr).errmag[is,*]
				endfor
				;
				;	close data file
				;
				FREE_LUN, lu_out
				;
				;	update textboxes
				;
				WIDGET_CONTROL, (*w_ptr).S4_nstars_textbox_wid, SET_VALUE=STRING(format='(I)',nstars)
				;
				;	set LED to green
				;
				image_colors[this_LED] = "green"
				WCT_LED_stack, LED1_wid, image_colors
			endif else begin
				if(status eq -99) then begin
					str = FILE_BASENAME(image_path) + ': no PSF stars. USE flag -> 0'
				endif else begin
					str = FILE_BASENAME(image_path)+" failed. No phot file; USE flag -> 0"
				endelse
				print_text, Event,"Warning -  "+str
				SAll_add_to_history, Event, h_wid, 4, str
				;
				;	and display it
				;
				SAll_populate_history, Event, h_wid, 4
				;
				;	turn image flag to "off"
				;
				(*g_ptr).group_image_flags[g_ind,i_image] = 0
				;
				image_colors[this_LED] = "orange"
				WCT_LED_stack, LED1_wid, image_colors
				WIDGET_CONTROL, (*w_ptr).S4_nstars_textbox_wid, SET_VALUE='failed'
			endelse
			;
			;	this image done, go to next
			;
			S4_SPF_image_skip_end:
			;
			;	clear up memory management
			;
			wait, 0.5
			HEAP_GC,/PTR
			wait, 0.5
		endfor
		;
		;	all images in this group are done
		;
		print_text, Event, "Finished PSF Fitting Group = "+group_name
		;
		;	change color of group LED to green
		;
		group_colors[ig] = "green"
		WCT_LED_stack, LED2_wid, group_colors
		;
		;	clear queue tag for this group and rewrite listbox
		;
		(*g_ptr).S4_group_queue[g_ind] = 0
		;
		;	and update step status value for this group
		;
		(*g_ptr).group_step[g_ind] = 4
		S4_populate_group_list, Event
		if( g_ind gt 10) then begin
			WIDGET_CONTROL, (*w_ptr).S4_group_listbox_wid, SET_LIST_TOP= ( g_ind - 6)
		endif
		MSP_PPOL_Save_Group_File, Event
		;
		S4_SPF_group_end:
	endfor
	;
	;	all groups done.
	;
	print_text, Event, "All Queued Groups have been PSF Fit"
	;
	S4_SPF_end:
	SAll_populate_history, Event, h_wid, 4
end