;
;	Step 9 Event Handler Routines - polarization calibration, correction
;
pro S9_Init, Event
	;
	;	initialize S9 stuff when tabbing onto Step 9
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	g_ptr = PTR_struct.PTR_group
	;
	h_wid = (*w_ptr).S9_history_textbox_wid
	;
	;	populate the group, metagroup table based on current
	;		meta-group mode
	;
	mg_mode = (*w_ptr).S9_meta_group_mode
	;
	case mg_mode of
		0:	begin
				S9_Select_group_mode, Event, 0
				n_groups = (*g_ptr).n_groups
				if(n_groups gt 0) then begin
					ind = WHERE((*g_ptr).S9_group_queue[0:n_groups-1] ge 1,count)
					if(count gt 0) then (*g_ptr).S9_group_queue[ind] = -2
				endif
				S9_populate_group_list, Event
			end
		1:	begin
				S9_Select_group_mode, Event, 1
				n_meta_groups = (*g_ptr).n_meta_groups
				if(n_meta_groups gt 0) then begin
					ind = WHERE((*g_ptr).S9_meta_group_queue[1:n_meta_groups] ge 1,count)
					if(count gt 0) then (*g_ptr).S9_meta_group_queue[ind+1] = -2
				endif
				S9_populate_meta_group_list, Event
			end
		else: print_text, Event, "ERROR - illegal meta-group mode number for Step 9"
	endcase
	;
	;	clear LEDs to red
	;
	S9_LED_sensitize, Event, 1, 0
	;
	;	initialize options buttons and buffers
	;
	(*w_ptr).S9_textbox_pathname = 'Default H-Pol P_inst Files'
	S9_Show_Textbox, Event					; PInst table pathname
	;
	(*w_ptr).S9_Pinst_mode = 1 - (*w_ptr).S9_Pinst_mode
	S9_Toggle_pinst, Event
	;
	(*w_ptr).S9_POLCAT_mode = 1 - (*w_ptr).S9_POLCAT_mode
	S9_Toggle_polcat, Event
	;
	(*w_ptr).S9_P_Scale_mode = 1 - (*w_ptr).S9_P_Scale_mode
	S9_P_Scale_override, Event
	;
	S9_Show_P_Scale_Textbox,Event
	;
	SAll_populate_history, Event, h_wid, 9
end
;-----------------------------------------------
;
pro S9_Test, Event
	;
	;	tests the 2MASS code
	;
	phot_struct_path = dialog_pickfile(Title="select phot_struct file")
	;
	S8_Merge_2MASS, Event, phot_struct_path, cal_struct, status
	;
	print_text, Event, "On return from S8_Merge_2MASS - status = "+STRING(status,format='(I)')
end
;
;------------------------------------------------
;
pro S9_Pinst_override, Event
	;
	;	throws up file chooser to select new P_Inst files,
	;	updates text box
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	;
	new_dir = dialog_pickfile(Title="Select Directory with New P_Inst Files",/directory)
	if(new_dir ne '') then begin
		(*w_ptr).S9_textbox_pathname = FILE_DIRNAME(new_dir) + PATH_SEP() + FILE_BASENAME(new_dir)
		S9_Show_Textbox, Event
	endif
	;
	;	echo to screen
	;
	S9_Read_Pinst_Files, Event, 9		; the 9 says use S9 pathname
end
;-----------------------------------------------
pro S9_Show_Textbox, Event
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	;
	wid = (*w_ptr).S9_Pinst_textbox_wid
	str = STRTRIM((*w_ptr).S9_textbox_pathname,2)
	;
	WIDGET_CONTROL, wid, SET_VALUE=str
end
;-----------------------------------------------
pro S9_Show_P_Scale_Textbox, Event
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	;
	wid = (*w_ptr).S9_P_scale_textbox_wid
	str = STRTRIM(STRING( (*w_ptr).S9_textbox_value,format='(i)') ,2)
	;
	WIDGET_CONTROL, wid, SET_VALUE=str
end
;----------------------------------------------
pro S9_Select_group_mode, Event, meta_group_mode
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	;
	(*w_ptr).S9_meta_group_mode = meta_group_mode
	;
	WIDGET_CONTROL, (*w_ptr).S9_show_groups_wid, SET_BUTTON=(1-meta_group_mode)
	WIDGET_CONTROL, (*w_ptr).S9_show_meta_groups_wid, SET_BUTTON=meta_group_mode
	;
	case meta_group_mode of
		0: begin
			print_text, Event, "Selecting Single Group Mode"
			S9_populate_group_list, Event
		   end
		1: begin
			print_text, Event, "Selecting Meta-Group Mode"
			S9_populate_meta_group_list, Event
		   end
	endcase
end
;
;-----------------------------------------------
pro S9_Toggle_pinst, Event
	;
	;	toggle this mode
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	;
	(*w_ptr).S9_pinst_mode = 1 - (*w_ptr).S9_pinst_mode
	;
	WIDGET_CONTROL, (*w_ptr).S9_use_pinst_correction_wid, SET_BUTTON=(*w_ptr).S9_pinst_mode
	;
	case (*w_ptr).S9_pinst_mode of
		0: print_text, Event, "Changing mode to NOT use P_Inst corrections"
		1: print_text, Event, "Changing mode to USE P_Inst corrections"
	endcase
	;
	;	update echo to screen
	;
	S9_Read_Pinst_Files, Event, 9
	;
end
;----------------------------------------------
pro S9_P_Scale_override, Event
	;
	;	toggle this mode
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	;
	(*w_ptr).S9_P_Scale_mode = 1 - (*w_ptr).S9_P_Scale_mode
	;
	WIDGET_CONTROL, (*w_ptr).S9_P_scale_override_button_wid, SET_BUTTON=(*w_ptr).S9_P_Scale_mode
	;
	case (*w_ptr).S9_P_Scale_mode of
		0: begin
			print_text, Event, "Changing mode to use default Pol scaling in region files"
			(*w_ptr).S9_textbox_value = -1
			;
			;	write this value to the textbox
			;
			S9_Show_P_Scale_Textbox,Event
			;
			;	and then desensitize it
			;
			WIDGET_CONTROL, (*w_ptr).S9_P_scale_textbox_wid, SENSITIVE=0
		   end
		1: begin
			print_text, Event, "Changing mode to override Pol scaling in region files"
			;
			;	make the textbox sensitive to allow mods
			;
			WIDGET_CONTROL, (*w_ptr).S9_P_scale_textbox_wid, SENSITIVE=1
			S9_Show_P_Scale_Textbox,Event
		   end
	endcase
	;
end
;
;-----------------------------------------------
pro S9_Toggle_polcat, Event
	;
	;	toggle this mode
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	;
	(*w_ptr).S9_polcat_mode = 1 - (*w_ptr).S9_polcat_mode
	;
	WIDGET_CONTROL, (*w_ptr).S9_deposit_into_polcat_wid, SET_BUTTON=(*w_ptr).S9_polcat_mode
	;
	case (*w_ptr).S9_polcat_mode of
		0: print_text, Event, "Changing mode to NOT save results in POLCAT form"
		1: print_text, Event, "Changing mode to SAVE results in POLCAT form"
	endcase
	;
end
;
;-----------------------------------------------
pro S9_Comments_text, Event
	;
	;	handle insert character events for FITS comments
	;		- text is always valid, CRs ignored --
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	g_ptr = PTR_struct.PTR_group
	;
	;	first, fetch current pointer into text box buffer
	;
	nch = (*w_ptr).S9_comments_nch
	wid = (*w_ptr).S9_comments_textbox_wid
	;
	;	if it is zero, check whether something is already there
	;
	if(nch eq 0) then begin
		mch = WIDGET_INFO(wid,/TEXT_NUMBER)
		if(mch ne 0) then begin
			WIDGET_CONTROL, wid, GET_VALUE=existing_text
			mch = STRLEN(existing_text[0])
			;
			;	pack into character buffer
			;
			(*w_ptr).S9_comments_buffer[*] = ''
			;
			for im = 0, mch-1 do begin
				(*w_ptr).S9_comments_buffer[im] = STRMID(existing_text, im, 1)
			endfor
			(*w_ptr).S9_comments_nch = mch
			nch = mch
		endif
	endif
	;
	if(Event.type eq 0) then begin
		;
		;	if character not CR, put it into buffer
		;
		ch = Event.ch
		if(ch ne 10B) then begin
			WIDGET_CONTROL, wid, GET_VALUE=existing_text
			(*w_ptr).S9_comments_nch = STRLEN(existing_text)
			for il = 0, STRLEN(existing_text[0])-1 do (*w_ptr).S9_comments_buffer[il] = STRMID(existing_text, il, 1)
			WIDGET_CONTROL, wid, SET_TEXT_SELECT=[Event.offset,0]
		endif else begin
			;
			;	was a character return, ignore
		endelse
		;
	endif else begin
		if(Event.type eq 1) then begin
			WIDGET_CONTROL, wid, GET_VALUE=existing_text
			;
			name = STRTRIM(Event.str,2)
			length = STRLEN(name)
			if(length gt 1) then begin
				this_name = strarr(length)
				for il=0, length-1 do begin
					this_name[il] = STRMID(name,il,1)
				endfor
			endif else begin
				if(length eq 0) then goto, S9CT_end
				;
				this_name = strarr(1)
				this_name[0] = name
			endelse
		endif else begin
			if(Event.type eq 2) then begin
				WIDGET_CONTROL, wid, GET_VALUE=existing_text
				if(Event.length eq 0) then goto, S9CT_end
			endif else begin
				print_text, Event, "Event not recognized"
			endelse
			new_name = 0
		endelse
		(*w_ptr).S9_comments_nch = STRLEN(existing_text)
		for il = 0, STRLEN(existing_text[0])-1 do (*w_ptr).S9_comments_buffer[il] = STRMID(existing_text, il, 1)
		WIDGET_CONTROL, wid, SET_TEXT_SELECT=[Event.offset,0]
	endelse
	;
	S9CT_end:
end
;
;----------------------------------------------------------------
pro S9_Modify_textbox, Event
	;
	;	handle insert character events for P scale override value
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	g_ptr = PTR_struct.PTR_group
	;
	;	first, fetch current pointer into text box buffer
	;
	wid = (*w_ptr).S9_P_scale_textbox_wid
	;
	nch = (*w_ptr).S9_textbox_nch
	;
	;	if it is zero, check whether something is already there
	;
	;** debug - dump some stuff
	;
	;print, "wid, nch = ",wid, nch
	;help, Event, /struct
	;
	existing_text = ''
	if(nch eq 0) then begin
		mch = WIDGET_INFO(wid,/TEXT_NUMBER)
		if(mch gt 1) then begin
			WIDGET_CONTROL, wid, GET_VALUE=existing_text
			mch = STRLEN(existing_text[0])
			;
			;	pack into character buffer
			;
			(*w_ptr).S9_textbox_buffer[*] = ''
			;
			for im = 0, mch-1 do begin
				(*w_ptr).S9_textbox_buffer[im] = STRMID(existing_text, im, 1)
			endfor
			(*w_ptr).S9_textbox_nch = mch
			nch = mch
		endif
	endif
	;
	;print, "after looking at existing text on arrival, "
	;print, "existing text = ",existing_text
	;print, "nch = ",nch
	;
	if(Event.type eq 0) then begin
		;
		;	if character not CR, put it into buffer
		;
		ch = Event.ch
		if(ch ne 10B) then begin
;			WIDGET_CONTROL, wid, GET_VALUE=existing_text
;			print, "existing text = ",existing_text
;			(*w_ptr).S9_textbox_nch = STRLEN(existing_text)
;			print, "string length = ",(*w_ptr).S9_textbox_nch
;			for il = 0, STRLEN(existing_text[0])-1 do (*w_ptr).S9_textbox_buffer[il] = STRMID(existing_text, il, 1)
;			WIDGET_CONTROL, wid, SET_TEXT_SELECT=[Event.offset,0]
			(*w_ptr).S9_textbox_nch = 1
			(*w_ptr).S9_textbox_buffer[0] = STRING(Event.ch)
			reads, STRING(Event.ch),format='(i)',value
			;print, "decoded value = ",value
			(*w_ptr).S9_textbox_value = value
		endif else begin
			;
			;	was a character return, interpret and load value buffer
			;
			;		- first, check for non-zero buffer length
			;
			if((*w_ptr).S9_textbox_nch gt 0) then begin
				;
				;		- load the buffer into a single string
				;
				name = ''
				for ich = 0, (*w_ptr).S9_textbox_nch-1 do begin
					name = name + (*w_ptr).S9_textbox_buffer[ich]
				endfor
				;
				name = STRTRIM(name,2)
				;
				;	new name available - read it into value buffer
				;
				reads, name, format='(i)',value
				(*w_ptr).S9_textbox_value = value
				;
				print_text, Event, "Setting P Scale Value to "+name+" percent"
				S9_Show_P_Scale_Textbox, Event
				;
			endif else begin
				print_text, Event, "Warning - no characters in textbox"
			endelse
			;
		endelse
		;
		;print_text, Event, "Insert Character Event - Character = "+STRING(format='(a)',Event.ch)
	endif else begin
		if(Event.type eq 1) then begin
			WIDGET_CONTROL, wid, GET_VALUE=existing_text
			;
			name = STRTRIM(Event.str,2)
			length = STRLEN(name)
			if(length gt 1) then begin
				this_name = strarr(length)
				for il=0, length-1 do begin
					this_name[il] = STRMID(name,il,1)
				endfor
			endif else begin
				if(length eq 0) then goto, S9PT_end
				;
				this_name = strarr(1)
				this_name[0] = name
			endelse
			existing_text = this_name
		endif else begin
			if(Event.type eq 2) then begin
				;
				;	delete type event, go to offset, delete length
				;
				WIDGET_CONTROL, wid, GET_VALUE=existing_text
				if(Event.length eq 0) then goto, S9PT_end
				existing_text = ''
			endif else begin
				print_text, Event, "Event not recognized"
			endelse
		endelse
		(*w_ptr).S9_textbox_nch = STRLEN(existing_text)
		if( (*w_ptr).S9_textbox_nch gt 0) then begin
			for il = 0, STRLEN(existing_text[0])-1 do (*w_ptr).S9_textbox_buffer[il] = STRMID(existing_text, il, 1)
			WIDGET_CONTROL, wid, SET_TEXT_SELECT=[Event.offset,0]
		endif else begin
			(*w_ptr).S9_textbox_buffer[*] = ''
			(*w_ptr).S9_textbox_buffer[0] = ''
			(*w_ptr).S9_textbox_nch = 0
			(*w_ptr).S9_textbox_value = -1
		endelse
	endelse
	;
	S9PT_end:
	S9_Show_P_Scale_Textbox, Event
end
;-----------------------------------------------
pro S9_Select_Groups, Event
	;
	;	select groups or metagroups
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	s_ptr = PTR_struct.PTR_summary
	w_ptr = PTR_struct.PTR_wid_id
	g_ptr = PTR_struct.PTR_group
	;
	;	get group/metagroup mode
	;
	tab_no = (*w_ptr).S9_meta_group_mode
	;
	wid = (*w_ptr).S9_group_listbox_wid
	;
	index = WIDGET_INFO( wid, /LIST_SELECT)
    n = N_ELEMENTS(index)
    skip = 0
	if(n eq 1 and index[0] eq -1) then skip = 1
	if(skip eq 0) then begin
		;
		case tab_no of
			0:	begin
					ind = WHERE((*g_ptr).group_step[index] ge 8 and $
			    				(*g_ptr).group_step[index] ne 31 and $
			    				(*g_ptr).group_step[index] ne 32, count)
					if(count gt 0) then begin
						index = index[ind]
						if(n ne count) then print_text, Event, "Warning - some groups not processed through Step 8 - skipping those"
						n = count
					endif else begin
						print_text, Event, "Warning - no groups processed through Step 8. Go back"
						goto, S9_SG_end
					endelse
					;
					(*g_ptr).n_groups_selected = n
					(*g_ptr).selected_groups[0:n-1] = index[*]
					if(n eq 1) then begin
						group = (*g_ptr).group_names[index[0]]
						print_text, Event, "Selected Group "+group+" from Group List"
					endif else begin
						print_text, Event, "Selected "+STRING(format='(I)',n)+" groups from list"
					endelse
				end
			1:	begin
					ind = WHERE((*g_ptr).meta_group_step[index+1] ge 8, count)
					if(count gt 0) then begin
						index = index[ind]
						if(n ne count) then print_text, Event, "Warning - some meta-groups not processed through Step 8 - skipping those"
						n = count
					endif else begin
						print_text, Event, "Warning - no meta-groups processed through Step 8. Go back"
						goto, S9_SG_end
					endelse
					(*g_ptr).n_meta_groups_selected = n
					(*g_ptr).selected_meta_groups[1:n] = index[*]+1
					if(n eq 1) then begin
						group = (*g_ptr).meta_group_names[index[0]+1]
						print_text, Event, "Selected Group "+group+" from Group List"
					endif else begin
						print_text, Event, "Selected "+STRING(format='(I)',n)+" groups from list"
					endelse
				end
		endcase
	   ;
	endif else print_text, Event, "Warning - no groups selected"
	S9_SG_end:
end
;
;-----------------------------------------------
pro S9_Add_groups, Event
	;
	;	selected groups are added to the group or metagroup queues
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	s_ptr = PTR_struct.PTR_summary
	g_ptr = PTR_struct.PTR_group
	;
	;	get mode
	;
	tab_no = (*w_ptr).S9_meta_group_mode
	;
	case tab_no of
		0:	begin
				;
				;	how many selected?
				;
				n = (*g_ptr).n_groups_selected
				;
				if(n gt 0) then begin
					for ig = 0, n-1 do begin
						j = (*g_ptr).selected_groups[ig]
						;
						;	set queue pointer to 2 (enqueued)
						;
						(*g_ptr).S9_group_queue[j] = 2
					endfor
				endif
				;
				;	reload listbox
				;
				S9_populate_group_list, Event
				;
				;	clear selections
				;
				(*g_ptr).n_groups_selected = 0
				if(n gt 0) then (*g_ptr).selected_groups[0:n-1] = 0
			end
		1: 	begin
				;
				n = (*g_ptr).n_meta_groups_selected
				;
				if(n gt 0) then begin
					(*g_ptr).S9_meta_group_queue[0]= -1
					for ig = 1, n do begin
						j = (*g_ptr).selected_meta_groups[ig]
						;
						;	set queue pointer to 2 (enqueued)
						;
						(*g_ptr).S9_meta_group_queue[j] = 2
					endfor
					print_text, Event, "Added "+STRING(n,format='(I)')+" meta-groups to queue"
				endif
				;
				;	reload listbox
				;
				S9_populate_meta_group_list, Event
				;
				(*g_ptr).n_meta_groups_selected = 0
				if(n gt 0) then (*g_ptr).selected_meta_groups[1:n] = 0
			end
	endcase

end
;-----------------------------------------------
pro S9_Remove_groups, Event
	;
	;	selected groups are removed (skipped) indicated queue
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	s_ptr = PTR_struct.PTR_summary
	g_ptr = PTR_struct.PTR_group
	;
	;	get mode
	;
	tab_no = (*w_ptr).S9_meta_group_mode
	;
	case tab_no of
		0:	begin
				;
				;	how many selected?
				;
				n = (*g_ptr).n_groups_selected
				;
				if(n gt 0) then begin
					for ig = 0, n-1 do begin
						j = (*g_ptr).selected_groups[ig]
						;
						(*g_ptr).S9_group_queue[j] = -1
					endfor
				endif
				;
				;	reload listbox
				;
				S9_populate_group_list, Event
				;
				;	clear selections
				;
				(*g_ptr).n_groups_selected = 0
				if(n gt 0) then (*g_ptr).selected_groups[0:n-1] = 0
			end
		1:	begin
				;
				;	how many selected?
				;
				n = (*g_ptr).n_meta_groups_selected
				;
				if(n gt 0) then begin
					for ig = 1, n do begin
						j = (*g_ptr).selected_meta_groups[ig]
						;
						(*g_ptr).S9_meta_group_queue[j] = -1
					endfor
				endif
				;
				S9_populate_meta_group_list, Event
				;
				(*g_ptr).n_meta_groups_selected = 0
				if(n gt 0) then (*g_ptr).selected_meta_groups[1:n] = 0
			end
	endcase
end
;
;-----------------------------------------------
pro S9_LED_sensitize, Event, S9_LED_number, sense
	;
	;	initializes LED stacks
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	;
	if(S9_LED_number eq 1) then begin
		wid = (*w_ptr).S9_LED1_groups_wid
	endif
	;
	WIDGET_CONTROL, wid, SENSITIVE=sense
	;
	;	set all colors to red upon sensitization
	;
	WIDGET_CONTROL, wid, GET_UVALUE=nLED
	;
	n_ele = N_ELEMENTS(nLED)
	if(n_ele eq 0) then nLED = 25
	if(nLED eq 0) then nLED = 25
	WIDGET_CONTROL, wid, SET_UVALUE=nLED
	colors = strarr(nLED)
	if(sense eq 1) then colors[*] = "red" else colors[*] = "gray"
	WCT_LED_stack, wid, colors
end
;
;-----------------------------------------------
pro S9_populate_group_list, Event
	;
	;	lists current groups and step status by name on the listbox
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	s_ptr = PTR_struct.PTR_summary
	g_ptr = PTR_struct.PTR_group
	;
	n_groups = (*g_ptr).n_groups
	if(n_groups gt 0) then begin
		;
		;	load group name,step and meta status, and queue status
		;
		text_vec = strarr(n_groups)
		for ig = 0, n_groups-1 do begin
			if((*g_ptr).group_step[ig] lt 8) then (*g_ptr).S9_group_queue[ig] = -2
			i_queue = (*g_ptr).S9_group_queue[ig]
			;
			case i_queue of
				-3: queue = "to Meta"
				-2:	queue = '?'
				-1: queue = 'Skip'
				 0: queue = 'Done'
				 1: queue = 'Active'
				 2: queue = 'Queued'
				else: queue = '...'
			endcase
			;
			grp_step = (*g_ptr).group_step[ig]
			if((grp_step ge 30 and grp_step le 40) or $
			   (grp_step ge 60 and grp_step le 70) or $
			   (grp_step ge 110 and grp_step le 119)) then begin
			   step_0 = FLOOR(grp_step / 10)
			   step_1 = grp_step - 10 * step_0
			   text_vec[ig] = STRTRIM(STRING(format='(A," Step=",I2.1,".",I1.1,", Meta=",I2.1,", Action= ",A)',$
					(*g_ptr).group_names[ig],step_0,step_1,(*g_ptr).group_meta[ig],queue),1)

			endif else begin
				text_vec[ig] = STRTRIM(STRING(format='(A," Step=",I2.1,", Meta=",I2.1,", Action= ",A)',$
					(*g_ptr).group_names[ig],(*g_ptr).group_step[ig],(*g_ptr).group_meta[ig],queue),1)
			endelse
			;
			;text_vec[ig] = STRTRIM(STRING(format='(A," Step=",I2.1,", Meta=",I2.1,", Action= ",A)',$
			;		(*g_ptr).group_names[ig],(*g_ptr).group_step[ig],(*g_ptr).group_meta[ig],queue),1)
		endfor
	endif else begin
		text_vec = ['','']
	endelse
	;
	WIDGET_CONTROL, (*w_ptr).S9_group_listbox_wid, SET_VALUE=text_vec
	;
	(*g_ptr).n_groups_selected = 0
	(*g_ptr).selected_groups[*] = 0
end
;
;------------------------------------------------------------------
pro S9_populate_meta_group_list, Event
	;
	;	loads the meta-name listbox on the indicated tab_number
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	g_ptr = PTR_struct.PTR_group
	;
	wid = (*w_ptr).S9_group_listbox_wid
	;
	n_meta = (*g_ptr).n_meta_groups
	if(n_meta gt 0) then begin
		;
		text_vec = strarr(n_meta)
		for img = 1, n_meta do begin
			;
			(*g_ptr).S9_meta_group_queue[0] = -1
			if((*g_ptr).meta_group_step[img] lt 8) then (*g_ptr).S9_meta_group_queue[img] = -2
			case (*g_ptr).S9_meta_group_queue[img] of
				-2:	queue = '?'
				-1: queue = 'Skip'
				 0: queue = 'Done'
				 1: queue = 'Active'
				 2: queue = 'Queued'
				else: queue = '...'
			endcase
			;
			text_vec[img-1] = STRTRIM(STRING(format='(A," Step=",I2.1," Action= ",A)',$
				(*g_ptr).meta_group_names[img],(*g_ptr).meta_group_step[img],queue),1)
		endfor
	endif else begin
		text_vec = ['','']
	endelse
	;
	WIDGET_CONTROL, wid, SET_VALUE=text_vec
	;
	(*g_ptr).n_meta_groups_selected = 0
	(*g_ptr).selected_meta_groups[*] = 0
end
;
;------------------------------------------------------
pro S9_Read_Pinst_Files, Event, step_no
	;
	;	read in the indicated Pinst images and values, echoing some to the screen
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	s_ptr = PTR_struct.PTR_summary
	g_ptr = PTR_struct.PTR_group
	;
	case step_no of
		9: begin
				h_wid = (*w_ptr).S9_history_textbox_wid
				inst_corr_flag = (*w_ptr).S9_Pinst_mode
				polcat_flag = (*w_ptr).S9_polcat_mode
				the_dir = (*w_ptr).S9_textbox_pathname
			end
		11: begin
				h_wid = (*w_ptr).S11_history_textbox_wid
				inst_corr_flag = (*w_ptr).S11_Pinst_mode
				the_dir = (*w_ptr).S11_Pinst_pathname
	  		end
	endcase
	;
	;	if P_inst correction selected, read in the desired
	;	P_inst files (onto heap)
	;
	if(inst_corr_flag eq 1) then begin
		;
		;CD, CURRENT=this_dir
		;
		;this_dir = FILE_DIRNAME(file_which('MSP_PPOL_Step_9_Event_Handler_Routines.pro'))
		;
		this_dir = (*s_ptr).PPOL_dir
		;
		p_inst_dir = this_dir + PATH_SEP() + 'H-Pol_Pinst_Default'
		;
		if(the_dir eq 'Default H-Pol P_inst Files') then begin
			the_p_inst_dir = p_inst_dir
		endif else begin
			the_p_inst_dir = the_dir ; this_dir + PATH_SEP() + 'Pinst_Default'
		endelse
		;
		;	check that all 5 key files exist
		;
		p_inst_files = the_p_inst_dir + PATH_SEP() + ['u_inst.fits','q_inst.fits',$
						'u_sig_inst.fits','q_sig_inst.fits','P_inst_values.dat']
		ibad = 0
		for itest = 0, N_ELEMENTS(p_inst_files)-1 do begin
			this_test = FILE_INFO(p_inst_files[itest])
			if(~this_test.exists) then ibad++
		endfor
		if(ibad gt 0) then begin
			print_text, Event, 'ERROR - one or more P_inst files missing from designated directory. Stopping'
			stop
		endif
		;
		;	ok, all here, create heap variables for P_inst files
		;
		(*w_ptr).S9_u_inst_ptr = PTR_NEW(readfits(p_inst_files[0],header,/silent))
		(*w_ptr).S9_q_inst_ptr = PTR_NEW(readfits(p_inst_files[1],header,/silent))
		(*w_ptr).S9_u_sig_inst_ptr = PTR_NEW(readfits(p_inst_files[2],header,/silent))
		(*w_ptr).S9_q_sig_inst_ptr = PTR_NEW(readfits(p_inst_files[3],header,/silent))
		;
		openr, lu_values, p_inst_files[4], /GET_LUN
		value_ptr = PTR_NEW(strarr(100))
		line = ''
		ipl = 0
		WHILE ~EOF(lu_values) do begin
			readf, lu_values, line
			(*value_ptr)[ipl] = line
			ipl++
		ENDWHILE
		FREE_LUN, lu_values
		;
		pol_efficiency 		= SXPAR((*value_ptr), 'P_EFFIC')
		s_pol_effic 		= SXPAR((*value_ptr), 'S_P_EFF')
		;
		;	load the offset angles, uncertainties, and time block end points
		;
		n_pa_blocks 		= SXPAR((*value_ptr), 'N_P_BLKS')
		;
		pol_off_angle 		= fltarr(n_pa_blocks)
		s_pol_off_angle		= fltarr(n_pa_blocks)
		end_yr_block		= fltarr(n_pa_blocks)
		;
		pa_count = 0
		s_pa_count = 0
		yr_count = 0
		;
		for iblk = 0, n_pa_blocks-1 do begin
			;
			;	names of variables
			;
			if(iblk+1 le 9) then begin
				pa_name   = 'P_OFF_' + STRING(iblk+1,format='(i1.1)')
				s_pa_name = 'SP_OF_' + STRING(iblk+1,format='(i1.1)')
				yr_name   = 'ENDYR_' + STRING(iblk+1,format='(i1.1)')
			endif else begin
				pa_name   = 'P_OFF_' + STRING(iblk+1,format='(i2.2)')
				s_pa_name = 'SP_OF_' + STRING(iblk+1,format='(i2.2)')
				yr_name   = 'ENDYR_' + STRING(iblk+1,format='(i2.2)')
			endelse
			;
			pol_off_angle[iblk] 	= SXPAR((*value_ptr), pa_name,COUNT=this_pa_count)
			s_pol_off_angle[iblk]   = SXPAR((*value_ptr), s_pa_name,COUNT=this_s_pa_count)
			end_yr_block[iblk]      = SXPAR((*value_ptr), yr_name,COUNT=this_yr_count)
			;
			pa_count 	+= this_pa_count
			s_pa_count 	+= this_s_pa_count
			yr_count 	+= this_yr_count
			;
		endfor
		;
		check_count = pa_count + s_pa_count + yr_count
		if(check_count ne 3*n_pa_blocks) then begin
			print_text, event, 'Failed to read Pol Offset Angles correctly. Stopping'
			stop
		endif
		;
;		pol_off_year			= [2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014]
		date_stamp			= SXPAR((*value_ptr), 'DATESTMP')
		this_pol_off_ind = 0L
		;
		PTR_FREE, value_ptr
		;
	endif else begin
		;
		;	no instrumental correction to be applied
		;
		date_stamp = '1999.99'
		pol_efficiency = 1.0
		pol_off_angle = fltarr(1)
		s_pol_off_angle = fltarr(1)
		end_yr_block = fltarr(1)
		n_pa_blocks = 1
		s_pol_effic = 1.0
		this_pol_off_ind = 0L
	endelse
	;
	(*w_ptr).S9_P_inst_values_ptr = PTR_NEW({	date_stamp: date_stamp,$			; scalar
												pol_efficiency: pol_efficiency,$	; scalar
												s_pol_effic: s_pol_effic,$			; scalar
											    n_pa_blocks:n_pa_blocks, $			; scalar integer
											    pol_off_angle: pol_off_angle,$		; vector
											    s_pol_off_angle: s_pol_off_angle, $	; vector
											    end_yr_block: end_yr_block,$		; vector
											    this_pol_off_ind: this_pol_off_ind}); scalar - pointer to pa block element
	;
	;	echo to screen
	;
	case step_no of
		9:	begin
				wid1 = (*w_ptr).S9_date_stamp_textbox_wid
				wid2 = (*w_ptr).S9_HWP_offset_angle_wid
				wid3 = (*w_ptr).S9_polarization_efficiency_wid
				;
				WIDGET_CONTROL, wid1, SET_VALUE=STRING(LONG(date_stamp), format='(I8)')
				WIDGET_CONTROL, wid2, SET_VALUE=STRING(n_pa_blocks, $
								format='(2x,i4.1)')
				WIDGET_CONTROL, wid3, SET_VALUE=$
					STRING(100.0*pol_efficiency, 100.0*s_pol_effic, format='(2x,f7.2," +/- ",f4.2, " %")')
				;
			end
		11:	begin
			end
	endcase
	;

end
;
;------------------------------------------------------
pro S9_Start_analysis, Event
	;
	;	Begin processing of all enqueued requests for polarimetry processing
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	s_ptr = PTR_struct.PTR_summary
	g_ptr = PTR_struct.PTR_group
	;
	h_wid = (*w_ptr).S9_history_textbox_wid
	;
	inst_corr_flag = (*w_ptr).S9_Pinst_mode
	polcat_flag = (*w_ptr).S9_polcat_mode
	;
	meta_flag = (*w_ptr).S9_meta_group_mode
	;
	LED1_wid = (*w_ptr).S9_LED1_groups_wid
	;
	;	if correction with instrumental polarization flag set, reload the images, values
	;
	if(inst_corr_flag eq 1) then S9_Read_Pinst_Files, Event, 9
	;
	case meta_flag of
		0:	begin
				this_queue = (*g_ptr).S9_group_queue[0:(*g_ptr).n_groups-1]
				these_names = (*g_ptr).group_names[0:(*g_ptr).n_groups-1]
			end
		1:  begin
				this_queue = (*g_ptr).S9_meta_group_queue[1:(*g_ptr).n_meta_groups]
				these_names = (*g_ptr).meta_group_names[1:(*g_ptr).n_meta_groups]
			end
	endcase
	;
	;	how many active items on the queue?
	;
	ind_queue = WHERE(this_queue[*] eq 2,count)
	if(count le 0) then begin
		; if none, quit
		print_text, Event, "WARNING - no items in queue to process."
		goto, S9_SA_end
	endif
	;
	print_text, Event, "Found "+STRING(format='(I)',count)+" groups in the queue. Processing starting"
	;
	;	Clear LED1 to red with the number of groups to process
	;
	WIDGET_CONTROL, LED1_wid, SET_UVALUE=count
	group_colors = strarr(count)
	group_colors[*] = "red"
	S9_LED_sensitize, Event, 1, 1
	;
	;	for each of the enqueued groups:
	;
	for ig = 0, count-1 do begin
		;
		g_ind = ind_queue[ig]
		group_name = these_names[g_ind]
		;
		case meta_flag of
			0: begin
				print_text, Event, "Starting Polarimetry Analysis for Group = "+group_name
				(*g_ptr).S9_group_queue[g_ind] = 1
				S9_populate_group_list, Event
				if( g_ind gt 10) then begin
					WIDGET_CONTROL, (*w_ptr).S9_group_listbox_wid, SET_LIST_TOP= ( g_ind - 6)
				endif
			   end
			1: begin
				print_text, Event, "Starting Polarimetry Analysis for Meta-Group = "+group_name
				(*g_ptr).S9_meta_group_queue[g_ind+1] = 1
				S9_populate_meta_group_list, Event
				if( g_ind gt 10) then begin
					WIDGET_CONTROL, (*w_ptr).S9_group_listbox_wid, SET_LIST_TOP= ( g_ind - 6)
				endif
			   end
		endcase
		;
		;	change the LED1 bar to blue
		;
		group_colors[ig] = "blue"
		WCT_LED_stack, LED1_wid, group_colors
		;
		S9_polarimetric_analysis, Event, g_ind, group_name, LED1_wid, group_colors, ig, $
									meta_flag, inst_corr_flag, $
									pol_struct, n_pol_stars, status
		;status = -1
		;
		case meta_flag of
			0:	begin
					print_text, Event, "Finished Polarimetric Analysis for Group = "+group_name+" finding "+$
									STRING(n_pol_stars,format='(I)')+" with polarimetry"
				end
			1:	begin
					print_text, Event, "Finished Polarimetric Analysis for Meta-Group = "+group_name+" finding "+$
									STRING(n_pol_stars,format='(I)')+" with polarimetry"
				end
		endcase
		;
		wait, 1
		;
		;	evaluate completion status: 0=OK, -1=bad
		;
		case status of
			0: begin
				;
				;	clear queue tag for this group and rewrite listbox
				;
				case meta_flag of
					0:	begin
							(*g_ptr).S9_group_queue[g_ind] = 0
							;
							;	and update step status value for this group
							;
							(*g_ptr).group_step[g_ind] = 9
							S9_populate_group_list, Event
							if( g_ind gt 10) then begin
								WIDGET_CONTROL, (*w_ptr).S9_group_listbox_wid, SET_LIST_TOP= ( g_ind - 6)
							endif
						end
					1:  begin
							(*g_ptr).S9_meta_group_queue[g_ind+1] = 0
							;
							;	and update step status value for this group
							;
							(*g_ptr).meta_group_step[g_ind+1] = 9
							S9_populate_meta_group_list, Event
							if( g_ind gt 10) then begin
								WIDGET_CONTROL, (*w_ptr).S9_group_listbox_wid, SET_LIST_TOP= ( g_ind - 6)
							endif
						end
				endcase
				;
				;	if the POLCAT flag is set, write POLCAT file
				;
				if(polcat_flag) then begin
					print_text, Event, "Writing POLCAT entry"
					S9_Write_POLCAT_entry, Event, group_name, pol_struct, n_pol_stars
				endif
				;
				;	change color of group LED to green
				;
				group_colors[ig] = "green"
				WCT_LED_stack, LED1_wid, group_colors
				;
			  end
			  ;
		  -1: begin
		  		;
				;	change color of group LED to yellow
				;
				group_colors[ig] = "yellow"
				WCT_LED_stack, LED1_wid, group_colors
				;
				;	clear queue tag for this group and rewrite listbox
				;
				case meta_flag of
					0:	begin
							(*g_ptr).S9_group_queue[g_ind] = -2
							;
							;	and don't step status value for this group
							;
							(*g_ptr).group_step[g_ind] = 8
							S9_populate_group_list, Event
							if( g_ind gt 10) then begin
								WIDGET_CONTROL, (*w_ptr).S9_group_listbox_wid, SET_LIST_TOP= ( g_ind - 6)
							endif
						end
					1:  begin
							(*g_ptr).S9_meta_group_queue[g_ind+1] = -2
							;
							;	and update step status value for this group
							;
							(*g_ptr).meta_group_step[g_ind+1] = 8
							S9_populate_meta_group_list, Event
							if( g_ind gt 10) then begin
								WIDGET_CONTROL, (*w_ptr).S9_group_listbox_wid, SET_LIST_TOP= ( g_ind - 6)
							endif
						end
				endcase
			 end
		endcase
		MSP_PPOL_Save_Group_File, Event
	endfor
	;
	;	all groups done.
	;
	print_text, Event, "All Queued Groups have had Polarimetry Analysis performed"
	;
	;	if instrumental correction was selected, unload all the heap variables
	;
	if(inst_corr_flag eq 1) then begin
		PTR_FREE, (*w_ptr).S9_u_inst_ptr
		PTR_FREE, (*w_ptr).S9_q_inst_ptr
		PTR_FREE, (*w_ptr).S9_u_sig_inst_ptr
		PTR_FREE, (*w_ptr).S9_q_sig_inst_ptr
		PTR_FREE, (*w_ptr).S9_P_inst_values_ptr
	endif
	;
	S9_SA_end:
end
