pro S2_Ski_Jump_Repair, Event, image_path, repair_method, ski_jump_code
	;
	;	Fits and fixes ski jumps at top and bottom of
	;	Mimir images
	;
	;	DPC	20100109 - added "abnormal" ski jump processing
	;						[delete top, bottom 5 rows]
	;				ski_jump_code = 00, 01, 10, 20, 02, 22
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_Struct
	s_ptr = PTR_Struct.PTR_Summary
	w_ptr = PTR_Struct.PTR_wid_id
	;
	bad_pix_value = -1.0e6
	;
	;	get save model mode
	;
	model_flag = (*w_ptr).S2_save_models_mode
	;
	;	extended target?
	;
	short_fit_flag = (*w_ptr).S2_extended_mode
	;
	;	create output pathname
	;
	output_filename = FILE_BASENAME(image_path)
	output_directory = (*s_ptr).analysis_dir + PATH_SEP() + 'S2_Ski_Jump_Fixes' + PATH_SEP()
	;
	output_pathname = output_directory + output_filename
	parts = STRSPLIT(output_pathname, '.fits',/extract, /regex)
	;
	model_pathname = parts[0] + '_model_bg.fits'
	;
	image = readfits(image_path, header, /SILENT)
	;
	image_size = SIZE(image)
	nc = image_size[1]
	nr = image_size[2]
	;
	resid_image = ski_jump_fit_sub(image, ski_jump_code, short_fit_flag, MODEL=model)
	;
	;	update FITS header
	;
	FXADDPAR, header, 'HISTORY', "PPOL-Step 2-version "+(*s_ptr).version_short_string
	FXADDPAR, header, 'HISTORY', "          "+(*w_ptr).S2_mode_FITS_string
	FXADDPAR, header, 'HISTORY', "          Repaired with Extended Flag = "+STRING(short_fit_flag,format='(i1)')
	;
	;	if method is subtract, save residuals
	;
	if(repair_method eq 'subtract') then begin
		FXADDPAR, header, "HISTORY", "Subtracted Model Ski Jump using MSP_PPOL, Step 2"
		;
		;	determine mean sky from center of original image
		;
		image_center = image[fix(7*nc/16):fix(9*nc/16),fix(7*nr/16):fix(9*nr/16)]
		vec = REFORM(image_center[*,*])
		image_mean = median_filtered_mean(vec)
		;
		ind = WHERE(resid_image ne bad_pix_value, count)
		if(count gt 0) then resid_image[ind] = resid_image[ind] + image_mean[0]	; add sky back in
		;
		writefits, output_pathname, resid_image, header
		;
		;	save model, if mode set
		;
		if(model_flag eq 1) then writefits, model_pathname, model, header
		;
	endif else begin
		;
		;	wants to do division - normalize to center of model
		;
		model_center = model[fix(7*nc/16):fix(9*nc/16),fix(7*nr/16):fix(9*nr/16)]
		vec = REFORM(model_center[*,*])
		model_mean = median_filtered_mean(vec)
		;
		model = model / model_mean[0]
		ratio = model * 0.0
		;
		ind = WHERE(model lt 0.1,count)
		if(count gt 0) then model[ind] = 1.0		; force low values high
		;
		ind_good = WHERE(image ne bad_pix_value, gcount)
		if(gcount gt 0) then begin
			ratio[ind_good] = image[ind_good] / model[ind_good]
		endif
		ind_bad = WHERE(image eq bad_pix_value, bcount)
		if(bcount gt 0) then begin
			ratio[ind_bad] = bad_pix_value
		endif
		;

		FXADDPAR, header, "HISTORY", "Divided by Model Ski Jump using MSP_PPOL, Step 2"
		writefits, output_pathname, resid_image, header
		;
		;	save model
		;
		if(model_flag eq 1) then writefits, model_pathname, model, header
	endelse
	;
	print_text, Event, "Saved Repaired Image "+output_pathname
	;
end