pro S4_defocus_aphot, Event, image_data, image_header, image_pathname, status, residual_image, $
				NG=ng, SAVE_RESIDUALS=isave_f, $
				SIGMA=sigma, SAT_LIMIT=sat_limit
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_Struct
	d_ptr = PTR_Struct.PTR_Data
	s_ptr = PTR_Struct.PTR_summary
	w_ptr = PTR_struct.PTR_wid_id
	;
	;	perform psf fitting on one image
	;
	;	DPC 20100804 	copied from S4_PSF_fit, but cut down to aphot for defocussed stars
	;
	debug = 1
	this_debug = debug
	;
	;---------------------------------------------------------------------------------
	image = image_data					; save copy of original image with bad pixels in place
	start_time = systime(/seconds)
	;
	;	create PSFs in grid (here ng = 1)
	;--------------------------------------------------------------------------------------
	;
	;		constants to pass
	;
	if(N_ELEMENTS(SAT_LIMIT) eq 0) then sat_limit = 6000L	; a reasonable value for Mimir images.
	;
	save_region_files 	= 1									; 0 - none, 1 - all
	;
	flatten_pre_flag 	= (*w_ptr).S4_flatten_image_flag
	fix_bad_pixels_flag = (*w_ptr).S4_fix_bad_pixels_flag
	;
	ap_rad 				= 2.80						; times FWHM
	clean_rad 			= 3.6						; times ap_rad times FWHM
	ap_vec 				= [0.4,0.475,0.55,0.625,$
						   0.7,0.775,0.85,0.925,$
						   1.0,1.05,1.1,1.15]			; times FWHM times ap_rad
	n_ap 				= N_ELEMENTS(ap_vec)
	ref_ap 				= [1.2, 1.7]				; times ap_rad times FWHM
	;
	ronois				= 3.1					; single image readout noise in ADU
	;
	step_dir_text 		= 'S4_First_PSF_Fits'
	ng_textbox_wid 		= (*w_ptr).S4_ng_textbox_wid
	nsig_textbox_wid 	= (*w_ptr).S4_nsig_textbox_wid
	analysis_dir		= (*s_ptr).analysis_dir
	;
	badpix = 		[-1.0e4,2.0*sat_limit]	; 2x sat ought to allow most
	bright_limit = 2.0 * sat_limit		; sat_limit is the limit for PSF building stars, bright_limit for unsat. stars
	;
	if(fix_bad_pixels_flag eq 1) then begin
		print_text, Event, "Fixing bad pixels in image"
		first_time = systime(/seconds)
		this_image = fix_bad_pixels(image)
	endif else this_image = image
	;
	if(flatten_pre_flag eq 1) then begin
		print_text, Event, "Flattening image background"
		find_image = model_background(this_image)	; models, flattens background
	endif
	;
	;	find, then do aperture phot on all stars
	;
	nsig = sigma
	faint_sigma = 4.75
	max_stars = 500
	max_group = 75
	bad_pix_value = -1.0e6
	do_phot = 'Aper'
	;
	;	get astrometry from header
	;
	header = image_header
	extast,header,astr,noparams
	;
	image_size = SIZE(find_image)
	xsize = image_size[1]
	ysize = image_size[2]
	;
	border = 6
	xborder = border
	yborder = 3 * border
	;
	;	get mean sky, mean_FWHM for this image
	;
	;	look to header for coadding counters (MPP_NSCL, MPP_DADD)
	;
	n_coadds = SXPAR(header, "MPP_NSCL", COUNT=n_test)	; this one is preferred
	m_coadds = SXPAR(header, "MPP_DADD", COUNT=m_test)
	;
	if(fix_bad_pixels_flag eq 0) then working_image = fix_bad_pixels(this_image) else working_image = this_image
	;
	if(n_test gt 0) then begin
		working_image = working_image / float(n_coadds)
		ind = WHERE(this_image gt -1.0e5,count)
		if(count gt 0) then this_image[ind] = this_image[ind] / float(n_coadds)
	endif else begin
		if(m_test gt 0) then begin
			working_image = working_image / float(m_coadds)
			ind = WHERE(this_image gt -1.0e5,count)
			if(count gt 0) then this_image[ind] = this_image[ind] / float(m_coadds)
		endif
	endelse
	;
	;    Find sky background statistics
	;
	g_sky, working_image, skymode, sky_sigma, mfm_mean, mfm_stdev, /SILENT
	if (sky_sigma eq -1.0) then sky_sigma = 6.
	if (skymode eq 0) then skymode = mfm_mean
	;
	if(this_debug gt 0) then print_text, Event, "Sky mean, stdev = "+STRING(mfm_mean)+" "+STRING(mfm_stdev)
	;
	; get FWHM the fast way
	;
	fwhm_debug = 0
	fwhm_loop = 0
	;
	fwhm_restart:
	;
	g_defocus_FWHM, this_image, working_image, skymode, sky_sigma, mfm_mean, mfm_stdev, $
				mean_FWHM, sig_FWHM, sat_limit, fwhm_debug, AB=ab, STATUS=status
	;
	if(status lt 0) then begin
		;
		; turn on debug and retry
		;
		fwhm_debug = 1
		fwhm_loop++
		if(fwhm_loop lt 2) then goto, fwhm_restart
		;
		;	image did not pass FWHM tests - flag as bad image and return
		;
		print_text, Event, 'ERROR - FWHM determination failed - Bad Image - turning off USE flag'
		star_list = -1
		goto, S4PSFF_end
	endif else begin
		;
		;	also, test for other error conditions
		;
		;	1. no good stars
		;
		if(sig_FWHM eq 3.0 or ab[0] eq 0.0 or ab[1] eq 0.0) then begin
			print_text, Event, 'ERROR - No good stars - Bad Image - turning off USE flag'
			star_list = -2
			goto, S4PSFF_end
		endif
		;
	endelse
	;
	fwhm = mean_FWHM
	;
	hmin = nsig * mfm_stdev
	;hmin4 = faint_sigma * mfm_stdev
	;
	;	debug=0
	;
	roundlim = [-1.0,0.85]
	sharplim = [0.1,0.85]
	;
	if(mean_FWHM lt 3.0) then begin
		roundlim = [-1.1,0.9]
		sharplim = [0.05,0.95]
	endif
	;
	g_find, this_image, x, y, flux, sharp, roundness, hmin, fwhm, roundlim, sharplim, $
			/SILENT, /NO_BAD_PIX_FIX
	;
	ind = WHERE(x gt xborder and x lt (xsize-xborder) and y gt yborder and $
			y lt (ysize-yborder) and FINITE(flux) and flux lt bright_limit, good_stars)
	;
	if(good_stars gt 0) then begin
		x = x[ind]
		y = y[ind]
		flux = flux[ind]
	endif
	;
	;	brightness rank, then remove near neighbors of bright stars
	;
	ind_bright = REVERSE(SORT(flux))
	;
	no_keep = intarr(good_stars)
	;
	dr_test = 5.0 * fwhm
	;
	for is1 = 0, good_stars-1 do begin
		;
		;	start from brightest stars to faintest
		;
		ist = ind_bright[is1]
		dx = x - x[ist]
		dy = y - y[ist]
		;
		dr = sqrt(dx^2 + dy^2)
		;
		ind_s = SORT(dr)
		;
		ind_del = WHERE(dr[ind_s] lt dr_test,n_kill)	; skip self
		if(n_kill gt 1) then begin
			no_keep[ind_s[ind_del[1:(n_kill-1)]]] = 1
		endif
	endfor
	;
	n_no_keep = TOTAL(no_keep)
	if(n_no_keep ge good_stars) then begin
		print_text, Event, 'ERROR - no isolated stars in image'
		status=-1
		goto, S4PSFF_end
	endif
	;
	if(n_no_keep gt 0) then begin
		ind_keep = WHERE(no_keep eq 0)
		x = x[ind_keep]
		y = y[ind_keep]
		flux = flux[ind_keep]
	endif
	;
	;	aperture phot all stars
	;
	apr = 			ap_rad*fwhm*ap_vec
	skyrad = 		ref_ap * ap_rad * fwhm 		;
	phpadu = 		8.21
	;
	g_aper,this_image,x,y,mags,errap,sky,skyerr,$
			phpadu,apr,skyrad,badpix,bright_limit,$
			/SILENT, /NO_BAD_PIX_FIX
	;
	magtest = REFORM( mags[0,*] )
	errap_test = REFORM( errap[0,*] )
	;
	ind_good = WHERE( FINITE(magtest) and errap_test lt 0.3 and magtest gt -10. and magtest lt 25., ngood)
	if(ngood gt 0) then begin
		mags = mags[*,ind_good]
		x = x[ind_good]
		y = y[ind_good]
		errap = errap[*,ind_good]
		sky = sky[ind_good]
		skyerr = skyerr[ind_good]
		nbright_stars = ngood
	endif
	;
	print_text, Event, 'Aperture photometered '+STRING(nbright_stars,format='(I)')+' stars '
	;
	;	dump values
	;
;	if(debug eq 1) then begin
;		for ic = 0, count-1 do begin
;			print_text, Event, "x,y,mag,sky = "+STRING(xp[ic],yp[ic],magp[ic],skyp[ic])
;		endfor
;	endif
	;
	FXADDPAR, header, 'HISTORY', "PPOL-Step 4-version "+(*s_ptr).version_short_string
	SXADDPAR, header, 'PSF_SIG', faint_sigma, "Detection threshold in sigma of sky"
	SXADDPAR, header, 'PSF_SAT', sat_limit, "Saturation Limit for allowed stars"
;	SXADDPAR, header, 'PSF_FWHM', fwhm,"FWHM in pixels"
;	SXADDPAR, header, 'PSF_FRAD', fit_rad, "PSF fitting radius, in pixels"
;	SXADDPAR, header, 'PSF_RSCL', rad_scale, "PSF model size, in FWHM units"
	;
	;	add aperture phot info
	;
	SXADDPAR, header, 'PP4_FWHM', fwhm, "FWHM used in aperture phot, in pixels"
	SXADDPAR, header, 'PP4_ARAD', ap_rad, "Maximum aperture radius in FWHM units"
	SXADDPAR, header, 'PP4_REF0', ref_ap[0],"Reference aperture inner radius in FWHM units"
	SXADDPAR, header, 'PP4_REF1', ref_ap[1],"Reference aperture outer radius in FWHM units"
	SXADDPAR, header, 'PP4_CLEN', clean_rad, "Radius to PSF clean to, in FWHM * ARAD units"
	SXADDPAR, header, 'PP4_NAP', n_ap,"Number of apertures used in Ap Phot"
	for iap = 0, n_ap-1 do begin
		name = 'PP4_AP'+STRING(iap,format='(I2.2)')
		SXADDPAR, header, name, ap_vec[iap], "Aperture "+STRING(iap+1,format='(I2)')+" in FWHM * ARAD units"
	endfor
	;
	image_header = header
	;
	;print_text,Event, "Finished fitting "+STRING(format='(I)',nstars)+" stars. Ready to save final photometry"
	;
	;	sort by RA (reverse X)
	;
	igood = 0L
	;
	if(nbright_stars gt 0) then begin
		ind = REVERSE(SORT(x))
		;
		nstars1 = nbright_stars-1
		for is = 0, nstars1 do begin
			;
			;	only keep stars with uncertainties under 0.25 mag	(20090720 DPC)
			;
			(*d_ptr).x[igood] = x[ind[is]]
			(*d_ptr).y[igood] = y[ind[is]]
			(*d_ptr).ra[igood] = 0.0
			(*d_ptr).dec[igood] = 0.0
			for iap = 0, n_ap-1 do begin
				(*d_ptr).mag[igood,iap] = (mags[iap,ind[is]] > (-9.999)) < 25.0	 ; trap bad at -10 mag and +25 mag
				(*d_ptr).errmag[igood,iap] = errap[iap,ind[is]] < 9.999  ; errors, too
			endfor
			(*d_ptr).sky[igood] = sky[ind[is]]
			(*d_ptr).psf_group[igood] = 0
			igood++
			;
		endfor
		nstars_good = igood
		(*d_ptr).nstars = nstars_good
		;
		;	if astrometry present, include RA, Dec output info
		;
		if(noparams eq -1) then begin
			;
			;	no astrometry
			;
		endif else begin
			;
			;xy2ad, xmerge[ind[0:nstars-1]], ymerge[ind[0:nstars-1]], astr, a, d
			ns1 = nstars_good-1
			if(ns1 gt 0) then begin
				xvec = (*d_ptr).x[0:ns1]
				yvec = (*d_ptr).y[0:ns1]
				xy2ad, xvec, yvec, astr, a, d
				for i = 0, ns1 do begin
						(*d_ptr).ra[i] = a[i]
						(*d_ptr).dec[i] = d[i]
				endfor
			endif
		endelse
		;
		end_time = SYSTIME(/SECONDS)
		print_text,Event, "** Completed photometry for "+STRING(nstars_good)+" stars in "+STRING(format='(I)',end_time - start_time)+" sec."
		print_text, Event, "**------------------------------------**"
		;
		;
		status = 0			; all worked !
		;
	endif else status = -1
	;
	S4PSFF_end:
	;
;	stop
	;
end
