pro S5_match_star_lists, Event, group_ind, status
	;
	;	takes output from PSF-fitter for a bunch of
	;	files and matches stars, based on RA, Dec positions
	;
	;	generates a table of magnitudes for each star
	;	with missing data listed as 99.99 mag
	;
	;	DPC	20080724
	;		20081020	enhanced to do adaptive windowing
	;		20081101	exports match lists
	;		20081203	sorts list by JD before matching
	;		20081209	imported to PPOL
	;		20081227	added computation of pixel offsets
	;		20090326	reduced clamp level for sigma_mag, added sigma_mag cutoff
	;		20090403	reduced clamp level to 0.0005 from 0.005 mag
	;		20090602	upped clamp level for mag to 0.01, added area aperture for delta-mag computation
	;		20090910	opened mag window
	;		20090917	implemented multi-aperture phot input file
	;		20091228	implemented new reference image identification for "rotated" HWP data
	;						ref image is the first HWP=0 image, no longer required to be the first image
	;		20091230	added mean star match count and mean dmag reporting textboxes
	;		20100124	released radius limit to full field
	;		20100915 	fixed HWP bug for linux
	;		20120411	made dmag_clamp work properly
	;	    20130606	added HWP test skip if photometry
	;		20131005	added MB_CAMRA and MB_FILTR to output files
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	s_ptr = PTR_struct.PTR_summary
	g_ptr = PTR_struct.PTR_group
	h_ptr = PTR_struct.PTR_history
	;
	h_wid = (*w_ptr).S5_history_textbox_wid
	match_wid = (*w_ptr).S5_textbox_matches_wid
	rms_wid = (*w_ptr).S5_textbox_rms_dmag_wid
	;
	WIDGET_CONTROL, match_wid, SET_VALUE=' '
	WIDGET_CONTROL, rms_wid, SET_VALUE=' '
	;
	n_ap = 12							; number of apertures - better to read this, but for now...
	ap_match = 10						; aperture to use for matching (*not* scaling)
	;
	dmag_clamp = 0.0075					; mag trap (no lower)
	;
	;	compute pixel radius inside of which to compute magnitude offsets
	;
	aperture_rad_scale = sqrt((*w_ptr).S5_match_area / !PI)		; multiply by nx or ny to get radius in pixels
	;
	;	get photometry files to group
	;
	dir_phot = (*s_ptr).analysis_dir + 'S4_First_PSF_Fits' + PATH_SEP()
	;
	;	number of images
	;
	n_img = (*g_ptr).group_numbers[group_ind]
	group_name = (*g_ptr).group_names[group_ind]
	;
	;	images with "use" flags (as opposed to "don't use" flags)
	;
	g_ind = WHERE((*g_ptr).group_image_flags[group_ind,0:n_img-1] eq 1, count)
	if(count gt 0) then begin
		n_img = count				; fix
		group_images = (*g_ptr).group_images[group_ind,g_ind]
		;
		;	convert to photometry file names
		;
		phot_names = FILE_BASENAME(group_images,'.fits') + '_phot.dat'
		file_in = dir_phot + phot_names
	endif else begin
		str = group_name+" had no photometry files. Skipping this group"
		print_text, Event, "ERROR - "+str
		SAll_add_to_history, Event, h_wid, 5, str
		status=-1
		goto, S5_MSL_end
	endelse
	;
	;	read the photometry files in
	;
	nlists = N_ELEMENTS(file_in)
	;
	;	clear mean match and rms save vectors
	;
	T_match_vec = fltarr(nlists)
	T_rms_vec = fltarr(nlists)
	;
	;	set up LED1
	;
	;	if > 130, use LED scaling
	;
	LED1_wid = (*w_ptr).S5_LED1_images_wid
	nLED = 130
	LED_scale = float(nlists) / float(nLED)
	if(LED_scale le 1.0) then LED_scale = 1.0
	;
	WIDGET_CONTROL, LED1_wid, SET_UVALUE=(nLED < nlists)
	image_colors = strarr(nLED < nlists)
	image_colors[*] = "red"
	S5_LED_sensitize, Event, 1, 1
	;
	hwp_angle = fltarr(nlists)
	jd = dblarr(nlists)
	header = strarr(500)
	;
	data = {id: 0L, x: 0.0d0, y: 0.0d0, ra: 0.0d0, dec: 0.0d0, $
				mag: dindgen(12), errmag: dindgen(12), sky: 0.0d0}
	base = replicate(data, nlists, 501)			; up to 501 stars per file
	base_nstars = lonarr(nlists)
	;
	base_header = strarr(nlists, 500)
	base_n_header = lonarr(nlists)
	;
	max_stars = 0.0
	file_max_stars = 0L
	nlists1 = nlists-1
	for ifile = 0, nlists1 do begin
		;
		;	determine number of lines to skip
		;
		openr, lu_in, file_in[ifile], /GET_LUN
		line = ''
		nskip = 0
		nheader = 0
		WHILE ~EOF(lu_in) do begin
			readf, lu_in, line, format='(a)'
			test = STRPOS(line, ";;")
			if(test ne -1) then begin
				;
				;	reconstruct headers, skipping otherwise blank lines
				;
				short_line = STRTRIM(line,2)
				if(STRLEN(short_line) gt 4) then begin
					header[nheader] = STRMID(line,2)
					nheader++
				endif
				nskip++
			endif
		ENDWHILE
		FREE_LUN, lu_in
		;
		base_n_header[ifile] = nheader
		base_header[ifile,0:nheader-1] = header[0:nheader-1]
		;
		;	extract HWP angle
		;
		hwp_angle[ifile] = FXPAR(header, "MB_HWP")
		if(hwp_angle[ifile] eq 0.0) then begin
			test_hwp_angle = FXPAR(header, "HWP_ANG")
			if(test_hwp_angle ne hwp_angle[ifile]) then begin
				;
				;	if this is photometry, not polarimetry, then skip this next stuff
				;
				if(STRTRIM(FXPAR(header,"FILTNME1"),2) ne 'Open') then begin
					;
					;	set HWP to nearest multiple of 22.5 deg
					;
					n = (test_hwp_angle) / 22.5
					nr = ROUND(n)
					nrm = (nr+32) MOD 16	; in case of any neg
					hwp = float(nrm) * 22.5
					hwp_angle[ifile] = hwp
				endif
			endif
		endif
		date_obs = FXPAR(header, "DATE-OBS")
		;
		;	also get camera, filter info
		;
		this_camera = FXPAR(header,"MB_CAMRA")
		this_filter = FXPAR(header,"MB_FILTR")
		;
		;	parse date / time
		;
		reads, date_obs, year, month, day, hour, minute, second, format='(i4,1x,i2,1x,i2,1x,i2,1x,i2,1x,f4.2)
		;
		;	convert to JD
		;
		jd[ifile] = JULDAY(Month, Day, Year, Hour, Minute, Second)
		;
		;print, "For image ",file_in[ifile], " HWP angle = ",hwp_angle[ifile]
		;
		readcol, file_in[ifile], id, x, y, ra, dec, sky, grp, mag0,mag1,mag2,mag3,mag4,mag5,$
				mag6,mag7,mag8,mag9,mag10,mag11,$
				smag0,smag1,smag2,smag3,smag4,smag5,smag6,smag7,smag8,smag9,smag10,smag11,  skipline=nskip, $
				FORMAT='L,D,D,D,D,D,I,D,D,D,D,D,D,D,D,D,D,D,D,D,D,D,D,D,D,D,D,D,D,D,D', /SILENT
		n = N_ELEMENTS(x)
		;
		igood = 0L
		;
		n1 = n-1
		for i = 0, n1 do begin
			;
			;	only let into the data base those stars with mag uncertainties smaller than 0.2 mag and larger than 0
			;
			this_mag = [mag0[i],mag1[i],mag2[i],mag3[i],mag4[i],mag5[i],mag6[i],mag7[i],mag8[i],mag9[i],mag10[i],mag11[i]]
			this_unc = [smag0[i],smag1[i],smag2[i],smag3[i],smag4[i],smag5[i],smag6[i],smag7[i],smag8[i],smag9[i],smag10[i],smag11[i]]
			ind_good = WHERE(this_unc le 0.3 and this_unc ge 0.001,ngood,COMPLEMENT=ind_bad,NCOMPLEMENT=nbad)
			if(ngood gt 0) then begin
				base[ifile,igood].id = id[i]
				base[ifile,igood].x = x[i]
				base[ifile,igood].y = y[i]
				base[ifile,igood].ra = ra[i]
				base[ifile,igood].dec = dec[i]
				for ig = 0, ngood-1 do begin
					ii = ind_good[ig]
					base[ifile,igood].mag[ii] = this_mag[ii]
					base[ifile,igood].errmag[ii] = this_unc[ii]
				endfor
				;
				if(nbad gt 0) then begin
					for ib = 0, nbad-1 do begin
						ii = ind_bad[ib]
						base[ifile,igood].mag[ii] = 99.99
						base[ifile,igood].errmag[ii] = 99.99
					endfor
				endif
				;
				base[ifile,igood].sky = sky[i]
				igood++
			endif
		endfor
		;
		n = igood
		base_nstars[ifile] = n					; number of good stars in this list
		;
		;FREE_LUN, lu_in
		print_text, Event, "Read "+STRING(n,format='(I)')+" stars (with s_mag < 0.3) from file "+FILE_BASENAME(file_in[ifile])
		if(n gt max_stars) then begin
			max_stars = n
			file_max_stars = ifile
		endif
	endfor
	;
	;	if nlist more than 1, open file for saving results
	;
	if(nlists gt 1) then begin
		out_dir = (*s_ptr).analysis_dir + 'S5_Star_Matching' + PATH_SEP()
		group_name = (*g_ptr).group_names[group_ind]
		;
		;	convert any embedded spaces to underscores in the name
		;
		new_group_name = replace_underscores(group_name)
		;
		out_path = out_dir + new_group_name + '_match.dat'
		;
		print_text, Event, "Opening output file "+out_path
		;
		openw, lu_out, out_path, /GET_LUN
	endif
	;
	;	match each file to the reference one, where the reference one is the
	;	HWP=~0 image with the least Julian date
	;
	xoff = fltarr(nlists)
	yoff = fltarr(nlists)
	;
	;	sort by JD
	;
	jd_ind = SORT(jd)
	;
	;	and replace in place
	;
	base = base[jd_ind,*]
	base_nstars = base_nstars[jd_ind]
	hwp_angle = hwp_angle[jd_ind]
	jd = jd[jd_ind]
	file_in = file_in[jd_ind]
	base_n_header = base_n_header[jd_ind]
	base_header = base_header[jd_ind,*]
	;
	;	find the first image that has zero HWP angle
	;
	ind_0 = WHERE(abs(hwp_angle) le 2.0, n_0)
	;
	;	check that there is at least one
	;
	if(n_0 eq 0) then begin
		print_text, Event, "ERROR - No images have HWP = 0 deg. Matching halted"
		status=-1
		goto, S5_MSL_end
	endif
	;
	;	ok, the first is the first in the sorted list (ind_0[0])
	;
	ref_ind = ind_0[0]
	;
	file_ref_name = FILE_BASENAME(file_in[ref_ind])
;	print, "Ref filename = ",file_ref_name
	;
	list_ref = load_star_list(ref_ind, base, base_nstars)
;	print, "List_ref = ",list_ref
	;
	;	fetch astrometry information
	;
	ref_header = ''
	ref_header = REFORM(base_header[ref_ind,0:base_n_header[ref_ind]-1])
	ref_header = ref_header[0:base_n_header[ref_ind]-1]
	;
	;help, ref_header
	;
	extast, ref_header, ref_astr_struct, noparams
	;
	;	check by computing the x,y reference pixel values
	;
	ra_cen = sxpar(ref_header, "CRVAL1")
	dec_cen = sxpar(ref_header, "CRVAL2")
	;
	ref_PSFmag = sxpar(ref_header,"PSFMAG")
	;
;	print, "Reference RA, DEC of image Center = ",ra_cen, dec_cen
	;
	ad2xy, ra_cen, dec_cen, ref_astr_struct, x_ref, y_ref
	;
	x_cen = sxpar(ref_header, "CRPIX1",count=kwcount)
	if(kwcount ne 1) then stop
	y_cen = sxpar(ref_header, "CRPIX2",count=kwcount)
	if(kwcount ne 1) then stop
	;
	CD1_1 = sxpar(ref_header, "CD1_1",count=kwcount)
	if(kwcount ne 1) then stop
	CD2_2 = sxpar(ref_header, "CD2_2",count=kwcount)
	if(kwcount ne 1) then stop
	platescale = 3600.0 * (abs(CD1_1) + abs(CD2_2)) / 2.
	;
;	print, "Ref Platescale = ",platescale, " arcsec per pix"
	;
;	print, "FITS header X,Y center = ",x_cen, y_cen
;	print, "computed    X,Y center = ",x_ref, y_ref
	;
	;	and produce a magnitude-ordered index
	;
	ref_mag_index = SORT(REFORM(list_ref[2,0:base_nstars[ref_ind]-1]))
	dtor = 180.0d0/!DPI
	;
	;	number of stars in the reference star list
	;
	n_stars = base_nstars[ref_ind]
	;
	;	compute magnitude matching area and radius, based on the number of stars
	;
	mag_match_area = 1000.0
	;
;	mag_match_area = 0.90			; 90 % by default
;	if(n_stars gt 30) then mag_match_area = 0.70
;	if(n_stars gt 100) then mag_match_area = 0.60
;	if(n_stars gt 200) then mag_match_area = 0.55
;	if(n_stars gt 500) then mag_match_area = 0.50
	;
	aperture_rad_scale = sqrt(mag_match_area / !PI)		; multiply by nx or ny to get radius in pixels
	;
	;
	nlists1 = nlists-1
	for ifile = 0, nlists1 do begin
		;
		file_comp_name = FILE_BASENAME(file_in[ifile])
		;
		print_text, Event, "Matching file "+file_comp_name+" to reference file "+file_ref_name
		;
		;	set LED1 to blue at start
		;
		this_LED = ROUND(float(ifile) / LED_scale) & this_LED = 0 > ((nLED-1) < this_LED)
		image_colors[this_LED] = "blue"
		WCT_LED_stack, LED1_wid, image_colors
		;--------------------------------------------
		;
		;	get the comp header and compute pixel offsets
		;
		this_header = ''
		this_header = REFORM(base_header[ifile,0:base_n_header[ifile]-1])
		this_header = this_header[0:base_n_header[ifile]-1]
		this_ra_cen = sxpar(this_header, "CRVAL1",count=kwcount)
		if(kwcount ne 1) then stop
		this_dec_cen = sxpar(this_header, "CRVAL2",count=kwcount)
		if(kwcount ne 1) then stop
		;
		this_nx = sxpar(this_header, "NAXIS1",count=kwcount)
		if(kwcount ne 1) then stop
		this_ny = sxpar(this_header, "NAXIS2",count=kwcount)
		if(kwcount ne 1) then stop
		;
		comp_PSFmag = sxpar(this_header, "PSFMAG",count=kwcount)
		if(kwcount ne 1) then stop
		comp_nPSF = sxpar(this_header, "NPSF",count=kwcount)
		if(kwcount ne 1) then stop
		comp_FWHM = sxpar(this_header, "PSF_FWHM",count=kwcount)
		if(kwcount ne 1) then stop
		;
		;	get the aperture information
		;
		PP4_ARAD = sxpar(this_header,"PP4_ARAD",count=kwcount)
		if(kwcount ne 1) then stop
		PP4_REF0 = sxpar(this_header,"PP4_REF0",count=kwcount)
		if(kwcount ne 1) then stop
		PP4_REF1 = sxpar(this_header,"PP4_REF1",count=kwcount)
		if(kwcount ne 1) then stop
		PP4_CLEN = sxpar(this_header,"PP4_CLEN",count=kwcount)
		if(kwcount ne 1) then stop
		PP4_NAP = sxpar(this_header,"PP4_NAP",count=kwcount)
		if(kwcount ne 1) then stop
		PP4_AP = dblarr(PP4_NAP)
		PP4_NAP1 = PP4_NAP-1
		for iap = 0, PP4_NAP1 do begin
			ap_name = 'PP4_AP'+STRING(iap,format='(i2.2)')
			PP4_AP[iap] = sxpar(this_header,ap_name,count=kwcount)
			if(kwcount ne 1) then stop
		endfor
		;
		;	comparison camera, filter - they must match to the reference
		;
		comp_camera = SXPAR(this_header, "MB_CAMRA")
		comp_filter = SXPAR(this_header, "MB_FILTR")
		;
		if(comp_camera ne this_camera or comp_filter ne this_filter) then begin
			print_text, Event, "In matching, reference and comparison cameras or filters do not match. Stopping"
			stop
		endif
		;
		aper_rad = aperture_rad_scale * ( this_nx < this_ny )	; get radius to ap limit
		aper_rad = 1.0e5		; turn off this test - DPC  - 20100308
		;
		x_cen = ROUND(this_nx / 2.0)
		y_cen = ROUND(this_ny / 2.0)
		;
;		print, "This RA, Dec image center = ",this_ra_cen, this_dec_cen
		;
		;	center to center offsets
		;
		ad2xy, this_ra_cen, this_dec_cen, ref_astr_struct, this_x_cen, this_y_cen
		;
;		print, "for ifile = ",ifile
;		print, "this_x_cen, y_cen = ",this_x_cen, this_y_cen," on ref image"
		this_x_offset = this_x_cen - x_ref
		this_y_offset = this_y_cen - y_ref
;		print, "offset to ADD to comp image to match REF image positions (Xoff,Yoff) = ",this_x_offset, this_y_offset
		;
		;	for each of 6 passes, used to refine the offsets
		;
		;	initial guesses
		;
		angle_window = 5.0		; match items within 5" of each other
		win_mag = 4.0			; and 4 mag of each other
		win_deg = angle_window / 3600.0d0
		n_mag = 0L
		;
		ra_zero = 0.0d0			; these will be updated with each pass
		dec_zero = 0.0d0
		mag_zero = 0.0d0
		mag_zero_ap = dblarr(6)
		;
		n_ra_dec = 0L
		ra_width = win_deg
		dec_width = win_deg
		mag_width = win_mag
		;
		for ipass = 0, 5 do begin
			;
			;	led blink between white and blue on alternate passes
			;
			if(ipass eq 2*(FIX(ipass/2))) then color = 'white' else color = 'blue'
			image_colors[this_LED] = color
			WCT_LED_stack, LED1_wid, image_colors
			;
			nmatch = 0L
			ra_off = dblarr(n_stars)
			dec_off = dblarr(n_stars)
			mag_off = dblarr(12, n_stars)
			s_mag_off = dblarr(12, n_stars)
			mag_use = intarr(12, n_stars)
			;
			list_pointers= lonarr(3,n_stars)	; points to match in referece, target lists
								; 3rd position is 1/0 for match/no-match
			;
			;	match to first list
			;
			;	set up two star lists and match
			;
			this_list = load_star_list(ifile, base, base_nstars)
;			print,"this_list = ",this_list
			;
			;	start with magnitude ordered first list
			;
			n_stars1 = n_stars-1
			for iref = 0, n_stars1 do begin
				;
				ref_ra = list_ref[0,ref_mag_index[iref]]
				ref_dec = list_ref[1,ref_mag_index[iref]]
				ref_mag = list_ref[2,ref_mag_index[iref]]
				;
				;	test this mag. if deleted, skip
				;
				if(ref_mag gt 25.0 or ref_mag lt -5.0) then goto, ref_loop_end
				;
				ref_s_mag = list_ref[3,ref_mag_index[iref]]
				;
				if(ref_s_mag gt 1.0 or ref_s_mag lt 0.001) then goto, ref_loop_end
				;
				ref_x = list_ref[4,ref_mag_index[iref]]
				ref_y = list_ref[5,ref_mag_index[iref]]
				;
				ref_rad = sqrt((x_cen - ref_x)^2 + (y_cen - ref_y)^2)
				if(ref_rad gt aper_rad) then ref_mag_use = 0 else ref_mag_use = 1	; select whether to use this star in the mag comparison
				;
				list_pointers[0,iref] = ref_mag_index[iref]	; save pointer into reference list
				;
				cosdec = cos(ref_dec / dtor)
				;
				;	compare to each star on second list
				;
				ddist_match = 10.0								; smallest match
				ddist_index = -1L
				ddist_ra_off = 0.0d0
				ddist_dec_off = 0.0d0
				ddist_mag_off = 0.0d0
				;
				for icomp = 0, base_nstars[ifile]-1 do begin
					;
					comp_ra = this_list[0,icomp]
					comp_dec = this_list[1,icomp]
					comp_mag = this_list[2,icomp]
					;
					; test comp mag. if out of bounds, skip
					;
					if(comp_mag gt 25.0 or comp_mag lt -5.0) then goto, comp_loop_end
					;
					comp_s_mag = this_list[3,icomp]
					;
					;	unc mag, too
					;
					if(comp_s_mag gt 1.0 or comp_s_mag lt 0.00001) then goto, comp_loop_end
					;
					comp_x = this_list[4,icomp]
					comp_y = this_list[5,icomp]
					comp_rad = sqrt((x_cen - comp_x)^2 + (y_cen - comp_y)^2)
					if(comp_rad gt aper_rad) then comp_mag_use = 0 else comp_mag_use = 1
					;
					ddec = comp_dec - ref_dec - dec_zero			; in decimal degrees on sky
					dra = (comp_ra - ref_ra) * cosdec - ra_zero		; in decimal degrees on sky
					;
					ddist = sqrt(ddec * ddec + dra * dra)
					dmag = (comp_mag - ref_mag) - mag_zero
					;
					if(abs(ddec) le dec_width and abs(dra) le ra_width) then begin
						if(ref_mag_use eq 0 or comp_mag_use eq 0 or abs(dmag) le mag_width) then begin
							;
							;	found a match for this star - is it closer than any other previous match?
							;
							if(ddist le ddist_match) then begin
								ddist_match = ddist
								ddist_index = icomp
								ddist_ra_off = dra + ra_zero
								ddist_dec_off = ddec + dec_zero
								ddist_mag_off = dmag + mag_zero
								ddist_s_mag_off = sqrt(ref_s_mag ^2 + comp_s_mag^2) > dmag_clamp  ; trap to no smaller than clamp value
								if(ref_mag_use eq 1 and comp_mag_use eq 1) then ddist_mag_use = 1 else ddist_mag_use = 0
							endif
							;
						endif
					endif
					;
					comp_loop_end:
				endfor
				;
				;	was there a match?
				;
				if(ddist_index ne -1L) then begin
					;
					;	yes -- save the match
					;
					ra_off[nmatch] = ddist_ra_off
					dec_off[nmatch] = ddist_dec_off
					;
					;	for each aperture
					;
					n_ap1 = n_ap-1
					for iap = 0, n_ap1 do begin
						mag_i = 6 + iap*2
						emag_i = mag_i + 1
						dmag = this_list[mag_i,ddist_index] - list_ref[mag_i,ref_mag_index[iref]]
						;
;						print, "iap, this_mag, ref_mag, mag_zero, dif_mag = ", iap, this_list[mag_i,ddist_index], $
;								list_ref[mag_i,ref_mag_index[iref]], mag_zero_ap[iap], dmag
						;
						if(abs(dmag) gt 1.0) then goto, ref_loop_end
						;
						mag_off[iap,nmatch] = dmag
						s_mag_off[iap,nmatch] = sqrt(list_ref[emag_i,ref_mag_index[iref]]^2 + this_list[emag_i,ddist_index]^2) > dmag_clamp
						mag_use[iap,nmatch] = ddist_mag_use
					endfor
					;
					;	and update pointer
					;
					list_pointers[1,iref] = ddist_index
					list_pointers[2,iref] = 1
					;
					nmatch++
				endif
				;
				ref_loop_end:
			endfor
			;
			;	end of this pass, recompute the offsets
			;
			if(nmatch eq 0) then begin
				;
				;	shouldn't get here - post error message, but don't put anything into output file
				;
				print_text, Event, 'ERROR - no matches - should not happen - astrometry error???'
				;
				;	go on to next image
				;
				mag_mean = -99.9
				s_mag_mean = -99.9
				ra_mean = -99.9
				dec_mean = -99.9
				x_tot = 99.99
				y_tot = 99.99
				;
;				printf, lu_out, ifile, FILE_BASENAME(file_in[ifile]), jd[ifile], HWP_angle[ifile], nmatch, $
;					mag_mean, s_mag_mean, ra_mean, dec_mean, x_tot, y_tot, $
;		 			format='(i3,1x,a,1x,f13.5,1x,f6.1,1x,i4,1x,f12.5,1x,f12.5,1x,f12.5,1x,f12.5,1x,f7.2,1x,f7.2)'
		 		;
		 		image_colors[this_LED] = "red"
		 		;
		 		SAll_add_to_history, Event, h_wid, 5, group_name + ": no matched stars for " + file_comp_name + ' USE flag -> 0'
				ind_bad = WHERE(group_images[ifile] eq (*g_ptr).group_images[group_ind,*],n_bad)
				if(n_bad eq 1) then begin
					(*g_ptr).group_image_flags[group_ind,ind_bad] = 0
				endif else begin
					stop
				endelse
			;
		 		goto, S5_file_end
		 	endif
		 	;
			ra_off = ra_off[0:nmatch-1]
			dec_off = dec_off[0:nmatch-1]
			mag_off = mag_off[*,0:nmatch-1]
			s_mag_off = s_mag_off[*,0:nmatch-1]
			mag_use = mag_use[*,0:nmatch-1]
			;
			zz_ra = median_filtered_mean(ra_off)
			zz_dec = median_filtered_mean(dec_off)
			;
			ind_mag_use = WHERE(mag_use[ap_match,*] eq 1,n_mag_use)
			if(n_mag_use ge 1) then begin
				zz_mag_ap = dblarr(12,2)
				for iap=0, n_ap-1 do begin
					zz_mag = median_filtered_mean(REFORM(mag_off[iap,ind_mag_use]))
					zz_mag_ap[iap,*] = zz_mag[*]
				endfor
			endif else begin
				print_text, Event, "ERROR - no matching stars inside match aperture of "+STRING(aper_rad)+" pixels"
				image_colors[this_LED] = "orange"
		 		;
		 		SAll_add_to_history, Event, h_wid, 5, group_name + " had no matching stars in center for image " + file_comp_name
			;
		 		goto, S5_file_end
			endelse
			;
			ra_zero = zz_ra[0]
			dec_zero = zz_dec[0]
			mag_zero = zz_mag_ap[2,0]
			mag_zero_ap = REFORM(zz_mag_ap[*,0])
			;
			;	for high precision work, compute mean mag off and uncertainly using propagated uncer.
			;
			mag_mean = dblarr(12)		; 0.0d0
			mag_wt = dblarr(12)			; 0.0d0
			s_mag_mean = dblarr(12)
			;
			for iap = 0, n_ap-1 do begin
				for k_match = 0, nmatch-1 do begin
					if(mag_use[iap,k_match] eq 1) then begin
						wt = 1./ (s_mag_off[iap,k_match]^2)
						mag_mean[iap] = mag_mean[iap] + mag_off[iap,k_match] * wt
						mag_wt[iap] = mag_wt[iap] + wt
					endif
				endfor
				;
				mag_mean[iap] = mag_mean[iap] / mag_wt[iap]
				s_mag_mean[iap] = 1./sqrt(mag_wt[iap])
			endfor
			;
			;	widths
			;
			ra_width = ((2.5+0.10*ipass)*zz_ra[1]) > (0.3d0 / 3600.0d0)		; 2.5-3.3 sigma
			dec_width = ((2.5+0.10*ipass)*zz_dec[1]) > (0.3d0 / 3600.0d0)
			mag_width = ((2.5+0.10*ipass)*zz_mag_ap[ap_match,1]) > 0.3	; trapped at no smaller than 0.3 mag
			;
			;if(ipass le 5) then mag_width = 6.0*zz_mag[1] else mag_width = 10.0	; only trim on early loops
			;if(ipass le 5) then mag_width = (2.5+0.15*ipass)*zz_mag[1] else mag_width = (2.0 < zz_mag[1]*2.0)
			;
	;		print, "pass ",ipass+1," Nmatches = ",nmatch," RA offset = ",ra_zero*3600.0,"+/-",zz_ra[1]*3600.0,$
	;		   " RA Window = ",ra_width*3600.0," Dec offset = ",dec_zero*3600.0,"+/-",$
	;		   zz_dec[1]*3600.0," Dec Window = ",dec_width*3600.0," Mag Offset = ",$
	;		   mag_zero,"+/-",zz_mag[1]," Mag Window = ",mag_width, " Mean-Mag-offset = ",mag_mean,"+/-",s_mag_mean
		endfor
		;
		;	end of this file loop, report matches
		;
		print_text, Event, "For "+STRING(n_stars,format='(I)')+" ref stars and "+$
			STRING(base_nstars[ifile],format='(I)')+" comp stars, "+STRING(nmatch,format='(I)')+" matched"
		;
		;	and plot distribution functions into windows
		;
		ra_off = (ra_off[0:nmatch-1])*3600.0d0
		dec_off = (dec_off[0:nmatch-1])*3600.0d0
		mag_off = mag_off[*,0:nmatch-1]
		s_mag_off = s_mag_off[*,0:nmatch-1]
		mag_use = mag_use[*,0:nmatch-1]
		;
		zz_ra = median_filtered_mean(ra_off)
		zz_dec = median_filtered_mean(dec_off)
		;
		zz_mag_ap = dblarr(12,2)
		;
		n_mag_use_save = 0L
		for iap = 0, n_ap-1 do begin
			ind_mag_use = WHERE(mag_use[iap,*] eq 1,n_mag_use)
			if(n_mag_use ge 1) then begin
				zz_mag = median_filtered_mean(mag_off[iap,ind_mag_use])
				zz_mag_ap[iap,*] = zz_mag[*]
				if(iap eq ap_match) then n_mag_use_save = n_mag_use
			endif
		endfor
		;
		ra_mean = zz_ra[0]
		dec_mean = zz_dec[0]
		;
		mag_mean = dblarr(12)		; 0.0d0
		mag_wt = dblarr(12)			; 0.0d0
		s_mag_mean = dblarr(12)
		;
		nmag_match = 0L
		for iap = 0, n_ap-1 do begin
			for k_match = 0, nmatch-1 do begin
				if(mag_use[iap,k_match] eq 1) then begin
					wt = 1./ (s_mag_off[iap,k_match]^2)
					mag_mean[iap] = mag_mean[iap] + mag_off[iap,k_match] * wt
					mag_wt[iap] = mag_wt[iap] + wt
					if(iap eq ap_match) then nmag_match++
				endif
			endfor
			;
			mag_mean[iap] = ((mag_mean[iap] / mag_wt[iap]) > (-9.999)) < 9.999
			s_mag_mean[iap] = (1./sqrt(mag_wt[iap])) < 9.999
		endfor
		;
		print_text, Event, "Mean RA offset = "+STRING(zz_ra[0])+" arcsec,  with dispersion = "+STRING(zz_ra[1])+" arcsec"
		print_text, Event, "Mean Dec offset = "+STRING(zz_dec[0])+" arcsec,  with dispersion = "+STRING(zz_dec[1])+" arcsec"
	;	 print_text, Event, "Mean Mag offset = "+STRING(zz_mag[0])+" mags,  with dispersion = "+STRING(zz_mag[1])+" mags"
		print_text, Event, "Weighted Mean Mag Offset = "+STRING(mag_mean[ap_match])+" mag, w/uncert.= "+STRING(s_mag_mean[ap_match])+" mag for "+STRING(n_mag_use_save)+" stars"
;		print_text, Event, "PSFmag difference = "+STRING(comp_PSFmag - ref_PSFmag)+" for "+STRING(comp_nPSF)+" PSF stars"
		 ;
		if(ifile ne ref_ind) then begin						; don't plot ref matched to itself
			;
			; windows are on the tab page of the gui
			;
			zoom = 5.0					; zoom factor
			;
			WIDGET_CONTROL, (*w_ptr).S5_ra_offsets_window_wid, GET_VALUE=win_no
			wset, win_no
			erase
			scale = (CEIL(zoom*ra_width*3600.0)/zoom) > (CEIL(zoom*dec_width*3600.0)/zoom)
			center = ROUND(zoom*zz_ra[0])/zoom
			xradec = dindgen(51)*(scale/25.0d0) - double(scale) + double(center)
			plot, xradec,  dpc_histogram(ra_off, NBINS=51, MIN = xradec[0], MAX = xradec[50]), $
			 	xtitle='RA offset [arcsec]',ytitle='Number of stars',psym=10
			;
			WIDGET_CONTROL, (*w_ptr).S5_dec_offsets_window_wid, GET_VALUE=win_no
			wset, win_no
			erase
			center = ROUND(zoom*zz_dec[0])/zoom
			xradec = dindgen(51)*(scale/25.0d0) - double(scale) + double(center)
			plot, xradec,  dpc_histogram(dec_off, NBINS=51, MIN = xradec[0], MAX = xradec[50]), $
		 		xtitle='Dec offset [arcsec]',ytitle='Number of stars',psym=10
		 	;
		 	WIDGET_CONTROL, (*w_ptr).S5_mag_offsets_window_wid, GET_VALUE=win_no
			wset, win_no
			erase
			center = ROUND(zoom*zz_mag_ap[ap_match,0])/zoom
			m_scale = CEIL(zoom*mag_width)/zoom
			xmag = dindgen(51)*(m_scale/25.0d0) - double(m_scale) + double(center)
			plot, xmag,  dpc_histogram(mag_off[ap_match,ind_mag_use], NBINS=51, MIN = xmag[0], MAX = xmag[50]), $
			 	xtitle='Mag offset [mag]',ytitle='Number of stars',psym=10
		 endif
		 ;
		 ;	compute pixel offsets from RA, Dec centers and match offsets, using saved platescale
		 ;
		 dx = -1.0 * ra_mean / platescale
		 dy = -1.0* dec_mean / platescale
		 total_x_offset = this_x_offset + dx
		 total_y_offset = this_y_offset + dy
		 ;
;		 print, "RA, Dec offsets (in arcsec) [This-Ref] = " ,ra_mean, dec_mean
;		 print, "X, Y offsets (in pixels)               = ", dx, dy
;		 print, "Center offsets (X, Y) in pixels        = ", this_x_offset, this_y_offset
;		 print, "Total image offsets to add to ref (X,Y)= ", total_x_offset, total_y_offset
		 ;
	 	 ;if(ifile eq 1) then stop
	 	 ;
	 	 ;	write to output file
	 	 ;
	 	 if(ifile eq 0) then begin
			nch_file = STRLEN(FILE_BASENAME(file_in[ifile]))
			nbuf2 = (nch_file - 8) / 2
			chbuf = ''
			if(nbuf2 gt 0) then begin
				for ib = 0, nbuf2-1 do begin
					chbuf = chbuf + ' '
				endfor
			endif
			;
			;	add header info regarding multiple apertures....
			;
			printf, lu_out, '; Phot matching file with multiple apertures created - ',systime()
			printf, lu_out, ';  key aperture info follows:'
			printf, lu_out, ';HISTORY   PPOL-Step 5-version '+(*s_ptr).version_short_string
			printf, lu_out, ';PP5_RFIM=         '+STRING(ref_ind,format='(I4)')+'/ Reference image index for Matching'
			printf, lu_out, ';PP4_ARAD=         '+STRING(PP4_ARAD,format='(F8.3)')+'/ Maximum Aper Size, in FWHM units'
			printf, lu_out, ';PP4_REF0=         '+STRING(PP4_REF0,format='(F8.3)')+'/ Inner ref radius, in FHWM * ARAD units'
			printf, lu_out, ';PP4_REF1=         '+STRING(PP4_REF1,format='(F8.3)')+'/ Outer ref radius, in FHWM * ARAD units'
			printf, lu_out, ';PP4_CLEN=         '+STRING(PP4_CLEN,format='(F8.3)')+'/ Outer cleaning radius, in FHWM * ARAD units'
			printf, lu_out, ';PP4_NAP =                '+STRING(PP4_NAP,format='(I2.2)')+'/ Number of apertures'
			;
			for iap = 0, n_ap-1 do begin
				printf, lu_out, ';PP4_AP'+STRING(iap,format='(I2.2)')+'=         '+STRING(PP4_AP[iap],format='(F8.6)')+'/ Aperture size in FWHM * ARAD units'
			endfor
			;
			;	add camera, filter
			;
			printf, lu_out, ';MB_CAMRA=         '+comp_camera+'/ Mimir Camera Optics Used'
			printf, lu_out, ';MB_FILTR=         '+comp_filter+'/ Mimir Filter Option Used'
			;
			printf, lu_out, ';'
			long_string = ';No. '+chbuf+'filename'+chbuf+' Julian DT/TM  '+' HWPA  '+'Nmtch'+$
	 	                   '  d_RA_match '+' d_Dec_match '+'XOffset '+'YOffset '+'dPSFmag '+'nPSF '+'nMagMtch'+'  PsfFwhm'+$
	 	                   ' dMag[0]  sdMag[0] dMag[1]  sdMag[1] dMag[2]  sdMag[2] dMag[3]  sdMag[3] dMag[4]  sdMag[4] dMag[5]  sdMag[5]'+$
	 	                   '  dMag[6]  sdMag[6] dMag[7]  sdMag[7] dMag[8]  sdMag[8] dMag[9]  sdMag[9]dMag[10]sdMag[10]dMag[11]sdMag[11]'
	 	 	printf, lu_out, format='(a)',long_string
	 	 endif
	 	 printf, lu_out, ifile, FILE_BASENAME(file_in[ifile]), jd[ifile], hwp_angle[ifile], nmatch, $
		 			ra_mean, dec_mean, total_x_offset, total_y_offset, comp_PSFmag - ref_PSFmag, comp_nPSF, nmag_match, comp_fwhm, $
		 			mag_mean[0],s_mag_mean[0],mag_mean[1],s_mag_mean[1],mag_mean[2],s_mag_mean[2],mag_mean[3],s_mag_mean[3],$
		 			mag_mean[4],s_mag_mean[4],mag_mean[5],s_mag_mean[5],$
		 			mag_mean[6],s_mag_mean[6],mag_mean[7],s_mag_mean[7],mag_mean[8],s_mag_mean[8],mag_mean[9],s_mag_mean[9],$
		 			mag_mean[10],s_mag_mean[10],mag_mean[11],s_mag_mean[11],$
		 			format='(i3,1x,a,1x,f15.7,1x,f6.1,1x,i4,1x,f12.5,1x,f12.5,1x,f7.2,1x,f7.2,1x,f7.4,1x,i4,5x,i4,1x,f8.3,1x,12(f8.5,1x,f8.5,1x))'
		 ;
		 print_text, Event, "X,Y Offsets to add to this image in coadding = "+STRING(total_x_offset, total_y_offset)
		 ;
		 ;	save match info to vectors
		 ;
		 T_match_vec[ifile] = float(n_mag_use_save)
		 T_rms_vec[ifile] = s_mag_mean[ap_match]
		 ;
		 ;	led to green
		 ;
		 image_colors[this_LED] = "green"
		 ;
		 S5_file_end:
		 WCT_LED_stack, LED1_wid, image_colors
	endfor
	;
	if(nlists gt 1) then FREE_LUN, lu_out
	status=0									; worked fine
	;
	;	populate textboxes
	;
	ind_good = WHERE(T_rms_vec ne 0, ngood)
	if(ngood gt 0) then begin
		zz = median_filtered_mean(T_match_vec[ind_good])
		yy = median_filtered_mean(T_rms_vec[ind_good])
		yy = yy * 1000.0								; report in mmag
		;
		WIDGET_CONTROL, match_wid, SET_VALUE=STRTRIM(STRING(zz[0],zz[1],format='(2x,F7.1,"+/-",F5.1)'),2)
		WIDGET_CONTROL, rms_wid, SET_VALUE=STRTRIM(STRING(yy[0],yy[1],format='(2x,F7.2,"+/-",F5.2)'),2)
	endif
	;
	S5_MSL_end:
	;
end

		;
