pro S6_meta_match, Event, meta_group_number, meta_group_name, output_summary_path, status
	;
	;	for designated meta-group:
	;
	;		1. access and ingest group-level filenames
	;		2. match reference images across groups, compute relative
	;			scale factors (group-to-group)
	;		3. output list of groups, relative scale factors, and offsets from Step 5
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	s_ptr = PTR_struct.PTR_summary
	g_ptr = PTR_struct.PTR_group
	;
	h_wid = (*w_ptr).S6_history_textbox_wid
	match_wid = (*w_ptr).S6C_textbox_matches_wid
	rms_wid = (*w_ptr).S6C_textbox_rms_dmag_wid
	nbad_wid = (*w_ptr).S6C_textbox_nbad_wid
	;
	WIDGET_CONTROL, match_wid, SET_VALUE=' '
	WIDGET_CONTROL, rms_wid, SET_VALUE=' '
	WIDGET_CONTROL, nbad_wid, SET_VALUE=' '
	;
	dmag_clamp = 0.005
	n_ap = 12
	;
	;	get the group names from the meta-group buffer
	;
	ngroups = (*g_ptr).meta_group_numbers[meta_group_number]	; number of groups in this meta
	;
	;	clear mean match and rms save vectors
	;
	T_match_vec = fltarr(ngroups)
	T_rms_vec = fltarr(ngroups)
	;
	if(ngroups le 0) then begin
		print_text, Event, "Error - meta-group number "+STRING(meta_group_number,format='(I)')+" does not exist"
		status = -1
		goto, MM_end
	endif
	;
	group_ids = REFORM((*g_ptr).meta_group_groups[meta_group_number,0:ngroups-1])	; group numbers in this meta
	group_names = (*g_ptr).group_names[group_ids]
	;
	print_text, Event, "group names in this meta-group = "
	for ig = 0, ngroups-1 do begin
		print_text, Event, "        " + group_names[ig]
	endfor
	;
	;	names of the reference images in each group
	;
	group_images = strarr(ngroups)
	for ig = 0, ngroups-1 do begin
;		igg = group_ids[ig]
		;
		;	have to read the S5 matching summary to get reference image for each group
		;
		match_phot = (*s_ptr).analysis_dir + 'S5_Star_Matching' + PATH_SEP() + group_names[ig] + '_match.dat'
		print_text, Event, "Opening S5 Star Matching file "+FILE_BASENAME(match_phot)+" for group "+group_names[ig]
		;
		openr, lu_match, match_phot, /GET_LUN
		;
		;	 read header info to get reference image name (from image index)
		;
		ref_ind = -1
		n_line = 0L
		WHILE ~EOF(lu_match) do begin
			line = ''
			readf, lu_match, line
			n_line++
			;
			;	look for header keywords
			;
			ind = STRPOS(line, 'PP5_RFIM')
			if(ind ne -1) then begin
				header = strarr(1)
				header[0] = STRMID(line,ind)
				ref_index = SXPAR(header, 'PP5_RFIM',COUNT=count)
				if(count eq 1) then begin
					print_text, Event, 'Reference Image number = '+STRING(ref_index)
					goto, found_ref_im
				endif
			endif
		ENDWHILE
		found_ref_im:
		;
		;	with index in hand, go get the image name
		;
		if(ref_index ne -1) then begin
			;
			;	read past all lines staring with semicolons
			;
			WHILE ~EOF(lu_match) do begin
				line = ''
				readf, lu_match, line
				n_line++
				ind = STRPOS(line, ';')
				if(ind eq -1) then goto, start_file_list
			ENDWHILE
			start_file_list:
			;
			;	and into the stack of image listings
			;
			if(ref_index ne 0) then begin
				for ii = 0, ref_index-1 do begin
					readf, lu_match, line
					n_line++
				endfor
			endif
			;
			;	have already read the relevant line
			;
;			readf, lu_match, line
;			n_line++
			;
			;	and parse to get filename
			;
			parts = STRSPLIT(line, ' ',/REGEX,/EXTRACT,COUNT=n_parts)
			for iparts = 0, n_parts-1 do begin
				ind_path = STRPOS(parts[iparts],'phot.dat')
				if(ind_path ne -1) then goto, found_name
			endfor
			found_name:
			;
			group_images[ig] = STRTRIM(parts[iparts],2)
			;
			print_text, Event, "For group "+group_names[ig]+" reference image was at line, index, filename = "+STRING(n_line,ref_index, format='(I3,", ",I2)')+', '+group_images[ig]
		endif else begin
			print_text, Event, "S6 Meta Error - Couldn't parse the S5 Match file "+match_phot+" to find reference image"
			status = -1
			goto, MM_end
		endelse
		;
		FREE_LUN, lu_match
		;
;		group_images[ig] = FILE_BASENAME((*g_ptr).group_images[igg,0])
	endfor
	;
	;	get photometry files directory
	;
	dir_phot = (*s_ptr).analysis_dir + 'S4_First_PSF_Fits' + PATH_SEP()
	;
	;	and directory to the first scaling
	;
	dir_scale = (*s_ptr).analysis_dir + 'S6_Scaling' + PATH_SEP()
	;
	;	number of reference images
	;
	n_img = ngroups
	;
	;	convert to photometry file names
	;
	phot_names = FILE_BASENAME(group_images) ; + '_phot.dat'
	;
	;	and scaling names
	;
	scale_names = FILE_BASENAME(group_names) + '_group_scale_factors.dat'
	;
	file_in = dir_phot + phot_names
	scale_in = dir_scale + scale_names
	;
	print_text, Event, "Pathnames to scaling files = "
	n_g = N_ELEMENTS(scale_in)
	for ig = 0, n_g-1 do print_text, Event, "Group "+group_names[ig]+" has scaling file = "+scale_in[ig]
	;
;	stop
	;
	;	read the photometry files in
	;
	nlists = N_ELEMENTS(file_in)
	;
	;	set up LED1
	;
	;	if > 10, use LED scaling
	;
	LED1_wid = (*w_ptr).S6C_LED1_group_wid
	nLED = 10
	LED_scale = float(nlists) / float(nLED)
	if(LED_scale le 1.0) then LED_scale = 1.0
	;
	WIDGET_CONTROL, LED1_wid, SET_UVALUE=(nLED < nlists)
	image_colors = strarr(nLED < nlists)
	image_colors[*] = "red"
	S6_LED_sensitize, Event, 2, 1, 1
	;
	hwp_angle = fltarr(nlists)
	jd = dblarr(nlists)
	header = strarr(500)
	;
	data = {id: 0L, x: 0.0d0, y: 0.0d0, ra: 0.0d0, dec: 0.0d0, $
				mag: 0.0d0, errmag: 0.0d0, sky: 0.0d0}
	base = replicate(data, nlists, 500)			; up to 500 stars
	base_nstars = lonarr(nlists)
	;
	base_header = strarr(nlists,500)
	base_n_header = lonarr(nlists)
	;
	max_stars = 0.0
	file_max_stars = 0L
	best_ap = intarr(nlists)
	;
	for ifile = 0, nlists-1 do begin
		;
		;	read in the selected best aperture for this group
		;
		openr, lu_scale, replace_underscores(scale_in[ifile]), /GET_LUN
		line = ''
		WHILE ~EOF(lu_scale) do begin
			readf, lu_scale, line, format='(a)'
			test = STRPOS(line, 'corrections follow')
			if(test ne -1) then begin
				;
				parts = STRSPLIT(line, ' ', /REGEX, /EXTRACT)
				reads, parts[6], this_ap, format='(i)'
				best_ap[ifile] = this_ap - 1		; reduce to IDL indexing
;				best_ap[ifile] = LONG(parts[6])
				;
				print_text, Event, "Best aperture for group "+group_names[ifile]+" = "+STRING(best_ap[ifile]+1,format='(I2)')
			endif
		ENDWHILE
		FREE_LUN, lu_scale
		;
		;	determine number of lines to skip
		;
		openr, lu_in, file_in[ifile], /GET_LUN
		line = ''
		nskip = 0
		WHILE ~EOF(lu_in) do begin
			readf, lu_in, line, format='(a)'
			test = STRPOS(line, ";;")
			if(test ne -1) then begin
				;
				;	reconstruct headers
				;
				header[nskip] = STRMID(line,2)
				nskip++
			endif
		ENDWHILE
		FREE_LUN, lu_in
		;
		base_header[ifile,0:nskip-1] = header[0:nskip-1]
		base_n_header[ifile] = nskip
		;
		;	extract HWP angle
		;
		hwp_angle[ifile] = FXPAR(header, "MB_HWP")
		date_obs = FXPAR(header, "DATE-OBS")
		;
		;	parse date / time
		;
		reads, date_obs, year, month, day, hour, minute, second, format='(i4,1x,i2,1x,i2,1x,i2,1x,i2,1x,f4.2)
		;
		;	convert to JD
		;
		jd[ifile] = JULDAY(Month, Day, Year, Hour, Minute, Second)
		;
		;print, "For image ",file_in[ifile], " HWP angle = ",hwp_angle[ifile]
		;
;		readcol, file_in[ifile], id, x, y, ra, dec, mag, errmag, sky, grp, skipline=nskip, $
;				FORMAT='L, D, D, D,D, D, D, D, I', /SILENT
		readcol, file_in[ifile], id, x, y, ra, dec, sky, grp, mag0,mag1,mag2,mag3,mag4,mag5,$
				mag6,mag7,mag8,mag9,mag10,mag11,$
				smag0,smag1,smag2,smag3,smag4,smag5,smag6,smag7,smag8,smag9,smag10,smag11,  skipline=nskip, $
				FORMAT='L,D,D,D,D,D,I,D,D,D,D,D,D,D,D,D,D,D,D,D,D,D,D,D,D,D,D,D,D,D,D', /SILENT

		;
		n = N_ELEMENTS(x)
		base_nstars[ifile] = n					; number of stars in this list
		;
		for i = 0, n-1 do begin
			base[ifile,i].id = id[i]
			base[ifile,i].x = x[i]
			base[ifile,i].y = y[i]
			base[ifile,i].ra = ra[i]
			base[ifile,i].dec = dec[i]
;			base[ifile,i].mag = mag[i]
;			base[ifile,i].errmag = errmag[i]
			base[ifile,i].sky = sky[i]
			;
			;
			;	select aperture data for best aperture for this group
			;
			case best_ap[ifile] of
				0: begin
					base[ifile,i].mag = mag0[i]
					base[ifile,i].errmag = smag0[i]
				   end
				1: begin
					base[ifile,i].mag = mag1[i]
					base[ifile,i].errmag = smag1[i]
				   end
				2: begin
					base[ifile,i].mag = mag2[i]
					base[ifile,i].errmag = smag2[i]
				   end
				3: begin
					base[ifile,i].mag = mag3[i]
					base[ifile,i].errmag = smag3[i]
				   end
				4: begin
					base[ifile,i].mag = mag4[i]
					base[ifile,i].errmag = smag4[i]
				   end
				5: begin
					base[ifile,i].mag = mag5[i]
					base[ifile,i].errmag = smag5[i]
				   end
				6: begin
					base[ifile,i].mag = mag6[i]
					base[ifile,i].errmag = smag6[i]
				   end
				7: begin
					base[ifile,i].mag = mag7[i]
					base[ifile,i].errmag = smag7[i]
				   end
				8: begin
					base[ifile,i].mag = mag8[i]
					base[ifile,i].errmag = smag8[i]
				   end
				9: begin
					base[ifile,i].mag = mag9[i]
					base[ifile,i].errmag = smag9[i]
				   end
				10: begin
					base[ifile,i].mag = mag10[i]
					base[ifile,i].errmag = smag10[i]
				   end
				11: begin
					base[ifile,i].mag = mag11[i]
					base[ifile,i].errmag = smag11[i]
				   end
				else: print_text, Event, "ERROR - aperture not recognized"
			endcase
		endfor
		;
		;FREE_LUN, lu_in
		print_text, Event, "Read "+STRING(n,format='(I)')+" stars from file "+FILE_BASENAME(file_in[ifile])
		if(n gt max_stars) then begin
			max_stars = n
			file_max_stars = ifile
		endif
	endfor
	;
;	stop
	;
	;	if nlist more than 1, open file for saving results
	;
	if(nlists gt 0) then begin		; 20130606 changed from 1 to allow single large meta-groups for phot
		;
		print_text, Event, "Opening output file " + output_summary_path
		;
		openw, lu_out, output_summary_path, /GET_LUN
	endif
	;
	;	match each file to the reference one, where the reference one is the HWP zero one with the least Julian date
	;
	xoff = fltarr(nlists)
	yoff = fltarr(nlists)
	;
	;	sort by JD
	;
	jd_ind = SORT(jd)
	;
	;	and replace in place
	;
	base = base[jd_ind,*]
	base_nstars = base_nstars[jd_ind]
	hwp_angle = hwp_angle[jd_ind]
	jd = jd[jd_ind]
	file_in = file_in[jd_ind]
	group_ids = group_ids[jd_ind]
	group_names = group_names[jd_ind]
	;
	base_header = base_header[jd_ind,*]
	base_n_header = base_n_header[jd_ind]
	;
	;	test that first has zero HWP angle
	;
	if(hwp_angle[0] ne 0.0) then begin
		str = FILE_BASENAME(file_in[0])+" does not have HWP = 0 deg. Matching halted"
		print_text, Event, "ERROR - first image "+str
		SAll_add_to_history, Event, h_wid, 6, str
		status=-1
		goto, MM_end
	endif
	;
	list_ref = load_star_list_for_meta(0, base, base_nstars)
	;
	file_ref_name = FILE_BASENAME(file_in[0])
	;
	;	fetch astrometry information
	;
	ref_header = REFORM(base_header[0,0:base_n_header[0]-1])
	;
	;help, ref_header
	;
	extast, ref_header, ref_astr_struct, noparams
	;
	;	check by computing the x,y reference pixel values
	;
	ra_cen = sxpar(ref_header, "CRVAL1")
	dec_cen = sxpar(ref_header, "CRVAL2")
	;
;	print, "Reference RA, DEC of image Center = ",ra_cen, dec_cen
	;
	ad2xy, ra_cen, dec_cen, ref_astr_struct, x_ref, y_ref
	;
	x_cen = sxpar(ref_header, "CRPIX1")
	y_cen = sxpar(ref_header, "CRPIX2")
	;
	CD1_1 = sxpar(ref_header, "CD1_1")
	CD2_2 = sxpar(ref_header, "CD2_2")
	platescale = 3600.0 * (abs(CD1_1) + abs(CD2_2)) / 2.
	;
	;	identify the mean, best aperature
	;
	PP6_APAV = ROUND((TOTAL(best_ap)/FLOAT(N_ELEMENTS(best_ap)))+1.05)
	;
	PP4_NAP = SXPAR(header, "PP4_NAP")
	PP4_ARAD = SXPAR(header, "PP4_ARAD")
	PP4_REF0 = SXPAR(header, "PP4_REF0")
	PP4_REF1 = SXPAR(header, "PP4_REF1")
	PP4_CLEN = SXPAR(header, "PP4_CLEN")
	;
	save_camera = SXPAR(header, "MB_CAMRA")
	save_filter = SXPAR(header, "MB_FILTR")
	;
;	print, "Ref Platescale = ",platescale, " arcsec per pix"
	;
;	print, "FITS header X,Y center = ",x_cen, y_cen
;	print, "computed    X,Y center = ",x_ref, y_ref
	;
	;	and produce a magnitude-ordered index
	;
	ref_mag_index = SORT(REFORM(list_ref[2,0:base_nstars[0]-1]))
	dtor = 180.0d0/!DPI
	;
	nbad = 0
	for ifile = 0, nlists-1 do begin
		igroup = group_ids[ifile]
		group_name = group_names[ifile]
		;
		file_comp_name = FILE_BASENAME(file_in[ifile])
		;
		print_text, Event, "Matching file "+file_comp_name+" to reference file "+file_ref_name
		;
		;	set LED1 to blue at start
		;
		this_LED = ROUND(float(ifile) / LED_scale) & this_LED = 0 > ((nLED-1) < this_LED)
		image_colors[this_LED] = "blue"
		WCT_LED_stack, LED1_wid, image_colors
		;--------------------------------------------
		;
		;	get the comp header and compute pixel offsets
		;
		this_header = REFORM(base_header[ifile,0:base_n_header[ifile]-1])
		this_ra_cen = sxpar(this_header, "CRVAL1")
		this_dec_cen = sxpar(this_header, "CRVAL2")
		;
;		print, "This RA, Dec image center = ",this_ra_cen, this_dec_cen
		;
		;	center to center offsets
		;
		ad2xy, this_ra_cen, this_dec_cen, ref_astr_struct, this_x_cen, this_y_cen
		;
;		print, "for ifile = ",ifile
;		print, "this_x_cen, y_cen = ",this_x_cen, this_y_cen," on ref image"
		this_x_offset = this_x_cen - x_ref
		this_y_offset = this_y_cen - y_ref
;		print, "offset to ADD to comp image to match REF image positions (Xoff,Yoff) = ",this_x_offset, this_y_offset
		;
		;	for each of 6 passes, used to refine the offsets
		;
		;	initial guesses (from textboxes)
		;
;		if((*w_ptr).S6_textbox_values[0] gt 0.0) then begin
;			ra_width_start = (*w_ptr).S6_textbox_values[0] / 3600.0d0
;		endif else begin
;			ra_width_start = 60.0 / 3600.0d0
;			(*w_ptr).S6_textbox_values[0] = 60.0							; default value
;		endelse
;		;
;		if((*w_ptr).S6_textbox_values[1] gt 0.0) then begin
;			dec_width_start = (*w_ptr).S6_textbox_values[1] / 3600.0d0
;		endif else begin
;			dec_width_start = 60.0 / 3600.0d0
;			(*w_ptr).S6_textbox_values[1] = 60.0							; default value
;		endelse
;		;
;		if((*w_ptr).S6_textbox_values[2] gt 0.0) then begin
;			mag_width_start = (*w_ptr).S6_textbox_values[2]
;		endif else begin
;			mag_width_start = 0.2
;			(*w_ptr).S6_textbox_values[2] = 0.2							; default value
;		endelse
		;
		;	initial guesses
		;
		angle_window = 60.0		; match items within 1 arcmin of each other (initial window)
		win_mag = 4.0			; and 4 mag of each other
		win_deg = angle_window / 3600.0d0
		n_mag = 0L
		;
		ra_zero = 0.0d0			; these will be updated with each pass
		dec_zero = 0.0d0
		mag_zero = 0.0d0
		;
		ra_width = 60.0 / 3600.0d0
		dec_width = 60.0 / 3600.0d0
		mag_width = 1.0
		;
		WIDGET_CONTROL, nbad_wid, SET_VALUE=STRTRIM(STRING(nbad,format='(I2)'))
		for ipass = 0, 5 do begin
			;
			;	led blink between white and blue on alternate passes
			;
			if(ipass eq 2*(FIX(ipass/2))) then color = 'white' else color = 'blue'
			image_colors[this_LED] = color
			WCT_LED_stack, LED1_wid, image_colors
			;
			nmatch = 0L
			ra_off = dblarr(base_nstars[0])
			dec_off = dblarr(base_nstars[0])
			mag_off = dblarr(base_nstars[0])
			s_mag_off = dblarr(base_nstars[0])
			;
			list_pointers= lonarr(3,base_nstars[0])	; points to match in referece, target lists
													; 3rd position is 1/0 for match/no-match
			;
			;	match to first list
			;
			;	set up two star lists and match
			;
			this_list = load_star_list_for_meta(ifile, base, base_nstars)
			;
			;	start with magnitude ordered first list
			;
			for iref = 0, base_nstars[0]-1 do begin
				;
				ref_ra = list_ref[0,ref_mag_index[iref]]
				ref_dec = list_ref[1,ref_mag_index[iref]]
				ref_mag = list_ref[2,ref_mag_index[iref]]
				ref_s_mag = list_ref[3,ref_mag_index[iref]]
				ref_x = list_ref[4,ref_mag_index[iref]]
				ref_y = list_ref[5,ref_mag_index[iref]]
				;
				list_pointers[0,iref] = ref_mag_index[iref]	; save pointer into reference list
				;
				cosdec = cos(ref_dec / dtor)
				;
				;	compare to each star on second list
				;
				ddist_match = 10.0								; smallest match
				ddist_index = -1L
				ddist_ra_off = 0.0d0
				ddist_dec_off = 0.0d0
				ddist_mag_off = 0.0d0
				;
				for icomp = 0, base_nstars[ifile]-1 do begin
					;
					comp_ra = this_list[0,icomp]
					comp_dec = this_list[1,icomp]
					comp_mag = this_list[2,icomp]
					comp_s_mag = this_list[3,icomp]
					;
					ddec = comp_dec - ref_dec - dec_zero			; in decimal degrees on sky
					dra = (comp_ra - ref_ra) * cosdec - ra_zero		; in decimal degrees on sky
					;
					ddist = sqrt(ddec * ddec + dra * dra)
					dmag = (comp_mag - ref_mag) - mag_zero
					;
					if(abs(ddec) le dec_width and abs(dra) le ra_width and abs(dmag) le mag_width) then begin
						;
						;	found a match for this star - is it closer than any other previous match?
						;
						if(ddist le ddist_match) then begin
							ddist_match = ddist
							ddist_index = icomp
							ddist_ra_off = dra + ra_zero
							ddist_dec_off = ddec + dec_zero
							ddist_mag_off = dmag + mag_zero
							ddist_s_mag_off = sqrt(ref_s_mag ^2 + comp_s_mag^2) > dmag_clamp  ; trap at no smaller than 0.08 per star
						endif
						;
					endif
				endfor
				;
				;	was there a match?
				;
				if(ddist_index ne -1L) then begin
					;
					;	yes -- save the match
					;
					ra_off[nmatch] = ddist_ra_off
					dec_off[nmatch] = ddist_dec_off
					mag_off[nmatch] = ddist_mag_off
					s_mag_off[nmatch] = ddist_s_mag_off
					;
					;	and update pointer
					;
					list_pointers[1,iref] = ddist_index
					list_pointers[2,iref] = 1
					;
					nmatch++
				endif
			endfor
			;
			;	end of this pass, recompute the offsets
			;
			if(nmatch eq 0) then begin
				;
				;	shouldn't get here - dump debug if so
				;
				print_text, Event, 'ERROR - no matches - should not happen - astrometry error???'
				;
				;	go on to next image
				;
				mag_mean = -99.9
				s_mag_mean = -99.9
				ra_mean = -99.9
				dec_mean = -99.9
				x_off = -99.9
				y_off = -99.9
				;
				printf, lu_out, ifile, igroup, group_name, file_in[ifile], jd[ifile], HWP_angle[ifile], nmatch, $
					mag_mean, s_mag_mean, ra_mean, dec_mean, $
		 			format='(i3,1x,i4,1x,a,1x,a,1x,f13.5,1x,f6.1,1x,i4,1x,f12.5,1x,f12.5,1x,f12.5,1x,f12.5,1x,f7.2,1x,f7.2)'
		 		;
		 		image_colors[this_LED] = "orange"
		 		SAll_add_to_history, Event, h_wid, 6, group_name + " had no matching stars for image " + file_comp_name
			;
				nbad++
				WIDGET_CONTROL, nbad_wid, SET_VALUE=STRTRIM(STRING(nbad,format='(I2)'))
				;
		 		goto, file_end
		 	endif
		 	;
			ra_off = ra_off[0:nmatch-1]
			dec_off = dec_off[0:nmatch-1]
			mag_off = mag_off[0:nmatch-1]
			s_mag_off = s_mag_off[0:nmatch-1]
			;
			zz_ra = median_filtered_mean(ra_off)
			zz_dec = median_filtered_mean(dec_off)
			zz_mag = median_filtered_mean(mag_off)
			;
			ra_zero = zz_ra[0]
			dec_zero = zz_dec[0]
			mag_zero = zz_mag[0]
			;
			;	for high precision work, compute mean mag off and uncertainly using propagated uncer.
			;
			mag_mean = 0.0d0
			mag_wt = 0.0d0
			for k_match = 0, nmatch-1 do begin
				wt = 1./ (s_mag_off[k_match]^2)
				mag_mean = mag_mean + mag_off[k_match] * wt
				mag_wt = mag_wt + wt
			endfor
			;
			mag_mean = mag_mean / mag_wt
			s_mag_mean = 1./sqrt(mag_wt)
			;
			;	widths
			;
			ra_width = ((2.5+0.10*ipass)*zz_ra[1]) > (0.3d0 / 3600.0d0)		; 2.5-4.5 sigma
			dec_width = ((2.5+0.10*ipass)*zz_dec[1]) > (0.3d0 / 3600.0d0)
			mag_width = ((2.5+0.10*ipass)*zz_mag[1]) > 0.3
			;
		endfor
		;
		;	end of this file loop, report matches
		;
		print_text, Event, "For "+STRING(base_nstars[0],format='(I)')+" ref stars and "+$
			STRING(base_nstars[ifile],format='(I)')+" comp stars, "+STRING(nmatch,format='(I)')+" matched"
		;
		;	and plot distribution functions into windows
		;
		ra_off = (ra_off[0:nmatch-1])*3600.0d0
		dec_off = (dec_off[0:nmatch-1])*3600.0d0
		mag_off = mag_off[0:nmatch-1]
		s_mag_off = s_mag_off[0:nmatch-1]
		;
		zz_ra = median_filtered_mean(ra_off)
		zz_dec = median_filtered_mean(dec_off)
		zz_mag = median_filtered_mean(mag_off)
		;
		ra_mean = zz_ra[0]
		dec_mean = zz_dec[0]
		;
		mag_mean = 0.0d0
		mag_wt = 0.0d0
		for k_match = 0, nmatch-1 do begin
			wt = 1./ (s_mag_off[k_match]^2)
			mag_mean = mag_mean + mag_off[k_match] * wt
			mag_wt = mag_wt + wt
		endfor
		;
		mag_mean = mag_mean / mag_wt
		s_mag_mean = 1./sqrt(mag_wt)
		;
		print_text, Event, "Mean RA offset = "+STRING(zz_ra[0])+" arcsec,  dispersion = "+STRING(zz_ra[1])+" arcsec"
		print_text, Event, "Mean Dec offset = "+STRING(zz_dec[0])+" arcsec,  dispersion = "+STRING(zz_dec[1])+" arcsec"
		print_text, Event, "Weighted Mean Mag offset = "+STRING(mag_mean)+" mag,  w/uncert. = "+STRING(s_mag_mean)+" mags"
		 ;
		if(ifile ge 1) then begin						; don't plot first matched to first
			; windows are on the tab page of the gui
			;
			zoom = 5.0
			;
			WIDGET_CONTROL, (*w_ptr).S6C_RA_plot_wid, GET_VALUE=win_no
			wset, win_no
			erase
			scale = (CEIL(zoom*ra_width*3600.0)/zoom) > (CEIL(zoom*dec_width*3600.0)/zoom)
			center = ROUND(zoom*zz_ra[0])/zoom
			xradec = dindgen(51)*(scale/25.0d0) - double(scale) + double(center)
			plot, xradec,  dpc_histogram(ra_off, NBINS=51, MIN = xradec[0], MAX = xradec[50]), $
			 	xtitle='RA offset [arcsec]',ytitle='Number of stars',psym=10
			;
			WIDGET_CONTROL, (*w_ptr).S6C_Dec_plot_wid, GET_VALUE=win_no
			wset, win_no
			erase
			center = ROUND(zoom*zz_mag[0])/zoom
			xradec = dindgen(51)*(scale/25.0d0) - double(scale) + double(center)
			plot, xradec,  dpc_histogram(dec_off, NBINS=51, MIN = xradec[0], MAX = xradec[50]), $
		 		xtitle='Dec offset [arcsec]',ytitle='Number of stars',psym=10
		 	;
		 	WIDGET_CONTROL, (*w_ptr).S6C_Mag_plot_wid, GET_VALUE=win_no
			wset, win_no
			erase
			center = ROUND(zoom*zz_mag[0])/zoom
			m_scale = CEIL(zoom*mag_width)/zoom
			xmag = dindgen(51)*(m_scale/25.0d0) - double(m_scale) + double(center)
			plot, xmag,  dpc_histogram(mag_off, NBINS=51, MIN = xmag[0], MAX = xmag[50]), $
			 	xtitle='Mag offset [mag]',ytitle='Number of stars',psym=10
		 endif
		  ;
		 ;	compute pixel offsets from RA, Dec centers and match offsets, using saved platescale
		 ;
		 dx = -1.0 * ra_mean / platescale
		 dy = -1.0* dec_mean / platescale
		 total_x_offset = this_x_offset + dx
		 total_y_offset = this_y_offset + dy
		 ;
;		 print, "RA, Dec offsets (in arcsec) [This-Ref] = " ,ra_mean, dec_mean
;		 print, "X, Y offsets (in pixels)               = ", dx, dy
;		 print, "Center offsets (X, Y) in pixels        = ", this_x_offset, this_y_offset
;		 print, "Total image offsets to add to ref (X,Y)= ", total_x_offset, total_y_offset
		 ;
	 	 ;if(ifile eq 1) then stop
	 	 ;
	 	 ;	write to output file
	 	 ;
		  if(ifile eq 0) then begin
			nch_file = STRLEN(FILE_BASENAME(file_in[ifile]))
			nbuf2 = (nch_file - 8) / 2
			chbuf = ''
			if(nbuf2 gt 0) then begin
				for ib = 0, nbuf2-1 do begin
					chbuf = chbuf + ' '
				endfor
			endif
			;
			;	add header info regarding multiple apertures....
			;
			printf, lu_out, '; Meta-group Phot matching file created - ',systime()
			printf, lu_out, ';  aperture info follows:'
			printf, lu_out, ';HISTORY   PPOL-Step 6C-version '+(*s_ptr).version_short_string
			;
			printf, lu_out, ';PP4_ARAD=         '+STRING(PP4_ARAD,format='(F8.3)')+'/ Maximum Aper Size, in FWHM units'
			printf, lu_out, ';PP4_REF0=         '+STRING(PP4_REF0,format='(F8.3)')+'/ Inner ref radius, in FHWM * ARAD units'
			printf, lu_out, ';PP4_REF1=         '+STRING(PP4_REF1,format='(F8.3)')+'/ Outer ref radius, in FHWM * ARAD units'
			printf, lu_out, ';PP4_CLEN=         '+STRING(PP4_CLEN,format='(F8.3)')+'/ Outer cleaning radius, in FHWM * ARAD units'
			printf, lu_out, ';PP4_NAP =               '+STRING(PP4_NAP,format='(I2.2)')+'/ Number of apertures'
			;
			printf, lu_out, ';PP6_APAV=               '+STRING(PP6_APAV,format='(I2)')+'/ Average Meta-Group Aperture Number
			printf, lu_out, '; '
			printf, lu_out, ';MB_CAMRA=         '+save_camera+'/ Mimir Camera Optics Used'
			printf, lu_out, ';MB_FILTR=         '+save_filter+'/ Mimir Filter Option Used'
			printf, lu_out, ';'
			printf, lu_out, ';No.  Grp  Group Name     Reference File                JD          HWP Ap  Nstrs      dMag        s_dMag         dRA          dDec     X_off   Y_off'
	 	 endif
	 	 ;
		 ;	save match info to vectors
		 ;
		 T_match_vec[ifile] = float(nmatch)
		 T_rms_vec[ifile] = s_mag_mean
		 ;
		 ;
	 	 printf, lu_out, ifile, igroup, replace_underscores(group_name), FILE_BASENAME(file_in[ifile]), jd[ifile], HWP_angle[ifile], $
	 	 				best_ap[ifile], nmatch, mag_mean, s_mag_mean, ra_mean, dec_mean, total_x_offset, total_y_offset, $
		 			format='(i3,1x,i4,3x,a,1x,a,1x,f13.5,1x,f6.1,1x,i2,3x,i4,1x,f12.6,1x,f12.6,1x,f12.5,1x,f12.5,1x,f7.2,1x,f7.2)'
	 	 ;
		 ;	led to green
		 ;
		 image_colors[this_LED] = "green"
		 ;
		 file_end:
		 WCT_LED_stack, LED1_wid, image_colors
	endfor
	;
	if(nlists gt 1) then FREE_LUN, lu_out
	status=0									; worked fine
	;
	;	populate textboxes
	;
	ind_good = WHERE(T_rms_vec ne 0, ngood)
	if(ngood gt 0) then begin
		zz = median_filtered_mean(T_match_vec[ind_good])
		yy = median_filtered_mean(T_rms_vec[ind_good])
		yy = yy * 1000.0								; report in mmag
		;
		WIDGET_CONTROL, match_wid, SET_VALUE=STRING(zz[0],zz[1],format='(2x,F7.1,"+/-",F5.1)')
		WIDGET_CONTROL, rms_wid, SET_VALUE=STRING(yy[0],yy[1],format='(2x,F7.2,"+/-",F5.2)')
	endif
	;
	MM_end:
	;
end
