function S7_new_coadd, coadd_struct
;
;	performs coaddition on images (the new way)
;	using vectorization
;
if(coadd_struct.double_precision_flag) then begin
	output_image = dblarr(coadd_struct.new_naxis1, coadd_new_struct.new_naxis2)
	sky_image = coadd_struct.sky_image
	scale = coadd_struct.scale
	scaled_mean_sky = coadd_struct.scaled_mean_sky
	fn_mean_images = coadd_struct.fn_mean_images
endif else begin
	output_image = fltarr(coadd_struct.new_naxis1, coadd_struct.new_naxis2)
	sky_image = FLOAT(coadd_struct.sky_image)
	scale = FLOAT(coadd_struct.scale)
	scaled_mean_sky = FLOAT(coadd_struct.scaled_mean_sky)
	fn_mean_images = FLOAT(coadd_struct.fn_mean_images)
endelse
;
n_in_pix = intarr(coadd_struct.new_naxis1, coadd_struct.new_naxis2)
plane = fltarr(coadd_struct.new_naxis1, coadd_struct.new_naxis2)
;
;	for all input images, compute mask images	(3D)
;
good_mask = coadd_struct.raw_images + 0.5e6
good_mask >= 0.
good_mask <= 1.
;
bad_mask = 1.0 - good_mask
;
for ii = 0, coadd_struct.n_images-1 do begin
	;
	;	load scaled raw image plane into plane
	;
	plane *= 0.0
	;
	x0 = coadd_struct.x_off[ii]
	x1 = x0+coadd_struct.naxis11
	y0 = coadd_struct.y_off[ii]
	y1 = y0+coadd_struct.naxis21
	;
	plane[x0:x1,y0:y1] = (coadd_struct.raw_images[ii,*,*] - sky_image[ii]) * scale[ii] * good_mask[ii,*,*]
	;
	;	accumulate
	;
	output_image += plane
	;
	;	count
	;
	n_in_pix[x0:x1,y0:y1] += good_mask[ii,*,*]
	;
endfor
;
;	make output masks
;
good_output_mask = n_in_pix
good_output_mask <= 1
bad_output_mask = 1 - good_output_mask
;
;	add bad to count, then normalize, then mask
;
n_in_pix += bad_output_mask		; this fills the zeros with ones so division doesn't fail
output_image -= bad_output_mask
;
output_image /= n_in_pix
output_image *= fn_mean_images
;
;	now, mask bad
;
output_image += (fn_mean_images - 1.0e6) * bad_output_mask + (scaled_mean_sky * fn_mean_images) * good_output_mask
;
return, output_image
;
end

