pro S7_new_meta_group_coadding, Event, g_ind, group_name, LED_wid, status
	;
	;	handles coadding of meta_groups, using sub-image parsing if needed
	;
	;	"new" version (20100528) does a better job of fetching the needed data for each block
	;			of rows in the output image
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	s_ptr = PTR_struct.PTR_summary
	w_ptr = PTR_struct.PTR_wid_id
	g_ptr = PTR_struct.PTR_group
	;
	; for debugging of astrometry, set jump_to_astrometry to 1
	;
	jump_to_astrometry = 0
	if(jump_to_astrometry eq 1) then begin
		output_path = dialog_pickfile(Title='Select Deep Coadded Image')
		goto, do_astrometry
	endif
	;
	o_s_f = (*w_ptr).S7_override_scaling_mode 	; = 0 normal, = 1 override scaling
	;
	status = -1
	bad_pixel_value = -1.0e6
	;
	max_hwp = 15
	;
	row513_flag = (*w_ptr).S7_mask_row513_mode					; kill row 513 if set
	flatten_pre_flag = (*w_ptr).S7_flatten_images_pre_flag
	flatten_post_flag = (*w_ptr).S7_flatten_images_post_flag
	fix_bad_pixels_flag = (*w_ptr).S7_fix_bad_pixels_flag
	;
	;	background removal modes
	;
	if( (*w_ptr).S7_options_mode_and_tab eq 0) then begin
		print_text, Event, 'Starting Coadding with Star-Field type background removal'
		;
		;	Star Field type backgrounds
		;
		if( (*w_ptr).S7_stars_mode eq 0) then begin
			print_text, Event, '   and using SuperSky method'
			low_order_flag = 0
			supersky_flag = 1
			use_step11_points_flag = 0
			apply_illumination_flag = 0
			save_supersky_flag = (*w_ptr).S7_save_supersky_mode
			smooth_supersky_flag = (*w_ptr).S7_smooth_supersky_mode
			supersky_boxcar = (*w_ptr).S7_box_width_value
		endif else begin
			print_text, Event, '    and using Modeling of sky method'
			low_order_flag = 0
			supersky_flag = 0
			apply_illumination_flag = 1	; if set, compute illumination function and scale by it
			save_supersky_flag = 0
			smooth_supersky_flag = 0
			use_step11_points_flag = 0
			supersky_boxcar = 0
		endelse
	endif else begin
		print_text, Event, 'Starting Coadding with Extended Object(s) type background removal'
		;
		;	Extended Objects type backgrounds
		;
		if( (*w_ptr).S7_extended_mode eq 0) then begin
			print_text, Event, '     and using Constant sky method'
			low_order_flag = 1
			use_step11_points_flag = 0
			fit_option_mode = -1
			supersky_flag = 0
			apply_illumination_flag = 0
		endif else begin
			print_text, Event, '     and using S11 grid points fitting'
			low_order_flag = 1
			use_step11_points_flag = 1
			supersky_flag = 0
			apply_illumination_flag = 0
			fit_option_mode = (*w_ptr).S7_fit_options_mode	; 0 = 4Q together; 1 = 4Q independent; 2 = T/B indep.
			case fit_option_mode of
				0: print_text, Event,'     with 4 Quads fit together'
				1: print_text, Event,'     with 4 Quads fit independently'
				2: print_text, Event,'     with Top and Bottom pairs of Quads fit independently'
			   -1: print_text, Event,'ERROR - should not have gotten here. Quad-mode = -1'
			endcase
		endelse
		;
		;	either way, no supersky
		;
		supersky_flag = 0								; dpc 20111223
		supersky_boxcar = 0
		;
		;	or illumination correction
		;
		apply_illumination_flag = 0						; dpc 20111223
	endelse
	;
	star_sky_flag = (*w_ptr).S7_star_sky_flag			; 1 = distinguish stars from sky in mfm, 0 = same (for comets)
	;
	supersky_mode = 'subtract'										; subtract (sky) or divide (2nd order FF)
;	supersky_mode = 'divide'
	supersky_kernel = supersky_boxcar												; * FWHM = pixels for median filtering
	;
	double_precision_flag = 0
	print_text, Event,"Starting Coadding with DoublePrecisionFlag = "+STRING(double_precision_flag,format='(I1)')
	;
	h_wid = (*w_ptr).S7_history_textbox_wid
	;
	;	any comments to add?
	;
	comment_flag = (*w_ptr).S7_comments_mode
	if(comment_flag) then comment_str = (*w_ptr).S7_comments_string
	;
	astro_dir = (*s_ptr).analysis_dir + 'S3_Astrometry' + PATH_SEP()
	scale_dir = (*s_ptr).analysis_dir + 'S6_Scaling' + PATH_SEP()
	psf_dir = (*s_ptr).analysis_dir + 'S4_First_PSF_Fits' + PATH_SEP()
	s11_dir = (*s_ptr).analysis_dir + 'S11_Full_Field_Polarimetry' + PATH_SEP()
	;
	;	create name of summary scaling file for this meta-group
	;
	this_group = replace_underscores(group_name)
	;
	if( use_step11_points_flag eq 1) then begin
		s11_file_path = s11_dir + this_group + '_Deep_Coadd_bg_points.dat'
		s11_ptr = s7_read_points_file(Event, s11_file_path)
		if(NOT(PTR_VALID(s11_ptr))) then begin
			str = "Could not read S11 points file"
			print_text, Event, "ERROR - "+str+" - ending"
			SAll_add_to_history, Event, h_wid, 7, str
			status = -1
			goto, S7_MGC_end
		endif
	endif
	;
	scale_path = scale_dir + this_group + '_meta_scale.dat'
	;
	;	check that it exists
	;
	test = FILE_INFO(scale_path)
	if(NOT(test.exists)) then begin
		str = group_name+" has no scaling summary file"
		print_text, Event, "ERROR - group "+str+" - ending"
		SAll_add_to_history, Event, h_wid, 7, str
		status = -1
		goto, S7_MGC_end
	endif
	;
	;	first time accessing file, determine how many lines in it
	;
	n_lines = FILE_LINES(scale_path)
	print_text, Event, "Opened Meta-group scale file "+FILE_BASENAME(scale_path)
	print_text, Event, " and found "+STRING(n_lines,format='(I)')+" lines (files+comments)"
	;
	;	prepare variables
	;
	file_names = strarr(n_lines)
	hwp_angles = fltarr(n_lines)
	scale_factors = dblarr(n_lines)
	x_offsets = intarr(n_lines)
	y_offsets = intarr(n_lines)
	;
	group_name_vector = strarr(n_lines)			; the names of each group (won't be n_lines long)
	image_group_name = strarr(n_lines)			; the group name for this particular image
	image_group_number = intarr(n_lines)		; the group number for this particular image
	;
	n_files = 0L
	;
	openr, lu_scale, scale_path, /GET_LUN
	line = ''
	;
	header = strarr(64,200)
	nheader = lonarr(64)
	;
	ngroups= -1L
	;
	WHILE ~EOF(lu_scale) do begin
		readf, lu_scale, line
		;
		;	pull off any line starting with semicolon into header
		;
		s_test = STRPOS(line, ';')
		if(s_test ne -1) then begin
			;
			;	if the word "gain" is present, pull off the group name and increment the groups counter
			;
			if(STRPOS(line,'gain') ne -1) then begin
				ngroups++
				nheader[ngroups] = 0L
				;
				;	parse the line to get the group name
				;
				parts = STRSPLIT(line,' ',/REGEX,COUNT=nparts,/EXTRACT)
				group_name_vector[ngroups] = STRTRIM(parts[2])
				;
			endif
			;
			;	but skip lines with two semicolons or "; -"
			;
			two_semi = STRPOS(line, ';;')
			semi_dash = STRPOS(line, '; -')
			if(two_semi eq -1 and semi_dash eq -1) then begin
				if(ngroups ge 0) then begin
					header[ngroups,nheader[ngroups]] = STRMID(line, s_test + 1)
					nheader[ngroups]++
				;
				endif
			endif
			;
		endif else begin
			;
			;	test for "phot" in line - skip if missing
			;
			test = STRPOS(line,"phot")
			if(test ne -1) then begin
				;
				parts = STRSPLIT(line,' ',/extract)
				file_names[n_files] = psf_dir + parts[0]
				image_group_name[n_files] = group_name_vector[ngroups]
				image_group_number[n_files] = ngroups
				;
				reads, parts[1], hwp, format='(F)'
				hwp_angles[n_files] = hwp
				reads, parts[2], scale, format='(D)'
				scale_factors[n_files] = scale
				reads, parts[3], x_off, format='(F)'
				x_offsets[n_files] = ROUND(x_off)
				reads, parts[4], y_off, format='(F)'
				y_offsets[n_files] = ROUND(y_off)
				;
				;	cleck for reasonableness and limits
				;
				if(scale lt 0.3 or scale gt 3.0) then begin
					str = FILE_BASENAME(file_names[n_files])+" has scale = "+parts[2]
					print_text, Event, "Warning - Image "+str
					SAll_add_to_history, Event, h_wid, 7, str
				endif
				;
				n_files++
			endif
		endelse
	ENDWHILE
	FREE_LUN, lu_scale
	;
	ngroups++
	;
	group_name_vector = group_name_vector[0:ngroups-1]	; trim group name vector to number of groups
	;
	print_text, Event, "Read in "+STRING(ngroups,format='(I2)')+" groups"
	;
	pp4_arad = dblarr(ngroups)
	pp4_ref0 = dblarr(ngroups)
	pp4_ref1 = dblarr(ngroups)
	pp4_clen = dblarr(ngroups)
	pp6_aprd = dblarr(ngroups)
	;
	for ig = 0, ngroups-1 do begin
		this_header = REFORM(header[ig,0:nheader[ig]-1])
		;
		pp4_arad[ig] = SXPAR(this_header, "PP4_ARAD")
		pp4_ref0[ig] = SXPAR(this_header, "PP4_REF0")
		pp4_ref1[ig] = SXPAR(this_header, "PP4_REF1")
		pp4_clen[ig] = SXPAR(this_header, "PP4_CLEN")
		pp6_aprd[ig] = SXPAR(this_header, "PP6_APRD")
	endfor
	;
	;	capture camera type
	;
	this_camera_type = STRTRIM(SXPAR(this_header, "MB_CAMRA"),2)
	if(this_camera_type eq 'F5') then f17 = 0 else f17 = 1
	;
	;	compute average properties of each
	;
	ave_arad = TOTAL(pp4_arad) / float(ngroups)
	ave_ref0 = TOTAL(pp4_ref0) / float(ngroups)
	ave_ref1 = TOTAL(pp4_ref1) / float(ngroups)
	ave_clen = TOTAL(pp4_clen) / float(ngroups)
	ave_aprd = TOTAL(pp6_aprd) / float(ngroups)
	;
	print_text, Event, "Ave ARAD, REF0, REF1, CLEN, APRD = "+$
		STRING(ave_arad, ave_ref0, ave_ref1, ave_clen, ave_aprd,format='(5(f7.3,1x))')
	;
	file_names = file_names[0:n_files-1]
	hwp_angles = hwp_angles[0:n_files-1]
	scale_factors = scale_factors[0:n_files-1]
	;
	image_group_name = image_group_name[0:n_files-1]
	image_group_number = image_group_number[0:n_files-1]
	;
	;	evaluate against scaling override
	;
	ind_so = WHERE(scale_factors ne 0.0,n_scale)
	if(n_scale gt 0 and o_s_f eq 1) then begin
		scale_factors[ind_so] = 1.0
	endif else begin
		;
		;	otherwise, push all other lists down to match good scaling factors
		;
		n_to_skip = n_files - n_scale
		if(n_to_skip gt 0) then	print_text, Event,'Skipping '+STRING(n_to_skip)+' images with UseFlag = 0'
		x_offsets = x_offsets[ind_so]
		y_offsets = y_offsets[ind_so]
		file_names = file_names[ind_so]
		hwp_angles = hwp_angles[ind_so]
		scale_factors = scale_factors[ind_so]
		;
		image_group_name = image_group_name[ind_so]
		image_group_number = image_group_number[ind_so]
		;
		n_files = n_scale
	endelse
	;
	x_offsets = x_offsets[0:n_files-1]
	y_offsets = y_offsets[0:n_files-1]

	x_o_min = FLOOR(MIN(x_offsets))
	x_o_max = CEIL(MAX(x_offsets))
	y_o_min = FLOOR(MIN(y_offsets))
	y_o_max = CEIL(MAX(y_offsets))
	;
	d_o_x = x_o_max - x_o_min
	d_o_y = y_o_max - y_o_min
	;
	;	compute final image dimensions and astrometry information
	;
	ref_header = S7_read_phot_file_header(file_names[0])
	;
	crpix1 = sxpar(ref_header, "CRPIX1")
	crpix2 = sxpar(ref_header, "CRPIX2")
	naxis1 = sxpar(ref_header, "NAXIS1")
	naxis2 = sxpar(ref_header, "NAXIS2")
	crval1 = sxpar(ref_header, "CRVAL1")
	crval2 = sxpar(ref_header, "CRVAL2")
	;
	y_size = naxis2								; save basic image y size
	;
	new_naxis1 = naxis1 + d_o_x
	new_naxis2 = naxis2 + d_o_y
	;
	new_crpix1 = crpix1 + (ROUND(x_offsets[0]) - x_o_min)		; this is where the old crpix1,2 will go in the
	new_crpix2 = crpix2 + (ROUND(y_offsets[0]) - y_o_min)		;    new image, if we keep the same old CRVAL1,2
	;
	new_x_center = float(new_naxis1) / 2.			; assuming IDL-like pixel coordinates...
	new_y_center = float(new_naxis2) / 2.			;   ("WWDS9T" - what would DS9 think???)
	;
	;	update header for new crpix1,2 values
	;
	SXADDPAR, ref_header, 'CRPIX1', new_crpix1
	SXADDPAR, ref_header, 'CRPIX2', new_crpix2
	;
	extast, ref_header, ref_astr_struct, noparams
	;
	;	compute ra, dec of new center
	;
	xy2ad, new_x_center, new_y_center, ref_astr_struct, ra_cen, dec_cen
	;
;	print, "old ra, dec center = ",crval1, crval2
;	print, "old crpix1, 2 = ",crpix1, crpix1
;	print, "xmin, xmax, dx = ", xmin, xmax, dx
;	print, "ymin, ymax, dy = ", ymin, ymax, dy
;	print, "new image size = ",new_naxis1, new_naxis2
;	print, "new crpix1, 2 = ",new_crpix1, new_crpix2
;	print, "final crpix1, 2 = ",new_x_center, new_y_center
;	print, "final ra, dec center = ",ra_cen, dec_cen
	;
	;	save new center ra, dec and new ref pixels
	;
	SXADDPAR, ref_header, 'CRPIX1', new_x_center
	SXADDPAR, ref_header, 'CRPIX2', new_y_center
	SXADDPAR, ref_header, 'CRVAL1', ra_cen
	SXADDPAR, ref_header, 'CRVAL2', dec_cen
	SXADDPAR, ref_header, 'NAXIS1', new_naxis1
	SXADDPAR, ref_header, 'NAXIS2', new_naxis2
	;
	;	amount by which a star's position should be incremented in X and Y
	;	from the pixel values in this image to find the approximate mean
	;	detector X,Y pixel values when detected
	;
	x_pix_offset = x_o_min - (float(TOTAL(x_offsets))/float(n_files))
	y_pix_offset = y_o_min - (float(TOTAL(y_offsets))/float(n_files))
	;
	SXADDPAR, ref_header, 'MPP_XOFF', x_pix_offset, 'Add this to X pix to get detector X'
	SXADDPAR, ref_header, 'MPP_YOFF', y_pix_offset, 'Add this to Y pix to get detector Y'
	SXADDPAR, ref_header, 'HISTORY', 'Created in MSP_PPOL Step 7 (meta-group coadding) on '+SYSTIME()
	;
	;	if comment flag set, add comments
	;
	if(comment_flag) then begin
		SXADDPAR, ref_header, 'HISTORY', comment_str
	endif
	;
	;	add info from Step 6 to pass onto Step 8
	;
	SXADDPAR, ref_header, 'PP4_ARAD', ave_arad,'Maximum Aper Size, in FWHM units'
	SXADDPAR, ref_header, 'PP4_REF0', ave_ref0,'Inner ref radius, in FHWM * ARAD units'
	SXADDPAR, ref_header, 'PP4_REF1', ave_ref1,'Outer ref radius, in FHWM * ARAD units'
	SXADDPAR, ref_header, 'PP4_CLEN', ave_clen,'Outer cleaning radius, in FHWM * ARAD units'
	SXADDPAR, ref_header, 'PP6_APRD', ave_aprd,'Aperture size in FWHM * ARAD units'
	;
	;	and reset X, Y image offsets to be referenced to 0,0
	;
	x_offsets -=  x_o_min
	y_offsets -=  y_o_min
	;
	;	parse into the 16 lists
	;
	hwp_values = [0,22.5,45.0,67.5,90.0,112.5,135.0,157.5,180.0,202.5,225.0,247.5,270.0,$
					292.5,315.0,337.5]
	hwp_values = hwp_values[0:max_hwp]
	;
	hwp_file_list = lonarr(max_hwp+1,n_files)
	hwp_list_n = lonarr(max_hwp+1)
	;
	;	Reset LED to reflect up to 17 sublists
	;
	WIDGET_CONTROL, LED_wid, SET_UVALUE=max_hwp+2
	image_colors = strarr(max_hwp+2)
	image_colors[*] = "red"
	S7_LED_sensitize, Event, 1, 1
	;
	;	compare each HWP to the list, finding the one closest (in case steps are
	;			bumped in the LOIS code...)
	;
	for ifile = 0, n_files-1 do begin
		dhwp = abs(hwp_angles[ifile] - hwp_values[*])
		min_dhwp = MIN(dhwp)
		ind = WHERE(dhwp eq min_dhwp and dhwp lt 5.0,count)		; must find one within 5 deg
		if(count eq 1) then begin
			hwp_file_list[ind,hwp_list_n[ind]] = ifile		; as a reference number
			hwp_list_n[ind]++
		endif else begin
			if(count ne 0) then begin
				str = "Multiple HWP angle matches for "+file_names[ifile]
				print_text, Event, "ERROR - "+str
				SAll_add_to_history, Event, h_wid, 7, str
			endif else begin
				str = "No HWP angle match for "+file_names[ifile]
				print_text, Event, "ERROR - "+str
				SAll_add_to_history, Event, h_wid, 7, str
			endelse
		endelse
	endfor
	;
	fn_mean_images = DOUBLE(CEIL(MEDIAN(hwp_list_n)))
	;
	if(fn_mean_images eq 0) then begin
		fn_mean_images = hwp_list_n[0]
		max_hwp = 0
	endif
	;
	;	if this was zero, then we have photometric data, all in HWP=0 - reset
	;
	this_is_polarimetry_flag = 1
	;
	if(fn_mean_images eq 0.0d0) then begin
		print_text, Event, "Non-Polarimetric Coadding Requested. Setting mean image number to HWP=0 number"
		fn_mean_images = hwp_list_n[0]
		this_is_polarimetry_flag = 0
	endif
	;
	print_text,Event, "fn_mean_images = "+STRING(fn_mean_images,format='(F6.1)')
	;
	SXADDPAR, ref_header, 'MPP_NSCL', fn_mean_images, 'Number of images this one is scaled to'
	;
;	for ihwp = 0, 15 do begin
;		print, "Found ",hwp_list_n[ihwp]," files with HWP = ",hwp_values[ihwp]
;	endfor
	;
	print_text, Event, "All HPW angles parsed"
	;
	hwp_coadd_paths = strarr(max_hwp+1)
	;
	;------------------------------------------------------------------------------
	;
	;	if supersky flag set, compute supersky images for each group for each hwp angle
	;		and write them to disk for fetching later
	;
	if(supersky_flag eq 1) then begin
		;
		disk_mode = 1					; save results to disk
		memory_mode = 0					; not in memory
		;
		S7_create_supersky, Event, group_name_vector, hwp_list_n, hwp_file_list, $
						file_names, scale_factors, image_group_number, $
						LED_wid, image_colors, $
						double_precision_flag, disk_mode, memory_mode, supersky, max_hwp
		;
		;	release supersky pointer, as it is on disk now
		;
		PTR_FREE, supersky
		;
	endif
	;
;	stop
	;
	;----------------------------- coadd 16 HWP images --------------------------------
	;
	nchunks = 1
	dy_chunk = new_naxis2
	old_new_naxis2 = new_naxis2
	;
	hwp_nchunks = intarr(max_hwp+1)
	hwp_dy_chunk = intarr(max_hwp+1)
	;
	if(double_precision_flag eq 1) then nsplit = 5 else nsplit = 10
	;
	for ihwp = 0, max_hwp do begin
		;
		;	don't coadd any that aren't there
		;
		if(hwp_list_n[ihwp] ge 1) then begin
			ind_list = REFORM(hwp_file_list[ihwp,0:hwp_list_n[ihwp]-1])
			;
			n_images = N_ELEMENTS(ind_list)
			;
			;	test for whether we need to deal with memory management
			;
			if(n_images gt nsplit) then begin
				;
				;	must parse calculation into multiple chunks of the output image
				;
				nchunks = CEIL(float(n_images) / float(nsplit))
			endif else begin
				nchunks = 1
			endelse
			;
			hwp_nchunks[ihwp] = nchunks
		endif
	endfor
	;
	for ihwp = 0, max_hwp do begin
		if(hwp_list_n[ihwp] ge 1) then begin
			hwp_dy_chunk[ihwp] = CEIL(float(new_naxis2) / float(hwp_nchunks[ihwp]))
		endif else begin
			hwp_dy_chunk[ihwp] = 0
		endelse
	endfor
	;
	if(old_new_naxis2 ne new_naxis2) then SXADDPAR, ref_header, 'NAXIS2', new_naxis2
	;
;	stop
	;
	;	for every HWP case
	;
	for ihwp = 0, max_hwp do begin
;	for ihwp = 1, 1 do begin			; test case
		;
		nchunks = hwp_nchunks[ihwp]
		dy_chunk = hwp_dy_chunk[ihwp]
		;
		if(dy_chunk gt 0) then begin
			;
			;	set LED1 to blue
			;
			this_LED = ihwp
			image_colors[this_LED] = "blue"
			WCT_LED_stack, LED_wid, image_colors
			;
			ind_list = REFORM(hwp_file_list[ihwp,0:hwp_list_n[ihwp]-1])
			;
			x_off = x_offsets[ind_list]
			y_off = y_offsets[ind_list]
			scale = scale_factors[ind_list]
			this_image_group_name = image_group_name[ind_list]
			;
	;		print, "Scale Factors = ",scale
			;
			;	get the vector of names
			;
			image_names = FILE_BASENAME(file_names[ind_list])
			;
			;	and convert back to FITS image names
			;
			n_images = N_ELEMENTS(image_names)
			;
			print_text, Event, "Beginning Coadding (Meta) of "+STRING(n_images,format='(I3)')+" images for HWP position "+STRING(ihwp+1,format='(I2)')
			;
			for i_image = 0, n_images-1 do begin
				parts = STRSPLIT(image_names[i_image],"_phot.dat",/REGEX,/EXTRACT)
				image_names[i_image] = parts[0] + '.fits'
				;
				;	set up its FITS header
				;
				if(i_image eq 0) then begin
					first_image_path = astro_dir + image_names[0]
					this_header = ref_header							; DPC 20090702
				endif
				;
				SXADDPAR, this_header, 'HISTORY', 'includes '+image_names[i_image]
				SXADDPAR, this_header, 'HISTORY', '  with X,Y offsets '+STRING(x_off[i_image],y_off[i_image],$
											format='(I3,1x,I3)')+" pixels"
				SXADDPAR, this_header, 'HISTORY', '  and multiplied by scale factor '+STRING(scale[i_image],$
											format='(F13.8)')
				SXADDPAR, this_header, 'MPP_NADD', n_images, 'Number of images coadded into this one'
			endfor
			;
			image_path = astro_dir + image_names
			;
			print_text, Event, "Computing final image as "+STRING(nchunks,format='(I)')+" sections of Y-size "+$
								STRING(dy_chunk,format='(I)')
			;
			yborder = new_naxis2 - naxis2
			xborder = new_naxis1 - naxis1
			;
			if(double_precision_flag) then output_image = PTR_NEW(dblarr(new_naxis1, new_naxis2)) else $
										   output_image = PTR_NEW(fltarr(new_naxis1, new_naxis2))
			;
			naxis11 = naxis1-1
			naxis21 = naxis2-1
			;
			;	read through all the images to establish sky levels and relative scalings
			;
			n_images_read = 0L
			sky_image = dblarr(n_images)
			sky_sigma = dblarr(n_images)
			;
		    for i_image = 0, n_images-1 do begin
				;
				sky_image[i_image] = 0.0d0
				sky_sigma[i_image] = 0.0d0
				;
				if(flatten_pre_flag eq 0) then begin
					if(low_order_flag eq 0 and supersky_flag eq 0) then begin
						ptr_new_raw_chunk = PTR_NEW(readfits(image_path[i_image], header, /SILENT))
						;
						;	LLQ fix
						;
						(*ptr_new_raw_chunk) = llq_clear( (*ptr_new_raw_chunk), header)
						;
						(*ptr_new_raw_chunk) = fix_low_pixels( (*ptr_new_raw_chunk), /TPS)
						;
						(*ptr_new_raw_chunk) = fix_high_pixels((*ptr_new_raw_chunk))
	;						print_text, Event, "Read image "+FILE_BASENAME(image_path[i_image])
						sky_image[i_image] = 0.0		; ie, don't correct
						sky_sigma[i_image] = 1.0e6
						n_images_read++
					endif else begin
						ptr_raw_chunk = PTR_NEW(readfits(image_path[i_image], header, /SILENT))
						;
						;	LLQ fix
						;
						(*ptr_raw_chunk) = llq_clear( (*ptr_raw_chunk), header)
						;
						(*ptr_raw_chunk) = fix_low_pixels( (*ptr_raw_chunk), /TPS)
						;
						(*ptr_raw_chunk) = fix_high_pixels((*ptr_raw_chunk))
						ptr_new_raw_chunk = model_background(ptr_raw_chunk,SKY=this_sky, SIGMA=this_sigma, /LOW_ORDER)
	;						print_text, Event, "Image "+FILE_BASENAME(image_path[i_image])+" has sky = "+STRING(this_sky)
						sky_image[i_image] = this_sky
						sky_sigma[i_image] = this_sigma
						n_images_read++
						PTR_FREE, ptr_raw_chunk
					endelse
				endif else begin
					ptr_new_raw_chunk =PTR_NEW(readfits(image_path[i_image], header, /SILENT))
					;
					(*ptr_new_raw_chunk) = llq_clear( (*ptr_new_raw_chunk), header)
					;
					(*ptr_new_raw_chunk) = fix_low_pixels( (*ptr_new_raw_chunk), /TPS)
					;
					(*ptr_new_raw_chunk) = fix_high_pixels((*ptr_new_raw_chunk))
					;
					;  turn off column flattening
					;
					ptr_raw_chunk = model_background(ptr_new_raw_chunk,SKY=this_sky,SIGMA=this_sigma,/NOCOLF, /TPS)
	;					print_text, Event, "Image "+FILE_BASENAME(image_path[i_image])+" has sky = "+STRING(this_sky)
					sky_image[i_image] = this_sky
					sky_sigma[i_image] = this_sigma
					n_images_read++
					PTR_FREE, ptr_raw_chunk
				endelse
				;
				PTR_FREE, ptr_new_raw_chunk
			endfor									; need to flush heap?
			;
			;	compute mean sky value for ALL images with non-zero scale factors
			;
			ind_sky = WHERE(scale ne 0.0,ngoodsky)
			if(ngoodsky gt 0) then begin
	;				zz = median_filtered_mean(sky_image[ind_sky]*scale[ind_sky])
	;				scaled_mean_sky = zz[0]
				scaled_mean_sky = TOTAL(sky_image[ind_sky] * scale[ind_sky]) / float(ngoodsky)
				mean_sky_sigma = TOTAL(sky_sigma[ind_sky] * scale[ind_sky]) / float(ngoodsky)
			endif else scaled_mean_sky = 0.0
			;
			print_text, Event, "Read in "+STRING(n_images_read,format='(I3)')+" images to find mean sky, sigma = "+$
								STRING(scaled_mean_sky,mean_sky_sigma)
			;
	;		stop
			;
			;	for nchunks repeats through the data...
			;
			for ichunk = 0, nchunks-1 do begin
				;
				if(nchunks gt 1) then print_text, Event, "Meta Coadding - Image Section number "+STRING(ichunk+1,format='(I)') $
					else print_text, Event, "Meta Coadding - full image without sectioning"
				;
				;	starting, ending output Y values
				;
				ymin_out = dy_chunk * ichunk
				ymax_out = (new_naxis2-1) < (ymin_out + dy_chunk - 1)
				ny_chunk = ymax_out - ymin_out + 1
				;
				if(double_precision_flag) then raw_images = PTR_NEW(dblarr(n_images, naxis1, ny_chunk)) else $
										       raw_images = PTR_NEW(fltarr(n_images, naxis1, ny_chunk))
				(*raw_images) += -1.0e6
				;
				print_text, Event, "Chunk number, Ymin_out, Ymax_out, Ny_chunk = "+$
						STRING(ichunk, ymin_out, ymax_out, ny_chunk, format='(4(I5,2x))')
				;
				;	fetch the raw images, shift by their x-offsets and select the chunk, paying attention to y-offsets
				;
				n_images_read = 0
				;
				n_in_raw = 0L
				ifirst = 0
				this_x_off = fltarr(n_images)
				this_y_off = fltarr(n_images)
				this_scale = fltarr(n_images)
				this_sky_image = fltarr(n_images)
				;
				load_heap_flag = 0			; will be set after loading heap superskys
				;
				for i_image = 0, n_images-1 do begin
					if(i_image eq 2*fix(i_image/2)) then image_colors[this_LED] = "orange" $
													else image_colors[this_LED] = "blue"
					;
					WCT_LED_stack, LED_wid, image_colors
					;
					;	do we need this particular image? Examine offsets
					;
					ymin_in = (y_size-1) < (ymin_out - y_off[i_image])
					ymax_in = (y_size-1) < (ymin_in + (ny_chunk-1))
					y0 = 0 > ymin_in
					y1 = 0 > ymax_in
					if( (y1-y0) ne 0) then dy = (y1-y0)+1 else dy = 0
					if( ymin_in lt 0) then ymin_raw = -1*ymin_in else ymin_raw = 0
					if( ymax_in gt 0) then begin
						if( dy gt 0 ) then ymax_raw =  ymin_raw + dy - 1 else ymax_raw = 0
					endif else ymax_raw = 0
					if(dy eq 0) then begin
						ymin_raw = 0
						ymax_raw = 0
					endif
					dy2 = ymax_raw - ymin_raw + 1
					if(dy ne dy2 and dy ne 0) then stop
					;
	;				if(ifirst eq 0) then begin
	;					ifirst++
	;					print, 'ichunk, image, ymin_out, ymax_out, y_off{image}, ymin_in, ymax_in, y0, y1, dy, ymin_raw, ymax_raw'
	;				endif
	;				print, ichunk, i_image, ymin_out, ymax_out, y_off[i_image], ymin_in, ymax_in, y0, y1, dy, ymin_raw, ymax_raw
					;
					;	if no portion of this image maps to the indicated output chunk, don't
					;	bother reading it in
					;
					if(dy eq 0) then goto, skip_for_now
					;
					;	compute pathnames to Astrometry directory
					;
					;	flatten if flag set
					;
					sky_image[n_in_raw] = 0.0d0
					sky_sigma[n_in_raw] = 0.0d0
					;
					if(flatten_pre_flag eq 0 and supersky_flag eq 0) then begin
						if( use_step11_points_flag eq 1 ) then begin
							;
							input_image = readfits(image_path[i_image], header, /SILENT)
							;
							input_image = llq_clear(input_image, header)
							;
							input_image = fix_low_pixels(input_image, /TPS)
							;
							input_image = fix_high_pixels(input_image)
							;
							S7_fit_using_S11_points, Event, input_image, fit_option_mode, s11_ptr, this_sky, $
													this_sigma, x_off[i_image], y_off[i_image], $
													image_path[i_image]
							ptr_new_raw_chunk = PTR_NEW(input_image)
							sky_image[n_in_raw] = this_sky
							sky_sigma[n_in_raw] = this_sigma
						endif else begin
							if(low_order_flag eq 0) then begin
								ptr_new_raw_chunk = PTR_NEW(readfits(image_path[i_image], header, /SILENT))
								;
								(*ptr_new_raw_chunk) = llq_clear( (*ptr_new_raw_chunk), header)
								;
								(*ptr_new_raw_chunk) = fix_low_pixels( (*ptr_new_raw_chunk), /TPS)
								;
								(*ptr_new_raw_chunk) = fix_high_pixels((*ptr_new_raw_chunk))
		;						print_text, Event, "Read image "+FILE_BASENAME(image_path[i_image])
							endif else begin
								ptr_raw_chunk = PTR_NEW(readfits(image_path[i_image], header, /SILENT))
								;
								(*ptr_raw_chunk) = llq_clear( (*ptr_raw_chunk), header)
								;
								(*ptr_raw_chunk) = fix_low_pixels( (*ptr_raw_chunk), /TPS)
								;
								(*ptr_raw_chunk) = fix_high_pixels((*ptr_raw_chunk))
								ptr_new_raw_chunk = model_background(ptr_raw_chunk,SKY=this_sky, SIGMA=this_sigma, /LOW_ORDER)
		;						print_text, Event, "Image "+FILE_BASENAME(image_path[i_image])+" has sky = "+STRING(this_sky)
								sky_image[n_in_raw] = this_sky
								sky_sigma[n_in_raw] = this_sigma
								PTR_FREE, ptr_raw_chunk
							endelse
						endelse
					endif else begin
						ptr_raw_chunk = PTR_NEW(readfits(image_path[i_image], header, /SILENT))
						;
						(*ptr_raw_chunk) = llq_clear( (*ptr_raw_chunk), header)
						;
						(*ptr_raw_chunk) = fix_low_pixels( (*ptr_raw_chunk), /TPS)
						;
						(*ptr_raw_chunk) = fix_high_pixels((*ptr_raw_chunk))
						ptr_new_raw_chunk = model_background(ptr_raw_chunk,SKY=this_sky,SIGMA=this_sigma,/NOCOLF,/TPS)
	;					print_text, Event, "Image "+FILE_BASENAME(image_path[i_image])+" has sky = "+STRING(this_sky)
						sky_image[n_in_raw] = this_sky
						sky_sigma[n_in_raw] = this_sigma
						PTR_FREE, ptr_raw_chunk
					endelse
					;
					;	if supersky flag is set, go fetch the correct supersky for this image
					;
					if(supersky_flag eq 1) then begin
						;
						ss_dir = (*s_ptr).analysis_dir + 'S7_Coadding' + PATH_SEP() + 'Supersky_Images' + PATH_SEP()
						ss_path = ss_dir + 'Supersky_' + STRTRIM(this_image_group_name[i_image]) + '_HWP_' + STRING(ihwp,format='(I2.2)') + '.fits'
						;
						;	if this is the first image in the stack and the first chunk, open the heap storage of ss images
						;
	;					if(i_image eq 0 and ichunk eq 0) then begin
						if(load_heap_flag eq 0 and ichunk eq 0) then begin
							ss_n = 0L
	;						print_text, Event, "resetting the supersky heap tables'
						endif
						;
						;	then, if the table has non-zero length, look for a name match
						;
	;					print_text, Event, 'For image '+STRING(i_image)+' file_name ' + FILE_BASENAME(image_path[i_image])
						if(ss_n gt 0) then begin
							ind_ss = WHERE( (*ss_name_PTR) eq ss_path, n_ss_match)
							if(n_ss_match eq 1) then begin
	;							print_text, Event, '..found supersky image match in table at location ' + STRING(ind_ss)
								;
								;	match found, load it
								;
								this_supersky_image = (*ss_image_PTR[ind_ss[0]])
	;							print_text, Event, '.....loaded the supersky image'
							endif else begin
	;							print_text, Event, '..no matching supersky in table, reading from disk'
								;
								;	no match found, read the image, then
								;		add this image and name to the list
								;
								this_supersky_image = readfits(ss_path, /silent)
								this_supersky_image = fix_high_pixels(this_supersky_image)
								if(N_ELEMENTS(this_supersky_image) eq 1) then begin
									print_text, Event, "ERROR - supersky image not found"
									stop
								endif
								;
	;							print_text, Event, '.....and adding to heap table'
								ss_image_PTR[ss_n] = PTR_NEW(this_supersky_image)
								(*ss_name_PTR) = [(*ss_name_PTR),ss_path]
								ss_n++
	;							print_text, Event, '.....heap updated, now holds ' +String(ss_n)+' entries'
								;
							endelse
						endif else begin
	;						print_text, Event, '..first image, so reading supersky from disk and starting heap table'
							;
							;	create the heap name and image lists
							;
							ptr_this_supersky_image = PTR_NEW(readfits(ss_path, /silent))
							(*ptr_this_supersky_image) = fix_high_pixels((*ptr_this_supersky_image))
							if(N_ELEMENTS((*ptr_this_supersky_image)) eq 1) then begin
									print_text, Event, "ERROR - supersky image not found"
									stop
							endif
							;
	;						print_text, Event, '...read file '+FILE_BASENAME(ss_path)+' from disk'
							ss_name_PTR = PTR_NEW(ss_path)
	;						ss_image_PTR = PTR_NEW(this_supersky_image)
							ss_image_PTR = PTRARR(100)						; some large maximum
							ss_image_PTR[0] = ptr_this_supersky_image
							ss_n = 1
							load_heap_flag = 1
	;						print_text, Event, '.....heap updated, now holds ' +String(ss_n)+' entries'
						endelse
						;
						;	and do the correction (scaled subtraction)
						;
						mask = PTR_NEW( (((*ptr_new_raw_chunk) + 1.0e5) > 0.0) < 1.0)
						(*ptr_new_raw_chunk) += (1.0 -  (*ptr_this_supersky_image)) * float(sky_image[i_image])
						(*ptr_new_raw_chunk) *= (*mask)
						(*ptr_new_raw_chunk) += (1.0 - (*mask)) * (-1.0e6)		; preserve bad pixels
	;					print_text, Event, "Supersky correction performed for image "+FILE_BASENAME(image_path[i_image])
						;
						;	also model and flatten image using TPS model
						;
						ptr_raw_chunk = model_background(ptr_new_raw_chunk, /NOCOLF, /TPS)		; creates new pointer
						PTR_FREE, ptr_new_raw_chunk								; revert to old name
						ptr_new_raw_chunk = PTR_NEW((*ptr_raw_chunk))
						PTR_FREE, ptr_raw_chunk
	;					print_text, Event, "Backgound flattened"
						;
						;	if this was the last image of the last chunk, flush all the heap variables
						;
						if(i_image eq (n_images-1) and ichunk eq (nchunks-1)) then begin
							PTR_FREE, ss_name_PTR
							PTR_FREE, ss_image_PTR
	;						print_text, Event, "Last image, so flushed heap table"
						endif
					endif
					;
					input_size = SIZE((*ptr_new_raw_chunk))
					xis = input_size[1]
					yis = input_size[2]
					(*ptr_new_raw_chunk)[0:2,*] = -1.0e6
					(*ptr_new_raw_chunk)[*,0:6] = -1.0e6
					(*ptr_new_raw_chunk)[xis-3:xis-1,*] = -1.0e6
					(*ptr_new_raw_chunk)[*,yis-14:yis-1] = -1.0e6
					;
					if(row513_flag eq 1) then (*ptr_new_raw_chunk)[*,513] = -1.0e6
					;
					(*raw_images)[n_in_raw,*,*] = -1.0e6
					(*raw_images)[n_in_raw,*,ymin_raw:ymax_raw] = (*ptr_new_raw_chunk)[*,y0:y1]
					;
					this_x_off[n_in_raw] = x_off[i_image]
					this_y_off[n_in_raw] = 0.0					; these are rectified out above
					this_scale[n_in_raw] = scale[i_image]
					this_sky_image[n_in_raw] = sky_image[i_image]
					n_in_raw++
					;
					;	clear the heap for this image
					;
					PTR_FREE, ptr_new_raw_chunk
					;
					;	skip to here if not including the current image
					;
					skip_for_now:
				endfor
				;
				;	trim key arrays
				;
				this_x_off = this_x_off[0:n_in_raw-1]
				this_y_off = this_y_off[0:n_in_raw-1]
				this_scale = this_scale[0:n_in_raw-1]
				this_sky_image = this_sky_image[0:n_in_raw-1]
				;
	;			stop
				;
				print_text, Event, "Combining "+STRING(n_in_raw,format='(I3)')+" images in this chunk range"
				;
				;	at this point, the input images should be lined up with output image
				;
				;------------------ compute output image --------------------------
				;
				;	load structure for passing to coadder
				;
				;if(ihwp eq 0) then debug = 1 else debug = 0
				coadd_debug = 0
				;
				;	replace full Y-axis extent with chunk extent
				;
				naxis2_chunk = ny_chunk
				new_naxis2_chunk = ny_chunk
				;
	;			print_text,Event,"Naxis21, New_naxis2 = "+STRING(naxis2_chunk, new_naxis2_chunk)
				;
	;			stop
				;
				if(star_sky_flag eq 0) then mean_sky_sigma = 1.0e5
				;
				coadd_struct = { new_naxis1:		new_naxis1,		$	x-axis of output image
	;							 new_naxis2: 		new_naxis2, 	$	y-axis
								 new_naxis2:		new_naxis2_chunk, $
								 naxis11:			naxis1-1, 		$
	;							 naxis21:			naxis2-1, 		$
								 naxis21:			naxis2_chunk-1, $
								 n_images:			n_in_raw, 			$	number of constituent images
								 x_off:				this_x_off,				$
								 y_off:				this_y_off, 				$
								 raw_images:		(*raw_images),			$	the input data cube
								 sky_image:			this_sky_image,		$
								 scale:				this_scale,				$
								 double_precision_flag: double_precision_flag, $
								 fn_mean_images: 	fn_mean_images, $
								 mean_sky_sigma: 	mean_sky_sigma, $
								 debug: 			coadd_debug, 			$
								 scaled_mean_sky:	scaled_mean_sky }
				;
				this_output_chunk = S7_coadd(coadd_struct)
				;
				;	load into output image
				;
				(*output_image)[*,ymin_out:ymax_out] = (*this_output_chunk)[*,*]
				;
				PTR_FREE, this_output_chunk
				PTR_FREE, raw_images
				;
			endfor
			;
			;	create file name and write to disk
			;
			output_name = replace_underscores(group_name) + "_HWP_" + STRTRIM(STRING(ihwp,format='(I2.2)'),2) + '.fits'
			output_dir = (*s_ptr).analysis_dir + 'S7_Coadding' + PATH_SEP()
			output_path = output_dir + output_name
			;
			hwp_coadd_paths[ihwp] = output_path
			;
			SXADDPAR, this_header, 'FILENAME', output_name, 'Created by MSP_PPOL Step 7'
			SXADDPAR, this_header, 'PPOL_HWP', hwp_values[ihwp], 'HWP angle in degrees, from PPOL'
			SXADDPAR, this_header, 'MB_HWP', hwp_values[ihwp],'Updated HWP angle from PPOL S7'
			;
			;	get current JD and add one min for each HWP angle
			;
			date_obs = FXPAR(this_header, "DATE-OBS")
			reads, date_obs, year, month, day, hour, minute, second, format='(i4,1x,i2,1x,i2,1x,i2,1x,i2,1x,f4.2)
			jd = JULDAY(Month, Day, Year, 0, ihwp, 0)
			SXADDPAR, this_header, 'MB_JD', jd, "JD manufactured by PPOL S7"
			;
			;	if apply_illumination flag set, find model background and divide by scaled background shape
			;		(DPC 20091128)
			;
			if(apply_illumination_flag eq 1) then begin
				flat_image = PTR_NEW(model_background( (*output_image) ,SKY=sky))
				model_image = PTR_NEW((*flat_image) * 0.0)
				;
				out_good_flag = PTR_NEW( (*output_image) + 0.5e6 )
				(*out_good_flag) >= 0.0
				(*out_good_flag) <= 1.0
				out_bad_flag = PTR_NEW(1.0 - (*out_good_flag))
				;
				(*model_image) = (( (*output_image) - (*flat_image)) + sky) * (*out_good_flag) + (*out_bad_flag)
				;
				;	scale the model to its mean sky value and then divide (correct) output image for this
				;	illumination pattern
				;
				(*output_image) /=  ( (*model_image) / sky)
				(*output_image) += (1.0 - 1.0e6) * (*out_bad_flag)
				SXADDPAR, this_header, 'HISTORY', 'Corrected for illumination, with mean sky = '+STRING(sky)
				;
				PTR_FREE, flat_image, model_image, out_good_flag, out_bad_flag
			endif
			;
			;	if flatten_post flag set, flatten image
			;
			if(flatten_post_flag eq 1) then begin
				(*output_image) = model_background( (*output_image), SKY=sky, /MCS, STEP_SIZE=27)
				SXADDPAR, this_header, 'HISTORY', 'Post Coadded TPF Flattened, with mean sky = '+STRING(sky)
			endif
			;
			;	if flag set, fix bad pixels remaining in image
			;
			if(fix_bad_pixels_flag eq 1) then begin
				(*output_image) = fix_bad_pixels( (*output_image) )
				SXADDPAR, this_header, 'HISTORY', 'Post coadding, fixed remaining bad pixels'
			endif
			;
			(*output_image) = fix_low_pixels( (*output_image), /TPS)
			;
			writefits, output_path, (*output_image), this_header
			;
			print_text, Event, "Wrote coadded image "+output_name
			;
			image_colors[this_LED] = "green"
			WCT_LED_stack, LED_wid, image_colors
			;
			PTR_FREE, output_image
			;
	;		stop
			print_text, Event, ' Performing Heap Garbage Collection '
			;
			HEAP_GC, /PTR
		endif else begin
			;
			;	put in a different color in the progress bar
			;
			print_text, Event, "No images coadded for HWP "+STRING(ihwp)
			image_colors[this_LED] = "palegreen light"
			WCT_LED_stack, LED_wid, image_colors
		endelse
	endfor
	;
;	stop
	;
;	if(ihwp eq 1) then goto, S7_MGC_end
	;
	;-------------------------- deep coadd ---------------------------------------------
	;
	;	reserve room, then read images
	;
	this_LED = max_hwp+1
	;
	if(this_is_polarimetry_flag eq 1) then begin
		image_colors[this_LED] = "blue"
		ihwp_max = max_hwp+1
	endif else begin
		image_colors[this_LED] = 'pink'
		ihwp_max = 1
	endelse
	;
	WCT_LED_stack, LED_wid, image_colors
	;
	if(double_precision_flag) then begin
		raw_images = PTR_NEW(dblarr(ihwp_max,new_naxis1, new_naxis2))
		(*raw_images) += -1.0d6
		output_image = PTR_NEW(dblarr(new_naxis1, new_naxis2))
		(*output_image) += -1.0d6
	endif else begin
		raw_images = PTR_NEW(fltarr(ihwp_max,new_naxis1, new_naxis2))
		(*raw_images) += -1.0e6
		output_image = PTR_NEW(fltarr(new_naxis1, new_naxis2))
		(*output_image) += -1.0e6
	endelse
	;
	n_total_files = 0L
	n_total_scaled = 0L
	sky_image = dblarr(ihwp_max)
	sky_sigma = dblarr(ihwp_max)
	for ihwp = 0, ihwp_max-1 do begin
		;
		;	flatten if flag set
		;
		sky_image[ihwp] = 0.0d0
		sky_sigma[ihwp] = 100.0
		;
		if(flatten_pre_flag eq 0) then begin
			(*raw_images)[ihwp,*,*] = fix_high_pixels(readfits(hwp_coadd_paths[ihwp],header,/SILENT))
		endif else begin
			(*raw_images)[ihwp,*,*] = model_background(readfits(hwp_coadd_paths[ihwp],header,/SILENT),SKY=this_sky,SIGMA=this_sigma,/LOW_ORDER)
			sky_image[ihwp] = this_sky
			sky_sigma[ihwp] = this_sigma
		endelse
		;
		n_this_files = sxpar(header, 'MPP_NADD')
		n_total_files = n_total_files + n_this_files		; update total file count
		n_scaled = sxpar(header, 'MPP_NSCL')
		n_total_scaled += ROUND(n_scaled)
		if(ihwp eq 0) then first_header = header
		SXADDPAR, first_header, 'HISTORY', 'deep coadd includes '+FILE_BASENAME(hwp_coadd_paths[ihwp])
	endfor
	;
	print_text, Event, "Beginning coadding of Deep image"
	;
	zz_sky = median_filtered_mean(sky_image)
	mean_sky = zz_sky[0]
	;
	ss = median_filtered_mean(sky_sigma)
	mean_sky_sigma = ss[0]
	;
	;---------------------------------------------------------
	x_off = fltarr(ihwp_max)
	y_off = fltarr(ihwp_max)
	naxis1 = new_naxis1
	naxis2 = new_naxis2
	;
	n_images = ihwp_max
	scale = fltarr(ihwp_max)
	scale[*] = 1.0
	;
	fn_mean_images = float(ihwp_max)
	scaled_mean_sky = mean_sky
	debug = 0
	;
	if(star_sky_flag eq 0) then mean_sky_sigma = 1.0e6
	;
	if(ihwp_max eq max_hwp+1) then begin
		coadd_struct = { new_naxis1:	new_naxis1,	$	x-axis of output image
						 new_naxis2: 	new_naxis2, 	$	y-axis
						 naxis11:		naxis1-1, $
						 naxis21:		naxis2-1, $
						 n_images:	n_images, 	$	number of constituent images
						 x_off:		x_off,		$
						 y_off:		y_off, 		$
						 raw_images: (*raw_images),	$	the input data cube
						 sky_image:		sky_image,		$
						 scale:		scale,		$
						 double_precision_flag: double_precision_flag, $
						 fn_mean_images: fn_mean_images, $
						 mean_sky_sigma: mean_sky_sigma, $
						 debug: debug, $
						 scaled_mean_sky:scaled_mean_sky }

		output_image = S7_coadd(coadd_struct)
	endif else begin
		output_image = PTR_NEW(readfits(hwp_coadd_paths[0],header,/SILENT))
	endelse
	;
	PTR_FREE, raw_images
	;
	;	output name
	;
	output_name = replace_underscores(group_name) + '_Deep_Coadd.fits'
	output_dir = (*s_ptr).analysis_dir + 'S7_Coadding' + PATH_SEP()
	output_path = output_dir + output_name
	;
	SXADDPAR, first_header, 'FILENAME', output_name, 'Created by MSP_PPOL Step 7'
	SXADDPAR, first_header, 'MPP_DADD', n_total_files, 'total number of files in this deep coadd'
	SXADDPAR, first_header, 'MPP_NSCL', n_total_scaled, 'Number of images this sum is scaled to'
	;
	;	flatten and fix bad pixels, if flags set
	;
	if(flatten_post_flag eq 1) then (*output_image) = model_background( (*output_image),/MCS,STEP_SIZE=22)
	if(fix_bad_pixels_flag eq 1) then (*output_image) = fix_bad_pixels( (*output_image) )
	;
	(*output_image) = fix_low_pixels( (*output_image), /TPS)
	;
	writefits, output_path, (*output_image), first_header
	;
	print_text, Event, "Wrote deep coadded image "+output_name
	;
	PTR_FREE, output_image
	;
	image_colors[this_LED] = "plum"
	WCT_LED_stack, LED_wid, image_colors
	;------------------------------------------- redo astrometry --------------------
	do_astrometry:
	;
	deep_image = readfits(output_path, header)
	;
	;	this needs to be updated for use with F17 images...
	;
	;f17 = 0
	tolerance = 0.0015
	platescale_est = 0.579
	if(f17 eq 1) then platescale_est = 0.179
	;
	print_text, Event, "Starting revised astrometry determination"
	deep_header = ppol_astrometry(Event, output_path, deep_image, header, return_flag, platescale_est, $
					tolerance, find_scale, $
					F17=f17, ARRAY_2MASS=array_2mass, TWO_MASS=two_mass, $
					FILE_2MASS=mass_array)
	;
	;	rewrite deep image with updated header
	;
	print_text, Event, "Resaving deep image with updated astrometry header"
	writefits, output_path, deep_image, deep_header
	;
	;-------------------------------------------------------------------------------
	;
	image_colors[this_LED] = "green"
	WCT_LED_stack, LED_wid, image_colors
	;
	status = 0
	S7_MGC_end:
end
