function S8_gaussian_test_one_star, image, nx, ny, this_x, this_y, ave_fwhm, debug, $
									AXIS=axis, SIGMA=sigma
;
;	does a 2d gaussian fit to one star in an image
;		and compares the fit (if successful) to the image averages
;		of fwhm and to a/b (> 1.66) - if wildly off, then sets status flag
;
;	return value:
;		status = 0		good star
;		       = 1		binary
;			   = -1		bad fit (usually CR or bad pix)
;
;	DPC	20110629	added axis ratio return
;		20110630	added sigma cut for peak
;
;debug = 1
;
if(N_ELEMENTS(sigma) eq 0) then sigma = 0.0
;
deleted_data = -1.0e6
sig_cut = ave_fwhm		; (2.354 * fwhm = fat limit)
;
if(debug gt 0) then begin
	print, " "
	print, "Starting gaussian fitting for star at X, Y, fwhm = ",this_x,this_y, ave_fwhm
endif
;
xg = findgen(nx)
yg = findgen(ny)
;
;	create 2D versions
;
MAKE_2D, xg, yg, xxg, yyg
;
xg = reform(xg)
yg = reform(yg)
;
gfit = gauss2dfit(image,coeffs,xg,yg,/TILT)
;
if(debug gt 0) then begin
	print, "Background Level = ",coeffs[0]
	print, "Peak Fit value = ",coeffs[1]
	print, "X, Y widths (FWHM) = ",coeffs[2]*2.3548,coeffs[3]*2.3548
	print, "X, Y centers = ",coeffs[4],coeffs[5]
	print, "Position Angle = ",coeffs[6]
	;
endif
;
;	test quality of fit
;
ind = WHERE(~FINITE(coeffs), nbad)
if(nbad gt 0) then goto, bad_fit
if(coeffs[4] lt 0 or coeffs[4] gt (nx-1)) then goto, bad_fit
if(coeffs[5] lt 0 or coeffs[5] gt (ny-1)) then goto, bad_fit
if(abs(coeffs[2]) lt 0.33 or abs(coeffs[3]) lt 0.33) then goto, bad_fit		; CR
if(abs(coeffs[2]) ge sig_cut or abs(coeffs[3]) ge sig_cut) then goto, binary_fit
if(coeffs[1] lt sigma) then goto, bad_fit
;
rad_wid = sqrt(coeffs[2]*coeffs[3])
if(rad_wid lt 0.33) then goto, bad_fit
if(rad_wid lt 0.6*ave_fwhm/2.354) then goto, bad_fit	; too narrow
if(rad_wid gt 1.6*ave_fwhm/2.354) then goto, binary_fit
;
a = coeffs[2] > coeffs[3]
b = coeffs[2] < coeffs[3]
axis = a/b
if(axis gt 1.75) then goto, binary_fit
;
if(debug gt 0) then begin
	pa = -1.0 * coeffs[6] * (57.29578)
	rad_wid = rad_wid * 2.3548
	wid_results = string(format='("FWHM =",F6.3," pixels")',rad_wid)
	print,wid_results
	cen_results = string(format='("X,Y Center = ",F6.1,",",F6.1," pixels")',coeffs[4],coeffs[5])
	print,cen_results
	axis_results = string(format='("Axis ratio, PA = ",F4.2,",",F6.1)',axis,pa)
	print,axis_results
endif
;
if(debug eq 1) then stop
;
good_fit:
status = 0
return, status
goto, the_end
;
bad_fit:
;
if(debug gt 0) then begin
	print, "Failed fit for star at X,Y = ",this_x,this_y
	print, "...fit coeffs = ",coeffs
	;
	stop
endif
;
status = -1
return, status
goto, the_end
;
binary_fit:
;
if(debug gt 0) then begin
	print, "Binary star most likely at X,Y = ",this_x,this_y
	print, "...fit coeffs = ",coeffs
	;
	stop
endif
;
status = 1
return, status
;
the_end:
;
end