pro S8_Merge_2MASS, Event, phot_struct, two_mass_struct, dif_mag, glimpse_struct, pmode, status
	;
	;	DPC	20091212	added list of mirror sites to Vizier call
	;		20130620	added pmode (H-Pol, K-Pol, J-Band, H-Band, K-Band)
	;
	;	1. takes an input phot_structure, determines the field
	;		center and extent
	;	2. calls Visier for 2MASS data for the field
	;	3. matches 2MASS to GPIPS stars
	;	4. determines magnitude calibration
	;	5. applies calibration to GPIPS	magnitudes
	;	6. creates 2MASS struct mirroring indexing into phot_struct
	;
	;	7. fetches GLIMPSE data, also
	;
	star_2mass_struct = {	id_phot:				  0L, $		; which phot star matched to
							desig:					   '', $
							ra:						0.0d0, $	; J2000
							dec:					0.0d0, $
							Jmag:					0.0d0, $
							E_Jmag:					0.0d0, $
							Hmag:					0.0d0, $
							E_Hmag:					0.0d0, $
							Kmag:					0.0d0, $
							E_Kmag:					0.0d0  $
						}
	star_glimpse_struct = {	id_phot:				0L, $
							desig:	  				'', $		GLIMPSE designation
							desig_2:				'', $		2mass desig
							b36mag:					0.0d0, $	band 1 (3.6um) mag
							e_b36mag:				0.0d0, $
							b45mag:					0.0d0, $
							e_b45mag:				0.0d0, $
							b58mag:					0.0d0, $
							e_b58mag:				0.0d0, $
							b80mag:					0.0d0, $
							e_b80mag:				0.0d0 $
						}
	;
	;phot_dir = FILE_DIRNAME(phot_struct_path)
	;
	;	1. determine field properties
	;
	;RESTORE, phot_struct_path
	;
	;help, phot_struct, /struct
	;
	idebug = 0
	;
	dmag_clamp = 0.0150							; trap uncertainties to no less than this for averaging purposes
	;
	max_2mass_stars = 149						; was 49 - boosted 20120502 to aim for better photometry transfer
	;
	nstars = N_ELEMENTS(phot_struct)
	;
	;print, "n_phot_stars as read it = ",n_phot_stars
	;
	two_mass_struct = REPLICATE(star_2mass_struct, nstars)
	glimpse_struct = REPLICATE(star_glimpse_struct, nstars)
	;
	;	set all designations to '_none_' and all magnitudes and uncertainties to 99.99 initially
	;
	two_mass_struct[*].desig = '_2_none_'
	glimpse_struct[*].desig_2 = '_g_none_'
	;
	two_mass_struct[*].Jmag = 99.99
	two_mass_struct[*].E_Jmag = 99.99
	two_mass_struct[*].Hmag = 99.99
	two_mass_struct[*].E_Hmag = 99.99
	two_mass_struct[*].Kmag = 99.99
	two_mass_struct[*].E_Kmag = 99.99
	;
	glimpse_struct[*].b36mag = 99.99
	glimpse_struct[*].e_b36mag = 99.99
	glimpse_struct[*].b45mag = 99.99
	glimpse_struct[*].e_b45mag = 99.99
	glimpse_struct[*].b58mag = 99.99
	glimpse_struct[*].e_b58mag = 99.99
	glimpse_struct[*].b80mag = 99.99
	glimpse_struct[*].e_b80mag = 99.99
	;
	RA_min = MIN(phot_struct[*].ra)
	RA_max = MAX(phot_struct[*].ra)
	Dec_min = MIN(phot_struct[*].dec)
	Dec_max = MAX(phot_struct[*].dec)
	;
	RA_cen = (RA_max + RA_min) / 2.
	Dec_cen = (Dec_max + Dec_min) / 2.
	;
	field_size = (60.0*(abs(Dec_max - Dec_min) * 1.5)) > 15.0 ; in arcmin, no smaller than 15 arcmin
	;
	print_text, Event, 'Calling Vizier with RA, Dec centers = '+STRING(RA_cen,Dec_cen)+' and field size = '+STRING(field_size)+' arcmin'
	;
	;	2. get data from visier
	;
;	print, "about to call Vizier via mp_QueryVizier"
;	stop
	;
	;	call vizier, bumping along mirror sites until one found
	;
	this_mirror = ['Jp', 'CA', 'CDS', 'CfA', 'UK']
	;
	nvloop = 0L
	WHILE nvloop lt 4320L do begin
		vizier_flag = 0
	;	if keyword_set(F17) then begin
	;  		for imirror = 0, N_ELEMENTS(this_mirror)-1 do begin
	;  			if(vizier_flag eq 0) then begin
	;	  		info = mp_QueryVizier('2MASS-PSC', [tel_ra,tel_dec], [3.5*(xsize/1024),3.5*(ysize/1026)], $
	;	  			MIRROR=this_mirror[imirror], constraint='Qflg==AAA')
	;	  		test = size(info)
	;	  		if test[0] ne 0 then vizier_flag = 1
	;  			endif
	;  		endfor
	;  		if(vizier_flag eq 0) then begin
	;      		info = 99
	;      		return, info		;No Vizier servers available
	;  		endif
	;	endif else begin				; f/5 camera
	  	for imirror = 0, N_ELEMENTS(this_mirror)-1 do begin
		  	if(vizier_flag eq 0) then begin
		  		info = mp_QueryVizier('2MASS-PSC', [RA_cen,Dec_cen], [field_size,field_size], $
		  			MIRROR=this_mirror[imirror]); , constraint='Qflg==AAA')
		  		test = size(info, /TYPE)
		  		glimpse_full_struct = -1L
		    	if (test ne 3) then begin
		    		vizier_flag = 1
		    		;
		    		;	fetch GLIMPSE data also
		    		;
		    		glimpse_full_struct = mp_QueryVizier('GLIMPSE', [RA_cen,Dec_cen], [field_size,field_size], $
		  			MIRROR=this_mirror[imirror]); , constraint='Qflg==AAA')
		  		endif else print, 'Failed 2MASS fetch'
		  		;
		  		;	if this query fails, though, try again
		  		;
		  		if( SIZE(glimpse_full_struct,/TYPE) eq 3) then begin
		  			;
		  			;	was a bad fetch
		  			;
		  			;	reset vizier_flag and continue
		  			;
		  			vizier_flag = 0
		  			print, 'Failed GLIMSE fetch'
		  		endif
		    endif
	  	endfor
	  	if(vizier_flag eq 0) then begin
	  		status = -99
	  		print, "No Vizier Servers at ",SYSTIME()," waiting 20s and retrying"
	  		wait, 20
	  		nvloop++
	   	endif else goto, done
	ENDWHILE
	;
	done:
  	;
  	n_2mass = N_ELEMENTS(info)
  	n_glimpse = N_ELEMENTS(glimpse_full_struct)
  	;
  	print_text, Event, "Read "+STRING(n_2mass,FORMAT='(I)')+" 2MASS stars and "+STRING(n_glimpse,FORMAT='(I)')+" GLIMPSE stars"
  	;
  	ra_2mass = info.RAJ2000
  	dec_2mass = info.DEJ2000
  	desig_2mass = info._2MASS
  	J_2mass = info.JMAG
  	s_J_2mass = info.E_JMAG
  	H_2mass = info.HMAG
  	s_H_2mass = info.E_HMAG
  	K_2mass = info.KMAG
  	s_K_2mass = info.E_KMAG
  	;
  	;	push down to only keep entries with good in-band magnitudes and uncertainties
  	;
  	case pmode of
  		'H-Pol': begin
  					M_2mass = H_2mass
  					s_M_2mass = s_H_2mass
  				 end
  		'K-Pol': begin
  					M_2mass = K_2mass
  					s_M_2mass = s_K_2mass
  				 end
  		'J-Band':begin
  					M_2mass = J_2mass
  					s_M_2mass = s_J_2mass
  				 end
  		'H-Band':begin
  					M_2mass = H_2mass
  					s_M_2mass = s_H_2mass
  				 end
  		'K-Band':begin
  					M_2mass = K_2mass
  					s_M_2mass = s_K_2mass
  				 end
  		else:	begin
  					print, 'No Pmode match - stopping'
  					stop
  				end
  	endcase
  	;
  	ind_2 = WHERE(M_2mass gt 1.0 and M_2mass lt 19.0 and s_M_2mass gt 0.0 and s_M_2mass lt 1.0,n_2mass)
  	print_text, Event, 'Selecting '+STRING(n_2mass)+' 2MASS stars within in-band cuts'
  	if(n_2mass gt 0) then begin
  		ra_2mass = ra_2mass[ind_2]
  		dec_2mass = dec_2mass[ind_2]
  		desig_2mass = desig_2mass[ind_2]
  		J_2mass = J_2mass[ind_2]
	  	s_J_2mass = s_J_2mass[ind_2]
	  	H_2mass = H_2mass[ind_2]
	  	s_H_2mass = s_H_2mass[ind_2]
	  	K_2mass = K_2mass[ind_2]
	  	s_K_2mass = s_K_2mass[ind_2]
	  	;
	  	M_2mass = M_2mass[ind_2]
	  	s_M_2mass = s_M_2mass[ind_2]
	endif else begin
		print_text, Event, "ERROR - no 2MASS stars had good H-band magnitudes"
	endelse
  	;
  	;	next, sort by in-band brightness
  	;
  	ind_2mass_M_mag = SORT(M_2mass)
  	;
  	ra_2mass = ra_2mass[ind_2mass_M_mag]
	dec_2mass = dec_2mass[ind_2mass_M_mag]
	desig_2mass = desig_2mass[ind_2mass_M_mag]
	J_2mass = J_2mass[ind_2mass_M_mag]
  	s_J_2mass = s_J_2mass[ind_2mass_M_mag]
  	H_2mass = H_2mass[ind_2mass_M_mag]
  	s_H_2mass = s_H_2mass[ind_2mass_M_mag]
  	K_2mass = K_2mass[ind_2mass_M_mag]
  	s_K_2mass = s_K_2mass[ind_2mass_M_mag]
  	;
  	M_2mass = M_2mass[ind_2mass_M_mag]
  	s_M_2mass = s_M_2mass[ind_2mass_M_mag]
  	;
  	;	also, determine in-band brightness order for GPIPS phot entries
  	;
  	ind_gpips_M_mag = SORT(phot_struct.mag)
  	n_phot_stars = N_ELEMENTS(ind_gpips_M_mag)
  	;
  	;	trim to brightest 150
  	;
  	n_phot_stars_501 = max_2mass_stars < (n_phot_stars - 1)
  	;
  	;	for each GPIPS star, find the closest 2MASS star within the mag window
  	;
  	;	find the appropriate window sizes for RA, Dec, Mag windowing
  	;	by determining the matching of all GPIPS stars to 2MASS star
  	;
	;	for each of max_pass+1 passes, used to refine the offsets
	;
	;	initial guesses
	;
	max_pass = 5
	;
	angle_window = 20.0		; match items initially within 20" of each other
	win_mag = 20.0			; and 20 mag of each other
	win_deg = angle_window / 3600.0d0
	;
	ra_zero = 0.0d0			; these will be updated with each pass
	dec_zero = 0.0d0
	mag_zero = 0.0d0
	ra_width = win_deg
	dec_width = win_deg
	mag_width = win_mag
	ang_limit = 0.9 / 3600.0d0		; 1 mimir pixel
	;
	dtor = 180.0/!DPI
	;
	for ipass = 0, max_pass do begin
		;
		nmatch = 0L
		;
		ra_off = dblarr(n_phot_stars)
		dec_off = dblarr(n_phot_stars)
		mag_off = dblarr(n_phot_stars)
		s_mag_off = dblarr(n_phot_stars)
		;
		mag_dif = dblarr(n_phot_stars)
		mag_mwt = dblarr(n_phot_stars)
		;
		ra_off *= 0.0d0
		dec_off *= 0.0d0
		mag_off *= 0.0d0
		s_mag_off *= 0.0d0
		;
		mag_dif *= 0.0d0
		mag_mwt *= 0.0d0
		;
		;	start with magnitude ordered first list
		;
		;	reset to full Mimir list on final pass
		;
		if(ipass eq max_pass) then n_phot_stars_501 = n_phot_stars - 1
		;
		n_gmatch = 0L
		;
		for iref = 0, n_phot_stars_501 do begin
			;
			ii = ind_gpips_M_mag[iref]
			;
			ref_ra = phot_struct[ii].ra
			ref_dec = phot_struct[ii].dec
			ref_mag = phot_struct[ii].mag
			ref_s_mag = phot_struct[ii].errmag
		 	;
			cosdec = cos(ref_dec / dtor)
			;
			;	compare to each star on second list
			;
			ddist_match = 10.0d0								; smallest match
			ddist_index = -1L
			ddist_ra_off = 0.0d0
			ddist_dec_off = 0.0d0
			ddist_mag_off = 0.0d0
			;
			for i2 = 0, n_2mass-1 do begin
				;
				comp_ra = ra_2mass[i2]
				comp_dec = dec_2mass[i2]
				comp_mag = M_2mass[i2]
				comp_s_mag = s_M_2mass[i2]
				;
				ddec = comp_dec - ref_dec - dec_zero			; in decimal degrees on sky
				dra = (comp_ra - ref_ra) * cosdec - ra_zero		; in decimal degrees on sky
				;
				ddist = sqrt(ddec * ddec + dra * dra)
				dmag = (comp_mag - ref_mag) - mag_zero
				;
				if(abs(ddec) le dec_width and abs(dra) le ra_width and abs(dmag) le mag_width) then begin
					;
					;	found a match for this star - is it closer than any other previous match?
					;
					if(ddist le ddist_match) then begin
						ddist_match = ddist
						ddist_index = i2
						ddist_ra_off = dra + ra_zero
						ddist_dec_off = ddec + dec_zero
						ddist_mag_off = dmag + mag_zero
						ddist_s_mag_off = sqrt(ref_s_mag ^2 + comp_s_mag^2) > dmag_clamp  ; trap to no smaller than 0.08
					endif
						;
				endif
			endfor
			;
			;	was there a match?
			;
			if(ddist_index ne -1L) then begin
				;
				;	yes -- save the match
				;
				ra_off[nmatch] = ddist_ra_off
				dec_off[nmatch] = ddist_dec_off
				mag_off[nmatch] = ddist_mag_off
				s_mag_off[nmatch] = ddist_s_mag_off
				;
				;	is this the final pass?
				;
				i2 = ddist_index
				;
				if(ipass eq max_pass) then begin
					two_mass_struct[ii].id_phot = ii
					two_mass_struct[ii].ra = ra_2mass[i2]
					two_mass_struct[ii].dec = dec_2mass[i2]
					two_mass_struct[ii].desig = 'J'+STRTRIM(desig_2mass[i2],2)
		  			two_mass_struct[ii].Jmag = J_2mass[i2]
				  	two_mass_struct[ii].E_Jmag = s_J_2mass[i2]
				  	two_mass_struct[ii].Hmag = H_2mass[i2]
				  	two_mass_struct[ii].E_Hmag = s_H_2mass[i2]
				  	two_mass_struct[ii].Kmag = K_2mass[i2]
				  	two_mass_struct[ii].E_Kmag = s_K_2mass[i2]
				  	;
				  	;	same for glimpse
				  	;
				  	;		but, have to find the 2mass name in the glimpse set that matches
				  	;
				  	ind_g = WHERE(two_mass_struct[ii].desig eq 'J'+STRTRIM(glimpse_full_struct._2MASS,2), n2g)
				  	if(n2g eq 1) then begin
				  		kg = ind_g[0]
				  		n_gmatch++
				  		glimpse_struct[ii].id_phot = ii
				  		glimpse_struct[ii].desig = 'J'+STRTRIM(glimpse_full_struct[kg].GLIMPSE,2)
				  		glimpse_struct[ii].desig_2 = STRTRIM(glimpse_full_struct[kg]._2MASS,2)
				  		glimpse_struct[ii].b36mag = glimpse_full_struct[kg]._3_6MAG
				  		glimpse_struct[ii].e_b36mag = glimpse_full_struct[kg].E_3_6MAg
				  		glimpse_struct[ii].b45mag = glimpse_full_struct[kg]._4_5MAG
				  		glimpse_struct[ii].e_b45mag = glimpse_full_struct[kg].E_4_5MAG
				  		glimpse_struct[ii].b58mag = glimpse_full_struct[kg]._5_8MAG
				  		glimpse_struct[ii].e_b58mag = glimpse_full_struct[kg].E_5_8MAG
				  		glimpse_struct[ii].b80mag = glimpse_full_struct[kg]._8_0MAG
				  		glimpse_struct[ii].e_b80mag = glimpse_full_struct[kg].E_8_0MAG
				  	endif
		  		endif
		  		;
		  		mag_dif(nmatch) = M_2mass[i2] - ref_mag
				mag_mwt(nmatch) = (sqrt(ref_s_mag^2 + s_M_2mass[i2]^2)) > dmag_clamp		; trap
				;
				nmatch++
			endif
		endfor
		;
		;	end of this pass, recompute the offsets
		;
		if(nmatch eq 0) then begin
			;
			;	shouldn't get here - dump debug if so
			;
			print_text, Event, 'ERROR - no GPIPS-2MASS matches - should not happen - astrometry error???'
			stop
			;
	 		goto, S8_M2_end
	 	endif
	 	;
		ra_off = ra_off[0:nmatch-1]
		dec_off = dec_off[0:nmatch-1]
		mag_off = mag_off[0:nmatch-1]
		s_mag_off = s_mag_off[0:nmatch-1]
		;
		zz_ra = median_filtered_mean(ra_off)
		zz_dec = median_filtered_mean(dec_off)
		zz_mag = median_filtered_mean(mag_off)
		;
		ra_zero = zz_ra[0]
		dec_zero = zz_dec[0]
		mag_zero = zz_mag[0]
		;
		;	for high precision work, compute mean mag off and uncertainly using propagated uncer.
		;
		mag_mean = 0.0d0
		mag_wt = 0.0d0
		for k_match = 0, nmatch-1 do begin
			wt = 1.d0/ (mag_mwt[k_match]^2)
			mag_mean += mag_dif[k_match] * wt
			mag_wt += wt
		endfor
		;
		mag_mean = mag_mean / mag_wt
		s_mag_mean = 1.d0/sqrt(mag_wt)
		;
		;	widths
		;
		; if entering final pass, turn off mag windowing
		;
		if(ipass eq (max_pass-1)) then begin
			mag_width = 10.0
			ang_limit = 1.2 / 3600.0d0		; 2 Mimir pixels
		endif
		;
		ra_width = ((2.5+0.20*ipass)*zz_ra[1]) > ang_limit		; 2.5-3.3 sigma
		dec_width = ((2.5+0.20*ipass)*zz_dec[1]) > ang_limit
		mag_width = ((2.5+0.20*ipass)*zz_mag[1]) > 0.4	; trapped at no smaller than 0.3 mag
		;
		;if(ipass le 5) then mag_width = 6.0*zz_mag[1] else mag_width = 10.0	; only trim on early loops
		;if(ipass le 5) then mag_width = (2.5+0.15*ipass)*zz_mag[1] else mag_width = (2.0 < zz_mag[1]*2.0)
		;
		print_text, Event, "pass "+STRING(ipass+1,format='(I2)')+" Nmatches = "+STRING(nmatch,format='(I5.1)')
		print_text, Event, " RA offset = "+STRING(ra_zero*3600.0,format='(F7.4)')+"+/-"+STRING(zz_ra[1]*3600.0,format='(F6.4)')+$
		   " RA Window = "+STRING(ra_width*3600.0,format='(F8.4)')
		print_text, Event, " Dec offset = "+STRING(dec_zero*3600.0,format='(F7.4)')+"+/-"+STRING(zz_dec[1]*3600.0,format='(F6.4)')+$
		   " Dec Window = "+STRING(dec_width*3600.0,format='(F8.4)')
		print_text, Event, " Mag Offset = "+STRING(mag_zero,format='(F8.4)')+"+/-"+STRING(zz_mag[1],format='(F8.4)')+" Mag Window = "+$
		   STRING(mag_width,format='(F8.4)')+" Mean-Mag-offset = "+STRING(mag_mean,format='(F7.4)')+$
		   "+/-"+STRING(s_mag_mean,format='(F6.4)')
	endfor
	;
	;	report matches
	;
	print_text, Event, "For "+STRING(n_2mass,format='(I)')+" 2mass stars and "+$
		STRING(n_glimpse,format='(I)')+" gpips stars, "+STRING(nmatch,format='(I)')+$
				" matched in 2MASS and "+STRING(n_gmatch,format='(I)')+" in GLIMPSE"
		;
	print_text, Event, "RA offset  = "+STRING(zz_ra[0]*3600.0,zz_ra[1]*3600.0,format='(f9.5," +/- ",f9.5)')
	print_text, Event, "Dec offset = "+STRING(zz_dec[0]*3600.0,zz_dec[1]*3600.0,format='(f9.5," +/- ",f9.5)')
	;
	; so, now we have the offsets for matches, for each 2mass star, find the nearest GPIPS one
	;
	print_text, Event, "Mean magnitude difference (2MASS - Mimir) = "+STRING(mag_mean,s_mag_mean,format='(f9.5," +/- ",f9.5)')
	print_text, Event, "For Pmode = "+pmode
	;
	;	correct all GPIPS magnitudes by this amount
	;
	phot_struct[*].mag += mag_mean
	;
	dif_mag = mag_mean			; 2mass - Mimir
	;
	status = 0
	;
	S8_M2_end:
end