pro S9_polarimetric_analysis, Event, g_ind, group_name, LED1_wid, group_colors, ig, $
									meta_flag, inst_corr_flag, $
									pol_struct, n_pol_stars, status
	;
	;	accesses saved .SAV file of photometry for a group,
	;	identifies stars with adequate polarimetry data,
	;	computes polarization properties (with or without instrumental
	;	correction).
	;
	;	saves a .SAV file of the polarimetry structure to Step 9 directory
	;	and returns the structure for writing as POLCAT entry (flag tested
	;	in calling program).
	;
	;	20110428	DPC 	added test to find years for lookup of correct
	;						polarimetric offset angle to apply in correction code called from here
	;	20120220	DPC		added GLIMPSE entries
	;	20120618	DPC		allowed 'old' phot_struct files to generate 'new' pol files
	;
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
	w_ptr = PTR_struct.PTR_wid_id
	s_ptr = PTR_struct.PTR_summary
	g_ptr = PTR_struct.PTR_group
	;
	h_wid = (*w_ptr).S9_history_textbox_wid
	;
	;---- debug ---
	;phot_status = 0
	;pol_status = 0
	;--------------
	;
	;	get year of observation for non-meta
	;
	if(meta_flag eq 0) then begin
		ind = WHERE( (*g_ptr).group_names eq group_name, nmatch)
		if(nmatch eq 1) then begin
			igrp = ind[0]
			first_image_path = (*g_ptr).group_images[igrp,0]
		endif else begin
			print_text, Event, 'ERROR - no match to group name for '+group_name+' in S9_polarimetric_analysis. Stopping'
			stop
		endelse
	endif else begin
		;
		;	and meta group, from first year
		;
		ind = WHERE( (*g_ptr).meta_group_names eq group_name, nmatch)
		if(nmatch eq 1) then begin
			igrp = ind[0]
			;
			;	get the first image in the first group of this meta-group
			;
			this_group = (*g_ptr).meta_group_groups[igrp,0]
			;
			first_image_path = (*g_ptr).group_images[this_group,0]
		endif else begin
			print_text, Event, 'ERROR - no match to meta group name for '+group_name+' in S9_polarimetric_analysis. Stopping'
			stop
		endelse
	endelse
	;
	;	check which type of directory separator is present
	;
	ind_fwd = STRPOS(first_image_path, '/')
	ind_rev = STRPOS(first_image_path, '\')
	;
	case ind_fwd of
		-1: begin
				case ind_rev of
					-1: begin
							stop
						end
				  else: begin
				  			delim = '\'
				  		end
				endcase
			end
	  else: begin
	  			case ind_rev of
	  				-1:	begin
	  						delim = '/'
	  					end
	  			  else:	begin
	  			  			stop
	  			  		end
	  			endcase
	  		end
	endcase
	;
	parts = STRSPLIT(first_image_path, delim, COUNT=count, /EXTRACT)
	;
	; trim to image name, no path
	;
	if(STRLEN(parts[count-1]) gt 6) then begin
		first_image_path = parts[count-1]
	endif else stop
	;
	reads, first_image_path, this_year, this_month, this_day, format='(i4,i2,i2)'
	;
	;	pack into ISO standard
	;
	iso_std = STRING(this_year,this_month,this_day,format='(i4.4,"-",i2.2,"-",i2.2," 03:00:00.00")')
	;
	;	and convert to get day in year
	;
	dt_vec = date_conv(iso_std,'V')
	;
	;	decimal year
	;
	;this_decimal_year = this_year + float(this_month-1) / 12.0
	this_decimal_year = double(this_year) + double((dt_vec[1]-1L)/365.25d0)
	;
	;	select calibration block from availables
	;
;	n_max_blocks = (*(*w_ptr).S9_P_inst_values_ptr)).n_pa_blocks
	end_yr_block = (*(*w_ptr).S9_P_inst_values_ptr).end_yr_block
	ind_block = WHERE(this_decimal_year lt end_yr_block,nyr)
	if(nyr gt 0) then begin
		this_block = ind_block[0]
		(*(*w_ptr).S9_P_inst_values_ptr).this_pol_off_ind = this_block
	endif else begin
		;
		;	if got here, either an error, or no Pinst correction selected
		;		check which
		;
		if(inst_corr_flag eq 0) then begin
			;
			;	no Pinst correction
			;
			(*(*w_ptr).S9_P_inst_values_ptr).this_pol_off_ind = -1
		endif else begin
			;
			;	nope, was bona fide error - stop
			;
			print, 'ERROR - could not match to calibration year-block. Stopping'
			stop
		endelse
	endelse
	;
	;	extract offset angle, uncertainty
	;
;	this_offset = (*(*w_ptr).S9_P_inst_values_ptr).pol_off_angle[this_block]
;	this_s_offset = (*(*w_ptr).S9_P_inst_values_ptr).s_pol_off_angle[this_block]
	;
;	image_wid = (*w_ptr).S8_image_textbox_wid
;	nstars_wid = (*w_ptr).S8_nstars_textbox_wid
	;
	;	1. create names for incoming phot_struct and outgoing pol_struct .SAV files
	;
	psf_dir = (*s_ptr).analysis_dir + 'S8_Deep_PSF_Fitting' + PATH_SEP()
	pol_dir = (*s_ptr).analysis_dir + 'S9_Polarimetry' + PATH_SEP()
	clean_group_name = replace_underscores(group_name)
	;
	phot_path = psf_dir + clean_group_name + '_phot_struct.sav'
	pol_path = pol_dir + clean_group_name + '_pol_struct.sav'
	;
	;	2. restore phot_struct for this group
	;
	RESTORE, phot_path			; restores n_phot_stars, phot_struct, two_mass_struct
	;
	;	test photstruct for presence of ierrmag keyword
	;		DPC 20120618
	;
	tn_ps = TAG_NAMES(phot_struct)
	ind_tnps = WHERE(STRUPCASE(tn_ps) eq 'IERRMAG',n_tnps)
	if(n_tnps eq 0) then tnps_flag = 0 else tnps_flag = 1
	;
	group_colors[ig] = 'orange'
	WCT_LED_stack, LED1_wid, group_colors
	;
	print_text, Event, "Restored SAV file for "+FILE_BASENAME(phot_path)
	;
	;	3. establish pointers to entries with good polarimetry
	;
	;		A. go through each phot star, flagging those with potential polarimetry
	;
	ind_pol = WHERE(phot_struct[*].pol_flag,p_count)
	if(p_count gt 0) then begin
		print_text, Event, "phot_struct contained "+STRING(p_count,format='(I)')+" stars with some HPW magnitudes"
		;
		;	B. Set up the pol_struct (large for now, trim later)
		;
		star_struct = {		id:					    	0L, $
							x:						0.0d0, $	; x position in coadded image
							y:						0.0d0, $
							detector_x:				0L,    $	; star location on Mimir detector
							detector_y:				0L,    $
							ra:						0.0d0, $	; J2000
							dec:					0.0d0, $
							mag:					0.0d0, $
							errmag:					0.0d0, $
							ierrmag:				0.0d0, $	; photometric mag uncert rss-ed of errmag and 45mmag
							sky:					0.0d0, $
							;
							;	polarimetry data
							;
							n_HWP:					0L,	   $
							HWP_angles:			fltarr(16),$
							HWP_mags:		    fltarr(16),$
							HWP_mag_errs:		fltarr(16),$
							u:						0.0d0, $
							q:						0.0d0, $
							u_err:					0.0d0, $
							q_err:					0.0d0, $
							p:						0.0d0, $
							p_err:					0.0d0, $
							theta:					0.0d0, $
							theta_err:				0.0d0, $
							;
							;	2mass data, if present
							;
							id_phot:				0L,		$
							desig:						'', $
							ra_2mass:				0.0d0 , $
							dec_2mass:				0.0d0 , $
							Jmag:					0.0d0 , $
							E_Jmag:					0.0d0, $
							Hmag:					0.0d0, $
							E_Hmag:					0.0d0, $
							Kmag:					0.0d0, $
							E_Kmag:					0.0d0, $
							;
							;	and GLIMPSE data
							;
							b36mag:					0.0d0,$
							e_b36mag:				0.0d0,$
							b45mag:					0.0d0,$
							e_b45mag:				0.0d0,$
							b58mag:					0.0d0,$
							e_b58mag:				0.0d0,$
							b80mag:					0.0d0,$
							e_b80mag:				0.0d0 $

						}
		pol_struct = REPLICATE(star_struct,p_count)
		;
		;	C. for each potential pol star, count the number of good HWP magnitudes
		;
		ngood_pol = 0L
		i_cal_read = 0
		nbad_pol = 0L
		nproblem_pol = 0L
		;
		for ip = 0, p_count-1 do begin
			;
			iphot = ind_pol[ip]
			hwp_mag_vec = phot_struct[iphot].hwp_mag[*]
			ind_hwp = WHERE(hwp_mag_vec lt 25.0 and hwp_mag_vec gt -10.0, hwp_count)
			;
			;	test that all four regions are represented
			;
			r1 = [0, 4, 8, 12]
			r2 = [1, 5, 9, 13]
			r3 = [2, 6, 10, 14]
			r4 = [3, 7, 11, 15]
			;
			nnr1 = 0
			nnr2 = 0
			nnr3 = 0
			nnr4 = 0
			for ir = 0, hwp_count-1 do begin
				ind_r1 = WHERE(ind_hwp[ir] eq r1,nr1)
				ind_r2 = WHERE(ind_hwp[ir] eq r2,nr2)
				ind_r3 = WHERE(ind_hwp[ir] eq r3,nr3)
				ind_r4 = WHERE(ind_hwp[ir] eq r4,nr4)
				;
				nnr1 += nr1
				nnr2 += nr2
				nnr3 += nr3
				nnr4 += nr4
			endfor
			;
			if(nnr1 gt 0 and nnr2 gt 0 and nnr3 gt 0 and nnr4 gt 0) then region_pass = 1 else region_pass = 0
			;
			;	if all regions represented, allow the star
			;
;			if(hwp_count ge 10) then begin
			if(region_pass eq 1) then begin
				;
				;	C. Assemble the star's information
				;
				hwp_magerr_vec = phot_struct[iphot].hwp_magerr[*]
				;
				zz_magerr = median_filtered_mean(hwp_magerr_vec)
				;
				hwp_angle_vec = phot_struct[iphot].hwp_angle[*]
				;
				;			check that there aren't wild outliers in the data, kill those if present
				;
				zz_mag = median_filtered_mean(hwp_mag_vec[ind_hwp])
				zz_mag[1] = (zz_mag[1] > zz_magerr[0]) > 0.05	; trap at no smaller than mean error or 0.05 mag
				ind_bad = WHERE(abs(hwp_mag_vec[ind_hwp] - zz_mag[0])/zz_mag[1] gt 7.0,nbad)
				if(nbad gt 0) then begin
;					print_text, Event, "Warning - fixing bad HWP data for star PHOT-ID "+STRING(iphot,format='(I5.1)')
					hwp_mag_vec[ind_hwp[ind_bad]] = -99.0
					hwp_magerr_vec[ind_hwp[ind_bad]] = 99.0
				endif
				;
				;	D. Call April's calculator, with appropriate flags set
				;
				star = {x: 0.0d0, y: 0.0d0, mag: dblarr(16), magerr: dblarr(16), hwp_angle: dblarr(16)}
				star.x = phot_struct[iphot].detector_x
				star.y = phot_struct[iphot].detector_y
				;
				;	limit detector values to physical detector
				;
				star.x = 0 > star.x < 1023
				star.y = 0 > star.y < 1023
				;
				star.mag = hwp_mag_vec
				star.magerr = hwp_magerr_vec
				star.hwp_angle = hwp_angle_vec
				;
				;	D.1. Find the mag uncert from the Stokes I dispersion
				;
			;	ierrmag = stokes_i_dispersion(hwp_mag_vec, hwp_magerr_vec,hwp_angle_vec)
				;
				;	D.2. compute polarization
				;
				;	on first call, read the instrumental pol cal images
				;
				ap_uq_with_inst_cor, Event, star, inst_corr_flag, pol, i_cal_read, pa_image, p_image, status
				;
				;	E. Evaluate results
				;
				if(status eq 0) then begin
					;
					;	F. Store results in pol_struct
					;
					;		pol stuff first
					;
					pol_struct[ngood_pol].n_HWP = hwp_count
					pol_struct[ngood_pol].HWP_angles = hwp_angle_vec
					pol_struct[ngood_pol].HWP_mags = hwp_mag_vec
					pol_struct[ngood_pol].HWP_mag_errs = hwp_magerr_vec
					pol_struct[ngood_pol].u = pol.u * 100.0
					pol_struct[ngood_pol].q = pol.q * 100.0
					pol_struct[ngood_pol].u_err = ((0.0 > pol.s_u) < 1.0) * 100.0
					pol_struct[ngood_pol].q_err = ((0.0 > pol.s_q) < 1.0) * 100.0
					pol_struct[ngood_pol].p = pol.p * 100.0
					pol_struct[ngood_pol].p_err = ((0.0 > pol.s_p) < 1.0) * 100.0
					pol_struct[ngood_pol].theta = pol.theta
					pol_struct[ngood_pol].theta_err = (0.0 > pol.s_theta) < 180.0
					;
					;		phot stuff second
					;
					pol_struct[ngood_pol].id = phot_struct[iphot].id
					pol_struct[ngood_pol].x = phot_struct[iphot].x
					pol_struct[ngood_pol].y = phot_struct[iphot].y
					pol_struct[ngood_pol].detector_x = phot_struct[iphot].detector_x
					pol_struct[ngood_pol].detector_y = phot_struct[iphot].detector_y
					pol_struct[ngood_pol].ra = phot_struct[iphot].ra
					pol_struct[ngood_pol].dec = phot_struct[iphot].dec
					pol_struct[ngood_pol].mag = phot_struct[iphot].mag
					pol_struct[ngood_pol].errmag = phot_struct[iphot].errmag
					;
					;	test for presence of ierrmag keyword - if lacking, substitute
					;	a value
					;		DPC 20120618
					;
					if(tnps_flag eq 1) then begin
						pol_struct[ngood_pol].ierrmag = phot_struct[iphot].ierrmag
					endif else begin
						pol_struct[ngood_pol].ierrmag = -9.99
					endelse
					;
					pol_struct[ngood_pol].sky = phot_struct[iphot].sky
					;
					;	and 2mass stuff, if present
					;
					if(two_mass_struct[iphot].desig ne '') then begin
						pol_struct[ngood_pol].id_phot = two_mass_struct[iphot].id_phot
						pol_struct[ngood_pol].desig = two_mass_struct[iphot].desig
						pol_struct[ngood_pol].ra_2mass = two_mass_struct[iphot].ra
						pol_struct[ngood_pol].dec_2mass = two_mass_struct[iphot].dec
						pol_struct[ngood_pol].Jmag = two_mass_struct[iphot].Jmag
						pol_struct[ngood_pol].E_Jmag = two_mass_struct[iphot].E_Jmag
						pol_struct[ngood_pol].Hmag = two_mass_struct[iphot].Hmag
						pol_struct[ngood_pol].E_Hmag = two_mass_struct[iphot].E_Hmag
						pol_struct[ngood_pol].Kmag = two_mass_struct[iphot].Kmag
						pol_struct[ngood_pol].E_Kmag = two_mass_struct[iphot].E_Kmag
						;
						;	and GLIMPSE
						;
						if(tnps_flag eq 1) then begin
							pol_struct[ngood_pol].b36mag = glimpse_struct[iphot].b36mag
							pol_struct[ngood_pol].e_b36mag = glimpse_struct[iphot].e_b36mag
							pol_struct[ngood_pol].b45mag = glimpse_struct[iphot].b45mag
							pol_struct[ngood_pol].e_b45mag = glimpse_struct[iphot].e_b45mag
							pol_struct[ngood_pol].b58mag = glimpse_struct[iphot].b58mag
							pol_struct[ngood_pol].e_b58mag = glimpse_struct[iphot].e_b58mag
							pol_struct[ngood_pol].b80mag = glimpse_struct[iphot].b80mag
							pol_struct[ngood_pol].e_b80mag = glimpse_struct[iphot].e_b80mag
						endif else begin
							pol_struct[ngood_pol].b36mag = 99.99
							pol_struct[ngood_pol].e_b36mag =  99.99
							pol_struct[ngood_pol].b45mag =   99.99
							pol_struct[ngood_pol].e_b45mag =99.99
							pol_struct[ngood_pol].b58mag =   99.99
							pol_struct[ngood_pol].e_b58mag = 99.99
							pol_struct[ngood_pol].b80mag =   99.99
							pol_struct[ngood_pol].e_b80mag =99.99
						endelse
						;
					endif else begin
						;
						;	fill with placeholders for no data
						;
						pol_struct[ngood_pol].id_phot = -99
						pol_struct[ngood_pol].desig = '_2_none_'
						pol_struct[ngood_pol].ra_2mass = 0.0
						pol_struct[ngood_pol].dec_2mass = 0.0
						pol_struct[ngood_pol].Jmag = 99.99
						pol_struct[ngood_pol].E_Jmag = 99.99
						pol_struct[ngood_pol].Hmag = 99.99
						pol_struct[ngood_pol].E_Hmag = 99.99
						pol_struct[ngood_pol].Kmag = 99.99
						pol_struct[ngood_pol].E_Kmag = 99.99
						;
						;	and GLIMPSE
						;
						pol_struct[ngood_pol].b36mag = 99.99
						pol_struct[ngood_pol].e_b36mag = 99.99
						pol_struct[ngood_pol].b45mag = 99.99
						pol_struct[ngood_pol].e_b45mag = 99.99
						pol_struct[ngood_pol].b58mag = 99.99
						pol_struct[ngood_pol].e_b58mag = 99.99
						pol_struct[ngood_pol].b80mag = 99.99
						pol_struct[ngood_pol].e_b80mag = 99.99
						;
					endelse
					;
					ngood_pol++
				endif else begin
					nproblem_pol++
					if(status eq -1) then begin
;						print_text, Event, "Star "+STRING(ip,format='(I)')+" failed pol analysis in ap_UQ"
					endif else begin
						print_text, Event, "Failed to find/open instrumental polarization images"
						status = -1
						goto, S9_PA_end
					endelse
				endelse
			endif else begin
				star_x = phot_struct[iphot].detector_x
				star_y = phot_struct[iphot].detector_y
;				print_text, Event, "Rejecting star (id_phot,x,y) "+STRING(iphot,star_x,star_y)+" with N_hwp = "+STRING(hwp_count,format='(I)')
				nbad_pol++
			endelse
		endfor
		;
		;	trim structure
		;
		pol_struct = pol_struct[0:ngood_pol-1]
		n_pol_stars = ngood_pol
		;
		print_text, Event, "Processed "+STRING(ngood_pol,format='(I)')+" stars for polarization signals"
		print_text, Event, "Rejected "+STRING(nbad_pol,format='(I)')+" stars for too few HWP detections"
		if(nproblem_pol gt 0) then print_text, Event, "Also, rejected "+STRING(nproblem_pol,format='(I)')+$
								" stars for failing polar analysis - ap_code"
		;
		;	save the pol_struct to disk as a .SAV file
		;
		SAVE, pol_struct, n_pol_stars, FILENAME=pol_path
		;
		print_text, Event, "Saved summary pol file "+FILE_BASENAME(pol_path)+" to disk"
		;
		status = 0
	endif else begin
		;
		;	no stars with HWP mags, reset status and quit
		;
		str = "No stars in "+FILE_BASENAME(phot_path)+" had HWP mags. Skipping group"
		print_text, Event, "Warning - "+str
		SAll_add_to_history, Event, h_wid, 9, str
	 	pol_struct = -1
	 	n_pol_stars = 0
		status = -1
	endelse
	;
	S9_PA_end:
	;
end