pro ap_uq_with_inst_cor, Event, star, do_instrumental_correction_flag, $
							    pol, i_cal_read, pa_image, p_image, status
;
;	if i_cal_read = 0, read images
;						else use input images
;
;	star struct should have format
;         star.x = detector x location of the star (integer pixel in IDL coords)
;         star.y = same, for y
;         star.mag[16] = vector of 16 hwp magnitudes
;         star.magerr[16] = uncertainties
;         star.hwp_angle[16] = vector of the HWP angles (floating point, in degrees)
;
;	processing flag:          do_instrumental_correction_flag  (0 for no, 1 for yes)
;
;	return a different structure containing:
;          pol.u = the computed U - stokes value (corrected for P_Inst, if flag set)
;          pol.q
;          pol.s_u
;          pol.s_q
;          pol.p
;          pol.s_p
;          pol.theta
;          pol.s_theta
;------------------------------------------
;	U-Q intensity balancing,
;	all folding,
;	all error propagation,
;	including propagating contributions from the instrumental corrections
;
;	DPC	20110126	added U flip
;
dtor=!pi/180.0
bad_mag_val=-90.
;
;--------------------------------------------------------
;
u_flip_flag = 1					; does early flip of U -> -U if set
;
;	set to 1 to perform pair normalization
;
pair_normalization = 0		; DPC 20110907 - this is already done in S6
;
;	grab hwp position array
;
hwp_pos	=	star.hwp_angle
;
;	get ff nums
;
ff_num		=	hwp_pos
ff_num[*]	=	-1
ff_ind_0	=	where(hwp_test(hwp_pos,0.) or hwp_test(hwp_pos,89.8) $
					or hwp_test(hwp_pos,180.2) or hwp_test(hwp_pos,270.0))
ff_num[ff_ind_0]=0.0
;
ff_ind_45	=	where(hwp_test(hwp_pos,22.3) or hwp_test(hwp_pos,112.7) $
					or hwp_test(hwp_pos,202.5) or hwp_test(hwp_pos,292.3))
ff_num[ff_ind_45]=45.0
;
ff_ind_90	=	where(hwp_test(hwp_pos,45.2) or hwp_test(hwp_pos,135.0) $
					or hwp_test(hwp_pos,224.8) or hwp_test(hwp_pos,315.2))
ff_num[ff_ind_90]=90.0
;
ff_ind_135	=	where(hwp_test(hwp_pos,67.5) or hwp_test(hwp_pos,157.3) $
					or hwp_test(hwp_pos,247.7) or hwp_test(hwp_pos,337.5))
ff_num[ff_ind_135]=135.0
;
foo		=	ff_num
ind_0	=	where(foo eq 0.0)
ind_45	=	where(foo eq 45.0)
ind_90	=	where(foo eq 90.0)
ind_135	=	where(foo eq 135.0)
;
;	convert mags to fluxes
;
mags       = star.mag
mags_uncer = star.magerr
;
;	convert to flux units, for a convenient reference point
;
fluxes_init = 10.0d0^(-0.40d0 * mags)
;
;		flux uncertainty
;
fluxerr_init = mags_uncer * fluxes_init * (1./1.086)
;
bad_mags_ind	=	where(floor(mags) lt bad_mag_val,cbad)
bad_fluxes		=	intarr(16)
if (cbad gt 0) then bad_fluxes[bad_mags_ind] = -2
;
pairs_ind	=	intarr(16)
pair_1		=	where(hwp_test(hwp_pos,0.)    or hwp_test(hwp_pos,45.2),c1)
pair_2		=	where(hwp_test(hwp_pos,22.3)  or hwp_test(hwp_pos,67.5),c2)
pair_3		=	where(hwp_test(hwp_pos,89.8)  or hwp_test(hwp_pos,135.0),c3)
pair_4		=	where(hwp_test(hwp_pos,112.7) or hwp_test(hwp_pos,157.3),c4)
pair_5		=	where(hwp_test(hwp_pos,180.2) or hwp_test(hwp_pos,224.8),c5)
pair_6		=	where(hwp_test(hwp_pos,202.5) or hwp_test(hwp_pos,247.7),c6)
pair_7		=	where(hwp_test(hwp_pos,270.0) or hwp_test(hwp_pos,315.2),c7)
pair_8		=	where(hwp_test(hwp_pos,292.3) or hwp_test(hwp_pos,337.5),c8)
pairs_ind[pair_1]	=	1
pairs_ind[pair_2]	=	2
pairs_ind[pair_3]	=	3
pairs_ind[pair_4]	=	4
pairs_ind[pair_5]	=	5
pairs_ind[pair_6]	=	6
pairs_ind[pair_7]	=	7
pairs_ind[pair_8]	=	8
;
;	do normalization
;
if (pair_normalization eq 1) then begin
	new_fluxes		= dblarr(n_elements(fluxes_init))
	new_fluxes_err	= dblarr(n_elements(fluxes_init))
	;
	for i = 0,7 do begin
		;
		;	go through the 8 pairs
		;
		current_pair = i+1
		ind_current_pair = where(pairs_ind eq current_pair)
		flux_a		=	fluxes_init[ind_current_pair[0]]
		flux_b		=	fluxes_init[ind_current_pair[1]]
		flux_a_err	=	fluxerr_init[ind_current_pair[0]]
		flux_b_err	=	fluxerr_init[ind_current_pair[1]]
		flux_sum	=	flux_a+flux_b
		new_flux_a	=	flux_a / (0.5 * flux_sum)
		new_flux_b	=	flux_b / (0.5 * flux_sum)
		;
		;flux_sum_err=sqrt(flux_a_err^2+flux_b_err^2)
		;new_flux_a_err=sqrt((2.*flux_a_err/flux_sum)^2+(2.*flux_a*flux_sum_err/flux_sum^2)^2)
		;new_flux_b_err=sqrt((2.*flux_b_err/flux_sum)^2+(2.*flux_b*flux_sum_err/flux_sum^2)^2)
		;********* just use scaled flux err
		;
		new_flux_a_err	=	flux_a_err / (0.5 * flux_sum)
		new_flux_b_err	=	flux_b_err / (0.5 * flux_sum)
		;
		;	check to see if fluxes were good
		;
		is_bad 		= bad_fluxes[ind_current_pair]
		is_bad_ind	= where(is_bad eq -2,c_is_bad)
		if (c_is_bad ne 0) then begin
			new_fluxes[ind_current_pair[0]]		=	-99.99
			new_fluxes[ind_current_pair[1]]		=	-99.99
			new_fluxes_err[ind_current_pair[0]]	=	99.99
			new_fluxes_err[ind_current_pair[1]]	=	99.99
		endif else begin
			new_fluxes[ind_current_pair[0]]		=	new_flux_a
			new_fluxes[ind_current_pair[1]]		=	new_flux_b
			new_fluxes_err[ind_current_pair[0]]	=	new_flux_a_err
			new_fluxes_err[ind_current_pair[1]]	=	new_flux_b_err
		endelse
	endfor			;iloop
	;
	fluxes		=	new_fluxes
	fluxerr		=	new_fluxes_err
	;
	new_bad_fluxes	=	where(new_fluxes eq -99.99,c_new_bad)
	if (c_new_bad gt 0) then bad_fluxes[new_bad_fluxes] = -2
endif else begin
	;
	;	if no pair normalization, then just load arrays
	;
	fluxes	=	fluxes_init
	fluxerr	=	fluxerr_init
endelse
;
;	setup dataout struct
;
placeholder={	u:		dblarr(1), 	$
				s_u:	dblarr(1),	$
				q:		dblarr(1),	$
				s_q:	dblarr(1),	$
				p:		dblarr(1),	$
				s_p:	dblarr(1),	$
				theta:	dblarr(1),	$
				s_theta:dblarr(1)	}
;
;	create an instance of this structure
;
pol	=	placeholder
;
xypos=[star.x, star.y]
;
;	fluxes at the ff posistions
;
pos0f	=	fluxes[ind_0]
pos45f	=	fluxes[ind_45]
pos90f	=	fluxes[ind_90]
pos135f	=	fluxes[ind_135]
;
;	grab the flux sigmas
;
pos0f_s		=	fluxerr[ind_0]
pos45f_s	=	fluxerr[ind_45]
pos90f_s	=	fluxerr[ind_90]
pos135f_s	=	fluxerr[ind_135]
;
;	the bad flux array
;
pos0bad		=	bad_fluxes[ind_0]
pos45bad	=	bad_fluxes[ind_45]
pos90bad	=	bad_fluxes[ind_90]
pos135bad	=	bad_fluxes[ind_135]
;
;	locate good indicies
;
pos0_good_ind	=	where(pos0bad ne -2,c0)
pos45_good_ind	=	where(pos45bad ne -2,c45)
pos90_good_ind	=	where(pos90bad ne -2,c90)
pos135_good_ind	=	where(pos135bad ne -2,c135)
;
;	if no good data at one of the ff positions, ignore star
;
if (c0 eq 0 or c45 eq 0 or c90 eq 0 or c135 eq 0) then begin
	status=-1
	goto,BADSTAR
endif
;
;	keep good fluxes and errs
;
pos0_good_flux		=	pos0f[pos0_good_ind]
pos45_good_flux		=	pos45f[pos45_good_ind]
pos90_good_flux		=	pos90f[pos90_good_ind]
pos135_good_flux	=	pos135f[pos135_good_ind]
pos0_good_fluxerr	=	pos0f_s[pos0_good_ind]
pos45_good_fluxerr	=	pos45f_s[pos45_good_ind]
pos90_good_fluxerr	=	pos90f_s[pos90_good_ind]
pos135_good_fluxerr	=	pos135f_s[pos135_good_ind]
;
;	compute averages for each posistion
;
if(n_elements(pos0_good_flux) gt 1) then begin
	pos0mfm = MOMENT(pos0_good_flux)
	pos0mfm_s = MOMENT(pos0_good_fluxerr)			; mean of uncertainties, too
	pos0mfm[1] = sqrt(pos0mfm[1])  > pos0mfm_s[0]	; larger of two uncertainties - scatter of means or mean of uncerts.
endif else begin
	pos0mfm = [pos0_good_flux[0],pos0_good_fluxerr[0]]
endelse
;
if(n_elements(pos45_good_flux) gt 1) then begin
	pos45mfm = MOMENT(pos45_good_flux)
	pos45mfm_s = MOMENT(pos45_good_fluxerr)
	pos45mfm[1] = sqrt(pos45mfm[1]) > pos45mfm_s[0]
endif else begin
	pos45mfm = [pos45_good_flux[0],pos45_good_fluxerr[0]]
endelse
;
if(n_elements(pos90_good_flux) gt 1) then begin
	pos90mfm = MOMENT(pos90_good_flux)
	pos90mfm_s = MOMENT(pos90_good_fluxerr)
	pos90mfm[1] = sqrt(pos90mfm[1]) > pos90mfm_s[0]
endif else begin
	pos90mfm = [pos90_good_flux[0], pos90_good_fluxerr[0]]
endelse
;
if(n_elements(pos135_good_flux) gt 1) then begin
	pos135mfm = MOMENT(pos135_good_flux)
	pos135mfm_s = MOMENT(pos135_good_fluxerr)
	pos135mfm[1] = sqrt(pos135mfm[1]) > pos135mfm_s[0]
endif else begin
	pos135mfm = [pos135_good_flux[0], pos135_good_fluxerr[0]]
endelse
;
;	filter out the bad points (> 8 sigma deviations - DPC 20100204)
;
mfm_ind0 = WHERE(  abs(pos0_good_flux - pos0mfm[0])/pos0mfm[1] lt 8.0, good_count0)
case good_count0 of
	0: begin
		status=-1
		goto,BADSTAR
	   end
	1: begin
		I_0 	= pos0_good_flux[mfm_ind0]
		s_I_0 	= pos0_good_fluxerr[mfm_ind0]
	   end
	else: begin
			good_mfm_flux0 		= pos0_good_flux[mfm_ind0]
			good_mfm_fluxerr0	= pos0_good_fluxerr[mfm_ind0]
			;
			;	recompute means of good points
			;
			good_0_mfm 		= MOMENT(good_mfm_flux0)
			good_0_mfm_s 	= MOMENT(good_mfm_fluxerr0)
			good_0_s	 	= sqrt(good_0_mfm[1]) > good_0_mfm_s[0]
			;
			I_0 	= good_0_mfm[0]
			s_I_0 	= good_0_s / sqrt(good_count0 - 1)
		  end
endcase
;
mfm_ind45 = WHERE(  abs(pos45_good_flux - pos45mfm[0])/pos45mfm[1] lt 8.0, good_count45)
case good_count45 of
	0: begin
		status =-1
		goto,BADSTAR
	   end
	1: begin
		I_45 	= pos45_good_flux[0]
		s_I_45 	= pos45_good_fluxerr[0]
	   end
	else: begin
			;
			good_mfm_flux45 	= pos45_good_flux[mfm_ind45]
			good_mfm_fluxerr45	= pos45_good_fluxerr[mfm_ind45]
			;
			good_45_mfm 	= MOMENT(good_mfm_flux45)
			good_45_mfm_s 	= MOMENT(good_mfm_fluxerr45)
			good_45_s 		= sqrt(good_45_mfm[1]) > good_45_mfm_s[0]

			I_45 	= good_45_mfm[0]
			s_I_45 	= good_45_s / sqrt(good_count45 - 1)
			;
		   end
endcase
;
mfm_ind90 = WHERE(  abs(pos90_good_flux - pos90mfm[0])/pos90mfm[1] lt 8.0, good_count90)
case good_count90 of
	0: begin
		status = -1
		goto,BADSTAR
	   end
	1: begin
		I_90 	= pos90_good_flux[0]
		s_I_90 	= pos90_good_fluxerr[0]
	   end
	else: begin
			;
			good_mfm_flux90 	= pos90_good_flux[mfm_ind90]
			good_mfm_fluxerr90	= pos90_good_fluxerr[mfm_ind90]
			;
			good_90_mfm 	= MOMENT(good_mfm_flux90)
			good_90_mfm_s 	= MOMENT(good_mfm_fluxerr90)
			good_90_s 		= sqrt(good_90_mfm[1]) > good_90_mfm_s[0]
			;
			I_90 	= good_90_mfm[0]
			s_I_90 	= good_90_s / sqrt(good_count90 - 1)
			;
		   end
endcase
;
mfm_ind135 = WHERE(  abs(pos135_good_flux - pos135mfm[0])/pos135mfm[1] lt 8.0, good_count135)
case good_count135 of
	0: begin
		status=-1
		goto,BADSTAR
	   end
	1: begin
		I_135 	= pos135_good_flux[0]
		s_I_135 = pos135_good_fluxerr[0]
	   end
	else: begin
			;
			good_mfm_flux135 	= pos135_good_flux[mfm_ind135]
			good_mfm_fluxerr135	= pos135_good_fluxerr[mfm_ind135]
			;
			good_135_mfm 	= MOMENT(good_mfm_flux135)
			good_135_mfm_s 	= MOMENT(good_mfm_fluxerr135)
			good_135_s 		= sqrt(good_135_mfm[1]) > good_135_mfm_s[0]
			;
			I_135 	= good_135_mfm[0]
			s_I_135 = good_135_s / sqrt(good_count135 - 1)
		  end
endcase
;
;	form weighted ave of good points
;
q = (I_0 - I_90)   / (I_0 + I_90)
u = (I_45 - I_135) / (I_45 + I_135)
;
if(u_flip_flag eq 1) then u *= -1.0
;
q_sig = (1./(I_0 + I_90))   * sqrt(s_I_0^2 + s_I_90^2)
u_sig = (1./(I_45 + I_135)) * sqrt(s_I_45^2 + s_I_135^2)
;
;	compute p and pa
;
p		=	sqrt(q^2 + u^2)
;
p_sig 	= (1./p) * sqrt( (q_sig * q)^2 + (u_sig * u)^2)			; 20100204 DPC
;
pa      = (0.5 / dtor) * ATAN(u, q)
pa      = (720.0 + pa) MOD 180.0
pa_sig  = (0.0 > ((0.5 / dtor) * (p_sig / p))) < 180.0
;
;print,p*100
;
;	make sure p is finite
;
checkfinite_nan	=	finite(p,/NAN)
checkfinite_inf	=	finite(p,/infinity)
if (checkfinite_nan eq 1 or checkfinite_inf eq 1) then begin
;	print_text, Event,'star is not finite'
	status = -1
	goto,BADSTAR
endif
;
;print,p*100,p_sig*100,pa,pa_sig,p/p_sig
;
;	instrumental correction (if selected)
;
if (do_instrumental_correction_flag eq 1) then begin
	;
	;	apply inst correction
	;
	ap_apply_inst_correction_sendimg, Event, xypos, [u, u_sig], [q, q_sig], ucor, qcor, status
 	;
	;	check to see if star was in a correctable position
	;
	if (status ne 0) then goto,BADSTAR
	;
	;	compute p and pa from corrected values
	;
	ap_uq2polpa, ucor, qcor, pcor, pacor
	;
endif else begin
	pcor  = [p, p_sig]
	pacor = [pa, pa_sig]
	ucor  = [u, u_sig]
	qcor  = [q, q_sig]
endelse
;
;check to see if Ricean correction can be done, if so do it
;
p_sig = pcor[1]
p = pcor[0]
pa = pacor[0]
pa_sig = pacor[1]
;
u = ucor[0]
q = qcor[0]
;
if ((1.-(p_sig/p)^2) lt 0.0) then begin
	;
	;	no, zero polarization
	;
	p 		= 0.0
	pa 		= 0.0
	pa_sig	= 180.0
endif else begin
	;
	;	yes, correct
	;
	p 		= p * SQRT(1.-(p_sig / p)^2)  ;	Ricean correction
	pa 		= (0.5 / dtor) * ATAN(u, q)
	pa 		= (720.0 + pa) mod 180.
	pa_sig 	= (0.0 > ((0.5 / dtor)*(p_sig / p))) < 180.0
endelse
;
;	repack pcor, pacor after Ricean correction
;
pcor  = [p, p_sig]
pacor = [pa, pa_sig]
;
;put data into struct
;
pol.q 		=	qcor[0]
pol.s_q 	=	qcor[1]
pol.u 		=	ucor[0]
pol.s_u 	=	ucor[1]
pol.p 		=	pcor[0]
pol.s_p 	= 	pcor[1]
pol.theta	= 	pacor[0]
pol.s_theta	=	pacor[1]
;
status = 0
;
BADSTAR:  ;if it's a bad star, skip
a = 0
;
end
