pro compute_P_efficiency_and_PA_offset
;
;	reads a data file of Mimir P, PA values and Whittet P, PA values
;	and determines least square fit solutions, with plotting and
;	data culling
;
;	DPC	20100822
;
file_in = dialog_pickfile(Title='Comma separated data file')
;
openr, lu_in, file_in, /GET_LUN
;
line = ''
;
one_line = {star_obs: '', P_M: 0.0, sP_M: 0.0, PA_M: 0.0, sPA_M: 0.0, $
                          P_W: 0.0, sP_W: 0.0, PA_W: 0.0, sPA_W: 0.0}
;
nlines = FILE_LINES(file_in)
;
pol_struct = REPLICATE(one_line, nlines)
;
f = '(f)'
;
iline = 0L
WHILE ~EOF(lu_in) do begin
	readf, lu_in, line
	parts = STRSPLIT(line,',',/REGEX,/EXTRACT,count=n)
	pol_struct[iline].star_obs = parts[0] + '_' + STRING(parts[1],format='(I1.1)')
	reads, parts[2], format=f, value
	pol_struct[iline].P_M = value
	reads, parts[3], format=f, value
	pol_struct[iline].sP_M = value
	reads, parts[4], format=f, value
	pol_struct[iline].PA_M = value
	reads, parts[5], format=f, value
	pol_struct[iline].sPA_M = value
	reads, parts[6], format=f, value
	pol_struct[iline].P_W = value
	reads, parts[7], format=f, value
	pol_struct[iline].sP_W = value
	reads, parts[8], format=f, value
	pol_struct[iline].PA_W = value
	reads, parts[9], format=f, value
	pol_struct[iline].sPA_W = value
	iline++
ENDWHILE
;
FREE_LUN, lu_in
;
;	plot P_W vs P_M with uncertainties
;
x = pol_struct.P_W
y = pol_struct.P_M
sx = pol_struct.sP_W
sy = pol_struct.sP_M
;
window, 22, title='P vs P', xsize = 750, ysize=750
plot, x, y, xr = [0,5], yr=[0,5], xs=1, ys = 1, psym=4, thick=3
errplot, x, (y-sy), (y+sy)
;
ind = sort(x)
xl = x[ind]
yl = y[ind]
lad = LADFIT(xl, yl)
;
xf0 = 0.0
yf0 = lad[0]
xf1 = 5.0
yf1 = lad[0] + xf1 * lad[1]
;
oplot, [xf0, xf1], [yf0, yf1], psym=-3, thick=4
;
;	and weighted least squares
;
lin = LINFIT( x, y , MEASURE_ERRORS=sy, SIGMA=s_lin, YFIT=yfit)
;
print, "Least squares solution to P(Mimir) = a[0] + a[1] * P(Whittet)"
print, "  a[0] = ",lin[0],"+/-",s_lin[0]
print, "  a[1] = ",lin[1],"+/-",s_lin[1]
;
;	over plot
;
oplot, x, yfit, psym=-4, thick=3, linestyle = 2
;
;	force zero offset by forming ratios and uncertainties
;
ratio = y / x
s_ratio_per_ratio = sqrt( (sy / y)^2 + (sx / x)^2)
s_ratio = s_ratio_per_ratio * ratio
;
wt = 0.0d0
a = 0.0d0
for il = 0, nlines-1 do begin
	this_wt = 1./ (s_ratio[il]^2)
	a += ratio[il] * this_wt
	wt += this_wt
endfor
a /= wt
s_a = 1./sqrt(wt)
;
print, "Forcing zero intercept, slope = ",a,"+/-",s_a
;
;	plot same
;
oplot, [0.,5.], [0., 5.0*a], psym=-3, thick=4, linestyle=3
;
cursor, xx, yy, /wait
wait, 1
;
wdelete, 22
end
