pro dpc_find_sky, image, sky_mean, sky_sigma
;
;   compute sky for this image
;
;   uses region inside 0.3-0.7 of full, skipping
;   innermost 0.4-0.6 to avoid big clusters and
;   whatnot
;
;   DPC   20060102
;
outer0 = 0.15
outer1 = 0.85
inner0 = 0.25
inner1 = 0.75
;
image_size = SIZE(image)
ncol = image_size[1]
nrow = image_size[2]
;
;
xmin = 0 > FLOOR(outer0*ncol)
xmax = FLOOR(outer1*ncol) < (ncol-1)
;
xban0 = 0 > FLOOR(inner0*ncol)
xban1 = FLOOR(inner1*ncol) < (ncol-1)
;
ymin = 5 > FLOOR(outer0*nrow)
ymax = FLOOR(outer1*nrow) < (nrow-1)
;
yban0 = 0 > FLOOR(inner0*nrow)
yban1 = FLOOR(inner1*nrow) < (nrow-1)
;
;   area
;
npix = (xmax-xmin+2)*(ymax-ymin+2) - (xban1-xban0+1)*(yban1-yban0+1)
;
nvec = 0L
sky = fltarr(npix)
for ix = xmin, xmax do begin
    for iy = ymin, ymax do begin
       if(ix lt xban0 or ix gt xban1) then begin
            if(iy lt yban0 or iy gt yban1) then begin
                if(image[ix,iy] gt -1.0e5 and image[ix,iy] lt 1.0e5) then begin
                   sky[nvec] = image[ix,iy]
                   nvec++
                endif
            endif
       endif
    endfor
endfor
sky = sky[0:nvec-1]
;
zz = median_filtered_mean(sky)
sky_mean = zz[0]
sky_sigma = zz[1]
;
;print, "sky mean, sigma = ",zz[0], zz[1]
;
end
