pro dpc_parse_HWP_list
;
;	reads selected images, parses into HWP PA lists,
;	writes batch files to run tp_image_combine on the lists
;	launches the batch job
;
;	DPC	20080618
;		20081102 changed to non-MFM to preserve photometry (MFM doesn't)
;
;-----------------------------------------------------------------
hwp_list = [0,22.3,45.2,67.5,89.8,112.7,135.0,157.3,180.2,202.5,224.8,247.7,$
            270.0,292.3,315.2,337.5]
list_list = strarr(16)
;
in_list = dialog_pickfile(Title="Select GPIPS Input Polarimetry Images",/MULTIPLE_FILES)
;
n_files = N_ELEMENTS(in_list)
if(n_files le 1) then stop
print, "Found ",n_files," files"
;
;	parse input path for GP number
;
header = headfits(in_list[0])
name = STRTRIM(FXPAR(header, 'OBJECT'),2)
;
;	get rid of blanks
;
ind = STRPOS(name,' ')
if(ind ne -1) then begin
	parts = STRSPLIT(name,' ',count=count,/extract)
	name = parts[0] + parts[1]
endif
;
;	and lower case
;
name = STRUPCASE(name)
print, "Field name = ",name
;
;	output directory
;
out_dir = dialog_pickfile(Title="Select Output Directory",/directory)
;
;	for each HWP, build the list of images in each band
;
for ihwp = 0, 15 do begin
	;print, " Starting HWP ",ihwp," loop"
	nlist = 0
	;
	;	open the list file
	;
	list_list[ihwp] = out_dir + STRING(format='(a,"_HWP_",I2.2)',name,ihwp) + '_list.dat'
	openw, lu_list, list_list[ihwp], /GET_LUN
	;
	for ifile = 0, n_files-1 do begin
		header = headfits(in_list[ifile])
		;print, "read image header for ",FILE_BASENAME(in_list[ifile])
		;
		this_hwp = FXPAR(header, 'MB_HWP')
		;print, "     checking file ",ifile," named ",FILE_BASENAME(in_list[ifile])," has MB_HWP = ",this_hwp
		;time = FXPAR(header, 'EXPTIME')
		;
		;print, "header filter name and integration time are ",filter,"  ",time
		;;
		if(abs(this_hwp - hwp_list[ihwp]) gt 0.3) then begin
			;print, "  --- this HWP doesn't match this HWP loop designation, skipping this image"
			goto, the_end
		endif
		;if(this_time ne time) then goto, the_end
		;
		;	ok, is correct hwp add pathname to list
		;
		printf, lu_list, STRTRIM(in_list[ifile],2)
		nlist++
		the_end:
	endfor
	;
	print, "For HWP ",ihwp,', found ',nlist," images to coadd"
	FREE_LUN, lu_list
	;
	list_list[ihwp] = FILE_BASENAME(list_list[ihwp])
endfor
;
;	all lists built, prep batch file
;
final_list = name + '_final_list.dat'
path_batch = out_dir + name + '_GPIPS-coadd_Batch.dat'
openw, lu_batch, path_batch, /GET_LUN
openw, lu_final, out_dir+final_list, /GET_LUN
;
printf, lu_batch, '; Batch file generated automatically by Coadd_GPIPS_field.pro on '
printf, lu_batch, '; ',SYSTIME()
cddir = out_dir
printf, lu_batch, "cd;",cddir
journal_path = 'journal_batch_' + name + '.dat'
printf, lu_batch, "journal;",journal_path

for ihwp = 0, 15 do begin
	coadd_image = name + '_PA_' + STRING(format='(i2.2)',ihwp) + '_coadd.fits'
	mfm_image = name + '_PA_' + STRING(format='(i2.2)',ihwp) + '_coadd_MFM.fits'
	printf, lu_batch, 'pol_image_combine_no_sky;',list_list[ihwp],';',coadd_image,';12'
	;
	;	add to final list
	;
	printf, lu_final, coadd_image
endfor
;
;	prep final coadd of all of these
;
FREE_LUN, lu_final
final_coadd = name + '_full_coadd.fits'
printf, lu_batch, 'pol_image_combine_no_sky;',final_list,';',final_coadd,';16'
;
;
printf, lu_batch, 'journal'
FREE_LUN, lu_batch
;
;	batch launched via system call
;
CALL_PROCEDURE, 'batch', path_batch
;
print, "done!"
end