function fix_low_pixels, this_input_image, TPS=tps
;
;	while respecting bad pixels, finds low pixels and
;	replaces them with local mean
;
;	if TPS set, use thin plate spline
;
if(KEYWORD_SET(tps)) then tps = 1 else tps = 0
;
;	is input_image an array or a pointer?
;
s = SIZE(this_input_image, /TYPE)
if(s eq 10) then begin
	is_pointer = 1
	input_image = (*this_input_image)
endif else begin
	is_pointer = 0
	input_image = this_input_image
endelse
;
bpm = (1.0 - ((input_image + 1.0e5) < 1.0)) < 1.0	; 0 where not bad pix, 1 where bad pix
;
;;  old version - 7/7/14 - doesn't account for gradients and such
;model =  model_background(input_image, SKY=sky, /LOW_ORDER,	SIGMA = sigma)
;
model = median(input_image, 13)
diff = input_image - model
zz = median_filtered_mean(diff[0:*:10,0:*:10])
sigma = zz[1]
;
yy = median_filtered_mean(input_image[0:*:10,0:*:10])
sky = yy[0]
;
;	don't let sigma fall below 4
;
sigma >= 4.0
;
;	find all pixels below 4 sigma threshold, but not bad pixels
;
low_pix_map = (1.0 - ((diff + 4.0*sigma) < 1.0)) < 1.0	; 1 where low pix
low_pix_map *= (1.0 - bpm)
;
;	for all bad pixels
;
if(tps eq 0) then begin
	;
	; replace with mean
	;
	return_image = input_image * (1.0 - low_pix_map) + low_pix_map * sky
endif else begin
	;
	;	or TPS
	;
	ns = SIZE(input_image)
	nx = ns[1]
	ny = ns[2]
	;
	;   create output array
	;
	return_image = input_image
	;
	bad_pix_value = -1.0e6
	;
	;   locate all bad pixel values in input array
	;
	ind_bad = WHERE(low_pix_map eq 1 or ~FINITE(input_image),nbad)
	if(nbad eq 0) then begin
	    ;
	    ;  no bad pixels, return
	    ;
	    return, return_image
	endif
	;
	;   interpret into set of (x,y) bad pixel pointers
	;
	ybad = fix(ind_bad / nx)
	xbad = ind_bad - nx * ybad
	;
	;----------------------
	;
	;   for each bad pixel
	;
	ibad = 0L
	for ibad = 0L, nbad-1L do begin
	    ;
	    ;  collect the nearest 31 pixels that have good data
	    ;
	    ix0 = 0 > (xbad[ibad]-5)
	    ix1 = (nx-1) < (xbad[ibad]+5)
	    iy0 = 0 > (ybad[ibad]-5)
	    iy1 = (ny-1) < (ybad[ibad]+5)
	    ;
	    inx = ix1 - ix0 + 1
	    iny = iy1 - iy0 + 1
	    ;
	    dist = fltarr(inx*iny)
	    ind = lonarr(inx*iny)
	    ;
	    nd = 0
	    for ix = ix0, ix1 do begin
	       for iy = iy0, iy1 do begin
	         if(input_image[ix,iy] ne bad_pix_value and finite(input_image[ix,iy]) and $
	         		low_pix_map[ix,iy] ne 1) then begin
	          ind[nd] = iy*nx + ix
	          dist[nd] = sqrt((ix-xbad[ibad])^2 + (iy-ybad[ibad])^2)
	          nd++
	         endif
	       endfor
	    endfor
	    ;
	    ;   if nd is zero, then the entire region was deleted --
	    ;   set pixel value to zero in this case.
	    ;
	    if(nd gt 0) then begin
	        ind = ind[0:nd-1]
	        dist = dist[0:nd-1]       ; trim to real values only
	        ;
	        ;  sort by dist
	        ;
	        ind_sort = SORT(dist)
	        ;
	        ;  for the nearest up to 31 values...
	        ;
	        max_ind = 31 < nd
	        ;
	        ;   if there are not at least 7 points, replace pixel with image mean
	        ;
	        if(max_ind ge 7) then begin
	            xpoint = intarr(max_ind)
	            ypoint = intarr(max_ind)
	            zpoint = dblarr(max_ind)
	            ;
	            for i = 0, max_ind-1 do begin
	                ;
	                ; unpack indices and load x, y, z vectors
	                ;
	                ypoint[i] = fix(ind[ind_sort[i]]/nx)
	                xpoint[i] = ind[ind_sort[i]] - nx * ypoint[i]
	                zpoint[i] = input_image[xpoint[i],ypoint[i]]
	            endfor
	            ;
	            ;  perform the TPS fit - return only the missing data value
	            ;
	            xs = xbad[ibad]
	            ys = ybad[ibad]
	            znew = GRID_TPS(xpoint, ypoint, zpoint, NGRID=2, START=[xs,ys], DELTA=1)
	            ;
	            ;  put this in the output array if it is within reason
	            ;
	            zvalue = znew[0,0]
	            if(zvalue le sky-10.0*sigma) then zvalue = sky
	            if(zvalue ge max(zpoint)+3.0*sigma) then zvalue = sky
	        endif else zvalue = sky
	        ;
	    endif else zvalue = sky
	    return_image[xbad[ibad],ybad[ibad]] = zvalue
	endfor
endelse
;
;
if(is_pointer eq 0) then begin
	return, return_image
endif else begin
	ptr_return_image = PTR_NEW(return_image)
	PTR_FREE, this_input_image
	return, ptr_return_image
endelse
;
end
