pro g_fit_and_remove_stars, ng, image, resid_image, x, y, mag, errmag, sky, star_id, $
	group_vector, PSF_model, PSF_gauss, PSF_rad, PSF_mag, sum_resid, $
		phpadu, ronois, fit_rad, PRINT=jprint, MESSAGE1=message1, IPLOT=iplot, $
		ISAVE=isave, MESSAGE2=message2, RESID_HEADER=resid_header, $
		RESID_FILENAME=resid_filename, idpc, status, NO_NSTAR=no_nstar
;
;	combines the action of the PSF generator, nstar (the fitter), and substar (the remover)
;	to operate on all groups in an image to fit and remove all stars
;
;	DPC	20080722
;
;		20090402 - added flag to skip nstar fitting
;
;----------------------------------------
if(N_ELEMENTS(jprint) eq 0) then iprint = 0 else iprint = jprint
;
;iprint = 1
;isave = 1
;iplot = 1
;
if(N_ELEMENTS(iplot) eq 0) then iplot = 0
if(N_ELEMENTS(isave) eq 0) then isave = 0
if(N_ELEMENTS(message1) eq 0) then message1 = ''
if(N_ELEMENTS(message2) eq 0) then message2 = ''
if(N_ELEMENTS(resid_header) eq 0) then resid_header = ''
if(N_ELEMENTS(resid_filename) eq 0) then resid_filename = 'none'
if(N_ELEMENTS(no_nstar) eq 0) then no_nstar = 0
;
;print, "in G_FIT_AND_REMOVE_STARS, no_nstar = ",no_nstar
;print, "... iprint, iplot, isave = ",iprint, iplot, isave
;if(no_nstar eq 1) then begin
;	print," no_nstar = ",no_nstar
;;	stop
;endif
;
;help, no_nstar
;stop
;
nstars = N_ELEMENTS(x)
;
output_group_vector = group_vector * 0
;
ngroups = N_ELEMENTS(UNIQ(group_vector,SORT(group_vector)))
;
;ngroups = MAX(group_vector)
;group_histo = histogram(group_vector,min=0,REVERSE_INDICES=R)
;
if(iprint) then begin
	print, "entered g_fit_and_remove_stars.pro with ",ngroups," groups to process"
;	print, "number of stars in each group = ",group_histo
endif
;
;max_group = max(group_histo)
;if(iprint) then print, "maximum stars in any group = ",max_group
;if(iprint) then print, "group_vector = ",group_vector
;
;	for each group, compute its PSF
;
xfit = dblarr(nstars)
yfit = dblarr(nstars)
magfit = dblarr(nstars)
errmagfit = dblarr(nstars)
chifit = dblarr(nstars)
skyfit = dblarr(nstars)
ifit = 0L
;
subimage = image
;
;	ngroups
;
for igroup = 0, ngroups-1 do begin
	ind = WHERE(group_vector eq igroup,count)
	if(count ne 0) then begin
		xc = x[ind]
		yc = y[ind]
		id = star_id[ind]
		magsg = mag[ind]
		skyg = sky[ind]
		errmagg = errmag[ind]
		group_histo = [igroup]
		;
		if(iprint) then print, "Group ",igroup," has ",count," stars"
		;
		mean_x = MEAN(x[ind])
		mean_y = MEAN(y[ind])
		;
		PSF_residuals = g_make_PSF(ng, mean_x, mean_y, PSF_model, PSF_gauss, PSF_rad, PSF_mag, sum_resid, status)
		;
		;	test for bad PSF
		;
		if(status eq -1) then begin
			print, "ERROR in g_fit_and_remove_stars: call to g_make_PSF returned with status=-1"
			;
			;	need to exit and reset NG down one (if possible)
			;
			goto, the_end
		endif
		;
		;print, "made new PSF at (x,y) = ",mean_x, mean_y," with PSF_mag = ",PSF_mag
		;
		 mkhdr, hdr, PSF_residuals         ;Create a minimal FITS header
		 sxaddpar, hdr, 'PHPADU', phpadu, 'Photons per Analog Digital Unit'
		 sxaddpar, hdr, 'RONOIS', ronois, 'Readout Noise'
		 sxaddpar, hdr, 'PSFRAD', PSF_rad, 'Radius where PSF is defined (pixels)'
		 sxaddpar, hdr, 'FITRAD', fit_rad, 'Fitting Radius'
		 sxaddpar, hdr, 'PSFMAG', PSF_mag, 'PSF Magnitude'
		 sxaddpar, hdr, 'GAUSS1', PSF_gauss[0], 'Gaussian Scale Factor'
		 sxaddpar, hdr, 'GAUSS2', PSF_gauss[1], 'Gaussian X Position'
		 sxaddpar, hdr, 'GAUSS3', PSF_gauss[2], 'Gaussian Y Position'
		 sxaddpar, hdr, 'GAUSS4', PSF_gauss[3], 'Gaussian Sigma: X Direction'
		 sxaddpar, hdr, 'GAUSS5', PSF_gauss[4], 'Gaussian Sigma: Y Direction'
		 ;
		 ;	report gaussians that are too narrow - DPC 20090307
		 ;
		 if(PSF_gauss[3] lt 0.35) then begin
		 	print, "G_FIT_AND_REMOVE_STARS: Warning - sigx = ",PSF_gauss[3]," for group ",igroup
		 endif
		 if(PSF_gauss[4] lt 0.35) then begin
		 	print, "G_FIT_AND_REMOVE_STARS: Warning - sigy = ",PSF_gauss[4]," for group ",igroup
		 endif
		 ;
		 sxaddpar, hdr, 'SUMRESID', sum_resid, 'Flux sum of resid array inside PSFRAD'
		 ;
		 x_w_psf = mean_x
		 y_w_psf = mean_y
		 sxaddpar, hdr, 'X_W_PSF', x_w_psf, 'Flux-Weighted X position'
		 sxaddpar, hdr, 'Y_W_PSF', y_w_psf, 'Flux-Weighted Y position'
		 ;
;		 if(igroup eq 0) then begin
;			 file_out = dialog_pickfile(Title="name of PSF fits file")
;			 writefits, file_out, PSF_residuals, hdr
;		 endif
		;
		;		if "NO_NSTAR" selected, skip over fitting
		;
		if(no_nstar eq 0) then begin
			;
			;	PSF-fit this group using this model PSF for this group's position
			;
			debug = 0
			silent=2
			varsky=1
			if(iprint eq 1) then begin
				if(igroup eq 0) then begin
					kprint = dialog_pickfile(title=message1)
					save_print = kprint
				endif else kprint = save_print
			endif else kprint=0
			;
	;		for ii= 0, N_ELEMENTS(id)-1 do begin
	;			print, "on input to nstar - id, x, y, mags, magerr, group = ",id[ii],xc[ii], yc[ii], $
	;							magsg[ii], errmagg[ii], group_histo[0]
	;		endfor

			ibk=0
			;
			;if(igroup eq 58 and idpc eq 1) then ibk = 1 else ibk = 0
			;
			if(iprint) then print," before g_nstar, ind = ",ind," magsg = ",magsg," errmag = ",errmagg," xc = ",xc," yc = ",yc
			;
			g_nstar,image,id,xc,yc,magsg,skyg,group_histo,phpadu,ronois,PSF_residuals,hdr,DEBUG=debug, $
	          errmagg,iter,chisq,peak,PRINT=kprint,SILENT=silent, VARSKY = varsky, ibk
	        ;
	        if(iprint) then print," after g_nstar, magsg = ",magsg," errmag = ",errmagg," xc = ",xc," yc = ",yc
	        ;
	        ;if(igroup eq 58 and idpc eq 1) then print, "on exit from nstar - id, x, y, mags, group = ",id,xc, yc, magsg, group
	        ;
	;        for ii= 0, N_ELEMENTS(id)-1 do begin
	;			print, "on exit from nstar - id, x, y, mags, magerr, group = ",id[ii],xc[ii], yc[ii], $
	;							magsg[ii], errmagg[ii], group_histo[0]
	;		endfor
			;
	        ;	save results (only well fit stars)
	        ;
	        ;new_count = N_ELEMENTS(errmagg)
        endif else begin
        	nxc = N_ELEMENTS(xc)
        	chisq = fltarr(nxc) + 1.0
        	if(iprint) then print, "in G_FIT_AND_REMOVE: skipped g_nstar for group ",igroup
        endelse
	    ;
	    good_ind = WHERE(abs(abs(magsg) - 99.99) gt 0.1,mcount)
	        ;
	    if(mcount eq 0) then if(iprint) then print, "Mcount = ",mcount," magsq = ",magsg
	;        if(mcount eq 0) then begin
	;        	print, "magsg = ",magsg
	;        	print, "errmagg = ",errmagg
	;        endif
	        ;
        if(mcount ne 0) then begin
        	;
        	xc = xc[good_ind]
        	yc = yc[good_ind]
        	magsg = magsg[good_ind]
        	errmagg = errmagg[good_ind]
        	chisq = chisq[good_ind]
        	skyg = skyg[good_ind]
        	;
	        jstar = 0
	        xsub = dblarr(mcount)
	        ysub = dblarr(mcount)
	        msub = dblarr(mcount)
	        ;
	        for istar = 0, mcount-1 do begin
	        	if(magsg[good_ind[istar]] le 25.0) then begin
	        		xfit[ifit] = xc[istar]
					yfit[ifit] = yc[istar]
					magfit[ifit] = magsg[istar]
					errmagfit[ifit] = errmagg[istar]
					chifit[ifit] = chisq[istar]
					skyfit[ifit] = skyg[istar]
					output_group_vector[ifit] = igroup
					;
					;	save for subtraction
					;
					xsub[jstar] = xc[istar]
					ysub[jstar] = yc[istar]
					msub[jstar] = magsg[istar]
					;
					ifit++
					jstar++
				endif
			endfor
			;
			;	remove all fitted stars to see what is left
			;
			;
			idsub = -1
			verbose = 0
			;
			;	check that there is actually a real star to remove - DPC 20081101
			;
			dpc_check = N_ELEMENTS(xsub)
			if(dpc_check ge 1) then begin
				k_good = WHERE(xsub ne 0.0,dpc_count)
				if(dpc_count ge 1) then begin			; only remove stars within image
					;
					g_substar,subimage,xsub,ysub,msub,idsub,PSF_residuals, hdr,VERBOSE = verbose
					;
				endif
			endif
			;
		endif else if(iprint) then print, "No good stars fit in group ", igroup
     endif else if(iprint) then print, "Group ",igroup," has no stars - ???"
endfor
;
;print, "All Groups fit. Total number of good stellar fits = ",ifit
;
;	plot of mag vs errmag
;
if(iplot eq 1) then begin
	window, 22
	xmin = FLOOR(MIN(magfit[WHERE(magfit gt 0.0)]))
	xmax = CEIL(MAX(magfit))
	plot, magfit, errmagfit, xtitle='mag',ytitle='error mag', psym=3, xrange=[xmin,xmax],$
		xstyle=1, yrange=[-0.05,0.75],ystyle=1
endif
;
;	save star-subracted image
;
if(isave eq 1) then begin
	if(resid_filename eq 'none') then begin
		;
		;	save residuals, but not in pre-named file
		;
		if(message2 eq '') then message2 = "image name for star subtracted image"
		sub_out = dialog_pickfile(title=message2)
	endif else begin
		sub_out = resid_filename
	endelse
	;
	if(N_ELEMENTS(resid_header) le 1) then begin
		header = ''					; dummy header
	endif else begin
		header = resid_header
	endelse
	;
	writefits, sub_out, subimage, header
endif
;
resid_image = subimage
;
;	put results back into input arrays
;
if(ifit gt 0) then begin
	x = xfit[0:ifit-1]
	y = yfit[0:ifit-1]
	mag = magfit[0:ifit-1]
	;
	ind_x = WHERE(mag lt -10.0, nbad)	; there could be stars brighter than PSF basis
	if(nbad gt 0) then begin
		print, 'ERROR - hugely negative mag (< -10) in g_fit_and_remove_stars'
		stop
	endif
	;
	sky = skyfit[0:ifit-1]
	errmag = errmagfit[0:ifit-1]
	;
	group_vector = output_group_vector[0:ifit-1]
endif else begin
	x[*] = 0.0
	y[*] = 0.0
	mag[*] = 99.90
	sky[*] = 0.0
	errmag[*] = 99.9
	;
	;print, "On exit, incoming group_vector = ",group_vector
	group_vector = -1
endelse
;
the_end:
end