function model_background, input_image, SKY=sky, LOW_ORDER=low_order, $
		SIGMA = sigma, NOROWF = rowf, NOCOLF = colf, TPS = this_tps, MCS = this_mcs, $
		STEP_SIZE = this_step
;
;	computes a smooth model of the background of an
;	image, and subtracts that from the
;	original image (putting back the mean sky)
;	to better reveal stars
;
;	if /LOW_ORDER, then only constant term is fit for - to enable removal
;					of gravity waves in backgrounds
;
;	DPC	20090701 	original code
;		20091008	added SKY keyword to return the sky value
;		20100123	added column shape removal after main fit
;		20110316	accepts input_image as pointer, returns same, if so, else image
;		20140720  added switches to turn off row, column flattening
;		20140801  added Thin Plate Spline (TPS) modeling (and switch) instead of polynomial modeling
;
;--------------------------------------------------
; switch for row, column flattening
;
if(N_ELEMENTS(rowf) eq 0) then row_flatten = 1 else row_flatten = 0
if(N_ELEMENTS(colf) eq 0) then col_flatten = 1 else col_flatten = 0
if(N_ELEMENTS(this_tps) eq 0) then do_tps = 0 else do_tps = 1
if(N_ELEMENTS(this_mcs) eq 0) then do_mcs = 0 else do_mcs = 1
if(N_ELEMENTS(this_step) eq 0) then step = 33 else step = this_step
;
debug = 0
;
if(debug ne 0) then print, 'echo: TPS= ',do_tps,', row_flatten=',row_flatten,', col_flatten=',col_flatten
;
;row_flatten = 1     ; or 1 to turn it on
;col_flatten = 1
;
if(N_ELEMENTS(low_order) eq 0) then low_order = 0 else low_order = 1
;
;	parses image into a grid of zones, and does mfm
;	for each zone, then fits a surface
;
is_ptr = SIZE(input_image,/TYPE)
if(is_ptr eq 10) then begin
	ptr_image = input_image
	;
	;	leave as pointer
	;
endif else begin
	;
	;	all others, turn into new heap variable, to be released later
	;
	ptr_image = PTR_NEW(input_image)
endelse
;
im_size = SIZE((*ptr_image))
xsize = im_size[1]
ysize = im_size[2]
;
;	if larger than one image, compute buffer size around edges
;
if(xsize gt 1024) then begin
	xborder = (xsize - 1024) / 2.0 + 5
endif else begin
	xborder = 2
endelse
;
if(ysize gt 1026) then begin
	yborder = (ysize - 1026) / 2.0 + 5
endif else begin
	yborder = 9
endelse
;
;step = 33
if(debug ne 0) then begin
    print, "Zone width = ",step
    time2 = systime(/seconds)
endif
;
;	how many zones? (ideally each zone would be 30x30 pixels
;			with adequate gaps between zones, of ~30 pixels)
;
nzone_x = FLOOR(FIX((xsize - 2*xborder) / step) )
nzone_y = FLOOR(FIX((ysize - 2*yborder)/ step) )
;ngap_x = FLOOR(((xsize - 2*xborder) - nzone_x * step) / float(nzone_x - 1))
;ngap_y = FLOOR(((ysize -2*yborder) - nzone_y * step) / float(nzone_y - 1))
x = indgen(nzone_x)
y = indgen(nzone_y)
;
if(debug ne 0) then begin
  print,'xsize, ysize = ',xsize, ysize
  print, 'nzone_x, nzone_y = ',nzone_x, nzone_y
;  print, 'ngap_x, ngap_y = ',ngap_x, ngap_y
endif
;
;
grid = fltarr(3, nzone_x * nzone_y)
raw_grid = fltarr(nzone_x, nzone_y)
x_grid = fltarr(nzone_x, nzone_y)
y_grid = fltarr(nzone_x, nzone_y)
sky_noise = fltarr(nzone_x * nzone_y)
;
for ix = 0, nzone_x-1 do begin
	x0 = FLOOR(xborder + ix * step + step / 2.)
	x1 = CEIL((x0 + step-1) < (xsize - xborder))
	mean_x = float(x1 + x0)/2.0
	for iy = 0, nzone_y-1 do begin
		y0 = FLOOR(yborder + iy * step + step / 2.)
		y1 = CEIL((y0 + step-1) < (ysize - yborder))
		mean_y = float(y0 + y1) / 2.0
		;
		ptr_subarray = PTR_NEW((*ptr_image)[x0:x1:3,y0:y1:3])
		vec = REFORM((*ptr_subarray)[*,*])
		ind = where(FINITE(vec) and vec gt -1.0e5,ngood)
		if(ngood gt 0) then zz = median_filtered_mean(vec[ind]) else zz = [-1.0e6,-1.0e6]
		;
;		print, "ix, iy, zz = ", ix, iy, zz[0], zz[1]
;		print, "x0, x1, y0, y1 = ",x0,x1,y0,y1
		;
		ixx = ix * nzone_y + iy
		grid[0,ixx] = mean_x
		grid[1,ixx] = mean_y
		grid[2,ixx] = zz[0]
		;
		sky_noise[ixx] = zz[1]
		;
		raw_grid[ix, iy] = zz[0]
		x_grid[ix, iy] = mean_x
		y_grid[ix, iy] = mean_y
		;
		PTR_FREE, ptr_subarray
	endfor
endfor
;
;	remove 0;s and other big deviations
;
zz = median_filtered_mean(REFORM(grid[2,*]))
mean_sky = zz[0]
;
;stop
;
ss = median_filtered_mean(sky_noise)
sigma = ss[0]
;
;	if low_order fit desired, return now
;
if(low_order) then begin
	ptr_output_image = PTR_NEW((*ptr_image))
	goto, the_end
endif
;
ind = WHERE(abs(grid[2,*] - zz[0])/zz[1] le 4.0,ngood)
if(ngood gt 0) then begin
	grid = grid[*,ind]
endif
;
; set up output image
; 
ptr_output_image = PTR_NEW((*ptr_image))
  (*ptr_output_image) *= 0.0
;
;  select TPS or polynomial modeling
;  
if(do_tps eq 1 or do_mcs eq 1) then begin
  ; 
  ; TPS
  ; 
  zvec = reform(grid[2,*])
  xvec = reform(grid[0,*])
  yvec = reform(grid[1,*])
  ;
 if(debug ne 0) then begin
    time0 = systime(/seconds)
    print, "Time to build grid (MFM) = ",time0-time2
  endif
  ;
  ; MCS has priority
  ;
  if(do_mcs eq 1) then begin
    ptr_sub_image = PTR_NEW(MIN_CURVE_SURF(zvec, xvec, yvec, GS=[6,6],$
        BOUNDS=[0, 0, xsize-1, ysize-1]))
  endif else begin
      ;
      ; TPS otherwise
      ; 
      ptr_sub_image = PTR_NEW(MIN_CURVE_SURF(zvec, xvec, yvec, /TPS, GS=[6,6],$
        BOUNDS=[0, 0, xsize-1, ysize-1]))
  endelse
;  (*ptr_output_image) = REBIN(sub_image, xsize, ysize)
  (*ptr_output_image) = CONGRID((*ptr_sub_image), xsize, ysize)
  if(debug ne 0) then begin
      time1 = systime(/seconds)
      print, "TPS time = ",time1-time0
  endif
;  stop
  PTR_FREE, ptr_sub_image
endif else begin
  ;
  ; polynomial fit
  ;
  ;	grid computed, fit to surface
  ;
  degree = ((nzone_x > nzone_y) - 4) < 4
  ;
  Result = SFIT( grid, degree, KX=coeffs, /IRREGULAR, /MAX_DEGREE )
  ;print, 'coeffs = ',coeffs
  ;
  ;surface, raw_grid, x_grid, y_grid, charsize=2
  ;cursor, xx, yy, /wait
  ;wait, 0.5
  ;
  ;	use coefficients to compute surface
  ;
  ;new_surface = fltarr(nzone_x, nzone_y)
  ;for ix = 0, nzone_x-1 do begin
  ;	for iy = 0, nzone_y-1 do begin
  ;		this_x = x_grid[ix,iy]
  ;		this_y = y_grid[ix,iy]
  ;		;
  ;		new_surface[ix,iy] = mean_sky + coeffs[0] + this_y * coeffs[1] + coeffs[2] * this_y * this_y + coeffs[3] * this_y^3 + $
  ;							 		coeffs[4] * this_y^4 + $
  ;							 this_x * coeffs[5] + this_x * this_y * coeffs[6] + this_x * this_y * this_y * coeffs[7] + $
  ;							 		this_x * this_y * this_y * this_y * coeffs[8] + $
  ;							 this_x * this_x * coeffs[9] + this_x * this_x * this_y * coeffs[10] + $
  ;							 		this_x * this_x * this_y * this_y * coeffs[11] + $
  ;							 this_x * this_x * this_x * coeffs[12] + this_x * this_x * this_x * this_y * coeffs[13] + $
  ;							 this_x * this_x * this_x * this_x * coeffs[14]
  ;	endfor
  ;endfor
  ;surface, new_surface, x_grid, y_grid, charsize = 2
  ;cursor, xx, yy, /wait
  ;wait, 0.5
  ;
  ;	difference
  ;
  ;surface, raw_grid - new_surface, x_grid, y_grid, charsize=2
  ;cursor, xx, yy, /wait
  ;wait, 0.5
  ;
  ;	compute output image
  ;
  ;
  ;for ix = 0, xsize-1 do begin
  ;	this_x = float(ix)
  ;	for iy = 0, ysize-1 do begin
  ;		this_y = float(iy)
  ;		output_image[ix,iy] = coeffs[0] + this_y * coeffs[1] + coeffs[2] * this_y * this_y + coeffs[3] * this_y^3 + $
  ;							 		coeffs[4] * this_y^4 + $
  ;							 this_x * coeffs[5] + this_x * this_y * coeffs[6] + this_x * this_y * this_y * coeffs[7] + $
  ;							 		this_x * this_y * this_y * this_y * coeffs[8] + $
  ;							 this_x * this_x * coeffs[9] + this_x * this_x * this_y * coeffs[10] + $
  ;							 		this_x * this_x * this_y * this_y * coeffs[11] + $
  ;							 this_x * this_x * this_x * coeffs[12] + this_x * this_x * this_x * this_y * coeffs[13] + $
  ;							 this_x * this_x * this_x * this_x * coeffs[14]
  ;	endfor
  ;endfor
  ;
  ;	vector version
  ;
  this_y = findgen(ysize)+0.5
  for ix = 0, xsize-1 do begin
  	this_x = float(ix)+0.5
  	;
  	(*ptr_output_image)[ix,*] = coeffs[0] + this_y * coeffs[1] + coeffs[2] * this_y * this_y + coeffs[3] * this_y^3 + $
  							 		coeffs[4] * this_y^4 + $
  							 this_x * coeffs[5] + this_x * this_y * coeffs[6] + this_x * this_y * this_y * coeffs[7] + $
  							 		this_x * this_y * this_y * this_y * coeffs[8] + $
  							 this_x * this_x * coeffs[9] + this_x * this_x * this_y * coeffs[10] + $
  							 		this_x * this_x * this_y * this_y * coeffs[11] + $
  							 this_x * this_x * this_x * coeffs[12] + this_x * this_x * this_x * this_y * coeffs[13] + $
  							 this_x * this_x * this_x * this_x * coeffs[14]
  endfor
endelse
;
;	compute background subtracted image, but put back the mean sky afterward
;
ptr_out_good_mask = PTR_NEW( (*ptr_image) + 0.5e6)
(*ptr_out_good_mask) >= 0.0
(*ptr_out_good_mask) <= 1.0
;
ptr_out_bad_mask = PTR_NEW(1.0-(*ptr_out_good_mask))
;
ptr_final_output = PTR_NEW( (*ptr_output_image) )
(*ptr_output_image) = (((*ptr_image) - (*ptr_output_image)) + mean_sky) * (*ptr_out_good_mask)
(*ptr_output_image) += (-1.0e6) * (*ptr_out_bad_mask)
;
;	--------------------------------------------------------------
;
if(row_flatten eq 1 or col_flatten eq 1) then begin
  ;
  ;		fit and remove the vertical wave present in most images
  ;
  column_means = MEDIAN( (*ptr_output_image), DIMENSION=2)
  ;
  ;	look for any bad columns in quadrants
  ;
  ind_neg = WHERE(column_means lt -1000,n_neg)
  if(n_neg gt 0) then begin
  	for icol = 0, n_neg-1 do begin
  		this_ind = ind_neg[icol]
  		col_vec = REFORM( (*ptr_output_image)[this_ind,*])
  		ind_good = WHERE(col_vec gt -1000.0, n_good)
  		if(n_good gt 0) then begin
  			column_means[this_ind] = MEDIAN(col_vec[ind_good])
  		endif else begin
  			column_means[this_ind] = 0.0
  		endelse
  	endfor
  endif
  ;
  the_column_sky = MEDIAN(column_means)
  ;
  if(col_flatten eq 1) then begin
      column_scales = column_means / the_column_sky
      column_excess = column_means - the_column_sky
  endif else begin
      column_scales = fltarr(N_ELEMENTS(column_means))
      column_scales += 1.0
      column_excess = fltarr(N_ELEMENTS(column_means))
      column_excess *= 0.0
  endelse
  ;
  ;   subtract deviations from mean sky, or multiply by scaled column deviations?
  ;   
  ptr_back_image = PTR_NEW((*ptr_output_image))
  bs = SIZE((*ptr_back_image))
  ;
  ; subtract excesses, only
  ; 
  for ic = 0, bs[1]-1 do (*ptr_back_image)[ic,*] -= column_excess[ic]
  ;   
  ;	scale each column by its mean (if the col_flatten flag was set)
  ;
;  for ic = 0, bs[1]-1 do (*ptr_back_image)[ic,*] *= column_scales[ic]
  ;-----------------------------------------------------
  ;	fast way:
  if(row_flatten eq 1) then begin
      row_mean = MEDIAN( (*ptr_back_image), DIMENSION=1)
  ;
  ;	slow way (but accurate?): - DPC 20100219 - didn't solve row pulldown, so going back to fast way
  ;row_mean = fltarr(bs[2])
  ;for ir = 0, bs[2]-1 do begin
  ;	zz = median_filtered_mean( REFORM(back_image[*,ir]) )
  ;	row_mean[ir] = zz[0]
  ;endfor
  ;-----------------------------------------------------
  ;
    the_row_sky = MEDIAN(row_mean)
  ;
    row_excess = row_mean - the_row_sky
  endif else begin
      row_excess = fltarr(bs[2])
      row_excess *= 0.0
  endelse
  ;
  ;			establish excesses - (*ptr_back_image) holds excesses to be subtracted from flattened image
  ;
  for ic = 0, bs[1]-1 do (*ptr_back_image)[ic,*] = row_excess[*] + column_excess[ic]
;  for ic = 0, bs[1]-1 do (*ptr_back_image)[ic,*] = row_mean[*] * column_scales[ic]
;  (*ptr_back_image) -= the_row_sky
  ;
  ;	correct output image while masked
  ;
  (*ptr_output_image) -= (*ptr_back_image) * (*ptr_out_good_mask)
  ;
  ;	and recompute sky
  ;
  ptr_y = PTR_NEW( model_background((*ptr_output_image), SKY=mean_sky, SIGMA=sigma, /LOW_ORDER) )
  PTR_FREE, ptr_back_image, ptr_y
endif
;
PTR_FREE, ptr_out_good_mask, ptr_out_bad_mask
HEAP_GC, /PTR
;
the_end:
;
sky = mean_sky
;
if(is_ptr eq 10) then begin
	;
	;	came in with pointer, leave with pointer (depends on calling program to release it)
	;
	return, ptr_output_image
endif else begin
	;
	;	came in with image, leave with image, clearing remaining pointers
	;
	output_image = (*ptr_output_image)
	PTR_FREE, ptr_output_image, ptr_image
	HEAP_GC, /PTR
	return, output_image
endelse
;
end
