Function mp_GPIPS_astrometry,Event,input_image, input_header,TWO_MASS = two_mass, F17 = f17, $
			ARRAY_2MASS=array_2mass, RESULT_FLAG=result_flag
  ;
  ;  Try to get astrometric solution for a GPIPS field:
  ;  1)  Use DAOFIND to get a star list for the GPIPS field
  ;  2)  Fetch the 2MASS-PSC data for that field (from RA/Dec in header)
  ;  3)  Pick the three brightest stars in each list and take ratio of their distances
  ;  4)  If they match, check a couple more stars
  ;      If they don't match, use different stars until a match is found
  ;
  ;  MP  20080611  Original Version
  ;  MP  20080722  Can process lists of images, only calls 2MASS when telescope moves significantly,
  ;                reference pixel now always (513,513).
  ;  MP  20080724  Added option to output 2MASS data with /TWO_MASS keyword.
  ;  MP  20080910  Added F17 platescale option, improved error message, and can accept any image dimensions
  ;  MP  20080929  Changed fwhm selection criteria to account for number of stars and relative error
  ;  MP  20081027  If an image truly fails, skip image instead of ending program
  ;  MP  20081030  Changed image cleanup, improved status messages, generalized 2MASS search size, OS independant
  ;  MP  20081112  Added Vizier server redundancy
  ;
  ;	 DC  20081204  changed to function to return new header

  if keyword_set(f17) then n=20 else n=10	;initial number of stars to use in 2MASS and found lists
  n_inc = 0									; number of time n has been increased
  a=0										; first star in DAOFIND list to use

  time1 = systime(1)
  ;
  num_images = size(input_image_name) & num_images = num_images[1]
  ;
    time_flag = 0
    time_start = systime(1)
    ;one step can write multiple HISTORY entries, so clear the history now, and put it back in later after cleaning
    old_history = sxpar(input_header,'HISTORY');save old history for later reentry
    sxdelpar,input_header,'HISTORY';delete old history

    ;Clean up image a little bit
    working_image = input_image > (-1)*500

    ;Find sky background statistics
    silent=1
    g_sky, working_image, skymode, sky_sigma, mfm_mean, mfm_stdev, SILENT=silent
    ;MMM, working_image, skymod, sky_sigma
    if (sky_sigma eq -1.0) then sky_sigma = 6.

    nstars_save = 0
    best_fwhm = 0.0
    low_err = 4.0
    grade_save = 0.0
    if keyword_set(f17) then fwhm_set = dindgen(31)/5.0+4.0 else fwhm_set = dindgen(31)/6.0+2.
    ;
    ;	get FWHM the fast way
    ;
    debug=0
    sat_limit=5000
    print_text, Event, "Starting g_get_FWHM"
    g_get_FWHM, working_image, mean_FWHM, sig_FWHM, sat_limit, debug
    dt = systime(1) - time_start
    print_text, Event, "Completed g_get_FWHM with value = "+STRING(format='(F7.3)',mean_FWHM)+$
    		" after "+STRING(format='(I)',dt)+" sec"
    ;
    ;for q = 0,30 do begin
    ;
    roundlim = [-0.8,0.6]
	sharplim = [0.2,0.8]
    scale = 7.0
    floop = 0L
    refind:
      level = mfm_mean+scale*mfm_stdev
      MP_FIND, working_image, single_x4, single_y4, single_flux4, sharp4, roundd4, $
      	level, (2.0 > mean_FWHM), roundlim, sharplim, relerr, /SILENT

      ind_weird_zeros = where((single_x4 eq 0.0) and (single_y4 eq 0.0),zeros_count)
      ;sometimes find returns all zeros, ignore this result
      nstars = size(single_x4)
      nstars = nstars[1]
      ;
      ;	if too many stars, bump scale
      ;
      if(nstars gt 200) then begin
      	scale = scale * 3.0
      	floop++
      	if(floop lt 10) then goto, refind
      endif
      ;
      print_text, Event, "Found "+$
      	STRING(format='(I)',nstars)+" stars after "+STRING(format='(I)',floop)+$
      	" loops. Final level = "+STRING(format='(F8.1)',level)
      ;
      ;grade = nstars*(relerr)
      ;print,fwhm_set[q],grade,nstars,relerr,nstars*relerr,nstars*sqrt(relerr)
      ;if (nstars-zeros_count gt nstars_save) then begin
      ;if (grade gt grade_save) then begin
        nstars_save = nstars
        stars_x_tot = single_x4
        stars_y_tot = single_y4
        stars_flux_tot = single_flux4
        low_err = relerr
        ;grade_save = grade
        best_fwhm = mean_FWHM
        ;best_fwhm = fwhm_set[q]
      ;endif
    ;endfor

    ;ignore stars near edges
    imsize = size(working_image)
    xsize = imsize[1]
    ysize = imsize[2]
    ind_edge = where((stars_x_tot lt (xsize/20)) or (stars_x_tot gt (0.95*xsize)) or (stars_y_tot lt (ysize/20)) or (stars_y_tot gt (0.95*ysize)))
    if (ind_edge[0] ne -1) then stars_flux_tot(ind_edge) = 1	; will cause them to have ridiculous magnitudes

    ;Convert rough fluxes to rough magnitudes
    stars_mag_tot = -2.5d*alog10(stars_flux_tot)+23.5;roughly
    sort_ind = sort(stars_mag_tot)

    ;sort magnitudes and associated x and y pixel coordinates by magnitude
    stars_x_tot = stars_x_tot(sort_ind)
    stars_y_tot = stars_y_tot(sort_ind)
    stars_mag_tot = stars_mag_tot(sort_ind)

    ;remove any star with a magnitude of 23.5 (bad star)
    bad_mag = where(stars_mag_tot eq 23.5,bad_mag_count)
    star_count = size(stars_mag_tot) & star_count = star_count[1]

    ;get estimated coordinates of center of image
    crpix1 = sxpar(input_header,'CRPIX1')
    crval1 = sxpar(input_header,'CRVAL1')
    crdelt1 = sxpar(input_header,'CRDELT1')
    crota1 = sxpar(input_header,'CROTA1')
    cfint1 = sxpar(input_header,'CFINT1')
    crpix2 = sxpar(input_header,'CRPIX2')
    crval2 = sxpar(input_header,'CRVAL2')
    if (crpix1 eq 1. and crval1 eq 1. and crdelt1 eq 1. and crota1 eq 0. and cfint1 eq 1.) then begin;pristine image
      tel_ra = 15.d*ten(strsplit(sxpar(input_header,'TELRA'),':',/EXTRACT)); telescope RA in degrees
      tel_dec = ten(strsplit(sxpar(input_header,'TELDEC'),':',/EXTRACT)); telescope Dec in degrees
    endif else begin; possibly altered, estimate coordinates at center of image
      tel_ra = (crpix1-(xsize/2))*(0.578/3600)+crval1
      tel_dec = (crpix2-(ysize/2))*(-0.578/3600)+crval2
    endelse

    ;Get corresponding 2MASS H magnitudes and coordinates
      if keyword_set(F17) then begin
        info = mp_QueryVizier('2MASS-PSC', [tel_ra,tel_dec], [3*(xsize/1024),3*(ysize/1026)], /CANADA, constraint='Qflg==AAA')
        test = size(info)
        if test[0] eq 0 then begin; server down, try France
          info = mp_QueryVizier('2MASS-PSC', [tel_ra,tel_dec], [3*(xsize/1024),3*(ysize/1026)], constraint='Qflg==AAA')
          test = size(info)
          if test[0] eq 0 then begin
            print_text,Event,'No Vizier servers available at this time'
            refined_header = header
            return, refined_header
          endif
        endif
      endif else begin
        info = mp_QueryVizier('2MASS-PSC', [tel_ra,tel_dec], [9*(xsize/1024),9*(xsize/1026)], /CANADA, constraint='Qflg==AAA')
        test = size(info)
        if test[0] eq 0 then begin; server down, try France
          info = mp_QueryVizier('2MASS-PSC', [tel_ra,tel_dec], [9*(xsize/1024),9*(ysize/1026)], constraint='Qflg==AAA')
          test = size(info)
          if test[0] eq 0 then begin
            print_text, Event,'No Vizier servers available at this time'
            refined_header = header
            return, refined_header
          endif
        endif
      endelse

    tel_ra_old = tel_ra
    tel_dec_old = tel_dec					;save these to compare with next image

    filter = strtrim(sxpar(input_header,'FILTNME1'),2)
    case filter of;regardless of filter, just call it extracted_Hmag
      'J':extracted_Hmag = info.Jmag
      'H':extracted_Hmag = info.Hmag
      'Ks1':extracted_Hmag = info.Kmag
      else:begin
        print,'No filter information found for image ' + file_basename(input_image_name[image])
        extracted_Hmag = info.Hmag;default to H magnitudes
        end
    endcase
    mass_sort_ind = sort(extracted_Hmag)
    mass_ra_tot = info(REVERSE(mass_sort_ind)).RAJ2000/15.d; also convert it to decimal hours
    mass_dec_tot = info(REVERSE(mass_sort_ind)).DEJ2000
    mass_H_tot = extracted_Hmag((REVERSE(mass_sort_ind)))
    mass_size = size(mass_H_tot) & mass_size = mass_size[1]

    if (mass_size gt 50) then begin; exclude bad 2MASS magnitudes
      bad_mass = where(mass_h_tot lt 4 or mass_h_tot gt 12,bad_mass_range); if enough stars, ignore dim ones
      if (mass_size-bad_mass_range lt 30) then bad_mass = where(mass_h_tot lt 4 or mass_h_tot gt 13,bad_mass_range)
      if (mass_size-bad_mass_range lt 30) then bad_mass = where(mass_h_tot lt 4 or mass_h_tot gt 14,bad_mass_range)
    endif else begin
      bad_mass = where(mass_h_tot lt 4)
    endelse

    if (bad_mass[0] ne -1) then mass_H_tot[bad_mass] = 0.0;set bad stars to 0 mag
    mass_sort2 = reverse(sort(mass_h_tot)); bad stars put at the end of the list
    mass_H_tot = mass_H_tot[mass_sort2]
    mass_ra_tot = mass_ra_tot[mass_sort2]
    mass_dec_tot = mass_dec_tot[mass_sort2]
    bad_mass2 = where(mass_H_tot eq 0.0,bad_mass_count);count how many stars have 0 mag
    ;remove bad stars from 2MASS lists
    mass_H_tot = mass_H_tot[0:mass_size-bad_mass_count-1]
    mass_ra_tot = mass_ra_tot[0:mass_size-bad_mass_count-1]
    mass_dec_tot = mass_dec_tot[0:mass_size-bad_mass_count-1]

    new_n:
    if keyword_set(F17) then begin
      star_sep01 = 96.6; min star seperations (arcsec), will reset seperations if more stars are needed
      star_sep02 = 96.6
      star_sep12 = 96.6
    endif else begin
      star_sep01 = 311.4; min star seperations (arcsec), will reset seperations if more stars are needed
      star_sep02 = 311.4
      star_sep12 = 311.4
    endelse

    found_stars = (star_count-bad_mag_count < n); use only n brightest stars
    stars_x = stars_x_tot(0:found_stars-1)
    stars_y = stars_y_tot(0:found_stars-1)
    stars_mag = stars_mag_tot(0:found_stars-1)

    mass_stars = (mass_size-bad_mass_count < n); only use a maximum of n stars
    ;mass_stars = (mass_size-bad_mass_count < 50); only use a maximum of n stars
    mass_H_tot = reverse(mass_H_tot);put brightest stars at the beginning of list
    mass_ra_tot = reverse(mass_ra_tot)
    mass_dec_tot = reverse(mass_dec_tot)
    mass_H = mass_H_tot[0:mass_stars-1]
    mass_ra = mass_ra_tot[0:mass_stars-1]
    mass_dec = mass_dec_tot[0:mass_stars-1]

    ;find the distance from every 2MASS star from every other 2MASS star
    distarr = fltarr(mass_stars,mass_stars)
    for coll=0,mass_stars-1 do begin
      for row=0,mass_stars-1 do begin
        gcirc,1,mass_ra[coll],mass_dec[coll],mass_ra[row],mass_dec[row],stardist
        distarr[coll,row] = stardist
      endfor
    endfor

    ;define ratios the ratios as
    ;ratio1 = dist01/dist02
    ;ratio2 = dist01/dist12
    ;ratio3 = dist02/dist12

    ;take three bright stars (a,b,c) in detector image and find the ratio of the distances between the stars
    star_retry = 0
    closer:
    if (star_retry gt 0) then begin;can't find stars at this seperation, allow closer matches
      if keyword_set(F17) then begin
        star_sep01 = star_sep01 - 15.5
        star_sep02 = star_sep02 - 15.5
        star_sep12 = star_sep12 - 15.5
      endif else begin
        star_sep01 = star_sep01 - 50
        star_sep02 = star_sep02 - 50
        star_sep12 = star_sep12 - 50
      endelse
    endif

    ;new_stars_post_failure:;loops back here if these three stars do not work
    ;defined 'a' at beginning of program
    b=a+1;b=1
    c=b+1;c=2

    reject_stars:;if the three stars are bad for any reason go back here and try again
    if (c gt found_stars-1) then begin;for this a and b, all c exhausted, reset c and increment b
      b = b+1
      c = 0
      if (b gt found_stars-1) then begin;for this a, all b exhausted, reset b and increment a
        b = 0
        a = a+1
        if (a gt found_stars-1) then begin;exhausted all a, b, c allow closer matches
          star_retry = star_retry+1
          if (star_retry eq 5) then begin
            if (n_inc eq 4) then begin
              ;print,'Coordinate matching unsuccessful, skipping image'
              goto, skip_image
            endif else begin
              ;print,'Increasing number of 2MASS stars by 15 for ' + file_basename(input_image_name[image])
              n=n+10
              n_inc = n_inc+1
              goto,new_n
            endelse
          endif
          ;print,'Coordinate matching unsuccessful, allowing closer stars for ' + file_basename(input_image_name[image])
          a = 0
          goto,closer
        endif
      endif
    endif

    if (a eq b) or (b eq c) or (a eq c) then begin
      c=c+1
      goto,reject_stars;two of the stars are the same
    endif

    dist01 = sqrt((stars_x[a]-stars_x[b])^2+(stars_y[a]-stars_y[b])^2)
    dist02 = sqrt((stars_x[a]-stars_x[c])^2+(stars_y[a]-stars_y[c])^2)
    dist12 = sqrt((stars_x[b]-stars_x[c])^2+(stars_y[b]-stars_y[c])^2)
    if (dist01 lt star_sep01) or (dist12 lt star_sep12) or (dist02 lt star_sep02) then begin;insufficient seperation
      c=c+1
      goto,reject_stars
    endif

    ;We know Mimir is roughly sky true, so the star with the highest y coordinate should have the
    ;highest declination, and vice versa.
    high_y = max([stars_y[a],stars_y[b],stars_y[c]])
    dec_high_ind = where([stars_y[a],stars_y[b],stars_y[c]] eq high_y)
    low_y = min([stars_y[a],stars_y[b],stars_y[c]])
    dec_low_ind = where([stars_y[a],stars_y[b],stars_y[c]] eq low_y)

    ratio1 = dist01/dist02
    ratio2 = dist01/dist12
    ;ratio3 = dist02/dist12

    matches ='0,0,0'
    grades=[1]

    ;For each star ss, assume it is the brightest (star 0) and that star rr is the next brightest (star 1)
    ;Find the ratio of distance from rr to ss with the distance from ss to each other star (pp).
    ;If the ratios are with 1% of the pixel ratios save those indicies, if not move on.

    For ss=0,mass_stars-1 do begin
      for rr=0,mass_stars-1 do begin
        if (rr ne ss) then begin; don't use one star twice
          distssrr = distarr(ss,rr)
          for pp=0,mass_stars-1 do begin
            if ((rr ne pp) and (ss ne pp)) then begin; don't use one star twice
              distsspp = distarr(ss,pp)
              mass_ratio1 = distssrr/distsspp
              ;if (abs(ratio1-mass_ratio1)/ratio1 lt 0.01) then begin;possible match, check other ratios
              if (abs(ratio1-mass_ratio1) lt 0.01) then begin;
                distrrpp = distarr(rr,pp)
                mass_ratio2 = distssrr/distrrpp
                ;mass_ratio3 = distsspp/distrrpp
                ;if (abs(ratio2-mass_ratio2)/ratio2 lt 0.01) and (abs(ratio3-mass_ratio3)/ratio3 lt 0.01) then begin
                ;if (abs(ratio2-mass_ratio2)/ratio2 lt 0.01) then begin
                if (abs(ratio2-mass_ratio2) lt 0.01) then begin
                  ;Check if declination order is correct
                  check_dec = [mass_dec(ss),mass_dec(rr),mass_dec(pp)];,mass_dec(nn)]
                  check_dec_high = where(max(check_dec) eq check_dec)
                  check_dec_low = where(min(check_dec) eq check_dec)
                  if ((check_dec_high eq dec_high_ind) and (check_dec_low eq dec_low_ind)) then begin
                    matches = [matches,string(ss,rr,pp)]
                    ;grade_new = 100^3.*(abs(ratio1-mass_ratio1)/ratio1)*(abs(ratio2-mass_ratio2)/ratio2)*(abs(ratio3-mass_ratio3)/ratio3)
                    grade_new = 100^3.*(abs(ratio1-mass_ratio1)/ratio1)*(abs(ratio2-mass_ratio2)/ratio2)
                    grades = [grades, grade_new]
                  endif
                endif
              endif
            endif
          endfor
        endif
      endfor
    Endfor

    ;print,'Testing possible solutions'

    new_solution:
    grades_ind = sort(grades);grades_ind[0] has the best grade
    grades = grades[grades_ind]; put best grade at the beginning of the list
    matches = matches[grades_ind]; put best match at beginning of the list
    star_indicies = FIX(STRSPLIT(matches[0],' ',/EXTRACT))

    if (grades[0] eq 1.0) then begin
      c=c+1
      goto,reject_stars
    endif

    solution_ra = [mass_ra[star_indicies[0]],mass_ra[star_indicies[1]],mass_ra[star_indicies[2]]]*15
    solution_dec = [mass_dec[star_indicies[0]],mass_dec[star_indicies[1]],mass_dec[star_indicies[2]]]
    solution_x = [stars_x[a],stars_x[b],stars_x[c]]
    solution_y = [stars_y[a],stars_y[b],stars_y[c]]
    starast, solution_ra, solution_dec, solution_x, solution_y, cd, HDR = input_header
    sxdelpar,input_header,'HISTORY';delete any previous history entries from STARAST.pro
    ;figure out coords for center pixel, and use it as reference pixel
    imsize = size(working_image)
    new_refx = float(imsize[1])/2.;allows for fractional pixels
    new_refy = float(imsize[2])/2.
    xyad, input_header, new_refx, new_refy, new_refra, new_refdec;new_refra already in degrees
    solution_ra = [new_refra,15.d*mass_ra[star_indicies[0]],15.d*mass_ra[star_indicies[1]]]
    solution_dec = [new_refdec,mass_dec[star_indicies[0]],mass_dec[star_indicies[1]]]
    solution_x = [new_refx,stars_x[a],stars_x[b]]
    solution_y = [new_refy,stars_y[a],stars_y[b]]
    starast, solution_ra, solution_dec, solution_x, solution_y, cd, HDR = input_header
    ast_hist_entry = sxpar(input_header,'HISTORY');save these history entries in case they are good
    sxdelpar,input_header,'HISTORY';delete history entries...again
    ;
    ;if taking a long time, run this program to use all stars once
    if time_flag eq 0 then begin
      time_now = systime(1)
      if time_now - time_start ge 1 then begin; if 3 min have passed try this
        time_flag = 1
        mp_GPIPS_astrometry_soft, stars_x_tot, stars_y_tot, stars_mag_tot, mass_h_tot, mass_ra_tot, mass_dec_tot, f17, input_header, cd_soft, flag
        if flag eq 1 then begin; success!
          goto, good_solution
        endif
      endif
    endif
    ;
    ;check coordinate description (CD) matrix elements
    if keyword_set(f17) then begin; F17 camera
      if ((abs(3600.0*cd[0]) lt 0.17) or (abs(3600.0*cd[0]) gt 0.19) or (abs(3600.0*cd[3]) lt 0.17) or (abs(3600.0*cd[3]) gt 0.19)  or (abs(cd[1]) gt 1.0e-6) or (abs(cd[2]) gt 1.0e-6)) then begin
          ;something is wrong with this solution, reject and try another
          grades[0] = 1.0; flunk
          goto,new_solution
        endif
    endif else begin; F5 camera
      if ((abs(3600.0*cd[0]) lt 0.55) or (abs(3600.0*cd[0]) gt 0.6) or (abs(3600.0*cd[3]) lt 0.55) or (abs(3600.0*cd[3]) gt 0.6) or (abs(cd[1]) gt 1.0e-6) or (abs(cd[2]) gt 1.0e-6)) then begin
        ;something is wrong with this solution, reject and try another
        grades[0] = 1.0; flunk
        goto,new_solution
      endif
    endelse

    good_solution:
    ;print,'Zeroth order plate scale:',3600.0*cd[3]
    sxaddpar,input_header,'CTYPE1','RA---TAN'
    sxaddpar,input_header,'CTYPE2','DEC--TAN'
    sxaddpar,input_header,'GPCOORD',1,'Status of GPIPS Astrometry';add header entry signifying this level of astrometry has been completed
    hist_size = size(old_history) & hist_size = hist_size(1)
    for hist_ind=0,hist_size-1 do begin
      sxaddpar,input_header,'HISTORY',old_history(hist_ind)
    endfor
    sxaddpar,input_header,'HISTORY',ast_hist_entry
    ;
    ;refine the astrometry
    ;print,input_image_name[image]
    refined_header = mp_refine_gpips_astrometry(mass_h_tot,mass_ra_tot,mass_dec_tot,working_image,input_header,skymod,sky_sigma,best_fwhm,f17)
    ;refined_header = mp_refine_gpips_astrometry_completely_new(mass_h_tot,mass_ra_tot,mass_dec_tot,working_image,input_header,skymod,sky_sigma,best_fwhm,f17)
    ;
    ;filename = file_basename(input_image_name[image])
    ;output_filename = output_directory + filename
    ;
    ;writefits,output_filename,input_image,refined_header
    ;
    if keyword_set(TWO_MASS) then begin
     ;If /twomass option selected, ouput 2MASS data to a file
     ;file = FILE_BASENAME(input_image_name[image], '.fits')
     ;path = output_directory + '2MASS_data_' + file + '.dat'
     num_mass_stars = size(info)
     num_mass_stars = num_mass_stars[1]
     ;
     array_2mass = strarr(num_mass_stars+1)
     ;
     extast,refined_header,astr					;get astrometry structure
     ;OPENW, Unit, path, /GET_LUN
       array_2mass[0]='   x       y       RA(deg)     Dec        L         B        mJ   s_mJ    mH   s_mH    mK   s_mK'
       for ii=0,num_mass_stars-1 do begin
         a = info[ii].raj2000
         d = info[ii].dej2000
         if(d ge 0.0) then dsign = '+' else dsign = '-'            ; dpc
         ad2xy, a, d, astr, x, y;convert to pixel coordinates
         glactc,a,d,2000,gl,gb,1
         if (x lt xsize) and (x gt 0) and (y lt ysize) and (y gt 0) then begin
           j = string(info[ii].jmag,format='(1x,f6.3)')
           je = string(info[ii].e_jmag,format='(1x,f5.3)')
           h = string(info[ii].hmag,format='(1x,f6.3)')
           he = string(info[ii].e_hmag,format='(1x,f5.3)')
           k = string(info[ii].kmag,format='(1x,f6.3)')
           ke = string(info[ii].e_kmag,format='(1x,f5.3)')
           outline = string(float(x),format='(f7.2)') + string(float(y),format='(1x,f7.2)') + $
                   string(a,format='(1x,f11.6)') + string(dsign,abs(d),format='(1x,a1,f9.6)') + $
                   string(gl,format='(1x,f9.5)') + string(gb,format='(1x,f9.5)') + $
                   j+je+h+he+k+ke
           array_2mass[ii+1] = outline
         endif
       endfor
   endif
   result_flag = 1
   goto, end_good
   skip_image:
   		result_flag = 0 ;bad image skips to here
   end_good:
   ;
  time2 = systime(1)
  ;print,'took ',strtrim(string(time2-time1),2),' seconds to process ',strtrim(string(num_images),2),' images'
  ;print,'took ',strtrim(string((time2-time1)/num_images),2),' seconds per images'
;
;
return, refined_header
end