function mp_refine_gpips_astrometry,mass_h,mass_ra,mass_dec,working_image,input_header,skymod,sky_sigma,best_fwhm,f17
;
;  Using the rough astrometry, already calculated, generate many plate scale coefficients and least-squares them
;
;  20081103  MP  Original Version
;

  details = 0; set to 1 to see plot of absolute error per star

  retry_astrometry=0; blank tag, will be read out of header
  old_history = sxpar(input_header,'HISTORY');save old history for later reentry
  sxdelpar,input_header,'HISTORY';delete old history

  ;f17 variable is 1 if in f/17 camera, 0 if in f/5 camera
  ;if f17 somehow didn't get set, assume f/5
  f17check = size(f17,/type)
  if f17check eq 0 then f17 = 0

  ;cleanup previous astrometry from header
  sxdelpar,input_header,'GPCD1_1'
  sxdelpar,input_header,'GPCD1_2'
  sxdelpar,input_header,'GPCD1_3'
  sxdelpar,input_header,'GPCD1_4'
  sxdelpar,input_header,'GPCD1_5'
  sxdelpar,input_header,'GPCD2_1'
  sxdelpar,input_header,'GPCD2_2'
  sxdelpar,input_header,'GPCD2_3'
  sxdelpar,input_header,'GPCD2_4'
  sxdelpar,input_header,'GPCD2_5'

  check_rough = sxpar(input_header,'GPCOORD')
  case check_rough of; check what astrometry has already been done
    0:  begin
          ;print,'First-cut astrometry not yet completed, ending program'
          return,input_header
        end
    1:  begin
          ;print,'Starting astrometric refinement...'
        end
    2:  begin
          ;print,'Astrometric refinement already completed'
          if (retry_astrometry = 0) then begin
            print,'Ending program'
            return,input_header
          endif else begin
            ;print,'Starting further astrometric refinement...'
          endelse
        end
  endcase

  ;bad_ind = where(mass_h lt 4, bad_mass)
  bad_ind = where(mass_h lt 8,bad_mass)
  if bad_mass ne 0 then mass_h[bad_ind] = 100; these will be put at end of the sorted list
  mass_ind = sort(mass_h);sort by decreasing brightness
  mass_h = mass_h[mass_ind]
  mass_dec = mass_dec[mass_ind]; in degrees
  mass_ra = 15.d*mass_ra[mass_ind]; in degrees

  ;count 2MASS stars, excluding bad stars
  num_mass_stars = size(mass_h) & num_mass_stars = num_mass_stars(1)
  star_count = num_mass_stars-bad_mass; the number of stars to use, ignoring known bad stars

  known_ra = mass_ra[0:star_count-1]
  known_dec = mass_dec[0:star_count-1]
  known_h = mass_h[0:star_count-1]
  star_x_est = dblarr(star_count)
  star_y_est = dblarr(star_count)
  star_x_exact = dblarr(star_count)
  star_y_exact = dblarr(star_count)

  ;calculating pixel positions from rough astrometry
  for i=0,star_count-1 do begin
    adxy, input_header, known_ra[i], known_dec[i], x, y
    star_x_est[i] = x
    star_y_est[i] = y
    cntrd, working_image, x, y, xcen, ycen, best_fwhm, /SILENT
    if xcen eq -1. or ycen eq -1. then begin; did not find star
      cntrd, working_image, x, y, xcen, ycen, best_fwhm, EXTENDBOX = 3, /SILENT
      if xcen eq -1. or ycen eq -1. then begin; did not find star
        cntrd, working_image, x, y, xcen, ycen, best_fwhm, EXTENDBOX = 5, /SILENT
        if xcen eq -1. or ycen eq -1. then begin; did not find star
          star_x_exact[i] = -1
          star_y_exact[i] = -1
        endif else begin
          star_x_exact[i] = xcen
          star_y_exact[i] = ycen
        endelse
      endif else begin
        star_x_exact[i] = xcen
        star_y_exact[i] = ycen
      endelse
    endif else begin
      star_x_exact[i] = xcen
      star_y_exact[i] = ycen
    endelse
  endfor

  ;eliminate any centers that were not found and marked with -1
  ind_bad_find = where(star_x_exact eq -1 or star_y_exact eq -1, bad_count)
  exact_sort = reverse(sort(star_x_exact))
  star_x_exact = star_x_exact[exact_sort]
  star_y_exact = star_y_exact[exact_sort]
  star_x_est = star_x_est[exact_sort]
  star_y_est = star_y_est[exact_sort]
  known_ra = known_ra[exact_sort]
  known_dec = known_dec[exact_sort]
  ;
  nstars = star_count-bad_count
  star_x_exact = star_x_exact[0:nstars-1]
  star_y_exact = star_y_exact[0:nstars-1]
  star_x_est = star_x_est[0:nstars-1]
  star_y_est = star_y_est[0:nstars-1]
  known_ra = known_ra[0:nstars-1]
  known_dec = known_dec[0:nstars-1]
  ;at this point the exact coordinates and 2MASS RA and Dec are matched up

;  ;pick a star and use it's exact pixel location and 2MASS coordinates as a reference point
;  ref_x = sxpar(input_header,'CRPIX1')
;  ref_y = sxpar(input_header,'CRPIX2')
;  ref_ra = sxpar(input_header,'CRVAL1')
;  ref_dec = sxpar(input_header,'CRVAL2')

  cd11 = 0.d
  cd12 = 0.d
  cd21 = 0.d
  cd22 = 0.d
  a = 0.d;ref RA
  d = 0.d;ref Dec
  for ind1=0, nstars-1 do begin
    for ind2=ind1+1, nstars-1 do begin
      for ind3=ind2+1, nstars-1 do begin;work through all sets of three stars
        dist12 = sqrt((star_x_exact[ind1]-star_x_exact[ind2])^2 + (star_y_exact[ind1]-star_y_exact[ind2])^2)
        dist23 = sqrt((star_x_exact[ind2]-star_x_exact[ind3])^2 + (star_y_exact[ind2]-star_y_exact[ind3])^2)
        dist13 = sqrt((star_x_exact[ind1]-star_x_exact[ind3])^2 + (star_y_exact[ind1]-star_y_exact[ind3])^2)
        if dist12 gt 500 and dist23 gt 500 and dist13 gt 500 then begin
          solution_ra = [known_ra[ind1],known_ra[ind2],known_ra[ind3]]
          solution_dec = [known_dec[ind1],known_dec[ind2],known_dec[ind3]]
          solution_x = [star_x_exact[ind1],star_x_exact[ind2],star_x_exact[ind3]]
          solution_y = [star_y_exact[ind1],star_y_exact[ind2],star_y_exact[ind3]]
          starast, solution_ra, solution_dec, solution_x, solution_y, cd
          xyad,input_header,513.d,513.d,a,d;find RA and Dec of reference pixel
          solution_ra = [a,known_ra[ind2],known_ra[ind3]]
          solution_dec = [d,known_dec[ind2],known_dec[ind3]]
          solution_x = [513,star_x_exact[ind2],star_x_exact[ind3]]
          solution_y = [513,star_y_exact[ind2],star_y_exact[ind3]]
          starast, solution_ra, solution_dec, solution_x, solution_y, cd
          ;check coordinate description (CD) matrix elements
          if (f17 eq 1) then begin; F17 camera
            if ((abs(3600.0*cd[0]) lt 0.17) or (abs(3600.0*cd[0]) gt 0.19) or (abs(3600.0*cd[3]) lt 0.17) or (abs(3600.0*cd[3]) gt 0.19)  or (abs(cd[1]) gt 1.0e-6) or (abs(cd[2]) gt 1.0e-6)) then begin
              ;something is wrong with this solution, reject and try another
            endif else begin
              ;save these plate constants
              if cd11[0] eq 0 then begin
                a_arr = a
                d_arr = d
                cd11 = cd[0,0]
                cd12 = cd[1,0]
                cd21 = cd[0,1]
                cd22 = cd[1,1]
              endif else begin
                a_arr = [a_arr,a]
                d_arr = [d_arr,d]
                cd11 = [cd11,cd[0,0]]
                cd12 = [cd12,cd[1,0]]
                cd21 = [cd21,cd[0,1]]
                cd22 = [cd22,cd[1,1]]
              endelse
            endelse
          endif else begin; F5 camera
            if ((abs(3600.0*cd[0]) lt 0.55) or (abs(3600.0*cd[0]) gt 0.6) or (abs(3600.0*cd[3]) lt 0.55) or (abs(3600.0*cd[3]) gt 0.6) or (abs(cd[1]) gt 5.0e-6) or (abs(cd[2]) gt 5.0e-6)) then begin
              ;something is wrong with this solution, reject and try another
            endif else begin
            ;save these plate constants
              if cd11[0] eq 0 then begin
                a_arr = a
                d_arr = d
                cd11 = cd[0,0]
                cd12 = cd[1,0]
                cd21 = cd[0,1]
                cd22 = cd[1,1]
              endif else begin
                a_arr = [a_arr,a]
                d_arr = [d_arr,d]
                cd11 = [cd11,cd[0,0]]
                cd12 = [cd12,cd[1,0]]
                cd21 = [cd21,cd[0,1]]
                cd22 = [cd22,cd[1,1]]
              endelse
            endelse
          endelse
        endif
      endfor
    endfor
  endfor

; DPC fix
  vec = abs(cd11/cd22)
  n_vec = N_ELEMENTS(vec)
  ind_good = WHERE(FINITE(vec),count)
  if(count ne n_vec) then print, "some bad values ..."

  zz = median_filtered_mean(abs(cd11[ind_good]/cd22[ind_good]))
  ind = where(vec[ind_good] lt zz[0]+zz[1] and vec[ind_good] gt zz[0]-zz[1])
;
  ind = ind_good[ind]
  ; end DPC FIX
  ;
  temp = median_filtered_mean(cd11[ind]) & cd1_1 = temp[0] & cd1_1err = temp[1]
  temp = median_filtered_mean(cd12[ind]) & cd1_2 = temp[0] & cd1_2err = temp[1]
  temp = median_filtered_mean(cd21[ind]) & cd2_1 = temp[0] & cd2_1err = temp[1]
  temp = median_filtered_mean(cd22[ind]) & cd2_2 = temp[0] & cd2_2err = temp[1]
  temp = median_filtered_mean(a_arr[ind]) & crval1 = temp[0]
  temp = median_filtered_mean(d_arr[ind]) & crval2 = temp[0]

  ;get reference coordinates, reference pixel is (513,513)
  zz4 = median_filtered_mean(a_arr[ind])
  refra = zz4[0]
  zz5 = median_filtered_mean(d_arr[ind])
  refdec = zz5[0]

  ;fix platescale so that it is the same in x and y directions
  tot_main_scales = [abs(cd11[ind]),abs(cd22[ind])]
  zz2 = median_filtered_mean(tot_main_scales)
  cd1_1 = -1.d*zz2[0]
  cd1_1err = zz2[1]
  cd2_2 = zz2[0]
  cd2_2err = zz[1]
  tot_rot_scales = [abs(cd12[ind]),abs(cd21[ind])]
  zz3 = median_filtered_mean(tot_rot_scales)
  cd2_1 = -1.d*zz3[0]
  cd2_1err = zz3[1]
  cd1_2 = -1.d*zz3[0]
  cd1_2err = zz3[1]

  ;find original distance sum
  orig_sum = 0
  for star=0, nstars-1 do begin
    x1 = star_x_exact[star]
    y1 = star_y_exact[star]
    adxy, input_header, known_ra[star], known_dec[star], x2, y2
    orig_sum = orig_sum + sqrt((x1-x2)^2 + (y1-y2)^2)
  endfor

  ;small perturbations on platescales
  if f17 eq 0 then begin; f/5 camera
;    d_epsilon = 0.0000001;change on major plate scale
;    d_delta = 1e-8; change on minor plate scale
    d_epsilon = 0.00000001;change on major plate scale
    d_delta = 1e-9; change on minor plate scale
  endif else begin; f/17 camera
    d_epsilon = 3.125e-8
    d_delta = 3.125e-9
  endelse

  original_header = input_header
  cd1_1_old = cd1_1
  cd2_2_old = -1.d*cd1_1_old
  cd2_1_old = cd2_1
  cd1_2_old = cd2_1_old

  done = 0
  step = 1
  old_sum = orig_sum
  done_count = 0
  d_maj = 0
  d_min = 0
  while done eq 0 do begin
    case step of
      1:  begin
            ;increment major plate scale
            cd1_1_new = cd1_1_old + d_epsilon
            cd2_2_new = -1.d*cd1_1_new
            cd1_2_new = cd2_1_old
            cd2_1_new = cd1_2_new
            d_maj = d_maj+1
          end
      2:  begin
            ;increment minor plate scale
            cd1_1_new = cd1_1_old
            cd2_2_new = -1.d*cd1_1_new
            cd1_2_new = cd2_1_old + d_delta
            cd2_1_new = cd1_2_new
            d_min = d_min+1
          end
      3:  begin
            ;decrement major plate scale
            cd1_1_new = cd1_1_old - d_epsilon
            cd2_2_new = -1.d*cd1_1_new
            cd1_2_new = cd2_1_old
            cd2_1_new = cd1_2_new
            d_maj = d_maj-1
          end
      4:  begin
            ;decrement minor plate scale
            cd1_1_new = cd1_1_old
            cd2_2_new = -1.d*cd1_1_new
            cd1_2_new = cd2_1_old - d_delta
            cd2_1_new = cd1_2_new
            d_min = d_min-1
          end
    endcase
    sxaddpar, input_header, 'CD1_1', cd1_1_new
    sxaddpar, input_header, 'CD1_2', cd1_2_new
    sxaddpar, input_header, 'CD2_1', cd2_1_new
    sxaddpar, input_header, 'CD2_2', cd2_2_new
    sum = 0.d
    for star=0, nstars-1 do begin
      x1 = star_x_exact[star]
      y1 = star_y_exact[star]
      adxy, input_header, known_ra[star], known_dec[star], x2, y2
      sum = sum + (x1-x2)^2 + (y1-y2)^2
    endfor
    if sum lt old_sum then begin
      ;found a better solution
      done_count = 0
      old_sum = sum
      cd1_1_old = cd1_1_new
      cd1_2_old = cd1_2_new
      cd2_1_old = cd2_1_new
      cd2_2_old = cd2_2_new
    endif else begin
      ;found a worse solution, try something else
      step = step+1
      if step eq 5 then step = 1
      done_count = done_count+1
      if done_count eq 5 then begin
        ;tried everything, this is the best solution
        done = 1
        ;undo last change to constants
        sxaddpar, input_header, 'CD1_1', cd1_1_old
        sxaddpar, input_header, 'CD1_2', cd1_2_old
        sxaddpar, input_header, 'CD2_1', cd2_1_old
        sxaddpar, input_header, 'CD2_2', cd2_2_old
      endif
    endelse
  endwhile

  x2 = dblarr(nstars)
  y2 = dblarr(nstars)
  err = dblarr(nstars)
  sum2 = 0.d
  for star=0, nstars-1 do begin
      x1 = star_x_exact[star]
      y1 = star_y_exact[star]
      adxy, input_header, known_ra[star], known_dec[star], x22, y22
      x2[star] = x22
      y2[star] = y22
      err[star] = sqrt((x1-x22)^2 + (y1-y22)^2)
      sum2 = sum2 + (x1-x22)^2 + (y1-y22)^2
  endfor

  if details eq 1 then begin;plot absolute errors for good stars
    device, decomposed=0
    loadct,39,/silent
    plot,star_x_exact,star_y_exact,psym=1
    for star=0,nstars-1 do begin
      oplot,[star_x_exact[star]],[star_y_exact[star]],psym=1,color=err[star]*200/max(err)+50
      xyouts,[star_x_exact[star]+5],[star_y_exact[star]],strmid(strtrim(string(err[star]),2),0,5)
    endfor
  endif

  sxaddpar,input_header,'GPCOORD',2
  hist_size = size(old_history)
  for i=0, hist_size[1]-1 do begin
    sxaddpar,input_header,'HISTORY',old_history[i]
  endfor

return,input_header

end