function ppol_1star, Event, this_tolerance, header, trimmed_mass_array, trimmed_star_list, $
					  platescale_est, STATUS=status, idebug
;
;	1 star astrometry routine
;
;	astrometry of the last resort...
;
;	find brightest star in Mimir image, get all 2MASS stars within 45 arcsec, then
;	rank to get brightest, force match and force CD matrix
;
;	DPC	20090128
;
final_header = header
old_hist = sxpar(header, "HISTORY")
;
flux_mimir = trimmed_star_list[3,*]
ind_mimir = REVERSE(SORT(flux_mimir))
;
x_mimir = [trimmed_star_list[0,ind_mimir[0]]]
y_mimir = [trimmed_star_list[1,ind_mimir[0]]]
;
print_text, Event, "Brightest Mimir star located at (X,Y) = "+STRING(x_mimir,y_mimir,format='(I4,1x,I4)')
;
;	find 2mass stars within 45 arcsec
;
;	first, convert all 2MASS coordinates into "standard coordinates"
;
;		what is the reference for this image?
;
xsize = sxpar(header, 'NAXIS1')
ysize = sxpar(header, 'NAXIS2')
test_crval1 = sxpar(header, 'CRVAL1')
test_crpix1 = sxpar(header, 'CRPIX1')
if(test_crval1 eq test_crpix1) then begin
	;
	;	"raw" image, get notional center coordinates from telescope coords
	;
	tel_ra = 15.d*ten(double(strsplit(sxpar(header,'TELRA'),':',/EXTRACT))); telescope RA in degrees
	tel_dec = ten(double(strsplit(sxpar(header,'TELDEC'),':',/EXTRACT))); telescope Dec in degrees
	;
	xcen = ROUND(xsize / 2.0)
	ycen = ROUND(ysize / 2.0)
	plate_scale = platescale_est
endif else begin
	;
	;	otherwise, fetch from the header
	;
	tel_ra = sxpar(header, 'CRVAL1')
	tel_dec = sxpar(header, 'CRVAL2')
	xcen = sxpar(header, 'CRPIX1')
	ycen = sxpar(header, 'CRPIX2')
	;
	;	is there a CD matrix? if so, calc plate scale from it
	;
	cd_00 = sxpar(header, "CD1_1",COUNT=count)
	if(count gt 0) then begin
		plate_scale = 3600.0 * abs(cd_00)
	endif else begin
		plate_scale = platescale_est				; only used in region plotting
	endelse
endelse
;
crval1  = [ tel_ra, tel_dec ]
;
;	reset center to zero
;
crpix1 = [0.0d0, 0.0d0]
;
; Convert RA, Dec to Eta, Xi
;
mass_ra_tot = reform(trimmed_mass_array[0,*])
mass_dec_tot = reform(trimmed_mass_array[1,*])
mass_mag_tot = reform(trimmed_mass_array[3,*])
;
num_mass = N_ELEMENTS(mass_ra_tot)
;
map_type = 2			; 'TAN'
wcssph2xy, 15.0*mass_ra_tot, mass_dec_tot, eta, xi, map_type, $
         latpole = 0.0, crxy = crpix1, ctype=['RA---TAN', 'DEC--TAN'], $
         crval = crval1
;
;	convert 2mass to notional X,Y pixel coords
;
x_2mass = fltarr(num_mass)
y_2mass = x_2mass
;
for i2m = 0, num_mass-1 do begin
	x_2mass[i2m] = xcen - eta[i2m] * 3600.0 / plate_scale
	y_2mass[i2m] = ycen + xi[i2m] * 3600.0 / plate_scale
endfor
;
;	locate brightest 2mass stars closer than 45 arcsec
;
rad45 = 45.0 / plate_scale
;
best_mag = 24.0
;
for istar = 0, 0 do begin
	;
	this_x = x_mimir[istar]
	this_y = y_mimir[istar]
	;
	for i2 = 0, num_mass-1 do begin
		dx = this_x - x_2mass[i2]
		dy = this_y - y_2mass[i2]
		dist = sqrt(dx * dx + dy * dy)
		if(dist lt rad45) then begin
			mag = mass_mag_tot[i2]
			if(mag le 20.0 and mag gt 0.0) then begin
				if(mag lt best_mag) then begin
					best_mag = mag
					ind_2mass = i2
				endif
			endif
		endif
	endfor
endfor
;
; 	if no star found, issue fail
;
if(best_mag eq 24.0) then begin
	sxaddpar, final_header, 'GPCOORD', 0
	status=-1
	return, final_header
endif
;
;	do a one-star solution
;
CD1_1 = -0.000160748738927d0
CD1_2 =  -4.42855242028D-007
CD2_1 =  -4.41641689571D-007
CD2_2 =   0.000160791326418d0
;
SXADDPAR, final_header, "CD1_1", CD1_1, "Assumed in PPOL"
SXADDPAR, final_header, "CD1_2", CD1_2
SXADDPAR, final_header, "CD2_1", CD2_1
SXADDPAR, final_header, "CD2_2", CD2_2
;
SXADDPAR, final_header, "CRPIX1", this_x, '1 Star Astrometry match, PPOL'
SXADDPAR, final_header, "CRPIX2", this_y
;
SXADDPAR, final_header, "CRVAL1", 15.0*mass_ra_tot[ind_2mass], '1 Star Astrometry match, PPOL'
SXADDPAR, final_header, "CRVAL2", mass_dec_tot[ind_2mass]
;
n_old = N_ELEMENTS(old_hist)
for i=0, n_old-1 do begin
	sxaddpar, final_header, "HISTORY", old_hist[i]
endfor
;
sxaddpar, final_header, 'GPCOORD', 1
status=1
if(idebug) then print_text, Event, "Leaving 1 star solution with good status = 1"
return,final_header
end