function ppol_2star, Event, this_tolerance, header, trimmed_mass_array, trimmed_star_list, $
					  platescale_est, STATUS=status, idebug
;
;	2 star astrometry routine
;
;	astrometry of the last resort...
;
;	find brightest 2 stars in Mimir image, and try to
;	match to 2mass stars
;
;	DPC	20090124
;
final_header = header
old_hist = sxpar(header, "HISTORY")
;
flux_mimir = trimmed_star_list[3,*]
ind_mimir = REVERSE(SORT(flux_mimir))
;
x_mimir = [trimmed_star_list[0,ind_mimir[0]],trimmed_star_list[0,ind_mimir[1]]]
y_mimir = [trimmed_star_list[1,ind_mimir[0]],trimmed_star_list[1,ind_mimir[1]]]
;
;	find nearest 2mass stars
;
;	first, convert all 2MASS coordinates into "standard coordinates"
;
;		what is the reference for this image?
;
xsize = sxpar(header, 'NAXIS1')
ysize = sxpar(header, 'NAXIS2')
test_crval1 = sxpar(header, 'CRVAL1')
test_crpix1 = sxpar(header, 'CRPIX1')
if(test_crval1 eq test_crpix1) then begin
	;
	;	"raw" image, get notional center coordinates from telescope coords
	;
	tel_ra = 15.d*ten(double(strsplit(sxpar(header,'TELRA'),':',/EXTRACT))); telescope RA in degrees
	tel_dec = ten(double(strsplit(sxpar(header,'TELDEC'),':',/EXTRACT))); telescope Dec in degrees
	;
	xcen = ROUND(xsize / 2.0)
	ycen = ROUND(ysize / 2.0)
	plate_scale = platescale_est
endif else begin
	;
	;	otherwise, fetch from the header
	;
	tel_ra = sxpar(header, 'CRVAL1')
	tel_dec = sxpar(header, 'CRVAL2')
	xcen = sxpar(header, 'CRPIX1')
	ycen = sxpar(header, 'CRPIX2')
	;
	;	is there a CD matrix? if so, calc plate scale from it
	;
	cd_00 = sxpar(header, "CD1_1",COUNT=count)
	if(count gt 0) then begin
		plate_scale = 3600.0 * abs(cd_00)
	endif else begin
		plate_scale = platescale_est				; only used in region plotting
	endelse
endelse
;
crval1  = [ tel_ra, tel_dec ]
;
;	reset center to zero
;
crpix1 = [0.0d0, 0.0d0]
;
; Convert RA, Dec to Eta, Xi
;
mass_ra_tot = reform(trimmed_mass_array[0,*])
mass_dec_tot = reform(trimmed_mass_array[1,*])
num_mass = N_ELEMENTS(mass_ra_tot)
;
map_type = 2			; 'TAN'
wcssph2xy, 15.0*mass_ra_tot, mass_dec_tot, eta, xi, map_type, $
         latpole = 0.0, crxy = crpix1, ctype=['RA---TAN', 'DEC--TAN'], $
         crval = crval1
;
;	convert 2mass to notional X,Y pixel coords
;
x_2mass = fltarr(num_mass)
y_2mass = x_2mass
;
for i2m = 0, num_mass-1 do begin
	x_2mass[i2m] = xcen - eta[i2m] * 3600.0 / plate_scale
	y_2mass[i2m] = ycen + xi[i2m] * 3600.0 / plate_scale
endfor
;
;	locate closest 2mass stars to our two
;
ind_2mass = [0L, 0L]
;
for istar = 0, 1 do begin
	dmin = 1.0e10
	;
	this_x = x_mimir[istar]
	this_y = y_mimir[istar]
	;
	for i2 = 0, num_mass-1 do begin
		dx = this_x - x_2mass[i2]
		dy = this_y - y_2mass[i2]
		dist = sqrt(dx * dx + dy * dy)
		if(dist lt dmin) then begin
			dmin = dist
			ind_2mass[istar] = i2
		endif
	endfor
endfor
;
;	do the two-star solution
;
solution_ra = 15.0*[mass_ra_tot[ind_2mass[0]],mass_ra_tot[ind_2mass[1]]]
solution_dec = [mass_dec_tot[ind_2mass[0]],mass_dec_tot[ind_2mass[1]]]
;
starast, solution_ra, solution_dec, x_mimir+1, y_mimir+1, cd, HDR = final_header, projection='TAN'
;
extast, final_header, astr_struct, noparams
xy2ad,  xcen,   ycen, astr_struct, ra_cen, dec_cen
;
;	check platescale (roughly)
;
if(idebug) then begin
	this_platescale = 1./cd[0]
;	;
	print_text, Event,'2 star solution: computed platescale '+STRING(abs(this_platescale))+' pixels per degree'
;	;
endif
;
ind = where(abs((cd[0] + cd[3])/cd[0]) lt 20.0*this_tolerance and $
		abs(abs(cd[0]*3600.0)-platescale_est)/platescale_est lt 40.0*this_tolerance,ngood)
;
;platescales must be the same and opposite signs and close to estimated platescale

if(ngood le 0) then begin
	print_text, Event, "CD matrix = "+STRING(cd[0],cd[1],cd[2],cd[3],format='(4(f12.8,1x))')
	status = -1
	sxaddpar, final_header, 'GPCOORD', 0
	return,final_header
endif
;
crval1 = [ra_cen, dec_cen]
crpix1 = [float(xcen), float(ycen)]
;
offset = 0.0					; different than in examples...
make_astr, astr_struct,CD = cd, crval = crval1, crpix = crpix1+offset, ctype = ['RA---','DEC--'] + 'TAN'
;
putast, final_header, astr_struct, equi=2000.0,cd_type=2
;
n_old = N_ELEMENTS(old_hist)
for i=0, n_old-1 do begin
	sxaddpar, final_header, "HISTORY", old_hist[i]
endfor
;
sxaddpar, final_header, 'GPCOORD', 1
status=1
if(idebug) then print_text, Event, "Leaving 2 star solution with good status = 1"
return,final_header
end