function ppol_find_stars, Event, image_pathname, image, input_header, mean_FWHM, max_quad_num, sat_limit, $
						  find_scale, i_way_more, bad_pixel_fix_flag, debug, nstars_found, $
						  defocus_mode, llq_flag
;
;	i_way_more is a flag (0 or 1) that sends back whether the star field is dense
;	and might benefit from zonal 2mass magnitude star analysis
;
;i_narrow = 1			; if the images have very short duration, the PSFs can be exceedingly narrow
;						; set this flag to '1' if so (HBC722, e.g.) or '0' otherwise - DPC 20121209
;
;	llq flag, if set, says ignore LLQ info
;
;	if Event is a structure, do the following, if not, don't
;
itype = SIZE(Event, /TYPE)
if(itype eq 8) then begin
	WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_Struct
	d_ptr = PTR_Struct.PTR_Data
	s_ptr = PTR_Struct.PTR_summary
	w_ptr = PTR_struct.PTR_wid_id
	;
	;	get the astrometry override flag value
	;
	astrom_flag = (*w_ptr).S7_astrom_override_flag	; if set, sets i_way_more to 1 always
endif
;
;debug = 1
;
test_columns = 1			; test for deleted columns in each quadrant, if found, kill all
;								stars in that quadrant
;
nfwhm = 1.0					; stars must be separated by at least 1 FWHM
;
this_debug = debug
;
reg_debug = 0				; set to 1 to save found, trimmed stars in DS9 reg files
;
max_stars = 350
;
;Clean up image a little bit
;
;	if bad pixel fix flag not set, then resort to original image. if set, use fixed image
;
if(bad_pixel_fix_flag eq 0) then begin
	this_image = fix_low_pixels(image)
	working_image = this_image
endif else begin
	this_image = image						; save the bad pixels
	working_image = fix_low_pixels(image)
	working_image = fix_bad_pixels(working_image)
endelse
;
image_size = SIZE(image)
nx_image = image_size[1]
ny_image = image_size[2]
;
;	classify image as raw (1024x1026), coadded (> 1024), or subframed (< 1024)
;
if(nx_image eq 1024) then coadd = 0 else if(nx_image gt 1024) then coadd = 1 else coadd = -1
;
if(coadd eq 0 and test_columns eq 1) then begin
	;
	;	test each quadrant for large fraction of deleted pixels - or llq flag set
	;
	good_quad = intarr(4)		; LL, LR, UL, UR order
	;
	buffer = 100
	for ixq = 0, 1 do begin
		xq0 = ixq * 512
		xq1 = xq0 + 511
		;
		for iyq = 0, 1 do begin
			yq0 = iyq * 513
			yq1 = yq0 + 512
			;
			sub_quad = double(image[xq0+buffer:xq1-buffer,yq0+buffer:yq1-buffer])
			;
			n_total = double(yq1 - yq0 + 1 - 2* buffer) * double(xq1 - xq0 + 1 - 2* buffer)
			;
			sub_quad += 1.0e5						; offset to large value to put deleted and good data opposite 0
			sub_quad <= 1.0
			sub_quad >= 0.0							; deleteds are all zero, now
			;
			good_total = TOTAL(sub_quad)
			;
			good_fraction = good_total / n_total
			;
			iquad = ixq + 2*iyq
			if(good_fraction lt 0.95) then good_quad[iquad] = 0 else good_quad[iquad] = 1
			;
			if(ixq+iyq eq 0 and llq_flag eq 1) then good_quad[iquad] = 0
		endfor
	endfor
endif else begin
	good_quad = [1-llq_flag,1,1,1]
endelse
;
;	look to header for coadding counters (MPP_NSCL, MPP_DADD)
;
n_coadds = SXPAR(input_header, "MPP_NSCL", COUNT=n_test)	; this one is preferred
m_coadds = SXPAR(input_header, "MPP_DADD", COUNT=m_test)
;
;	get camera
;
this_camera_type = STRTRIM(SXPAR(input_header, "MB_CAMRA", COUNT=n_camera))
if(n_camera eq 1) then begin
	if(this_camera_type eq 'F5') then f17 = 0 else f17 = 1
endif else begin
	print_text, Event, 'ERROR - could not get camera type in PPOL_FIND_STARS. Stopping'
	stop
endelse
;
if(n_test gt 0) then begin
	working_image = working_image / float(n_coadds)
	ind = WHERE(this_image gt -1.0e5,count)
	if(count gt 0) then this_image[ind] = this_image[ind] / float(n_coadds)
endif else begin
	if(m_test gt 0) then begin
		working_image = working_image / float(m_coadds)
		ind = WHERE(this_image gt -1.0e5,count)
		if(count gt 0) then this_image[ind] = this_image[ind] / float(m_coadds)
	endif
endelse
;
;    Find sky background statistics
;
g_sky, working_image, skymode, sky_sigma, mfm_mean, mfm_stdev, /SILENT, LLQ=llq_flag
if (sky_sigma eq -1.0) then sky_sigma = 6.
if (skymode eq 0) then skymode = mfm_mean
;
;	if column test turned on and any bad quads found, set them equal to the mean background
;
if(test_columns eq 1) then begin
	bad_q_tot = TOTAL(good_quad)
	if(bad_q_tot lt 4) then begin
		;
		;	at least one bad quad, do the clean-up
		;
		for ixq = 0, 1 do begin
			xq0 = ixq * 512
			xq1 = xq0 + 511
			;
			for iyq = 0, 1 do begin
				yq0 = iyq * 513
				yq1 = yq0 + 512
				;
				iquad = ixq + 2*iyq
				if(good_quad[iquad] eq 0) then begin
					working_image[xq0:xq1, yq0:yq1] = mfm_mean
					this_image[xq0:xq1, yq0:yq1] = mfm_mean
				endif
			endfor
		endfor
	endif
endif
;
if(this_debug gt 0) then print_text, Event, "Sky mean, stdev = "+STRING(mfm_mean)+" "+STRING(mfm_stdev)
;
; get FWHM the fast way
;
fwhm_debug = 0
fwhm_loop = 0
;
fwhm_restart:
;
if(defocus_mode eq 0) then begin
	;
	;	try both narrow and non-narrow options to see which returns better (higher S/N) fwhm
	;
	best_merit = -100.0
	i_merit = -1L
	for i_narrow = 0, 1 do begin
		g_get_FWHM, this_image, working_image, skymode, skysig, mfm_mean, mfm_stdev, $
				mean_FWHM, sig_FWHM, sat_limit, fwhm_debug, AB=ab, NSTARS=n_fwhm_stars, $
				NARROW = i_narrow, STATUS=status, F17=f17
		if(status eq 0) then begin
			merit = alog10(float(n_fwhm_stars)) - alog10(sig_FWHM)		; a merit function
			if(merit gt best_merit) then begin
				best_merit = merit
				i_merit = i_narrow
			endif
		endif
	endfor
	;
	if(best_merit ne -100.0) then begin
		;
		;	use the best one
		;
		g_get_FWHM, this_image, working_image, skymode, skysig, mfm_mean, mfm_stdev, $
					mean_FWHM, sig_FWHM, sat_limit, fwhm_debug, AB=ab, NSTARS=n_fwhm_stars, $
					NARROW = i_merit, STATUS=status, F17=f17
		;
		;	and set i_narrow
		;
		i_narrow = i_merit
		print_text, Event, 'Setting I_NARROW in PPOL_FIND_STARS to '+STRING(i_narrow,format='(i10)')
		;
	endif					; else keep bad status and drop to status check
endif else begin
	g_defocus_FWHM, this_image, working_image, skymode, skysig, mfm_mean, mfm_stdev, $
			mean_FWHM, sig_FWHM, sat_limit, fwhm_debug, AB=ab, NSTARS=n_fwhm_stars, STATUS=status
	i_narrow = 0
endelse
;
if(status lt 0) then begin
	;
	; turn on debug and retry
	;
	fwhm_debug = 1
	fwhm_loop++
	if(fwhm_loop lt 2) then goto, fwhm_restart
	;
	;	image did not pass FWHM tests - flag as bad image and return
	;
	print_text, Event, 'ERROR - FWHM determination failed - Bad Image - turning off USE flag'
	star_list = -1
	goto, the_end
endif else begin
	;
	;	also, test for other error conditions
	;
	;	1. no good stars
	;
	if(sig_FWHM eq 3.0 or ab[0] eq 0.0 or ab[1] eq 0.0) then begin
		print_text, Event, 'ERROR - No good stars - Bad Image - turning off USE flag'
		star_list = -2
		goto, the_end
	endif
	;
	;	2. test for axis ratio A/B > 3/2  (1.50) - declare bad image if so
	;			soften to 2.0 for few stars
	;
	if(i_narrow eq 1) then ab_criterion = 2.5 else ab_criterion = 1.5
	if(f17 eq 1) then ab_criterion = 10.0
	;
	if( (n_fwhm_stars ge 5 and (ab[0]-ab[1]) gt ab_criterion and defocus_mode eq 0) or $
	    (n_fwhm_stars lt 5 and ab[0] gt ab_criterion and defocus_mode eq 0) )  then begin
		print_text, Event, 'ERROR - Gaussian A/B axis ratio high - Bad Image - turning off USE flag'
		star_list = -3
		goto, the_end
	endif
	;
	;	3. merit-function of 5 small tests, if 3 pass then declare bad image
	;
	bad_merit = 0
	if(n_fwhm_stars ge 5) then begin
		if( ab[1] gt 0.15) then bad_merit++
		if( ab[0] gt ab_criterion-0.15) then bad_merit++
		if( ab[0]/ab[1] lt 5.0) then bad_merit++
		if( mean_FWHM/sig_FWHM lt 10.) then bad_merit++
	endif
	if( mean_FWHM gt 3.5 and ab[0] gt ab_criterion-0.25) then bad_merit++
	;
	if( bad_merit ge 3+2*f17 and defocus_mode eq 0) then begin
		print_text, Event, 'ERROR - Image quality poor (merit = '+STRING(bad_merit,format='(i1)')+') - turning off USE flag'
		star_list = -4
		goto, the_end
	endif
endelse
;
if(this_debug gt 0) then print_text, Event, "mean FWHM, axis_ratio = "+STRING(mean_FWHM,sig_FWHM,ab[0],format='(F7.2,"+/-",f7.2,5x,F7.3)')
;
roundlim = [-1.5,1.5]
sharplim = [0.1,0.9]
;
if(mean_FWHM lt 3.0*(1.0+2.0*f17)) then begin
	roundlim = [-0.9,0.8]
	sharplim = [0.15,0.9]
endif
;roundlim = [-1.5,1.5]
;sharplim = [0.1,0.9]
;
;	initial sigma multiplier for finding stars
;
scale = find_scale							; saved between images
scale_lots_o_stars = 8.0					; need scale equal or more to designate this image as "rich" in stars
floop = 0L
;
refind:
;
i_way_more = 0
level = (scale*mfm_stdev) < sat_limit
;
if(this_debug gt 0) then  print_text, Event, "detection level = "+STRING(level)
;
FIND, working_image, single_x4, single_y4, single_flux4, sharp4, round4, level, $
						(2.0 > mean_FWHM), roundlim, sharplim, SILENT=(1-this_debug)
;
;stop
;
nstars = N_ELEMENTS(single_x4)
if(this_debug gt 0) then begin
	print_text, Event, "found "+STRING(nstars)+" stars"
	for ist = 0, nstars-1 do begin
		print_text, Event, "Star "+STRING(ist+1)+"  X, Y  = "+STRING(single_x4[ist],single_y4[ist])+"  Flux = "+STRING(single_flux4[ist])
	endfor
endif
;
;check that each star is legitimate
;
imsize = size(image)
xsize = imsize[1]
ysize = imsize[2]
;
;if(idebug) then print_text, Event, "in PpolFindStars, X,Y sizes = "+STRING(xsize)+" "+STRING(ysize)
;
;	reject any within border_size pixels of X edges and border_size of Y edges
;
old_nstars = nstars
case coadd of
	0: begin
		xborder = 2
		yborder = 20
	   end
	1: begin
		xborder = 30
		yborder = 105
	   end
   -1: begin
   		xborder = 8
   		yborder = 30
   	   end
endcase
;border_size = 30.0
ind = WHERE(single_x4 ge xborder and single_x4 le (xsize-xborder), xcount)
if(xcount gt 0) then begin
	single_x4 = single_x4[ind]
	single_y4 = single_y4[ind]
	single_flux4 = single_flux4[ind]
	nstars = xcount
endif
ind = WHERE(single_y4 ge yborder and single_y4 le (ysize-yborder), ycount)
if(ycount gt 0) then begin
	single_x4 = single_x4[ind]
	single_y4 = single_y4[ind]
	single_flux4 = single_flux4[ind]
	nstars = ycount
endif
;
;	and reject any with fitted brightness below threshold, or deleted (99.9)
;
ind = WHERE(single_flux4 gt 0.35*level and single_flux4 lt 1.0e5, lcount)
if(lcount gt 0) then begin
	single_x4 = single_x4[ind]
	single_y4 = single_y4[ind]
	single_flux4 = single_flux4[ind]
	nstars = lcount
endif
;
;	reject stars within NFHWM FWHM of each other
;
;	rank by brightness first
;
ind_bright = REVERSE(SORT(single_flux4))
;
rad_test = nfwhm*mean_FWHM
no_keep = intarr(nstars)
ns1 = nstars-1
for i = 0, ns1 do begin
	ib = ind_bright[i]
	dx = single_x4 - single_x4[ib]
	dy = single_y4 - single_y4[ib]
	dr = sqrt(dx^2 + dy^2)
	ind_s = SORT(dr)
	ind_no = WHERE(dr[ind_s] lt rad_test,n_kill)
	if(n_kill gt 1) then begin
		no_keep[ind_s[ind_no[1:(n_kill-1)]]] = 1
	endif
endfor
;
n_dead = TOTAL(no_keep,/INTEGER)
if(n_dead ge nstars) then begin
	print, 'ERROR - ppol_find_stars - no isolated stars'
	nstars_found = 0
	star_list = -10
	goto, the_end
endif
;
if(n_dead gt 0) then begin
	ind_keep = WHERE(no_keep eq 0,nstars)
	single_x4 = single_x4[ind_keep]
	single_y4 = single_y4[ind_keep]
	single_flux4 = single_flux4[ind_keep]
	;
endif
;
;	compute median flux and keep only those stars within +/- 1 mag of it
;
;median_flux = MEDIAN(single_flux4)
;flux_low = median_flux / 2.512
;flux_high = median_flux * 2.512
;ind_flux = WHERE(single_flux4 ge flux_low and single_flux4 le flux_high,n_flux)
;n_flux_bad = nstars - n_flux
;if(n_flux gt 0) then begin
;	if(n_flux ne nstars) then begin
;		single_x4 = single_x4[ind_flux]
;		single_y4 = single_y4[ind_flux]
;		single_flux4 = single_flux4[ind_flux]
;		nstars = n_flux
;	endif
;endif else goto, nstars_zero
;
;	reject any star with more than two bad or saturated pixels within 0.5 FWHM of center
;
nbad = 0L
igood = 0L
dist = 0.5*mean_FWHM
for i = 0, nstars-1 do begin
	xmin = 0 > ROUND(single_x4[i] - dist)
	xmax = (xsize-1) < ROUND(single_x4[i] + dist)
	ymin = 0 > ROUND(single_y4[i] - dist)
	ymax = (ysize-1) < ROUND(single_y4[i] + dist)
	;
	sub_image = this_image[xmin:xmax, ymin:ymax]
	;
	ind = WHERE(sub_image lt -1.0e5 or sub_image ge sat_limit, badcount)
	if(badcount gt 2) then begin
		nbad++
	endif else begin
		single_x4[igood] = single_x4[i]
		single_y4[igood] = single_y4[i]
		single_flux4[igood] = single_flux4[i]
		igood++
	endelse
endfor
;
if(this_debug gt 0) then print_text, Event, "Number rejected by bad pixel test = "+STRING(nbad)
nstars = igood
;
nstars_zero:
;
if(nstars eq 0) then begin
	if(floop lt 7) then begin
		scale = scale * 0.9
		if(scale ge 4.5) then begin
			print_text, Event, "No stars, reducing threshold to "+STRING(scale)+" sigma"
			floop++
			goto, refind
		endif
	endif
	;
	print_text, Event, "No stars found - Bad Image - turning off USE flag"
	;
	star_list = -1
	goto, the_end
endif

single_x4 = single_x4[0:igood-1]
single_y4 = single_y4[0:igood-1]
single_flux4 = single_flux4[0:igood-1]
;
nreject = 0L
igood = 0L
;
if(this_debug gt 0) then begin
	print_text, Event, "Number of stars too close to the borders, so rejected = "+STRING(old_nstars - ycount)
	print_text, Event, "Number of stars too close to neighbor stars, so rejected = "+STRING(n_dead)
	print_text, Event, "Number of stars too faint to be detected, so rejected = "+STRING(ycount - lcount)
;	print_text, Event, "Number of stars outside the +/-1 mag flux range = "+STRING(n_flux_bad)
	print_text, Event, "Number of surviving stars = "+STRING(nstars)
endif
;
;	if we've remove lots of stars, then likely a globular, so set i_way_more value
;
if(n_dead gt 100) then i_way_more = 2
;
;	add gaussian test to check each star for compliance (not in defocus mode, though)
;
if((defocus_mode eq 0) or (f17 eq 1)) then begin
	ngauss = 0L
	for i = 0, nstars-1 do begin
		;
	;	this_debug = 0
		g_test = gauss_test(working_image, mfm_mean, mfm_stdev, single_x4[i], single_y4[i],$
						mean_FWHM, x_fit, y_fit, fwhm_fit, BASIS=basis, this_debug)
		;
		;	if flag > 0, good fit, update x, y, flux values
		;
		if(g_test gt 0.0) then begin
			;
			;	test for quality of fit
			;
			dx = single_x4[i] - x_fit
			dy = single_y4[i] - y_fit
			dfw = abs(fwhm_fit - mean_FWHM)/sig_FWHM
			if(dfw gt 5.0 and dfw*sig_FWHM gt 0.35) then fwtest = 1 else fwtest = 0
			if(abs(dx) gt 1.0 or abs(dy) gt 1.0 or fwtest eq 1) then begin
				if(this_debug gt 0) then begin
					print_text, Event, "Rejected star at X,Y = "+STRING(single_x4[i],single_y4[i],format='(I4,1x,I4)')+$
					'  Basis = dx, dy, or fwhm bad'
					print_text, Event, "  dx, dy, fwtest, fwhm_fit = "+STRING(dx,dy, fwtest, fwhm_fit)
				endif
			endif else begin
				single_x4[ngauss] = x_fit
				single_y4[ngauss] = y_fit
				single_flux4[ngauss] = g_test
				ngauss++
			endelse
		endif else if(this_debug gt 0) then print_text, Event, "Rejected star at X,Y = "+STRING(single_x4[i],single_y4[i],format='(I4,1x,I4)')+$
								'  Basis = '+basis
	endfor
	;
	if(ngauss ne nstars) then begin
		if(ngauss ne 0) then begin
			nstars = ngauss
			single_x4 = single_x4[0:nstars-1]
			single_y4 = single_y4[0:nstars-1]
			single_flux4 = single_flux4[0:nstars-1]
			;
			if(debug) then print_text, Event, "After gauss_test, number of good stars = "+STRING(ngauss,format='(I)')
			;
		endif else begin
			print_text, Event, "ERROR - no stars had good gaussian fits in PPOL_FIND_STARS"
			if(floop lt 7) then begin
				scale = scale * 0.9
				if(scale ge 4.5) then begin
					;print_text, Event, "Too few stars ("+STRING(nstars,format='(I)')+"), reducing threshold to "+STRING(scale)+" sigma"
					floop++
					goto, refind
				endif
			endif else begin
				;
				;	no stars with good gaussian fits, declare bad image
				;
				print_text, Event, "ERROR - no stars had good gaussian shapes - Bad Image - turning off USE flag"
				star_list = -4
				goto, the_end
			endelse
		endelse
	endif
endif
;
if(reg_debug gt 0) then begin
	;
	;	write a region file for this image
	;
	region_dir = (*s_ptr).analysis_dir + 'S3_Astrometry' + PATH_SEP() + 'Region_Files' + PATH_SEP()
	test_dir = FILE_INFO(region_dir)
	if(NOT(test_dir.exists)) then FILE_MKDIR, region_dir
	;
	file_reg = FILE_BASENAME(image_pathname) + '_found_stars.reg'
	region_path = region_dir + file_reg
	;
	openw, lu_find, region_path, /GET_LUN
	;
	printf, lu_find, 'global color=green font="helvetica 10 normal" select=1 highlite=1 edit=1 move=1 delete=1 include=1 fixed=0 source'
	printf, lu_find, 'physical'
	for isf = 0, nstars-1 do begin
		printf, lu_find, (single_x4[isf]+1),(single_y4[isf]+1),format='("circle(",f7.1,",",f7.1,",8)")'
	endfor
	FREE_LUN, lu_find
endif
;
;stop
;
;sort by brightness
;
;ind = reverse(sort(single_flux4))
;
; if too many stars, bump scale
;
;	test for way more stars than max_quad_num in any quadrant
;
over_ratio = 5.0
i_none_in_quad = 0
;
;	do by quads
;
;determine quadrant for each star
;
ind1 = where(single_x4 lt 0.5*xsize and single_y4 ge 0.5*ysize,num_quad_1)	; UL
ind2 = where(single_x4 ge 0.5*xsize and single_y4 ge 0.5*ysize,num_quad_2)	; UR
ind3 = where(single_x4 ge 0.5*xsize and single_y4 lt 0.5*ysize,num_quad_3)	; LR
ind4 = where(single_x4 lt 0.5*xsize and single_y4 lt 0.5*ysize,num_quad_4)	; LL
;
nqstars = max([num_quad_1, num_quad_2, num_quad_3, num_quad_4])
;
if(nqstars gt max_quad_num) then begin
	if(debug gt 0) then print_text, Event, "too many stars ("+STRING(nstars,format='(I)')+"), sorting and trimming list"
	;
	if num_quad_1 gt max_quad_num then begin
	  ;
	  if(num_quad_1 gt over_ratio*max_quad_num) then if(scale ge scale_lots_o_stars) then i_way_more++
	  ;
	  quad1_bright = single_flux4[ind1]
	  sort1 = reverse(sort(quad1_bright))
	  sort1 = sort1[0:max_quad_num-1]
	  ind1 = ind1[sort1]
	  num_quad_1 = max_quad_num
	endif else begin
		if(num_quad_1 eq 0) then i_none_in_quad++
	endelse
	;
	if num_quad_2 gt max_quad_num then begin
	  ;
	  if(num_quad_2 gt over_ratio*max_quad_num) then if(scale ge scale_lots_o_stars) then i_way_more++
	  ;
	  quad2_bright = single_flux4[ind2]
	  sort2 = reverse(sort(quad2_bright))
	  sort2 = sort2[0:max_quad_num-1]
	  ind2 = ind2[sort2]
	  num_quad_2 = max_quad_num
	endif else begin
		if(num_quad_2 eq 0) then i_none_in_quad++
	endelse
	;
	if num_quad_3 gt max_quad_num then begin
	  ;
	  if(num_quad_3 gt over_ratio*max_quad_num) then if(scale ge scale_lots_o_stars) then i_way_more++
	  ;
	  quad3_bright = single_flux4[ind3]
	  sort3 = reverse(sort(quad3_bright))
	  sort3 = sort3[0:max_quad_num-1]
	  ind3 = ind3[sort3]
	  num_quad_3 = max_quad_num
	endif else begin
		if(num_quad_3 eq 0) then i_none_in_quad++
	endelse
	;
	if num_quad_4 gt max_quad_num then begin
	  ;
	  if(num_quad_4 gt over_ratio*max_quad_num) then if(scale ge scale_lots_o_stars) then i_way_more++
	  ;
	  quad4_bright = single_flux4[ind4]
	  sort4 = reverse(sort(quad4_bright))
	  sort4 = sort4[0:max_quad_num-1]
	  ind4 = ind4[sort4]
	  num_quad_4 = max_quad_num
	endif else begin
		if(num_quad_4 eq 0) then i_none_in_quad++
	endelse
	;
	if(i_way_more ge 2) then i_way_more = 1			; must have at least two quads rich
	;
	if(i_none_in_quad gt 0) then begin
		;
		;	check to be sure we don't have no stars in the quads that are turned off
		;		if so, skip refind
		;
		find_skip = 0
		if(test_columns eq 1) then begin
			;
			if(num_quad_1 eq 0 and good_quad[2] eq 1) then find_skip++
			if(num_quad_2 eq 0 and good_quad[3] eq 1) then find_skip++
			if(num_quad_3 eq 0 and good_quad[1] eq 1) then find_skip++
			if(num_quad_4 eq 0 and good_quad[0] eq 1) then find_skip++
		endif else begin
			find_skip = 1
		endelse
		;
		if(floop lt 7 and find_skip ne 0) then begin
			scale = scale * 0.9
			if(scale ge 4.5) then begin
				print_text, Event, "No stars in at least one quadrant, reducing threshold to "+STRING(scale)+" sigma"
				floop++
				goto, refind
			endif
		endif
	endif
	;
	nstars = num_quad_1+num_quad_2+num_quad_3+num_quad_4
	;
	if(this_debug gt 0) then print_text, Event, "Sorted stars by quad - nq = "+$
		STRING(num_quad_1,num_quad_2,num_quad_3,num_quad_4)+"  final total = "+STRING(nstars)
	;
endif else begin
	;
	;	too few, lower threshold
	;
	if(nstars lt 1.5*max_quad_num) then begin
		if(floop lt 7) then begin
			if(nstars ge max_quad_num) then begin
				scale = (scale * 0.9) > 4.4
			endif else begin
				scale = (scale * 0.75) > 4.4
			endelse
			if(scale ge 4.5) then begin
				print_text, Event, "Too few stars ("+STRING(nstars,format='(I)')+"), reducing threshold to "+STRING(scale)+" sigma"
				floop++
				goto, refind
			endif
		endif
	endif
	;
endelse
;
x4 = fltarr(num_quad_1 + num_quad_2 + num_quad_3 + num_quad_4)
y4 = x4
s4 = x4
q4 = x4
;
nq = [num_quad_1, num_quad_2, num_quad_3, num_quad_4]
ii = 0L
for iq = 0, 3 do begin
	if(nq[iq] ne 0) then begin
		istart = ii
		iend = ii+nq[iq]-1
		;
		case iq of
			0:	ind = ind1
			1:  ind = ind2
			2:  ind = ind3
			3:  ind = ind4
		endcase
		;
		x4[istart:iend] = single_x4[ind]
		y4[istart:iend] = single_y4[ind]
		s4[istart:iend] = single_flux4[ind]
		q4[istart:iend] = iq+1
		;
		ii = iend+1
	endif
endfor
;
single_x4 = x4
single_y4 = y4
single_flux4 = s4
nstars_found = ii
;
stars_x_tot = single_x4
stars_y_tot = single_y4
stars_flux_tot = single_flux4
;
;reserve space to store quadrant they are located in
;
stars_quad = q4
;
if(reg_debug gt 0) then begin
	;
	;	write another region file
	;
	region_dir = (*s_ptr).analysis_dir + 'S3_Astrometry' + PATH_SEP() + 'Region_Files' + PATH_SEP()
	test_dir = FILE_INFO(region_dir)
	if(NOT(test_dir.exists)) then FILE_MKDIR, region_dir
	;
	file_reg = FILE_BASENAME(image_pathname) + '_trimmed_stars.reg'
	region_path = region_dir + file_reg
	;
	openw, lu_find, region_path, /GET_LUN
	;
	printf, lu_find, 'global color=red font="helvetica 10 normal" select=1 highlite=1 edit=1 move=1 delete=1 include=1 fixed=0 source'
	printf, lu_find, 'physical'
	for isf = 0, nstars-1 do begin
		printf, lu_find, (single_x4[isf]+1),(single_y4[isf]+1),format='("circle(",f7.1,",",f7.1,",6)")'
	endfor
	FREE_LUN, lu_find
endif
;
;stop
;
star_list = transpose([[stars_x_tot],[stars_y_tot],[stars_quad],[stars_flux_tot]])
;
the_end:
;
;	astrom flag
;
if(astrom_flag eq 1) then i_way_more = 1
;
return,star_list

end
