pro S11_PInst_correct, Event, deep_path, group_colors, ig, status
;
;	performs PInst corrections -
;
;		first, removal of U, Q inst terms
;
;		second, correction for efficiency
;
;		third, rotation into equatorial system
;
;	DPC	20110829
;
WIDGET_CONTROL, Event.top, GET_UVALUE=PTR_struct
w_ptr = PTR_struct.PTR_wid_id
s_ptr = PTR_struct.PTR_summary
g_ptr = PTR_struct.PTR_group
;
meta_flag = (*w_ptr).S11_meta_group_mode
;
LED1_wid = (*w_ptr).S11_LED1_groups_wid
;
print_text, Event, 'Beginning PInst Corrections of Raw Q, U images'
group_colors[ig] = 'white'
WCT_LED_stack, LED1_wid, group_colors
;
;	get correct value for obs year/date from header of deep image
;
deep_header = HEADFITS( deep_path, /silent)
;
date_obs = SXPAR(deep_header, 'DATE-OBS')
reads, date_obs, this_year, this_month, format='(I4,1x,I2)'
;
;		decimal year
;
this_decimal_year = this_year + float(this_month) / 12.0
;
;		select calibration block from availables
;
end_yr_block = (*(*w_ptr).S9_P_inst_values_ptr).end_yr_block
ind_block = WHERE(this_decimal_year lt end_yr_block,nyr)
if(nyr gt 0) then begin
	this_block = ind_block[0]
	(*(*w_ptr).S9_P_inst_values_ptr).this_pol_off_ind = this_block
endif
;
dtor=!pi / 180.
;
;	fetch P_inst contants directly
;
P_efficiency 	= (*(*w_ptr).S9_P_inst_values_ptr).pol_efficiency
sig_P_efficiency 	= (*(*w_ptr).S9_P_inst_values_ptr).s_pol_effic
datestamp 		= (*(*w_ptr).S9_P_inst_values_ptr).date_stamp
;
;	pointer to the right year's offset angle - from above
;
jpol			= (*(*w_ptr).S9_P_inst_values_ptr).this_pol_off_ind
;
print_text, Event, 'matched to PInst time block '+STRING(jpol+1,format='(I2)')
;
;	get the right offset angle
;
P_offset     		= (*(*w_ptr).S9_P_inst_values_ptr).pol_off_angle[jpol] * dtor
sig_P_offset   	= (*(*w_ptr).S9_P_inst_values_ptr).s_pol_off_angle[jpol] * dtor
;
;	the instrumental corrections from heap
;
u_inst_ptr = (*w_ptr).S9_u_inst_ptr
q_inst_ptr = (*w_ptr).S9_q_inst_ptr
sig_u_ptr  = (*w_ptr).S9_u_sig_inst_ptr
sig_q_ptr  = (*w_ptr).S9_q_sig_inst_ptr
;
;	read the raw Stokes Q, U and uncertainties
;
dir_in = FILE_DIRNAME( deep_path )
file_in = FILE_BASENAME( deep_path )
;
test_fits = STRPOS( STRUPCASE(file_in), 'DEEP_COADD.FITS')
if(test_fits ne -1) then begin
	QU_root = STRMID( file_in, 0, test_fits)
endif else begin
	print_text, Event, "ERROR - Could not parse filename"
	status = -1
	goto, S11PC_end
endelse
;
;		and directory
;
test_S7 = STRPOS( dir_in, 'S7_' )
if(test_S7 ne -1) then begin
	new_dir = STRMID( dir_in, 0, test_S7 ) + 'S11_Full_Field_Polarimetry'
	test = FILE_INFO(new_dir)
	if(test.exists eq 1 and test.directory eq 1) then begin
		dir_hwp = new_dir
	endif else begin
		print_text, Event, "ERROR - S11 Directory does not exist"
		status = -1
		goto, S11PC_end
	endelse
endif else begin
	print_text, Event, "ERROR - Deep filename does not contain 'S7_' "
	status = -1
	goto, S11PC_end
endelse
;

Q_path = dir_hwp + PATH_SEP() + QU_root + 'Q.fits'
Q_header = HEADFITS( Q_path )
ncol = SXPAR(Q_header, 'NAXIS1')
nrow = SXPAR(Q_header, 'NAXIS2')
;
mpp_xoff = SXPAR(Q_header, 'MPP_XOFF')	; amount to add to x pix to get detector x pix
mpp_yoff = SXPAR(Q_header, 'MPP_YOFF')
;
ptr_QU = PTR_NEW( {		raw_Q:		fltarr(ncol, nrow),	$
						sig_raw_Q:	fltarr(ncol, nrow), $
						raw_U:		fltarr(ncol, nrow),	$
						sig_raw_U:	fltarr(ncol, nrow) } )
;
ptr_out = PTR_NEW( {	  Q:		fltarr(ncol, nrow), $
						 sQ:		fltarr(ncol, nrow), $
						  U:		fltarr(ncol, nrow), $
						 sU:		fltarr(ncol, nrow) } )
;
(*ptr_QU).raw_Q = readfits(Q_path, Q_header, /silent)
;
sQ_path = dir_hwp + PATH_SEP() + QU_root + 'sQ.fits'
(*ptr_QU).sig_raw_Q = readfits(sQ_path, sQ_header, /silent)
;
U_path = dir_hwp + PATH_SEP() + QU_root + 'U.fits'
(*ptr_QU).raw_U = readfits(U_path, U_header, /silent)
;
sU_path = dir_hwp + PATH_SEP() + QU_root + 'sU.fits'
(*ptr_QU).sig_raw_U = readfits(sU_path, sU_header, /silent)
;
print_text, Event, 'Read Stokes Images from disk'
;
;
;	projection cos and sin for offset angle and efficiency
;
c		=	cos(P_offset * 2.0) / P_efficiency
s		=	sin(P_offset * 2.0) / P_efficiency
sig_c 	= 	(1./P_efficiency) * sqrt( (sig_P_efficiency^2)*(sin(P_offset * 2.0))^2 + $
				4.0 * sig_P_offset^2 * (cos(P_offset * 2.0))^2 )
sig_s 	= 	(1./P_efficiency) * sqrt( (sig_P_efficiency^2)*(cos(P_offset * 2.0))^2 + $
				4.0 * sig_P_offset^2 * (sin(P_offset * 2.0))^2 )
;
;	for every pixel in the image
;
for ix = 0, ncol-1 do begin
	for iy = 0, nrow-1 do begin
		;
		;	check that this isn't a deleted pixel
		;
		if( (*ptr_QU).sig_raw_U[ix,iy] gt 1.0e5 or (*ptr_QU).sig_raw_Q[ix,iy] gt 1.0e5) then begin
			(*ptr_out).Q[ix,iy] = 0.0
			(*ptr_out).sQ[ix,iy] = 1.0e6
			(*ptr_out).U[ix,iy] = 0.0
			(*ptr_out).sU[ix,iy] = 1.0e6
		endif else begin
			;
			;	scale U, Q values to percentages to match instrumental values
			;
			raw_u		=	100.0 * (*ptr_QU).raw_U[ix,iy]
			raw_q		=	100.0 * (*ptr_QU).raw_Q[ix,iy]
			sig_raw_u 	= 	100.0 * (*ptr_QU).sig_raw_U[ix,iy]
			sig_raw_q 	= 	100.0 * (*ptr_QU).sig_raw_Q[ix,iy]
			;
			;	image pixel's mean location on the detector array
			;
			starx = (0 > (ix + ROUND(mpp_xoff))) < 1023
			stary = (0 > (iy + ROUND(mpp_yoff))) < 1023
			;
			;	instrumental correction terms at that location
			;
			this_u_inst 	= (*u_inst_ptr)[starx, stary]
			this_q_inst 	= (*q_inst_ptr)[starx, stary]
			this_sig_u_inst = (*sig_u_ptr)[starx, stary]
			this_sig_q_inst = (*sig_q_ptr)[starx, stary]
			;
			;	perform the correction of the star's raw U, Q to the final U, Q
			;
			dQ 		= raw_q - this_q_inst
			dU 		= raw_u - this_u_inst
			sig_dU 	= sqrt(this_sig_u_inst^2 + sig_raw_u^2)
			sig_dQ 	= sqrt(this_sig_q_inst^2 + sig_raw_q^2)
			;
			;	rotate and scale
			;
			q_cor	= dQ * c  -  dU * s
			u_cor	= dU * c  +  dQ * s
			;
			;	compute uncertainties
			;
			sig_q_cor = sqrt( (c * sig_dQ)^2 + (dQ * sig_c)^2 + (s * sig_dU)^2 + (dU * sig_s)^2 )
			sig_u_cor = sqrt( (c * sig_du)^2 + (dU * sig_c)^2 + (s * sig_dQ)^2 + (dQ * sig_s)^2 )
			;
			;	save results (as fractions, not percentages)
			;
			(*ptr_out).Q[ix,iy] = q_cor / 100.0
			(*ptr_out).sQ[ix,iy] = sig_q_cor / 100.0
			(*ptr_out).U[ix,iy] = u_cor / 100.0
			(*ptr_out).sU[ix,iy] = sig_u_cor / 100.0
			;
		endelse
	endfor
endfor
;
;	save the images
;
Q_path = dir_hwp + PATH_SEP() + QU_root + 'Q_cor.fits'
sQ_path = dir_hwp + PATH_SEP() + QU_root + 'sQ_cor.fits'
U_path = dir_hwp + PATH_SEP() + QU_root + 'U_cor.fits'
sU_path = dir_hwp + PATH_SEP() + QU_root + 'sU_cor.fits'
;
;	update headers with PInst correction parameters
;
history = strarr(8)
history[0] = 'PINST_FL= '+(*w_ptr).S11_Pinst_pathname
history[1] = 'DATESTMP= '+STRING((*(*w_ptr).S9_P_inst_values_ptr).date_stamp,$
					format='(I8)')+" / YYYYMMDD of PInst Determination" 		;
this_block = (*(*w_ptr).S9_P_inst_values_ptr).this_pol_off_ind
history[2] = 'PINST_BK= '+STRING(this_block+1,format='(i2)')
history[3] = 'END_YR  = '+STRING((*(*w_ptr).S9_P_inst_values_ptr).end_yr_block[this_block],$
					format='(f10.3)')+" / This block end decimal year"
history[4] = 'PA_OFF  = '+STRING((*(*w_ptr).S9_P_inst_values_ptr).pol_off_angle[this_block],$
					format='(f10.3)')+" / PA offset to add to Mimir values [deg]"
history[5] = 'S_PA_OFF= '+STRING((*(*w_ptr).S9_P_inst_values_ptr).s_pol_off_angle[this_block],$
					format='(f10.3)')+" / PA offset uncertainty [deg]"
history[6] = 'P_EFFIC = '+STRING(100.0*(*(*w_ptr).S9_P_inst_values_ptr).pol_efficiency,$
					format='(f10.3)')+" / Polarization efficiency [%]"
history[7] = 'S_P_EFF = '+STRING(100.0*(*(*w_ptr).S9_P_inst_values_ptr).s_pol_effic,$
					format='(f10.3)')+" / Uncertainty of Pol. Effic. [%]"
;
SXADDHIST, history, q_header
SXADDHIST, history, sq_header
SXADDHIST, history, u_header
SXADDHIST, history, su_header
;
writefits, Q_path, (*ptr_out).Q, q_header
writefits, sQ_path, (*ptr_out).sQ, sq_header
writefits, U_path, (*ptr_out).U, u_header
writefits, sU_path, (*ptr_out).sU, su_header
;
print_text, Event, 'Wrote Stokes Images to disk'
group_colors[ig] = 'lawngreen'
WCT_LED_stack, LED1_wid, group_colors
print_text, Event, 'Completed PInst correction of Raw->Corr Q, U images'
;
PTR_FREE, ptr_QU, ptr_out
wait, 0.3
;
status = 0
;
S11PC_end:
;
end