function S7_mask_stars, input_image, SIGMA_SCALE = this_sigma_scale
;
;	finds and punches out pixels with stellar flux contributions
;		in an image
;
;	DPC  20100825
;	     20140707 added sigma_scale to allow overriding
;
if(N_ELEMENTS(this_sigma_scale) eq 0) then sigma_scale_value = 3.0 else sigma_scale_value = this_sigma_scale
;
im_size = SIZE(input_image)
nx = im_size[1]
ny = im_size[2]
;
image_data_ptr = PTR_NEW(input_image)
sky_image_ptr = PTR_NEW(fltarr(nx,ny))
;
;	remove variable sky for this image and save the rms value
;
(*sky_image_ptr) = model_background((*image_data_ptr), SKY=sky, $
		SIGMA = sky_sigma, /MCS, /NOROWF, /NOCOLF)
;
;	compute the sky sigma from the incoming sky values (not great, but inexpensive)
;
sky_cut = sky + (sigma_scale_value *  sky_sigma)
;
good_mask_ptr				= PTR_NEW(intarr(nx,ny))
is_sky_ptr					= PTR_NEW(intarr(nx,ny))
;
;	and compute whether each pixel is looking at "sky" or not - yes if under 3 sigma of sky noise
;		plus mean sky
;
(*is_sky_ptr) = ((1 + FLOOR( sky_cut - (*sky_image_ptr) ) ) > 0) < 1		; 1 is sky, 0 is non-sky
;
;	and test its neighbors, borrowing the good_mask_ptr array for now
;
xy_shift = [[0, -1, 0, 1, -2, -1, 1, 2, -1, 0, 1, 0], $		; x offsets
			[2, 1, 1, 1, 0, 0, 0, 0, -1, -1, -1, -2]]			; y offsets
for ixy = 0, 11 do begin
	(*good_mask_ptr) += SHIFT( (*is_sky_ptr), xy_shift[ixy,0], xy_shift[ixy,1] )
endfor
;
;	keep as sky any pixel with surrounding count > 10
;
(*good_mask_ptr) -= 10
(*good_mask_ptr) <= 1
(*good_mask_ptr) >= 0				; 0 is not-sky, 1 is sky
;
;	using new mask, either keep original sky value (if "sky" pixel) or
;		mean background sky value (if "star" pixel)
;
;	the background to use for the star pixels is the original minus the fitted, with the sky added back in
;

output_image = (*good_mask_ptr) * (*image_data_ptr) + (1 - (*good_mask_ptr)) * ( (*image_data_ptr) - (*sky_image_ptr) + sky)
;
PTR_FREE, image_data_ptr, sky_image_ptr
PTR_FREE, good_mask_ptr
PTR_FREE, is_sky_ptr
;
return, output_image
;
end

